//
// contact base term
//
// Description: contact term
//
//
// Author:  <Gauthier BECKER>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
#include "contactTerms.h"
#include "unknownField.h"
template<> void rigidContactLinearTermBase<double>::get(MElement *ele, int npts, IntPt *GP,fullVector<double> &m) const
{
  nbdofs = _spc->getNumKeys(ele);
  nbdofsGC = _spc->getNumKeysOfGravityCenter();
  m.resize(nbdofs);
  m.setAll(0.);
  nbvertex = ele->getNumVertices();
  nbcomp = (nbdofs-nbdofsGC)/nbvertex;
  nbcomptimenbvertex = nbvertex*nbcomp;
  // get displacement of vertices (To know the current node positions)
  disp.clear(); R.clear();
  _spc->getKeys(ele,R);

  _ufield->get(R,disp);

  verdisp[3] = disp[nbcomptimenbvertex];
  verdisp[4] = disp[nbcomptimenbvertex+1];
  verdisp[5] = disp[nbcomptimenbvertex+2];

  for(int i=0;i<nbvertex;i++){
    for(int j=0;j<6;j++) mvalue[j] =0.;
    ver = ele->getVertex(i);
    verdisp[0] = disp[i];
    verdisp[1] = disp[i+nbvertex];
    verdisp[2] = disp[i+2*nbvertex];

    this->get(ver,verdisp,mvalue);
    // look if there is contact force
    double val = mvalue[0]*mvalue[0]+mvalue[1]*mvalue[1]+mvalue[2]*mvalue[2];
    if (val > 0){
      // in conact
      _allcontactNode->insert(ele,i,_lastNormalContact);
      for(int j=0;j<nbcomp;j++){
        m(i+j*nbvertex) += mvalue[j];
        m(nbcomptimenbvertex+j) += mvalue[j+3]; // count two times
      }
    }
    /*
    int jj=0;
    while(jj<6){
      if(mvalue[jj] != 0.){
//        Msg::Info("Contact detected on elem %d vertex %d",ele->getNum(),ele->getVertex(i)->getNum());
        _allcontactNode->insert(ele,i,_lastNormalContact);
        for(int j=0;j<nbcomp;j++){
          m(i+j*nbvertex) += mvalue[j];
          m(nbcomptimenbvertex+j) += mvalue[j+3]; // count two times
        }
        break;
      }
      jj++;
    }
    */
  }
 // if(m.norm() != 0.)
//    m.print("contact");
}

template<> void defoDefoContactBilinearTermByPerturbation<double>::get(contactElement *cE, int npts, IntPt *GP, fullMatrix<double> &m) const
{
  int nbdofs = _spc->getNumKeys(cE);
  m.resize(nbdofs,nbdofs);
  m.setAll(0.); //here we need to fill slave node 1, x, slave node 2, x,slave node 3, x, ... slave node 1, y etc and then master node 1, x, master node 2, x, etc
  static std::vector<Dof> keys;
  keys.clear();
  _spc->getKeys(cE,keys);
  static fullVector<double> disp,Fplus,Fminus;
  disp.resize(nbdofs);
  disp.setAll(0.);
  _ufield->get(keys,disp); 
  defoDefoContactLinearTermBase<double> *defoDefoTerm = static_cast<defoDefoContactLinearTermBase<double> *> (_lterm);


  for(int j=0; j<nbdofs; j++)
  {
    disp(j)+=_pert;
    //defoDefoTerm->detectNodesInContact(cE,disp);  //we should do the detection before the perturbation if we put the function out
    defoDefoTerm->get(cE,npts,GP,Fplus,disp);
    disp(j)-=2.*_pert;
    //defoDefoTerm->detectNodesInContact(cE,disp);  //we should do the detection before the perturbation
    defoDefoTerm->get(cE,npts,GP,Fminus,disp);
    disp(j)+=_pert;
    for(int i=0; i<nbdofs; i++)
    {
      m(i,j)=(Fplus(i)-Fminus(i))/(-2.*_pert); //minus because stiffness related toof internal force
    }
  }
  defoDefoTerm->get(cE,npts,GP,Fminus,disp);// put back the right _nodesInContact
  //m.print("numerical defo defo stiffness matrix");
}

