//
// Description: Storing class for gurson Coalescence
//
//
// Author:  <J. Leclerc>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "ipCoalescence.h"
#include "restartManager.h"
#include "ipNonLocalPorosity.h"

// IPCoalescenceBase
// -------------------------------------------
IPCoalescence::IPCoalescence():  _acceleratedRate(1.),_porosityAtCoalescenceOnset(0.), 
                                  _coalescenceOnsetFlag(false),_ipvAtCoalescenceOnset(NULL),
																 _shearCoalescenceOnsetFlag(false)
{

};


IPCoalescence::IPCoalescence(const IPCoalescence &source) :
    _acceleratedRate(source._acceleratedRate), _porosityAtCoalescenceOnset(source._porosityAtCoalescenceOnset),
    _coalescenceOnsetFlag(source._coalescenceOnsetFlag), 
   _shearCoalescenceOnsetFlag(source._shearCoalescenceOnsetFlag)
{
  _ipvAtCoalescenceOnset = NULL;
  if (source._ipvAtCoalescenceOnset != NULL){
    _ipvAtCoalescenceOnset = dynamic_cast<IPNonLocalPorosity*>(source._ipvAtCoalescenceOnset->clone());
  }

};


IPCoalescence &IPCoalescence::operator=(const IPCoalescence &source)
{
  _acceleratedRate = source._acceleratedRate;
  _porosityAtCoalescenceOnset = source._porosityAtCoalescenceOnset;
  _coalescenceOnsetFlag = source._coalescenceOnsetFlag;
  _shearCoalescenceOnsetFlag = source._shearCoalescenceOnsetFlag;
  
  if (source._ipvAtCoalescenceOnset != NULL){
    if (_ipvAtCoalescenceOnset != NULL){
      _ipvAtCoalescenceOnset->operator=(*dynamic_cast<const IPVariable*>(source._ipvAtCoalescenceOnset));
    }
    else{
      _ipvAtCoalescenceOnset =dynamic_cast<IPNonLocalPorosity*>(source._ipvAtCoalescenceOnset->clone());
    }
  }
  else{
  if (_ipvAtCoalescenceOnset) {delete _ipvAtCoalescenceOnset; _ipvAtCoalescenceOnset = NULL;}
  }

  return *this;
};

IPCoalescence::~IPCoalescence(){
  if (_ipvAtCoalescenceOnset) {
    delete _ipvAtCoalescenceOnset;
    _ipvAtCoalescenceOnset = NULL;
  }
  
};


void IPCoalescence::restart()
{
  restartManager::restart(_acceleratedRate);
  restartManager::restart(_coalescenceOnsetFlag);
	restartManager::restart(_shearCoalescenceOnsetFlag);
  restartManager::restart(_porosityAtCoalescenceOnset);
  restartManager::restart(_ipvAtCoalescenceOnset);
};

const IPNonLocalPorosity* IPCoalescence::getIPvAtCoalescenceOnset() const{
  if (_ipvAtCoalescenceOnset == NULL){
    Msg::Error("IPCoalescence::getIPVoidStateAtCoalescenceOnset in an ipv does not exist");
  }
  else{
    return _ipvAtCoalescenceOnset;
  }
};
void IPCoalescence::setIPvAtCoalescenceOnset(const IPNonLocalPorosity& ipvVoid){
  if (_ipvAtCoalescenceOnset == NULL){
    _ipvAtCoalescenceOnset = dynamic_cast<IPNonLocalPorosity*>(ipvVoid.clone());
  } 
  else{
    _ipvAtCoalescenceOnset->operator =(*dynamic_cast<const IPVariable*>(&ipvVoid));
  }
};


// IPNoCoalescence
// -------------------------------------------
IPNoCoalescence::IPNoCoalescence() : IPCoalescence()
{
};


IPNoCoalescence::IPNoCoalescence(const IPNoCoalescence &source) : IPCoalescence(source)
{
};


IPNoCoalescence &IPNoCoalescence::operator=(const IPCoalescence &source)
{
  IPCoalescence::operator=(source);
  return *this;
};

IPCoalescence * IPNoCoalescence::clone() const
{
  return new IPNoCoalescence(*this);
};



// IPFstarCoalescence
// -------------------------------------------
IPFstarCoalescence::IPFstarCoalescence() : IPCoalescence()
{

};


IPFstarCoalescence::IPFstarCoalescence(const IPFstarCoalescence &source) : IPCoalescence(source)
{

};


IPFstarCoalescence &IPFstarCoalescence::operator=(const IPCoalescence &source)
{
  IPCoalescence::operator=(source);
  return *this;
};


IPCoalescence * IPFstarCoalescence::clone() const
{
  return new IPFstarCoalescence(*this);
};


/* IPThomasonCoalescence */
// -------------------------------------------
IPThomasonCoalescence::IPThomasonCoalescence(): IPCoalescence(),
  crackOffsetOnCft(1.), yieldOffset(0.), LodeOffset(1.)
{

};


IPThomasonCoalescence::IPThomasonCoalescence(const IPThomasonCoalescence& src): IPCoalescence(src),
  crackOffsetOnCft(src.crackOffsetOnCft), yieldOffset(src.yieldOffset),
  LodeOffset(src.LodeOffset)
{

};


IPThomasonCoalescence& IPThomasonCoalescence::operator = (const IPCoalescence& src){
  IPCoalescence::operator =(src);
  const IPThomasonCoalescence* psrc = dynamic_cast<const IPThomasonCoalescence*>(&src);
  if (psrc != NULL)
  {
    crackOffsetOnCft = psrc->crackOffsetOnCft;
    yieldOffset = psrc->yieldOffset;
    LodeOffset = psrc->LodeOffset;
  }
  return *this;
};


void IPThomasonCoalescence::restart(){
  IPCoalescence::restart();
  restartManager::restart(crackOffsetOnCft);
  restartManager::restart(yieldOffset);
  restartManager::restart(LodeOffset);

};

