//
// C++ Interface: terms
//
// Description: Class to compute Internal point
//
//
// Author:  <Gauthier BECKER>, (C) 2010
// <Van Dung NGUYEN>, (C) 2019
// Copyright: See COPYING file that comes with this distribution
//
//
# ifndef _IPFIELD_H_
# define _IPFIELD_H_

#ifndef SWIG
#include <vector>
#include "quadratureRules.h"
#include "unknownField.h"
#include "elementsField.h"
#include "GModel.h"
#include "ipstate.h"
#include "partDomain.h"
#include "InterfaceKeys.h"

class TwoNum{
  public:
    int small, large;
    TwoNum(){ //define for restartManager
      small=0;
      large=0;
    }
    TwoNum(const int e1, const int e2){
      if (e1 < e2) {
        small = e1;
        large = e2;
      }
      else{
        small = e2;
        large = e1;
      }
    }
    TwoNum(const int e1): small(e1),large(e1){}
    TwoNum(const TwoNum& src): small(src.small),large(src.large){}
    ~TwoNum(){};
    TwoNum& operator = (const TwoNum& other) {
      small = other.small;
      large = other.large;
      return *this;
    };
    bool operator < (const TwoNum &other) const{
      if (small < other.small) return true;
      if (small > other.small) return false;
      if (large < other.large) return true;
      return false;
    }
    bool operator == (const TwoNum &other) const{
      return (small == other.small && large == other.large);
    }
    bool operator != (const TwoNum &other) const{
      return (small != other.small  || large != other.large);
    };
    /*void restart()
    {
      restartManager::restart(small);
      restartManager::restart(large);
      return;
    }*/

};

#endif //SWIG

class IPField : public elementsField {
  public:
    enum  Output { SVM=0,SIG_XX, SIG_YY, SIG_ZZ, SIG_XY, SIG_YZ, SIG_XZ, DAMAGE, PLASTICSTRAIN, GL_EQUIVALENT_STRAIN,
                    STRAIN_XX, STRAIN_YY, STRAIN_ZZ, STRAIN_XY, STRAIN_YZ, STRAIN_XZ, HIGHSTRAIN_NORM,
                    HIGHSTRESS_NORM, GL_NORM, STRESS_NORM, P_STRESS_NORM , F_XX, F_XY, F_XZ, F_YX, F_YY,F_YZ,
                    F_ZX, F_ZY, F_ZZ, P_XX, P_XY, P_XZ,P_YX, P_YY, P_YZ, P_ZX, P_ZY, P_ZZ,
                    G_XXX, G_XXY, G_XXZ, G_XYX, G_XYY, G_XYZ,G_XZX, G_XZY, G_XZZ,
                    G_YXX, G_YXY, G_YXZ, G_YYX, G_YYY, G_YYZ,G_YZX, G_YZY, G_YZZ,
                    G_ZXX, G_ZXY, G_ZXZ, G_ZYX, G_ZYY, G_ZYZ,G_ZZX, G_ZZY, G_ZZZ,
                    Q_XXX, Q_XXY, Q_XXZ, Q_XYX, Q_XYY, Q_XYZ,Q_XZX, Q_XZY, Q_XZZ,
                    Q_YXX, Q_YXY, Q_YXZ, Q_YYX, Q_YYY, Q_YYZ,Q_YZX, Q_YZY, Q_YZZ,
                    Q_ZXX, Q_ZXY, Q_ZXZ, Q_ZYX, Q_ZYY, Q_ZYZ,Q_ZZX, Q_ZZY, Q_ZZZ,
                    MTX_SVM,MTX_SIG_XX,MTX_SIG_YY,MTX_SIG_ZZ,MTX_SIG_XY,MTX_SIG_YZ,MTX_SIG_XZ,
                    INC_SVM,INC_SIG_XX,INC_SIG_YY,INC_SIG_ZZ,INC_SIG_XY,INC_SIG_YZ,INC_SIG_XZ,
                    TEMPERATURE,THERMALFLUX_X,THERMALFLUX_Y,THERMALFLUX_Z,
                    VOLTAGE,ELECTRICALFLUX_X,ELECTRICALFLUX_Y,ELECTRICALFLUX_Z,
                    LOCAL_POROSITY, NONLOCAL_POROSITY, CORRECTED_POROSITY, YIELD_POROSITY,
                    NUCLEATION_ONSET, NUCLEATED_POROSITY, LIGAMENT_RATIO,
                    PRESSION, PLASTIC_POISSON_RATIO,PLASTICSTRAIN_COMPRESSION, PLASTICSTRAIN_TRACTION, PLASTICSTRAIN_SHEAR,
                    NONLOCAL_PLASTICSTRAIN,BACKSTRESS, JACOBIAN, ELASTIC_JACOBIAN, PLASTIC_JACOBIAN,
                    PLASTIC_RATE,FLOW_FACTOR_RATE, OCTAHEDRAL_SHEAR_STRAIN_RATE, OCTAHEDRAL_NORM_STRAIN_RATE,
                    K_XX, K_YY, K_ZZ, K_XY, K_YZ, K_XZ,FAILURE_ONSET, GL_EQUIVALENT_STRAIN_RATE,
                    NONLOCAL_FAILURE_PLASTICSTRAIN, FAILURE_PLASTICSTRAIN, DAMAGE_0, DAMAGE_1, LOCAL_EQUIVALENT_STRAINS, NONLOCAL_EQUIVALENT_STRAINS,
                    FIRST_PRINCIPAL_STRESS, SECOND_PRINCIPAL_STRESS, THIRD_PRINCIPAL_STRESS, GL_EFFECTIVE_STRAIN_RATE,
                    DISP_JUMP_X,DISP_JUMP_Y,DISP_JUMP_Z,STRESS_TRIAXIALITY,ACTIVE_DAMAGING_ZONE,
                    FIELD_CAPACITY, MECHANICAL_SOURCE, FIELD_SOURCE,INCOMPATIBLE_STRAIN_X,INCOMPATIBLE_STRAIN_Y,INCOMPATIBLE_STRAIN_Z,
                    COHESIVE_JUMP_X,COHESIVE_JUMP_Y,COHESIVE_JUMP_Z,DAMAGE_IS_BLOCKED,
                    DEFO_ENERGY,PLASTIC_ENERGY,IRREVERSIBLE_ENERGY,DAMAGE_ENERGY,
                    MTX_STRAIN_XX,MTX_STRAIN_YY,MTX_STRAIN_ZZ,MTX_STRAIN_XY,MTX_STRAIN_YZ,MTX_STRAIN_XZ,
                    INC_STRAIN_XX,INC_STRAIN_YY,INC_STRAIN_ZZ,INC_STRAIN_XY,INC_STRAIN_YZ,INC_STRAIN_XZ,
                    EXTRAFIELD_1,EXTRAFIELD_2,EXTRAFIELD_3,EXTRAFIELD_4,EXTRAFIELD_5,EXTRAFIELD_6,EXTRAFIELD_7,EXTRAFIELD_8,EXTRAFIELD_9,
                    EXTRAFIELD_NORM, DELETED,LOST_ELLIPTICITY,
                    LOCAL_0,LOCAL_1, LOCAL_2,
                    NONLOCAL_0, NONLOCAL_1, NONLOCAL_2, CONSTITUTIVEEXTRADOFDIFFUSION_0, CONSTITUTIVEEXTRADOFDIFFUSION_1, CONSTITUTIVEEXTRADOFDIFFUSION_2, FAILED,
                    GL_XX, GL_XY, GL_XZ, GL_YY, GL_YZ, GL_ZZ, COALESCENCE_CRITERION, COALESCENCE, COALESCENCE_NECKING, COALESCENCE_SHEAR,POROSITY_COALESCENCE_ONSET,
                    CL_XX, CL_YY, CL_ZZ, VOLUMETRIC_PLASTIC_STRAIN, LODE_PARAMETER, LODE_ANGLE,DEVIATORIC_PLASTIC_STRAIN,
                    ASPECT_RATIO, SHAPE_FACTOR, LIGAMENT_RATIO_COALESCENCE_ONSET,ASPECT_RATIO_COALESCENCE_ONSET, SHAPE_FACTOR_COALESCENCE_ONSET,
                    LIMIT_LOAD_FACTOR,
                    INC_VOLUME_RATIO, INC_ASPECT_RATIO_1, INC_ASPECT_RATIO_2, INC_R_XX, INC_R_XY, INC_R_XZ, INC_R_YX, INC_R_YY, INC_R_YZ,
                    INC_R_ZX, INC_R_ZY, INC_R_ZZ,
                    FTH_XX, FTH_YY, FTH_ZZ, FTH_XY, FTH_YZ, FTH_ZX,
                    FF_XX,  FF_YY,  FF_ZZ,  FF_XY,  FF_YZ,  FF_ZX,
                    FPG_XX, FPG_YY, FPG_ZZ, FPG_XY, FPG_YZ, FPG_ZX,
                    FG_XX,  FG_YY,  FG_ZZ,  FG_XY,  FG_YZ,  FG_ZX,
                    FP_XX,  FP_YY,  FP_ZZ,  FP_XY, FP_YX,  FP_YZ, FP_ZY, FP_ZX, FP_XZ,
                    FR_XX,  FR_YY,  FR_ZZ,  FR_XY,  FR_YZ,  FR_ZX,
                    ZG,     TIG,    TIR,    TT};
    enum  Operator { MEAN_VALUE=1, MIN_VALUE, MAX_VALUE, CRUDE_VALUE};
#ifndef SWIG
    // Struct for archiving
    struct ip2archive{
      double value; // current value
      std::vector< MElement * > ele; // element where the point is situated (if node is common to several element the first one met is chosen. Normally OK continuity)
      MVertex* _v;  // = vertex if archiving on node
      int numgauss; // local gauss num
      int dim; // dim, equal -1 if get value from "numgauss" of element "numphys"
      int comp;  // component to archive == ipval
      int numphys; //

      int minusNum; // ele to archive
      int plusNum; // element to archive  minusNum= plusNum for bulk elements otherwise for interface element

      int nstep;
      std::vector<const partDomain *> domain; // address of domain needed to access IP
      nlsField::ElemValue evalue;
      FILE *fp;

      void openFile()
      {
        std::string fname;
        std::ostringstream oss;
        if (numphys > 0)
          oss << numphys;
        else{
          oss<< ele[0]->getNum();
        }
        std::string s = oss.str();
        oss.str("");
        //oss << comp;
        //std::string s2 = oss.str();
        std::string s2 = IPField::ToString(comp);
        switch(evalue){
         case nlsField::mean:
          oss.str("Mean");
          break;
         case nlsField::min:
          oss.str("Min");
          break;
         case nlsField::max:
          oss.str("Max");
          break;
         default:
          oss.str("");
        }
        std::string s3 = oss.str();
        #if defined(HAVE_MPI)
        if(Msg::GetCommSize() != 1){
          oss.str("");
          oss << Msg::GetCommRank();
          s3 += "_part"+oss.str();
        }
        #endif // HAVE_MPI
        if (dim == -1){
          std::ostringstream numgss;
          if (evalue == nlsField::crude)
            numgss << numgauss;
          fname = "IP"+numgss.str()+"OnElement"+s+"val_"+s2+s3+".csv";
        }
        else if(dim == 0)
          fname = "IP"+s+"val_"+s2+s3+".csv";
        else if(dim == 1)
          fname = "IPLine"+s+"val_"+s2+s3+".csv";
        else if(dim == 2)
          fname = "IPFace"+s+"val_"+s2+s3+".csv";
        else
          fname = "IPVolume"+s+"val_"+s2+s3+".csv";

        if (restartManager::available())
        {
          fp = fopen(fname.c_str(),"a");
        }
        else
        {
          fp = fopen(fname.c_str(),"w");
        }
      }

      // constructor for solver
      ip2archive(const int dim_,const int nphys_,const int ipval_,const int eval_,
                 const int nstep_, const int numgau=0) : dim(dim_), numphys(nphys_), comp(ipval_), nstep(nstep_),fp(NULL),// the other has to be defined later
                                     numgauss(numgau),minusNum(0),plusNum(0.), value(0.), _v(NULL)
      {
        // empty vectors (they will be filled later) clear to be sure
        ele.clear();
        domain.clear();
        switch(eval_){
         case 1:
          evalue=nlsField::mean;
          break;
         case 2:
          evalue=nlsField::min;
          break;
         case 3:
          evalue=nlsField::max;
          break;
         default:
          evalue=nlsField::crude;
        }

        if (dim == 0){
          groupOfElements g(dim,numphys);
          groupOfElements::vertexContainer::iterator itv = g.vbegin();
          _v = *itv;
        }
      }

      ip2archive(const int ipval_,const int nstep_, MVertex* v): dim(0), numphys(v->getNum()), comp(ipval_), nstep(nstep_),fp(NULL),// the other has to be defined later
                                     numgauss(0),minusNum(0),plusNum(0.), value(0.), _v(v), evalue(nlsField::crude){
        ele.clear();
        domain.clear();
      };

      ip2archive(const int dim_,const int minusEle, const int plusEle,const int ipval_,const int eval_,
                 const int nstep_, const int numgau) : dim(dim_), numphys(0), comp(ipval_), nstep(nstep_),fp(NULL),// the other has to be defined later
                                     numgauss(numgau),minusNum(minusEle),plusNum(plusEle),value(0.), _v(NULL)
      {
        // empty vectors (they will be filled later) clear to be sure
        ele.clear();
        domain.clear();
        switch(eval_){
         case 1:
          evalue=nlsField::mean;
          break;
         case 2:
          evalue=nlsField::min;
          break;
         case 3:
          evalue=nlsField::max;
          break;
         default:
          evalue=nlsField::crude;
        }
      }

      ip2archive(const ip2archive &source) : ele(source.ele), domain(source.domain), fp(source.fp), numgauss(source.numgauss),
                                             numphys(source.numphys), comp(source.comp), evalue(source.evalue), dim(source.dim),
                                             nstep(source.nstep),plusNum(source.plusNum),minusNum(source.minusNum),value(source.value),
                                             _v(source._v) {}
      ~ip2archive(){
        if(fp!=NULL) fclose(fp);
        fp=NULL;
      }
    };
  protected :
    AllIPState *_AIPS;
    std::vector<ip2archive> viparch;
    nonLinearMechSolver* _solver; // solver to which ipField belongs

    bool _withMultiscale; // for synchronisation MPI
    bool _withDissipation;  // for active dissipation
    bool _withCohesiveFailure; // for failure synchronisation MPI

    int _numOfActiveDissipationIPs;
    int _numOfActiveDissipationIPsReached;
    int _numOfNewlyBrokenElements;
    int _totalNumOfNewlyBrokenElements; // sum in all procs when using MPI

     // current step data
    bool _failureAndInternalAreChecked;
    bool _failureStateIsFlushed;

    // Cracked elements management
    std::set<int> _newlyBrokenElements;    // (buffer) all interface elements where crack insertion occurs at the current step
    std::set<int> _brokenElements;         // all interface broken element is stored in order to avoid rechecking
    // Blocked elements management
    std::set<int> _newlyBlockedDissipationElements;  // (buffer) all bulk elements where damage blocking occurs at the current step
    std::set<int> _blockedDissipationElements;       // all blocked element is stored in order to avoid rechecking/reblocking

    // for damage blockage
    // for multiscale analysis
    std::set<int> _blockedDissipationIPs;
    std::set<int> _brokenIPs;
    std::set<int> _solverBrokenIPs;

    std::set<int> _nonBrokenIPs; // part of _brokenIPs must be set to nonBroken
    std::set<int> _nonBlockedDissipationIPs; // part of _blockedDissipationIPs must be set to non dissipation

    // for saving
    int _lastSaveStep;
    int _numStepBetweenTwoSaves;

 protected:
  void checkInternalState();
  void checkFailure();


 public :
  IPField(nonLinearMechSolver* sl, const std::vector<ip2archive> &vaip,
          std::vector<dataBuildView> &dbview_,std::string filename = "stress");
  virtual ~IPField();

  virtual void resetArchiving();
  virtual void archive(const double time,const int step, const bool forceView);
  double getIPValueOnPhysical(const int dim, const int physical, const int ipVal) const;

  static std::string ToString(const int i);

  AllIPState* getAips() {return _AIPS;}
  const AllIPState* getAips() const {return _AIPS;}

  const std::vector<partDomain*>& getDomainVector() const;


  void compute1state(IPStateBase::whichState ws, bool stiff);

  // num allows to select a particular gauss' point. Ins value is used only if ev = crude
  double getIPcomp(const partDomain *ef,MElement *ele, const IPStateBase::whichState ws,
                   const int cmp,const nlsField::ElemValue ev,const int num=0) const;
  // vol return the associated volume
  double getIPcompAndVolume(const partDomain *ef,MElement *ele, const IPStateBase::whichState ws,
                   const int cmp,const nlsField::ElemValue ev, double &vol) const;
  // function to archive
  virtual void get(partDomain *dom, MElement *ele, std::vector<double> &val, const int cc,
                   const nlsField::ElemValue ev)const{
    val[0]=  this->getIPcomp(dom,ele,IPStateBase::current,cc,ev);
  }

  // Interaction with Aips
  void copy(IPStateBase::whichState source, IPStateBase::whichState dest);

  void checkAll();
  bool isAllChecked() const {return _failureAndInternalAreChecked;}
  bool isFlushedFailureState() const {return _failureStateIsFlushed;};
  void flushFailureState();

  void nextStep(const double time=0.);
  int getNumOfActiveDissipationIPs() const {return _numOfActiveDissipationIPsReached;};
  int getNumOfActiveDissipationIPsCurrent() const {return _numOfActiveDissipationIPs;};
  int getLocalNumOfNewlyBrokenInterfaceElementInAllDomains() const {return _numOfNewlyBrokenElements;};
  int getNumOfNewlyBrokenInterfaceElementInAllDomains() const {return _totalNumOfNewlyBrokenElements;};

  void checkActiveDissipation();

  // initial broken
  void initialBroken(GModel* pModel, std::vector<int> &vnumphys);
  void initialBroken(MElement *iele, materialLaw *mlaw);
  void setFactorOnArchivingFiles(const int fact);


  template<class T> void getIPv(const MElement *ele,const T** vipv, const IPStateBase::whichState ws= IPStateBase::current) const
  {
    AllIPState::ipstateElementContainer* vips = _AIPS->getIPstate(ele->getNum());
    for(int i=0; i<vips->size(); i++){
      IPStateBase* ips = (*vips)[i];
      vipv[i] = static_cast<const T*>(ips->getState(ws));
    }
  }
  template<class T> void getIPv(const MElement *ele,const T** vipv, const T** vipvprev) const
  {
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(ele->getNum());
    for(int i=0; i<vips->size(); i++){
      IPStateBase* ips = (*vips)[i];
      vipv[i] = static_cast<const T*>(ips->getState(IPStateBase::current));
      vipvprev[i] = static_cast<const T*>(ips->getState(IPStateBase::previous));
    }
  }
  template<class T1,class T2> void getIPv(const MInterfaceElement *iele, const T1** vipv_m, const T2** vipv_p,
                                          const IPStateBase::whichState ws=IPStateBase::current) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    // SAME NUMBER OF GAUSS POINTS ON BOTH SIDES
    int npts = vips->size()/2;
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      IPStateBase *ipsp = (*vips)[i+npts];
      vipv_m[i] = dynamic_cast<const T1*>(ipsm->getState(ws));
      vipv_p[i] = dynamic_cast<const T2*>(ipsp->getState(ws));
    }
  }
  template<class T1,class T2> void getIPv(const MInterfaceElement *iele, const T1** vipv_m, const T1** vipvprev_m,
                                             const T2** vipv_p, const T2** vipvprev_p) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    // SAME NUMBER OF GAUSS POINTS ON BOTH SIDES
    int npts = vips->size()/2;
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      IPStateBase *ipsp = (*vips)[i+npts];
      vipv_m[i] = static_cast<const T1*>(ipsm->getState(IPStateBase::current));
      vipv_p[i] = static_cast<const T2*>(ipsp->getState(IPStateBase::current));
      vipvprev_m[i] = static_cast<const T1*>(ipsm->getState(IPStateBase::previous));
      vipvprev_p[i] = static_cast<const T2*>(ipsp->getState(IPStateBase::previous));
    }
  }
  template<class T1,class T2> void getIPv(const MInterfaceElement *iele, T1** vipv_m, T2** vipv_p,
                                             const IPStateBase::whichState ws=IPStateBase::current) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    // SAME NUMBER OF GAUSS POINTS ON BOTH SIDES
    int npts = vips->size()/2;
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      IPStateBase *ipsp = (*vips)[i+npts];
      vipv_m[i] = static_cast<T1*>(ipsm->getState(ws));
      vipv_p[i] = static_cast<T2*>(ipsp->getState(ws));
    }
  }
  template<class T1,class T2> void getIPv(const MInterfaceElement *iele, T1** vipv_m, T2** vipvprev_m,
                                             T2** vipv_p, T2** vipvprev_p) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    // SAME NUMBER OF GAUSS POINTS ON BOTH SIDES
    int npts = vips->size()/2;
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      IPStateBase *ipsp = (*vips)[i+npts];
      vipv_m[i] = static_cast<T1*>(ipsm->getState(IPStateBase::current));
      vipv_p[i] = static_cast<T2*>(ipsp->getState(IPStateBase::current));
      vipvprev_m[i] = static_cast<T1*>(ipsm->getState(IPStateBase::previous));
      vipvprev_p[i] = static_cast<T2*>(ipsp->getState(IPStateBase::previous));
    }
  }


  template<class T1> void getIPv(const MInterfaceElement *iele, const T1** vipv_m,
                                   const IPStateBase::whichState ws=IPStateBase::current) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    // SAME NUMBER OF GAUSS POINTS ON BOTH SIDES
    int npts = vips->size();
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      vipv_m[i] = static_cast<const T1*>(ipsm->getState(ws));
    }
  }
  template<class T1> void getIPv(const MInterfaceElement *iele, const T1** vipv_m, const T1** vipvprev_m) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    // SAME NUMBER OF GAUSS POINTS ON BOTH SIDES
    int npts = vips->size();
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      vipv_m[i] = static_cast<const T1*>(ipsm->getState(IPStateBase::current));
      vipvprev_m[i] = static_cast<const T1*>(ipsm->getState(IPStateBase::previous));
    }
  }

  template<class T1> void getIPv(const MInterfaceElement *iele, T1** vipv_m,
                                   const IPStateBase::whichState ws=IPStateBase::current) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    int npts = vips->size();
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      vipv_m[i] = static_cast<T1*>(ipsm->getState(ws));
    }
  }
  template<class T1> void getIPv(const MInterfaceElement *iele, T1** vipv_m, T1** vipvprev_m) const
  {
    const MElement* e = dynamic_cast<const MElement*>(iele);
    AllIPState::ipstateElementContainer *vips = _AIPS->getIPstate(e->getNum());
    int npts = vips->size();
    for(int i=0;i<npts;i++){
      IPStateBase *ipsm = (*vips)[i];
      vipv_m[i] = static_cast<T1*>(ipsm->getState(IPStateBase::current));
      vipvprev_m[i] = static_cast<T1*>(ipsm->getState(IPStateBase::previous));
    }
  }
  double computeBulkDeformationEnergy(MElement *ele, const partDomain *dom, const IPStateBase::whichState ws) const;
  double computeInterfaceDeformationEnergy(MElement *ele, const dgPartDomain *dom, const IPStateBase::whichState ws) const;
  double computeDeformationEnergy(const IPStateBase::whichState ws) const;

  double computeBulkPlasticEnergy(MElement *ele, const partDomain *dom, const IPStateBase::whichState ws) const;
  double computeInterfacePlasticEnergy(MElement *ele, const dgPartDomain *dom, const IPStateBase::whichState ws) const;
  double computePlasticEnergy(const IPStateBase::whichState ws) const;

  double computeBulkDamageEnergy(MElement *ele, const partDomain *dom, const IPStateBase::whichState ws) const;
  double computeInterfaceDamageEnergy(MElement *ele, const dgPartDomain *dom, const IPStateBase::whichState ws) const;
  double computeDamageEnergy(const IPStateBase::whichState ws) const;

  double computeBulkPathFollowingLocalValue(MElement *ele, const partDomain *dom, const IPStateBase::whichState ws) const;
  double computeInterfacePathFollowingLocalValue(MElement *ele, const dgPartDomain *dom, const IPStateBase::whichState ws) const;
  double computePathFollowingLocalValue(const IPStateBase::whichState ws) const;

  int computeFractureEnergy(MElement *ele,const dgPartDomain *dom,double* arrayEnergy, const IPStateBase::whichState ws) const;
  int computeFractureEnergy(double * arrayEnergy,const IPStateBase::whichState ws) const;

  virtual void restart();

  // block damage of problem
  void blockDissipation(const IPStateBase::whichState ws, const bool fl);

  // block dissipation at element level
  void addOneBlockedDissipationElement(MElement* ele);
  bool dissipationIsBlockedOnElement(MElement* ele) const;

  void addOneBrokenElement(MElement* ele);
  bool elementIsBroken(MElement* ele) const;

  void blockDissipationIP(const int ip);
  void brokenIP(const int ip);
  void solverBroken(const int ip);

  const std::set<int>& getConstRefToNonBrokenIPs() const{return _nonBrokenIPs;};
  std::set<int>& getRefToNonBrokenIPs() {return _nonBrokenIPs;};

  const std::set<int>& getConstRefToNonBlockedDissipationIPs() const{return _nonBlockedDissipationIPs;};
  std::set<int>& getRefToNonBlockedDissipationIPs() {return _nonBlockedDissipationIPs;};

  // _brokenIPs, _blockedDissipationIPs cannot be modifed outside class
  const std::set<int>& getConstRefToBrokenIPs() const{return _brokenIPs;};
  const std::set<int>& getConstRefToBlockedDissipationIPs() const{return _blockedDissipationIPs;};

  // new element
  const std::set<int>& getConstRefToNewlyBrokenElements() const {return _newlyBrokenElements;};
  std::set<int>& getRefToNewlyBrokenElements(){return _newlyBrokenElements;};
  //
  const std::set<int>& getConstRefToNewlyBlockedDissipationElements() const {return _newlyBlockedDissipationElements;};
  std::set<int>& getRefToNewlyBlockedDissipationElements() {return _newlyBlockedDissipationElements;};
#endif // swig
};
#endif // IPField

