//
//
// Description: define the IP data for finite strain cases
//
// Author:  <Van Dung NGUYEN>, (C) 2012
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "ipFiniteStrain.h"
#include "highOrderTensor.h"

ipFiniteStrain::ipFiniteStrain():_GPData(NULL),_interfaceGPData(NULL), IPVariableMechanics(){}
ipFiniteStrain::ipFiniteStrain(const ipFiniteStrain &source): IPVariableMechanics(source), _GPData(NULL),_interfaceGPData(NULL){
};
ipFiniteStrain &ipFiniteStrain::operator=(const IPVariable &source){
	// _GPData stores data of shape function, it is never copied
  IPVariableMechanics::operator=(source);
  return *this;
};
ipFiniteStrain::~ipFiniteStrain(){
  if(_GPData!= NULL){
    delete _GPData;
    _GPData = NULL;
  }
	if (_interfaceGPData!=NULL){
		delete _interfaceGPData;
		_interfaceGPData = NULL;
	}
}
  // get shape function information from ipvariable
std::vector<TensorialTraits<double>::ValType>& ipFiniteStrain::f(const FunctionSpaceBase* space, MElement* ele, IntPt &GP) const {
  if (_GPData == NULL){
		_GPData = new IntPtData<double>();
	};
  return _GPData->f(space,ele,GP);
};
std::vector<TensorialTraits<double>::GradType>& ipFiniteStrain::gradf(const FunctionSpaceBase* space, MElement* ele, IntPt &GP) const {
  if (_GPData == NULL){
		_GPData = new IntPtData<double>();
	};
	return _GPData->gradf(space,ele,GP);
};

std::vector<TensorialTraits<double>::HessType>& ipFiniteStrain::hessf(const FunctionSpaceBase* space, MElement* ele, IntPt &GP) const {
  if (_GPData == NULL){
		_GPData = new IntPtData<double>();
	};
	return _GPData->hessf(space,ele,GP);
};

std::vector<TensorialTraits<double>::ThirdDevType>& ipFiniteStrain::thirdDevf(const FunctionSpaceBase* space, MElement* ele, IntPt &GP) const {
  if (_GPData == NULL){
		_GPData = new IntPtData<double>();
	};
	return _GPData->thirdDevf(space,ele,GP);
};

double& ipFiniteStrain::getJacobianDeterminant(MElement* ele, IntPt &GP) const{
  if (_GPData == NULL){
		_GPData = new IntPtData<double>();
	};
	return _GPData->getJacobianDeterminant(ele,GP);
};


std::vector<TensorialTraits<double>::ValType>& ipFiniteStrain::f(const FunctionSpaceBase* space, MInterfaceElement* ele, IntPt &GP) const {
  if (_interfaceGPData == NULL){
		_interfaceGPData = new IntPtData<double>();
	};
	MElement* e = dynamic_cast<MElement*>(ele);
  return _interfaceGPData->f(space,e,GP);
};
std::vector<TensorialTraits<double>::GradType>& ipFiniteStrain::gradf(const FunctionSpaceBase* space, MInterfaceElement* ele, IntPt &GP) const {
  if (_interfaceGPData == NULL){
		_interfaceGPData = new IntPtData<double>();
	};
	MElement* e = dynamic_cast<MElement*>(ele);
	return _interfaceGPData->gradf(space,e,GP);
};

std::vector<TensorialTraits<double>::HessType>& ipFiniteStrain::hessf(const FunctionSpaceBase* space, MInterfaceElement* ele, IntPt &GP) const {
  if (_interfaceGPData == NULL){
		_interfaceGPData = new IntPtData<double>();
	};
	MElement* e = dynamic_cast<MElement*>(ele);
	return _interfaceGPData->hessf(space,e,GP);
};

std::vector<TensorialTraits<double>::ThirdDevType>& ipFiniteStrain::thirdDevf(const FunctionSpaceBase* space, MInterfaceElement* ele, IntPt &GP) const {
  if (_interfaceGPData == NULL){
		_interfaceGPData = new IntPtData<double>();
	};
	MElement* e = dynamic_cast<MElement*>(ele);
	return _interfaceGPData->thirdDevf(space,e,GP);
};

double& ipFiniteStrain::getJacobianDeterminant(MInterfaceElement* ele, IntPt &GP) const{
  if (_interfaceGPData == NULL){
		_interfaceGPData = new IntPtData<double>();
	};
	MElement* e = dynamic_cast<MElement*>(ele);
	return _interfaceGPData->getJacobianDeterminant(e,GP);
};
