//
// C++ Interface: ipHardening
//
// Description: Base class for ipHardening
//
//
// Author:  <L. Noels>, (C) 2013
//
// Copyright: See COPYING file that comes with this distribution
//
//
// class with the variables of IP (stress, deformation and localBasis)
#ifndef IPHARDENING_H_
#define IPHARDENING_H_
#include <stdlib.h>
#include <stdio.h>
#include "GmshConfig.h"
#include "ipvariable.h"

class IPJ2IsotropicHardening{

protected:
    double R0;         // initital yield stress (could be temperature dependent)
    double R;          // actual yield stress
    double dR;
    double ddR;
    double integR;
    bool _isSaturated; // harderning modulus become 0 under some conditions
    double dRdT; // temeprature dependence
    
 public:
  IPJ2IsotropicHardening();
  IPJ2IsotropicHardening(const IPJ2IsotropicHardening &source);
  virtual IPJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening * clone() const=0;
  
  virtual bool& getRefToSaturationState() {return _isSaturated;};
  virtual bool isSaturated() const {return _isSaturated;};

  virtual double getR0() const {return R0;}
  virtual double getR() const {return R;}
  virtual double getDR() const {return dR;}
  virtual double getDDR() const {return ddR;}
  virtual double getIntegR() const {return integR;}
  virtual double getDRDT() const{ return dRdT;};
  virtual void set(const double &_r0, const double &_r, const double &_dr, const double &_ddr, const double &_integR, const double& _dRdT)
  {
    R0 = _r0;
    R=_r;
    dR=_dr;
    ddR=_ddr;
    integR=_integR;      
    dRdT = _dRdT;
  }
};

class IPPerfectlyPlasticJ2IsotropicHardening : public IPJ2IsotropicHardening
{

 protected:

 public:
  IPPerfectlyPlasticJ2IsotropicHardening();
  IPPerfectlyPlasticJ2IsotropicHardening(const IPPerfectlyPlasticJ2IsotropicHardening &source);
  virtual IPPerfectlyPlasticJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPPerfectlyPlasticJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening * clone() const;
};

class IPPowerLawJ2IsotropicHardening : public IPJ2IsotropicHardening
{

 protected:
 public:
  IPPowerLawJ2IsotropicHardening();
  IPPowerLawJ2IsotropicHardening(const IPPowerLawJ2IsotropicHardening &source);
  virtual IPPowerLawJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPPowerLawJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening *clone() const;

};

class IPExponentialJ2IsotropicHardening : public IPJ2IsotropicHardening
{

 protected:

 public:
  IPExponentialJ2IsotropicHardening();
  IPExponentialJ2IsotropicHardening(const IPExponentialJ2IsotropicHardening &source);
  virtual IPExponentialJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPExponentialJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening * clone() const;

};

class IPSwiftJ2IsotropicHardening : public IPJ2IsotropicHardening
{

 protected:

 public:
  IPSwiftJ2IsotropicHardening();
  IPSwiftJ2IsotropicHardening(const IPSwiftJ2IsotropicHardening &source);
  virtual IPSwiftJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPSwiftJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening * clone() const;

};

class IPLinearExponentialJ2IsotropicHardening : public IPJ2IsotropicHardening
{
 protected:

 public:
  IPLinearExponentialJ2IsotropicHardening();
  IPLinearExponentialJ2IsotropicHardening(const IPLinearExponentialJ2IsotropicHardening &source);
  virtual IPLinearExponentialJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPLinearExponentialJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening * clone() const;

};



/* IPLinearFollowedByExponentialJ2IsotropicHardening */
class IPLinearFollowedByExponentialJ2IsotropicHardening : public IPJ2IsotropicHardening
{
 protected:
  // nothing to add to IPJ2IsotropicHardening basis
 public:
  IPLinearFollowedByExponentialJ2IsotropicHardening();
  IPLinearFollowedByExponentialJ2IsotropicHardening(const IPLinearFollowedByExponentialJ2IsotropicHardening &source);
  virtual IPLinearFollowedByExponentialJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPLinearFollowedByExponentialJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening * clone() const;

};



/* IPLinearFollowedByPowerLawJ2IsotropicHardening */
class IPLinearFollowedByPowerLawJ2IsotropicHardening : public IPJ2IsotropicHardening
{

 protected:
 public:
  IPLinearFollowedByPowerLawJ2IsotropicHardening();
  IPLinearFollowedByPowerLawJ2IsotropicHardening(const IPLinearFollowedByPowerLawJ2IsotropicHardening &source);
  virtual IPLinearFollowedByPowerLawJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPLinearFollowedByPowerLawJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening *clone() const;

};

class IPLinearFollowedByMultiplePowerLawJ2IsotropicHardening : public IPJ2IsotropicHardening
{

 protected:
 public:
  IPLinearFollowedByMultiplePowerLawJ2IsotropicHardening();
  IPLinearFollowedByMultiplePowerLawJ2IsotropicHardening(const IPLinearFollowedByMultiplePowerLawJ2IsotropicHardening &source);
  virtual IPLinearFollowedByMultiplePowerLawJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
  virtual ~IPLinearFollowedByMultiplePowerLawJ2IsotropicHardening(){}
  virtual void restart();
  virtual IPJ2IsotropicHardening *clone() const;

};




class IPPolynomialJ2IsotropicHardening : public IPJ2IsotropicHardening{
  public:
    IPPolynomialJ2IsotropicHardening();
    IPPolynomialJ2IsotropicHardening(const IPPolynomialJ2IsotropicHardening &source);
    virtual IPPolynomialJ2IsotropicHardening &operator=(const IPJ2IsotropicHardening &source);
    virtual ~IPPolynomialJ2IsotropicHardening(){}
    virtual void restart();
    virtual IPJ2IsotropicHardening * clone() const;
};

class IPTwoExpJ2IsotropicHaderning: public IPJ2IsotropicHardening{
  public:
    IPTwoExpJ2IsotropicHaderning();
    IPTwoExpJ2IsotropicHaderning(const IPTwoExpJ2IsotropicHaderning &source);
    virtual IPTwoExpJ2IsotropicHaderning &operator=(const IPJ2IsotropicHardening &source);
    virtual ~IPTwoExpJ2IsotropicHaderning(){}
    virtual void restart();
    virtual IPJ2IsotropicHardening * clone() const;
};
#endif //IPHARDENING_H_

