//
// Description: storing class for j2 linear elasto-plastic law
//
//
// Author:  <Gauthier BECKER>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "ipJ2linear.h"
#include "restartManager.h"
IPJ2linear::IPJ2linear() : IPVariableMechanics(), _j2lepspbarre(0.), _j2lepsp(1.), _j2ldsy(0.),  _elasticEnergy(0.),
_plasticPower(0.),_DplasticPowerDF(0.),_Ee(0.),_irreversibleEnergy(0.),_DirreversibleEnergyDF(0.),
_dissipationBlocked(false),_dissipationActive(false),_plasticEnergy(0.)
{
  ipvJ2IsotropicHardening=NULL;
  Msg::Error("IPJ2Linear::IPJ2Linear is not initialized with a hardening IP Variable");

};
IPJ2linear::IPJ2linear(const J2IsotropicHardening *j2IH) : IPVariableMechanics(), _j2lepspbarre(0.), _j2lepsp(1.), _j2ldsy(0.),
_elasticEnergy(0.),_plasticPower(0.),_DplasticPowerDF(0.),_Ee(0.),_irreversibleEnergy(0.),_DirreversibleEnergyDF(0.),
_dissipationBlocked(false),_dissipationActive(false),_plasticEnergy(0.) {
  ipvJ2IsotropicHardening=NULL;
  if(j2IH ==NULL) Msg::Error("IPJ2Linear::IPJ2Linear has no j2IH");
  j2IH->createIPVariable(ipvJ2IsotropicHardening);

};
IPJ2linear::IPJ2linear(const IPJ2linear &source) : IPVariableMechanics(source), _j2lepspbarre(source._j2lepspbarre),
                                                   _j2lepsp(source._j2lepsp), _j2ldsy(source._j2ldsy),
                                                   _elasticEnergy(source._elasticEnergy),
                                                   _plasticPower(source._plasticPower),_DplasticPowerDF(source._DplasticPowerDF),
                                                   _Ee(source._Ee),_irreversibleEnergy(source._irreversibleEnergy),
																									 _DirreversibleEnergyDF(source._DirreversibleEnergyDF),
																									 _dissipationBlocked(source._dissipationBlocked),_dissipationActive(source._dissipationActive),
                                                   _plasticEnergy(source._plasticEnergy)
{
  ipvJ2IsotropicHardening = NULL;
  if(source.ipvJ2IsotropicHardening != NULL)
  {
    ipvJ2IsotropicHardening = dynamic_cast<IPJ2IsotropicHardening*>(source.ipvJ2IsotropicHardening->clone());
  }
}
IPJ2linear& IPJ2linear::operator=(const IPVariable &source)
{
  IPVariableMechanics::operator=(source);
  const IPJ2linear* src = dynamic_cast<const IPJ2linear*>(&source);
  if(src!=NULL)
  {
    _j2lepspbarre = src->_j2lepspbarre;
    _j2lepsp = src->_j2lepsp;
    _j2ldsy = src->_j2ldsy;
    _elasticEnergy = src->_elasticEnergy;
    _plasticPower = src->_plasticPower;
    _DplasticPowerDF = src->_DplasticPowerDF;
    _Ee = src->_Ee;
		_irreversibleEnergy = src->_irreversibleEnergy;
		_DirreversibleEnergyDF = src->_DirreversibleEnergyDF;
		_dissipationBlocked = src->_dissipationBlocked;
    _dissipationActive = src->_dissipationActive;
    _plasticEnergy = src->_plasticEnergy;
    if(src->ipvJ2IsotropicHardening != NULL)
    {
      if (ipvJ2IsotropicHardening!=NULL){
        ipvJ2IsotropicHardening->operator=(*(dynamic_cast<IPJ2IsotropicHardening*>(src->ipvJ2IsotropicHardening)));
      }
      else
        ipvJ2IsotropicHardening= dynamic_cast<IPJ2IsotropicHardening*>(src->ipvJ2IsotropicHardening->clone());
    }
    else{
      if (ipvJ2IsotropicHardening) {
        delete ipvJ2IsotropicHardening;
        ipvJ2IsotropicHardening = NULL; 
      }
    }
  }
  return *this;
}

double IPJ2linear::defoEnergy() const
{
  return _elasticEnergy;

}
double IPJ2linear::plasticEnergy() const
{
  return _plasticEnergy;
}

void IPJ2linear::restart()
{
  IPVariableMechanics::restart();
  restartManager::restart(_j2lepspbarre);
  restartManager::restart(_j2lepsp);
  restartManager::restart(_j2ldsy);
  restartManager::restart(_elasticEnergy);
  restartManager::restart(ipvJ2IsotropicHardening);
  restartManager::restart(_plasticPower);
  restartManager::restart(_DplasticPowerDF);
  restartManager::restart(_Ee);
	restartManager::restart(_irreversibleEnergy);
	restartManager::restart(_DirreversibleEnergyDF);
	restartManager::restart(_dissipationBlocked);
  restartManager::restart(_dissipationActive);
  restartManager::restart(_plasticEnergy);
  return;
}

