//
// Description: storing class for non local damage
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "ipNonLocalDamage.h"
#include "ipField.h"
#include "restartManager.h"

double IPNonLocalDamage::defoEnergy() const
{
  return _elasticEne;
}
double IPNonLocalDamage::plasticEnergy() const
{
  return _plasticEne;
}

double IPNonLocalDamage::get(int comp) const
{


  //here to vizualize direction
  //const double *euler=&(_nldStatev[_pos_euler]);
  //if(comp == IPField::SIG_XX) return euler[0];
  //if(comp == IPField::SIG_YY) return euler[1];
  //if(comp == IPField::SIG_ZZ) return euler[2];

  if(comp >= IPField::SIG_XX and comp<=IPField::SIG_XY )
	  return _nldStress(comp-1);
  else if(comp==IPField::SIG_XZ) // order xx yy zz xy xz yz
	  return _nldStress(comp-2);
  else if(comp==IPField::SIG_YZ)
	  return _nldStress(comp);
  else if(comp == IPField::SVM) // von Mises
  {

     double svm= (_nldStress(0)-_nldStress(1))*(_nldStress(0)-_nldStress(1))+(_nldStress(2)-_nldStress(1))*(_nldStress(2)-_nldStress(1))+(_nldStress(2)-_nldStress(0))*(_nldStress(2)-_nldStress(0));
     svm += 6.*(_nldStress(3)*_nldStress(3)+_nldStress(4)*_nldStress(4)+_nldStress(5)*_nldStress(5));
     svm /= 2.;
     svm = sqrt(svm);
     return svm;
  }
  else if(comp == IPField::DAMAGE) // damage
     return _nldDamage;
  else if(comp == IPField::PLASTICSTRAIN)
     return _nldCurrentPlasticStrain;
  else if(comp >= IPField::STRAIN_XX and comp<IPField::STRAIN_XY )
	  return _nldStrain(comp-IPField::STRAIN_XX);
  else if(comp == IPField::STRAIN_XY)
	  return _nldStrain(comp-IPField::STRAIN_XX)/2.;
  else if(comp==IPField::STRAIN_XZ) // order xx yy zz xy xz yz
	  return _nldStrain(comp-IPField::STRAIN_XX-1)/2.;
  else if(comp==IPField::STRAIN_YZ)
	  return _nldStrain(comp-IPField::STRAIN_XX+1)/2.;
  else if(comp>=  IPField::MTX_SVM and comp<=IPField::INC_SIG_XZ and _pos_str_mtx >=0 and _pos_str_inc >=0)
  {
     const double *str = NULL;
     double sq2 = sqrt(2.);
     double Ja=1.;
     if(comp>=  IPField::MTX_SVM and comp<=IPField::MTX_SIG_XZ)
     { 
        str = &(_nldStatev[_pos_str_mtx]);
        if(_pos_mtx_Ja>=0) Ja=_nldStatev[_pos_mtx_Ja];
     }
     else
     { 
        str = &(_nldStatev[_pos_str_inc]);
        if(_pos_inc_Ja>=0) Ja=_nldStatev[_pos_inc_Ja];
     }
     static fullVector<double> _cauchy(6);
     //in composite cauchy is stored, but in phases this is Kirchhoff
     for(int i=0;i<3;i++)
     {
       _cauchy(i)=str[i]/Ja;
       _cauchy(i+3) = str[i+3]/sq2/Ja;
     }
     if(comp == IPField::MTX_SVM or comp == IPField::INC_SVM)
     {
        double svm= (_cauchy(0)-_cauchy(1))*(_cauchy(0)-_cauchy(1))+(_cauchy(2)-_cauchy(1))*(_cauchy(2)-_cauchy(1))+(_cauchy(2)-_cauchy(0))*(_cauchy(2)-_cauchy(0));
       svm += 6.*(_cauchy(3)*_cauchy(3)+_cauchy(4)*_cauchy(4)+_cauchy(5)*_cauchy(5));
       svm /= 2.;
       svm = sqrt(svm);
       return svm;
     }
     else if(comp == IPField::MTX_SIG_XX or comp == IPField::INC_SIG_XX)
       return _cauchy(0);
     else if(comp == IPField::MTX_SIG_YY or comp == IPField::INC_SIG_YY)
       return _cauchy(1);
     else if(comp == IPField::MTX_SIG_ZZ or comp == IPField::INC_SIG_ZZ)
       return _cauchy(2);
     else if(comp == IPField::MTX_SIG_XY or comp == IPField::INC_SIG_XY)
       return _cauchy(3);
     else if(comp == IPField::MTX_SIG_YZ or comp == IPField::INC_SIG_YZ)
       return _cauchy(5);
     else if(comp == IPField::MTX_SIG_XZ or comp == IPField::INC_SIG_XZ)
       return _cauchy(4);
     else
       return 0.;
  }
  else if(comp>=  IPField::MTX_STRAIN_XX and comp<=IPField::INC_STRAIN_XZ and _pos_str_mtx >=0 and _pos_str_inc >=0)
  {
     const double *stn = NULL;
     double sq2 = sqrt(2.);
     if(comp>=  IPField::MTX_STRAIN_XX and comp<=IPField::MTX_STRAIN_XZ) stn = &(_nldStatev[_pos_stn_mtx]);
     else stn = &(_nldStatev[_pos_stn_inc]);

     static fullVector<double> _strain(6);
     for(int i=0;i<3;i++)
     {
       _strain(i)=stn[i];
       _strain(i+3) = stn[i+3]*sq2;
     }
     if(comp == IPField::MTX_STRAIN_XX or comp == IPField::INC_STRAIN_XX)
       return _strain(0);
     else if(comp == IPField::MTX_STRAIN_YY or comp == IPField::INC_STRAIN_YY)
       return _strain(1);
     else if(comp == IPField::MTX_STRAIN_ZZ or comp == IPField::INC_STRAIN_ZZ)
       return _strain(2);
     else if(comp == IPField::MTX_STRAIN_XY or comp == IPField::INC_STRAIN_XY)
       return _strain(3);
     else if(comp == IPField::MTX_STRAIN_YZ or comp == IPField::INC_STRAIN_YZ)
       return _strain(5);
     else if(comp == IPField::MTX_STRAIN_XZ or comp == IPField::INC_STRAIN_XZ)
       return _strain(4);
     else
       return 0.;
  }
  else if(comp==IPField::INC_VOLUME_RATIO)
  {
    if(_pos_inc_vfi>0)
      return _nldStatev[_pos_inc_vfi];
    else
      return 0.;
  }
  else if(comp==IPField::INC_ASPECT_RATIO_1 || comp==IPField::INC_ASPECT_RATIO_2)
  {
    if(_pos_inc_ar>0)
      return _nldStatev[_pos_inc_ar+comp-IPField::INC_ASPECT_RATIO_1];
    else
      return 0.;
  }
  else if(comp>=IPField::INC_R_XX || comp<=IPField::INC_R_ZZ)
  {
    if(_pos_inc_R>0)
      return _nldStatev[_pos_inc_R+comp-IPField::INC_R_XX];
    else
      return 0.;
  }
  else
     return 0.;

}

void IPNonLocalDamage::restart()
{
  IPVariableMechanics::restart();
  restartManager::restart(_nldNsdv);
  restartManager::restart(_pos_str_mtx);
  restartManager::restart(_pos_str_inc);
  restartManager::restart(_pos_stn_mtx);
  restartManager::restart(_pos_stn_inc);
  restartManager::restart(_pos_inc_vfi);
  restartManager::restart(_pos_inc_ar);
  restartManager::restart(_pos_inc_R);        
  restartManager::restart(_pos_inc_Ja);
  restartManager::restart(_pos_mtx_Ja);
  restartManager::restart(_nldMaterialTensor.getDataPtr(),36);
  restartManager::restart(_nldCouplingStressEffectiveStrain.getDataPtr(),6);
  restartManager::restart(_nldCouplingEffectiveStrainStress.getDataPtr(),6);
  restartManager::restart(_nldStrain.getDataPtr(),6);
  restartManager::restart(_nldStress.getDataPtr(),6);
  restartManager::restart(_nldCharacteristicLengthMatrix);
  restartManager::restart(_nldEffectivePlasticStrain);
  restartManager::restart(_nldCurrentPlasticStrain);
  restartManager::restart(_nldDamage);
  restartManager::restart(_nldSpBar);
  restartManager::restart(_nldStatev,_nldNsdv);
  restartManager::restart(_elasticEne);
  restartManager::restart(_plasticEne);
  restartManager::restart(_dissipationBlocked);
  return;
}
