//
// C++ Interface: ipNucleation
//
// Description: Base class for ipNucleation
//
//
// Author:  <L. Noels>, (C) 2013
//
// Copyright: See COPYING file that comes with this distribution
//
//
// class with the variables of IP
#ifndef IPNUCLEATION_H_
#define IPNUCLEATION_H_
#include <stdlib.h>
#include <stdio.h>
#include "GmshConfig.h"
#include "ipvariable.h"
#include "STensorOperations.h"

class IPNucleation{
  protected:
    double nucleatedfV; // nucleated porosity
    double DeltafV; // increment compared to previous
    double DDeltafVDHatP;// derivatives

 public:
  IPNucleation();
  IPNucleation(const IPNucleation &source);
  virtual IPNucleation &operator=(const IPNucleation &source);
  virtual ~IPNucleation(){}
  virtual void restart();
  virtual IPNucleation * clone() const=0;
  
  virtual double getNucleatedPorosity() const {return nucleatedfV;};
  virtual double getNucleatedPorosityIncrement() const {return DeltafV;};
  virtual double getDNucleatedPorosityDMatrixPlasticDeformation() const {return DDeltafVDHatP;};
  
  virtual double& getRefToNucleatedPorosity() {return nucleatedfV;};
  virtual double& getRefToNucleatedPorosityIncrement() {return DeltafV;};
  virtual double& getRefToDNucleatedPorosityDMatrixPlasticDeformation() {return DDeltafVDHatP;};
  
  virtual void resetNucleation(const IPNucleation& ipv) {
    nucleatedfV = ipv.nucleatedfV;
    DeltafV = 0.;
    DDeltafVDHatP = 0.;
  };
};

class IPTrivialNucleationRate : public IPNucleation
{
 public:
  IPTrivialNucleationRate():IPNucleation(){};
  IPTrivialNucleationRate(const IPTrivialNucleationRate &source): IPNucleation(source){};
  virtual IPTrivialNucleationRate &operator=(const IPNucleation &source)
  {
    IPNucleation::operator =(source);
    return *this;
  }
  virtual ~IPTrivialNucleationRate(){}
  virtual IPNucleation * clone() const {return new IPTrivialNucleationRate();};
};


class IPLinearNucleationRate : public IPNucleation{


 public:
  IPLinearNucleationRate();
  IPLinearNucleationRate(const IPLinearNucleationRate &source);
  virtual IPLinearNucleationRate &operator=(const IPNucleation &source);
  virtual ~IPLinearNucleationRate(){}
  virtual IPNucleation * clone() const;
  virtual void restart();
};

class IPExponentialNucleationRate : public IPNucleation{


 public:
  IPExponentialNucleationRate();
  IPExponentialNucleationRate(const IPExponentialNucleationRate &source);
  virtual IPExponentialNucleationRate &operator=(const IPNucleation &source);
  virtual ~IPExponentialNucleationRate(){}
  virtual IPNucleation * clone() const;
  virtual void restart();
};
#endif //IPNUCLEATION_H_
