//
// C++ Interface: ipNucleationCriterion
//
// Description: Provide ip class for nucleation criterion 
//
//
// Author:  <J. Leclerc>, (C) 2019
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef IPNUCLEATIONCRITERION_H_
#define IPNUCLEATIONCRITERION_H_

#include "ipvariable.h"

class IPNucleationCriterion{
  // Base class for NucleationCriterionlaw-related IPV
  // nothing need to be added yet
 
public:
  // Constructors & destructor
  IPNucleationCriterion();
  IPNucleationCriterion(const IPNucleationCriterion &source);
  virtual IPNucleationCriterion &operator=(const IPNucleationCriterion &source);
  virtual ~IPNucleationCriterion(){};
  virtual IPNucleationCriterion* clone() const=0;
  
  // Standard functions
  virtual void restart();
  
  // Access functions
  virtual bool getNucleationActivatedFlag() const=0;
  virtual bool setNucleationActivatedFlag(const bool flag)=0;
};



class IPTrivialNucleationCriterion : public IPNucleationCriterion{
  // IPV for TrivialNucleationCriterionLaw
  // (nothing need to be added)
  // by default, the nucleation is considered as activated
  public:
    // Constructors & destructor
    IPTrivialNucleationCriterion();
    IPTrivialNucleationCriterion(const IPTrivialNucleationCriterion &source);
    virtual IPTrivialNucleationCriterion &operator=(const IPNucleationCriterion &source);
    virtual ~IPTrivialNucleationCriterion(){};
    virtual IPNucleationCriterion* clone() const;
    
   
    // Acces functions
    virtual bool getNucleationActivatedFlag() const{return true;};
    virtual bool setNucleationActivatedFlag(const bool flag){
      Msg::Error("IPTrivialNucleationCriterion::this function should not be used");
    };
};



class IPBereminNucleationCriterion : public IPNucleationCriterion{
  // IPV for TrivialNucleationCriterionLaw
protected:
    // State flag
    bool _activeNucleation;     // = true if nucleation is currently active 
                                // == if the criterion has been already fulfilled
                                // It is inititated to false.
    // Stress concentration factor under a tensorial form
    //STensor3 _StressConcentrationTensor;
    
  // (nothing need to be added)
public:
  // Constructors & destructor
  IPBereminNucleationCriterion();
  IPBereminNucleationCriterion(const IPBereminNucleationCriterion &source);
  virtual IPBereminNucleationCriterion &operator=(const IPNucleationCriterion &source);
  virtual ~IPBereminNucleationCriterion(){};
  virtual IPNucleationCriterion* clone() const;
  
  
  // Standard functions
  virtual void restart();
  
  // Acces functions
  virtual bool getNucleationActivatedFlag() const{return _activeNucleation;};
  virtual bool setNucleationActivatedFlag(const bool flag) {_activeNucleation = flag;};
};





#endif //IPNUCLEATIONCRITERION_H_