// Description: storing class for Phenomenological SMP
// Author:  <H. Gulasik, M. Pareja>, (C) 2017
// Copyright: See COPYING file that comes with this distribution

#include "ipPhenomenologicalSMP.h"
#include "restartManager.h"
#include "STensorOperations.h"

IPPhenomenologicalSMP::IPPhenomenologicalSMP(const double t0, const double Tg, const double Tonsetg, const double Tonsetr, const double alpha_trans)
                      :IPVariableMechanics(), _thermalEnergy(0.), _eps(0.), _SMPEnergy(0.), Tdot(0.), Tt(Tg), w(0.2), lambdaT(1.), Tmin(t0), alpha_exp_ekstra(0.)
{
    Tig= t0;
    Tir= t0;
    //alphaig= alpha_trans;
    alphaig=0.;
    STensorOperation::unity(Fpg);
    STensorOperation::unity(Fp);
    STensorOperation::unity(Ff);
    STensorOperation::unity(Fth);
    STensorOperation::unity(Uf);
    STensorOperation::unity(Fg);
    STensorOperation::unity(Fr);
    //zg= 0.;
    zg=1./(1.+exp(2.*w*(t0-Tt)));
    ipvJ2IsotropicHardening = NULL;
    Msg::Error("IPJ2Linear::IPJ2Linear is not initialized with a hardening IP Variable");
};



IPPhenomenologicalSMP::IPPhenomenologicalSMP(const double t0, const double Tg, const double Tonsetg, const double Tonsetr, const double alpha_trans, const J2IsotropicHardening *j2IH)
                      :IPVariableMechanics(), _thermalEnergy(0.), _eps(0.), _SMPEnergy(0.), Tdot(0.), Tt(Tg), w(0.2), lambdaT(1.), Tmin(t0), alpha_exp_ekstra(0.)
{
    Tig= t0;
    Tir= t0;
    //alphaig= alpha_trans;
    alphaig=0.;

    STensorOperation::unity(Fpg);
    STensorOperation::unity(Fp);
    STensorOperation::unity(Ff);
    STensorOperation::unity(Fth);
    STensorOperation::unity(Uf);
    STensorOperation::unity(Fr);
    STensorOperation::unity(Fg);
    //zg= 0.;
    zg=1./(1.+exp(2.*w*(t0-Tt)));
    ipvJ2IsotropicHardening = NULL;
    if(j2IH == NULL) Msg::Error("IPJ2Linear::IPJ2Linear has no j2IH");
    j2IH->createIPVariable(ipvJ2IsotropicHardening);
};



IPPhenomenologicalSMP::IPPhenomenologicalSMP(const IPPhenomenologicalSMP &source)
                      :IPVariableMechanics() //: IPLinearThermoMechanics(source)
{
    _SMPEnergy=     source._SMPEnergy;
    Fpg =           source.Fpg;
    Fp =            source.Fp;
    Ff =            source.Ff;
    Fth =           source.Fth;
    Uf =            source.Uf;
    Fr =            source.Fr;
    Fg =            source.Fg;
    zg =            source.zg;
    Tig =           source.Tig;
    Tir =           source.Tir;
    lambdaT =       source.lambdaT;
    Tmin =          source.Tmin;
    alpha_exp_ekstra= source.alpha_exp_ekstra;
    Tdot =          source.Tdot;
    Tt =            source.Tt;
    w =             source.w;
    alphaig =       source.alphaig;
    _thermalEnergy= source._thermalEnergy;
    _eps=           source._eps;
    _SMPEnergy=     source._SMPEnergy;

    ipvJ2IsotropicHardening = NULL;
    if(source.ipvJ2IsotropicHardening != NULL)
    {
        ipvJ2IsotropicHardening = dynamic_cast<IPJ2IsotropicHardening*>(source.ipvJ2IsotropicHardening->clone());
    }
}



IPPhenomenologicalSMP& IPPhenomenologicalSMP::operator=(const IPVariable &source)
{
    IPVariableMechanics::operator=(source);

    const IPPhenomenologicalSMP* src = dynamic_cast<const IPPhenomenologicalSMP*>(&source);
    if(src)
    {
        Fpg =               src->Fpg;
        Fp =                src->Fp;
        Ff =                src->Ff;
        Fr =                src->Fr;
        Fg =                src->Fg;
        Fth =               src->Fth;
        Uf =                src->Uf;
        zg =                src->zg;
        Tig =               src->Tig;
        Tir =               src->Tir;
        lambdaT =           src->lambdaT;
        Tmin =              src->Tmin;
        alpha_exp_ekstra=   src->alpha_exp_ekstra;
        Tdot =              src->Tdot;
        Tt =                src->Tt;
        w =                 src->w;
        alphaig =           src->alphaig;
        _thermalEnergy =    src->_thermalEnergy;
        _eps=               src->_eps;
        _SMPEnergy=         src->_SMPEnergy;

        if(src->ipvJ2IsotropicHardening != NULL)
        {
            if (ipvJ2IsotropicHardening!=NULL)
            {
                ipvJ2IsotropicHardening->operator=(*(dynamic_cast<IPJ2IsotropicHardening*>(src->ipvJ2IsotropicHardening)));
            }
            else
                ipvJ2IsotropicHardening= dynamic_cast<IPJ2IsotropicHardening*>(src->ipvJ2IsotropicHardening->clone());
        }
        else
        {
            if (ipvJ2IsotropicHardening)
            {
                delete ipvJ2IsotropicHardening;
                ipvJ2IsotropicHardening = NULL;
            }
        }
    }
    return *this;
}



void IPPhenomenologicalSMP::restart()
{
    IPVariableMechanics::restart();
    restartManager::restart(Fpg);
    restartManager::restart(Fp);
    restartManager::restart(Ff);
    restartManager::restart(Fg);
    restartManager::restart(Fr);
    restartManager::restart(Fth);
    restartManager::restart(Uf);
    restartManager::restart(zg);
    restartManager::restart(Tig);
    restartManager::restart(Tir);
    restartManager::restart(lambdaT);
    restartManager::restart(Tmin);
    restartManager::restart(alpha_exp_ekstra);
    restartManager::restart(Tdot);
    restartManager::restart(Tt);
    restartManager::restart(w);
    restartManager::restart(alphaig);
    restartManager::restart(_SMPEnergy);
    restartManager::restart(_thermalEnergy);
    restartManager::restart(_eps);
    restartManager::restart(ipvJ2IsotropicHardening);
    return;
}
