// Description: storing class for linear thermomechanical law it is a "virtual implementation" you have to define an ipvariable
// in your project which derive from this one.
// Author:  <H. Gulasik, M. Pareja>, (C) 2017
// Copyright: See COPYING file that comes with this distribution

#ifndef IPPHENOMENOLOGICALSMP_H_
#define IPPHENOMENOLOGICALSMP_H_
#include "ipLinearThermoMechanics.h"
#include "STensor3.h"
#include "ipvariable.h"
#include "ipHardening.h"
#include "j2IsotropicHardening.h"

class IPPhenomenologicalSMP : public IPVariableMechanics//: public IPLinearThermoMechanics  //, public mlawSMP
{
protected:
    IPJ2IsotropicHardening* ipvJ2IsotropicHardening;
    STensor3 Fpg,Fp,Ff,Fth,Uf;
    STensor3 Fr, Fg;
    double zg;
    double _SMPEnergy;
    double _thermalEnergy;
    double _eps;
    double Tig, Tir, alphaig, Tdot, Tt, w;
    double lambdaT, Tmin, alpha_exp_ekstra;

public:
    IPPhenomenologicalSMP(const double t0, const double Tg, const double Tonsetg, const double Tonsetr, const double alpha_trans);
    IPPhenomenologicalSMP(const double t0, const double Tg, const double Tonsetg, const double Tonsetr, const double alpha_trans, const J2IsotropicHardening *j2IH);
    IPPhenomenologicalSMP(const IPPhenomenologicalSMP &source);
    IPPhenomenologicalSMP& operator=(const IPVariable &source);

    virtual double defoEnergy() const{return _SMPEnergy;}
    virtual double getThermalEnergy() const {return _thermalEnergy;};

    virtual STensor3 &getRefToFpg() {return Fpg;};
    virtual const STensor3 &getConstRefToFpg() const {return Fpg;};
    virtual STensor3 &getRefToFp() {return Fp;};
    virtual const STensor3 &getConstRefToFp() const {return Fp;};
    virtual STensor3 &getRefToFf() {return Ff;};
    virtual const STensor3 &getConstRefToFf() const {return Ff;};
    virtual STensor3 &getRefToFth() {return Fth;};
    virtual const STensor3 &getConstRefToFth() const {return Fth;};
    virtual STensor3 &getRefToUf() {return Uf;};
    virtual const STensor3 &getConstRefToUf() const {return Uf;};
    virtual STensor3 &getRefToFr() {return Fr;};
    virtual const STensor3 &getConstRefToFr() const {return Fr;};
    virtual STensor3 &getRefToFg() {return Fg;};
    virtual const STensor3 &getConstRefToFg() const {return Fg;};
    virtual double &getRefTozg() {return zg;};
    virtual double getzg()  const {return zg;};
    virtual double& getRefToEquivalentPlasticDefo() {return _eps;};
    virtual const double& getConstRefToEquivalentPlasticDefo() const {return _eps;};
    virtual double& getRefToTig() {return Tig;};
    virtual const double& getConstRefToTig() const {return Tig;};
    virtual double& getRefToTir() {return Tir;};
    virtual const double& getConstRefToTir() const {return Tir;};

    virtual double& getRefToLambdaT() {return lambdaT;};
    virtual const double& getConstRefToLambdaT() const {return lambdaT;};
    virtual double& getRefToTmin() {return Tmin;};
    virtual const double& getConstRefToTmin() const {return Tmin;};
    virtual double& getRefToalpha_exp_ekstra() {return alpha_exp_ekstra;};
    virtual const double& getConstRefToalpha_exp_ekstra() const {return alpha_exp_ekstra;};

    virtual double& getRefToAlphaig() {return alphaig;};
    virtual const double& getConstRefToAlphaig() const {return alphaig;};
    virtual double& getRefToTdot() {return Tdot;};
    virtual const double& getConstRefToTdot() const {return Tdot;};
    virtual double& getRefToTt() {return Tt;};
    virtual const double& getConstRefToTt() const {return Tt;};
    virtual double& getRefTow() {return w;};
    virtual const double& getConstRefTow() const {return w;};

    virtual void restart();

    virtual IPVariable* clone() const {return new IPPhenomenologicalSMP(*this);};

    // Operational + post-processing purposes
    virtual double& getRefToFaux1() {return Ff(1,1);};
    virtual double& getRefToFaux2() {return Fp(1,1);};
    virtual double& getRefToFaux3() {return Uf(1,1);};
    virtual double& getRefToFaux4() {return Fg(1,1);};
    virtual double& getRefToFaux5() {return Fr(1,1);};

    virtual const IPJ2IsotropicHardening &getConstRefToIPJ2IsotropicHardening() const
    {
        if(ipvJ2IsotropicHardening==NULL)
            Msg::Error("IPJ2linear: ipvJ2IsotropicHardening not initialized");
        return *ipvJ2IsotropicHardening;
    }

    virtual IPJ2IsotropicHardening &getRefToIPJ2IsotropicHardening()
    {
        if(ipvJ2IsotropicHardening==NULL)
            Msg::Error("IPJ2linear: ipvJ2IsotropicHardening not initialized");
        return *ipvJ2IsotropicHardening;
    }
};
#endif // IPPHENOMENOLOGICALSMP_H_
