#ifndef STENSOROPERATIONS_H_
#define STENSOROPERATIONS_H_

#include "STensor3.h"
#include "STensor33.h"
#include "STensor43.h"
#include "STensor53.h"
#include "STensor63.h"
#include <cmath>


namespace STensorOperation{
  inline void zero(double& a){
    a= 0.;
  };

  inline void zero(SVector3& a){
    a(0) = 0.; a(1) = 0.; a(2) = 0.;
  };

  inline void zero(STensor3& a){
    a(0,0) = 0.; a(0,1) = 0.; a(0,2) = 0.;
    a(1,0) = 0.; a(1,1) = 0.; a(1,2) = 0.;
    a(2,0) = 0.; a(2,1) = 0.; a(2,2) = 0.;
  };
  inline void unity(STensor3& a){
    a(0,0) = 1.; a(0,1) = 0.; a(0,2) = 0.;
    a(1,0) = 0.; a(1,1) = 1.; a(1,2) = 0.;
    a(2,0) = 0.; a(2,1) = 0.; a(2,2) = 1.;
  };


  inline void diag(STensor3& a, const double s){
    a(0,0) = s; a(0,1) = 0.; a(0,2) = 0.;
    a(1,0) = 0.; a(1,1) = s; a(1,2) = 0.;
    a(2,0) = 0.; a(2,1) = 0.; a(2,2) = s;
  };

  inline void zero(STensor33& a){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          a(i,j,k) = 0.;
        }
      }
    }
  };

  inline void zero(STensor43& a){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            a(i,j,k,l) = 0.;
          }
        }
      }
    }
  };
  inline void zero(STensor53& a){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            for (int p=0; p<3; p++){
              a(i,j,k,l,p) = 0.;
            }
          }
        }
      }
    }
  };
  inline void zero(STensor63& a){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            for (int p=0; p<3; p++){
              for (int q=0; q<3; q++){
                a(i,j,k,l,p,q) = 0.;
              }
            }
          }
        }
      }
    }
  };
  
  inline void scale(const STensor3& a, double x, STensor3& ax)
  {
    for (int i=0; i<3; i++)
    {
      for (int j=0; j<3; j++)
      {
        ax(i,j) = a(i,j)*x;
      }
    }
  }
  inline void scale(const STensor43& a, double x, STensor43& ax)
  {
    for (int i=0; i<3; i++)
    {
      for (int j=0; j<3; j++)
      {
        for (int k=0; k<3; k++)
        {
          for (int l=0; l<3; l++)
          {
            ax(i,j,k,l) = a(i,j,k,l)*x;
          }
        }
      }
    }
  }

  inline bool isnan(const double a){
    if (a != a) return true;
    return false;
  };

  inline bool isnan(const SVector3& a){
    for (int i=0; i<3; i++){
      if (a(i) != a(i)) return true;
    }
    return false;
  };

  inline bool isnan(const STensor3& a){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        if (a(i,j) != a(i,j)) return true;
      }
    }
    return false;
  };

  inline bool isinf(const double a){
    return isinf(a);
  };

  inline bool isinf(const STensor3& a){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        if (isinf(a(i,j))) return true;
      }
    }
    return false;
  };
  
  inline double trace(const STensor3& a){
    return a(0,0) + a(1,1)+ a(2,2);
  };
  inline void decomposeDevTr(const STensor3& E, STensor3& devE, double& trE){
    trE = E.trace();
    devE = E;
    double trEover3 = trE/3.;
    devE(0,0) -=trEover3;
    devE(1,1) -=trEover3;
    devE(2,2) -=trEover3;
  };

  inline double determinantSTensor3(const STensor3 &a)
  {
    return (a(0,0) * (a(1,1) * a(2,2) - a(1,2) * a(2,1)) -
            a(0,1) * (a(1,0) * a(2,2) - a(1,2) * a(2,0)) +
            a(0,2) * (a(1,0) * a(2,1) - a(1,1) * a(2,0)));
  }
  
  inline double dot(const SVector3& a, const SVector3& b) 
  {
    return a(0)*b(0) + a(1)*b(1) + a(2)*b(2);
  }
  inline double doubledot(const STensor3& a, const STensor3& b){
    double val= 0.;
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        val += a(i,j)*b(i,j);
      }
    }
    return val;
  };
  
  
  inline void getThreeInvariantsSTensor3(const STensor3& a, double& I1, double& I2, double& I3){
    I1 = a(0,0)+a(1,1)+a(2,2);
    I2 = a(0,0)*a(1,1)+a(1,1)*a(2,2)+a(2,2)*a(0,0)-a(0,1)*a(1,0)-a(1,2)*a(2,1)-a(2,0)*a(0,2);
    I3 = determinantSTensor3(a);
  };
    
  inline void inverseSTensor3(const STensor3 &a, STensor3 &ainv)
  {
    double udet = 1./determinantSTensor3(a);
    ainv(0,0) =  (a(1,1) * a(2,2) - a(1,2) * a(2,1))* udet;
    ainv(1,0) = -(a(1,0) * a(2,2) - a(1,2) * a(2,0))* udet;
    ainv(2,0) =  (a(1,0) * a(2,1) - a(1,1) * a(2,0))* udet;
    ainv(0,1) = -(a(0,1) * a(2,2) - a(0,2) * a(2,1))* udet;
    ainv(1,1) =  (a(0,0) * a(2,2) - a(0,2) * a(2,0))* udet;
    ainv(2,1) = -(a(0,0) * a(2,1) - a(0,1) * a(2,0))* udet;
    ainv(0,2) =  (a(0,1) * a(1,2) - a(0,2) * a(1,1))* udet;
    ainv(1,2) = -(a(0,0) * a(1,2) - a(0,2) * a(1,0))* udet;
    ainv(2,2) =  (a(0,0) * a(1,1) - a(0,1) * a(1,0))* udet;
  };

  inline void inverseAndTransposeSTensor3(const STensor3 &a, STensor3 &ainvT)
  {
    double udet = 1./determinantSTensor3(a);
    ainvT(0,0) =  (a(1,1) * a(2,2) - a(1,2) * a(2,1))* udet;
    ainvT(0,1) = -(a(1,0) * a(2,2) - a(1,2) * a(2,0))* udet;
    ainvT(0,2) =  (a(1,0) * a(2,1) - a(1,1) * a(2,0))* udet;
    ainvT(1,0) = -(a(0,1) * a(2,2) - a(0,2) * a(2,1))* udet;
    ainvT(1,1) =  (a(0,0) * a(2,2) - a(0,2) * a(2,0))* udet;
    ainvT(1,2) = -(a(0,0) * a(2,1) - a(0,1) * a(2,0))* udet;
    ainvT(2,0) =  (a(0,1) * a(1,2) - a(0,2) * a(1,1))* udet;
    ainvT(2,1) = -(a(0,0) * a(1,2) - a(0,2) * a(1,0))* udet;
    ainvT(2,2) =  (a(0,0) * a(1,1) - a(0,1) * a(1,0))* udet;
  };

  inline bool invertFullMatrix4x4(const double m[16], double inv[16])
  {
    inv[0] = m[5]  * m[10] * m[15] -
             m[5]  * m[11] * m[14] -
             m[9]  * m[6]  * m[15] +
             m[9]  * m[7]  * m[14] +
             m[13] * m[6]  * m[11] -
             m[13] * m[7]  * m[10];

    inv[4] = -m[4]  * m[10] * m[15] +
              m[4]  * m[11] * m[14] +
              m[8]  * m[6]  * m[15] -
              m[8]  * m[7]  * m[14] -
              m[12] * m[6]  * m[11] +
              m[12] * m[7]  * m[10];

    inv[8] = m[4]  * m[9] * m[15] -
             m[4]  * m[11] * m[13] -
             m[8]  * m[5] * m[15] +
             m[8]  * m[7] * m[13] +
             m[12] * m[5] * m[11] -
             m[12] * m[7] * m[9];

    inv[12] = -m[4]  * m[9] * m[14] +
               m[4]  * m[10] * m[13] +
               m[8]  * m[5] * m[14] -
               m[8]  * m[6] * m[13] -
               m[12] * m[5] * m[10] +
               m[12] * m[6] * m[9];

    inv[1] = -m[1]  * m[10] * m[15] +
              m[1]  * m[11] * m[14] +
              m[9]  * m[2] * m[15] -
              m[9]  * m[3] * m[14] -
              m[13] * m[2] * m[11] +
              m[13] * m[3] * m[10];

    inv[5] = m[0]  * m[10] * m[15] -
             m[0]  * m[11] * m[14] -
             m[8]  * m[2] * m[15] +
             m[8]  * m[3] * m[14] +
             m[12] * m[2] * m[11] -
             m[12] * m[3] * m[10];

    inv[9] = -m[0]  * m[9] * m[15] +
              m[0]  * m[11] * m[13] +
              m[8]  * m[1] * m[15] -
              m[8]  * m[3] * m[13] -
              m[12] * m[1] * m[11] +
              m[12] * m[3] * m[9];

    inv[13] = m[0]  * m[9] * m[14] -
              m[0]  * m[10] * m[13] -
              m[8]  * m[1] * m[14] +
              m[8]  * m[2] * m[13] +
              m[12] * m[1] * m[10] -
              m[12] * m[2] * m[9];

    inv[2] = m[1]  * m[6] * m[15] -
             m[1]  * m[7] * m[14] -
             m[5]  * m[2] * m[15] +
             m[5]  * m[3] * m[14] +
             m[13] * m[2] * m[7] -
             m[13] * m[3] * m[6];

    inv[6] = -m[0]  * m[6] * m[15] +
              m[0]  * m[7] * m[14] +
              m[4]  * m[2] * m[15] -
              m[4]  * m[3] * m[14] -
              m[12] * m[2] * m[7] +
              m[12] * m[3] * m[6];

    inv[10] = m[0]  * m[5] * m[15] -
              m[0]  * m[7] * m[13] -
              m[4]  * m[1] * m[15] +
              m[4]  * m[3] * m[13] +
              m[12] * m[1] * m[7] -
              m[12] * m[3] * m[5];

    inv[14] = -m[0]  * m[5] * m[14] +
               m[0]  * m[6] * m[13] +
               m[4]  * m[1] * m[14] -
               m[4]  * m[2] * m[13] -
               m[12] * m[1] * m[6] +
               m[12] * m[2] * m[5];

    inv[3] = -m[1] * m[6] * m[11] +
              m[1] * m[7] * m[10] +
              m[5] * m[2] * m[11] -
              m[5] * m[3] * m[10] -
              m[9] * m[2] * m[7] +
              m[9] * m[3] * m[6];

    inv[7] = m[0] * m[6] * m[11] -
             m[0] * m[7] * m[10] -
             m[4] * m[2] * m[11] +
             m[4] * m[3] * m[10] +
             m[8] * m[2] * m[7] -
             m[8] * m[3] * m[6];

    inv[11] = -m[0] * m[5] * m[11] +
               m[0] * m[7] * m[9] +
               m[4] * m[1] * m[11] -
               m[4] * m[3] * m[9] -
               m[8] * m[1] * m[7] +
               m[8] * m[3] * m[5];

    inv[15] = m[0] * m[5] * m[10] -
              m[0] * m[6] * m[9] -
              m[4] * m[1] * m[10] +
              m[4] * m[2] * m[9] +
              m[8] * m[1] * m[6] -
              m[8] * m[2] * m[5];

    double det = m[0] * inv[0] + m[1] * inv[4] + m[2] * inv[8] + m[3] * inv[12];

    if (det == 0)
        return false;

    det = 1.0 / det;
    for (int i = 0; i < 16; i++)
        inv[i] *= det;
    return true;
}


  inline void prod(const STensor3& a, const STensor3& b, const double alp, STensor43& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j,k,l) = alp*a(i,j)*b(k,l);
          }
        }
      }
    }
  }
  // c += alp*axb
  inline void prodAdd(const STensor3& a, const STensor3& b, const double alp, STensor43& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j,k,l) += alp*a(i,j)*b(k,l);
          }
        }
      }
    }
  }

  inline void axpy(STensor43& y, const STensor43& x, const double alp){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            y(i,j,k,l) += alp*x(i,j,k,l);
          }
        }
      }
    }
  }
  inline void axpy(STensor3& y, const STensor3& x, const double alp){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        y(i,j) += alp*x(i,j);
      }
    }
  }


  inline void transposeSTensor3(const STensor3 &a, STensor3 &aT)
  {
    aT(0,0) = a(0,0); aT(0,1) = a(1,0); aT(0,2) = a(2,0);
    aT(1,0) = a(0,1); aT(1,1) = a(1,1); aT(1,2) = a(2,1);
    aT(2,0) = a(0,2); aT(2,1) = a(1,2); aT(2,2) = a(2,2);
  };



  inline void multSTensor3(const STensor3& a, const STensor3&b, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        c(i,j) = 0.;
        for (int k=0; k<3; k++){
          c(i,j) +=a(i,k)*b(k,j);
        }
      }
    }
  };

  inline void multSTensor3Add(const STensor3& a, const STensor3&b, const double alph, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          c(i,j) +=alph*a(i,k)*b(k,j);
        }
      }
    }
  };

  inline void multSTensor3FirstTranspose(const STensor3& a, const STensor3&b, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        c(i,j) = 0.;
        for (int k=0; k<3; k++){
          c(i,j) +=a(k,i)*b(k,j);
        }
      }
    }
  };
  inline void multSTensor3FirstTransposeAdd(const STensor3& a, const STensor3&b, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          c(i,j) +=a(k,i)*b(k,j);
        }
      }
    }
  };

  inline void multSTensor3STensor43(const STensor3& a, const STensor43& b, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        c(i,j) = 0.;
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j) += a(k,l)*b(k,l,i,j);
          }
        }
      }
    }
  };
  inline void multSTensor3STensor43Add(const STensor3& a, const STensor43& b, const double fact, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j) += fact*a(k,l)*b(k,l,i,j);
          }
        }
      }
    }
  };

  inline void multSTensor3SecondTranspose(const STensor3& a, const STensor3& b, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        c(i,j) = 0.;
        for (int k=0; k<3; k++){
          c(i,j) +=a(i,k)*b(j,k);
        }
      }
    }
  };

  inline void multSTensor43(const STensor43& a, const STensor43& b, STensor43& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j,k,l) = 0.;
            for (int p=0; p<3; p++){
              for (int q=0; q<3; q++){
                c(i,j,k,l) += a(i,j,p,q)*b(p,q,k,l);
              }
            }
          }
        }
      }
    }
  };
  
  inline void multSTensor43Add(const STensor43& a, const STensor43& b, const double alpha, STensor43& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            for (int p=0; p<3; p++){
              for (int q=0; q<3; q++){
                c(i,j,k,l) += alpha*a(i,j,p,q)*b(p,q,k,l);
              }
            }
          }
        }
      }
    }
  };

  inline void multSTensor43FirstTranspose(const STensor43& a, const STensor43& b, STensor43& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j,k,l) = 0.;
            for (int p=0; p<3; p++){
              for (int q=0; q<3; q++){
                c(i,j,k,l) += a(p,q,i,j)*b(p,q,k,l);
              }
            }
          }
        }
      }
    }
  };


  inline void multSTensor43STensor3(const STensor43& a, const STensor3& b, STensor3& c, double fact = 1.){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        c(i,j) = 0.;
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j) += fact*a(i,j,k,l)*b(k,l);
          }
        }
      }
    }
  };
  
  inline void multSTensor43STensor3Add(const STensor43& a, const STensor3& b, double fact, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j) += fact*a(i,j,k,l)*b(k,l);
          }
        }
      }
    }
  };


  inline void multSTensor43STensor3SecondTranspose(const STensor43& a, const STensor3& b, STensor3& c){
    for (int i=0; i<3; i++){
      for (int j=0; j<3; j++){
        c(i,j) = 0.;
        for (int k=0; k<3; k++){
          for (int l=0; l<3; l++){
            c(i,j) += a(i,j,k,l)*b(l,k);
          }
        }
      }
    }
  };

  inline void multSTensor3SVector3(const STensor3& a, const SVector3& b, SVector3& c){
    for (int i=0; i<3; i++){
      c(i) = 0.;
      for (int j=0; j<3; j++){
        c(i) += a(i,j)*b(j);
      }
    }
  };
  
  inline void multSVector3STensor3(const SVector3& a, const STensor3& b,  SVector3& c){
    for (int i=0; i<3; i++){
      c(i) = 0.;
      for (int j=0; j<3; j++){
        c(i) += a(i)*b(i,j);
      }
    }
  };
  

  // F = sum_{i=0, order) coeffs[i]*a^i
  bool polynomial(const STensor3& a, const int order, const fullVector<double>& coeffs, STensor3& F, STensor43* dF= NULL, STensor63* ddF=NULL);
  void logSTensor3(const STensor3& a, const int order, STensor3& loga, STensor43* dloga = NULL, STensor63* ddloga = NULL);
  void expSTensor3(const STensor3& a, const int order, STensor3& expa, STensor43* dexpa = NULL, STensor63* ddexpa = NULL);
  
  void VRDecomposition(const STensor3& a, STensor3& V, STensor3&R);
  void RUDecomposition(const STensor3& a, STensor3& U, STensor3&R);

  
  inline void Ddeta_Da_sym(const STensor3& a, const double deta, STensor3& Ddeta_Da){
    // with a symetric
    // D(det(a))Da = det(a)*inv(a) = a*a - a*tr(a) + 0.5*(tr(a)*tr(a) - tr(a*a))I
  
    multSTensor3(a,a,Ddeta_Da); 
    double tr_a = trace(a);
    double tr_aa = trace(Ddeta_Da);
    
    Ddeta_Da.daxpy(a,-tr_a);
    double factI = 0.5*(tr_a*tr_a -tr_aa);
    Ddeta_Da(0,0) += factI;
    Ddeta_Da(1,1) += factI;
    Ddeta_Da(2,2) += factI;
  };
  
};

#endif // STENSOROPERATIONS_H_
