#ifndef WEIGHT_PHASE_C  
#define WEIGHT_PHASE_C 1

#include "matrix_operations.h"
#include "Weight.h"
#include "rotations.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

//#define pi 3.14159265
//******************************************************
//**** orientation distrubution function for each direction fiber  */
//**************************************************************************************

void getClosure(double a1, double a2, double a3, double A[3][3][3][3])
{
  double a1d[3][3], a2d[3][3], a3d[3][3];
  double Aclosure1D[3][3][3][3],Aclosure2D[3][3][3][3],Aclosure3D[3][3][3][3];
  double alpha1, alpha2, alpha3;
  int i,j,k,l;

  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      a1d[i][j]=a2d[i][j]=a3d[i][j]=0.;
    }
  }
  a1d[0][0]=1.;
  getQuadraticClosure(a1d,Aclosure1D);

  a2d[0][0]=a1/(a2+a1);
  a2d[1][1]=a2/(a2+a1);
  getHybridClosure(a2d,Aclosure2D,true);

  a3d[0][0]=a1;
  a3d[1][1]=a2;
  a3d[2][2]=a3;
  getHybridClosure(a3d,Aclosure3D,false);

  alpha1=(a1-a2)/a1;
  alpha2=(a2-a3)/a1;
  alpha3=(a3)/a1;

  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          A[i][j][k][l]=alpha1*Aclosure1D[i][j][k][l]+alpha2*Aclosure2D[i][j][k][l]+alpha3*Aclosure3D[i][j][k][l];
        }
      }    
    }
  }
}
void getLinearClosure(double a[3][3], double Al[3][3][3][3], bool is2D)
{
  int i,j,k,l;
  double alphal,betal;
  double I[3][3];
  double Ia[3][3][3][3],Iap1[3][3][3][3],Is[3][3][3][3];


  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      I[i][j]=0.;
    }
  }
  for(i=0;i<3;i++)
  {
    I[i][i]=1.;
  }
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          Is[i][j][k][l]=0.5*I[i][k]*I[j][l]+0.5*I[i][l]*I[j][k];
          Ia[i][j][k][l]=0.5*a[i][k]*a[j][l]+0.5*a[i][l]*a[j][k];
          Iap1[i][j][k][l]=0.5*(a[i][k]+I[i][k])*(a[j][l]+I[j][l])+0.5*(a[i][l]+I[i][l])*(a[j][k]+I[j][k]);
        }
      }    
    }
  }
  alphal=-1./35.; betal=1./7.;
  if(is2D)
  {
    alphal=-1./24.; betal=1./6.;
  }

  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          Al[i][j][k][l]=alphal*I[i][j]*I[k][l]+2.*(alphal-betal)*Is[i][j][k][l]+betal*(I[i][j]*a[k][l]+a[i][j]*I[k][l])+2.*betal*(Iap1[i][j][k][l]-Ia[i][j][k][l]);
        }
      }    
    }
  }
}

void getQuadraticClosure(double a[3][3], double Aq[3][3][3][3])
{
  int i,j,k,l;
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          Aq[i][j][k][l]=a[i][j]*a[k][l];
        }
      }    
    }
  }
}
void getHybridClosure(double a[3][3], double Ah[3][3][3][3], bool is2D)
{
  int i,j,k,l;
  double deta, Nh, f;
  double Al[3][3][3][3], Aq[3][3][3][3]; 
  double **atmp;
  mallocmatrix(&atmp,3,3);
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      atmp[i][j]=a[i][j];
    }
  } 
  if(is2D)
  {
    for(i=0;i<2;i++)
    {
      atmp[i][2]=0.;
      atmp[2][i]=0.;
    }
    atmp[2][2]=1.;
  }

  deta=Determinant(atmp,3);
  Nh=27.;
  if(is2D)
  {
    deta=Determinant(atmp,2);
    Nh=4.;
  }
  f=1.-Nh*deta;

  getQuadraticClosure(a,Aq);
  getLinearClosure(a,Al,is2D);
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          Ah[i][j][k][l]=(1.-f)*Al[i][j][k][l]+f*Aq[i][j][k][l];
        }
      }    
    }
  }
  freematrix(atmp,3);
}
double getODF(double p[3], double bofa[3][3], double bigBofa[3][3][3][3], bool is2D)
{
  int i,j,k,l;
  double trfp;
  double pp[3][3], fp[3][3];
  double Aq[3][3][3][3], Al[3][3][3][3];
  double psi,psi1, psi2, psi3;

  psi1=1./4./pi;
  psi2=15./8./pi;
  psi3=315./32./pi;
  if(is2D)
  {
    psi1=1./2./pi;
    psi2=2./pi;
    psi3=8./pi;
  }
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      pp[i][j]=p[i]*p[j];
      fp[i][j]=p[i]*p[j];
    }
  }
  trfp=fp[0][0]+fp[1][1]+fp[2][2];
  for(i=0;i<3;i++)
  {
    fp[i][i]-=trfp/3.;
  }
  getLinearClosure(pp, Al, is2D);
  getQuadraticClosure(pp, Aq);
  psi=psi1;
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      psi+=psi2*bofa[i][j]*fp[i][j];
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          psi+=psi3*bigBofa[i][j][k][l]*(Aq[i][j][k][l]-Al[i][j][k][l]);
        }
      }
    }
  }
  return psi;
}
double generateWeight(int N, double **a, double **probDensityList, double **facetList, double **thetaList, double **phiList)
{
  int i,j,k,l,m,n,o,p;
  double *lambdaA;
  double **atmp, **eA, **Rot;
  int eigOrderLarger, eigOrderMid, eigOrderSmaller;
  double A[3][3][3][3],Atmp[3][3][3][3], bigBofa[3][3][3][3], Al[3][3][3][3];
  double bofa[3][3], a33[3][3];
  double dir[3];
  double deltaTheta,theta,surfFacet,surfTotal,phi,trbofa,probDensity;
  int *facesPerTheta;
  int totalFaces, Nphi;

  mallocmatrix(&atmp,3,3);
  mallocvector(&lambdaA,3);
  mallocmatrix(&eA,3,3);
  mallocmatrix(&Rot,3,3);

  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      atmp[i][j]=a[i][j];
      bofa[i][j]=a[i][j];
      a33[i][j]=a[i][j];
    }
  } 
  trbofa=bofa[0][0]+bofa[1][1]+bofa[2][2];
  for(i=0;i<3;i++)
  {
    bofa[i][i]-=trbofa/3.;
  }
  getLinearClosure(a33, Al, false);
  Jacobian_Eigen( 3, atmp, lambdaA, eA);
  //here we need to check the order
  if(lambdaA[0]>=lambdaA[1] && lambdaA[1]>=lambdaA[2])
  {
    eigOrderLarger=0;
    eigOrderMid=1;
    eigOrderSmaller=2;
  }
  else if(lambdaA[0]>=lambdaA[2] && lambdaA[2]>=lambdaA[1])
  {
    eigOrderLarger=0;
    eigOrderMid=2;
    eigOrderSmaller=1;
  }
  else if(lambdaA[1]>=lambdaA[2] && lambdaA[2]>=lambdaA[0])
  {
    eigOrderLarger=1;
    eigOrderMid=2;
    eigOrderSmaller=0;
  }
  else if(lambdaA[1]>=lambdaA[0] && lambdaA[0]>=lambdaA[2])
  {
    eigOrderLarger=1;
    eigOrderMid=0;
    eigOrderSmaller=2;
  }
  else if(lambdaA[2]>=lambdaA[1] && lambdaA[1]>=lambdaA[0])
  {
    eigOrderLarger=2;
    eigOrderMid=1;
    eigOrderSmaller=0;
  }
  else if(lambdaA[2]>=lambdaA[0] && lambdaA[0]>=lambdaA[1])
  {
    eigOrderLarger=2;
    eigOrderMid=0;
    eigOrderSmaller=1;
  }
  getClosure(lambdaA[eigOrderLarger], lambdaA[eigOrderMid], lambdaA[eigOrderSmaller], Atmp);
  for(i=0;i<3;i++)
  {
    Rot[i][0]=eA[i][eigOrderLarger];
    Rot[i][1]=eA[i][eigOrderMid];
    Rot[i][2]=eA[i][eigOrderSmaller];
  } 
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        for(l=0;l<3;l++)
        {
          A[i][j][k][l]=0.;
          for(m=0;m<3;m++)
          {
            for(n=0;n<3;n++)
            {
              for(o=0;o<3;o++)
              {
                for(p=0;p<3;p++)
                {
                  A[i][j][k][l]+=Rot[i][m]*Rot[j][n]*Rot[k][o]*Rot[l][p]*Atmp[m][n][o][p];
                }
              }
            }
          }
          bigBofa[i][j][k][l]=A[i][j][k][l]-Al[i][j][k][l];
        }
      }
    }
  }
  
  deltaTheta=pi/N; 
  facesPerTheta = new int[N];
  totalFaces=0;
  for(i=0;i<N;i++)
  {
    theta=i*deltaTheta+deltaTheta/2.;
    Nphi=(int)(((double)(N))*sin(theta));
    if(i==0 or i==N-1) Nphi=1;
    facesPerTheta[i]=Nphi;
    totalFaces+=Nphi;
  }
  mallocvector(probDensityList,totalFaces);
  mallocvector(facetList,totalFaces);
  mallocvector(thetaList,totalFaces);
  mallocvector(phiList,totalFaces);
  k=0;
  surfTotal=0.;
  probDensity=0.;
  for(i=0;i<N;i++)
  {
    theta=i*deltaTheta+deltaTheta/2.;
    surfFacet=2.*pi*sin(theta)*sin(deltaTheta/2.)/facesPerTheta[i];
    for(j=0;j<facesPerTheta[i];j++)
    {
      phi=(j+1./2.)*pi/facesPerTheta[i];
      dir[0]= cos(phi)*sin(theta);
      dir[1]= sin(phi)*sin(theta);
      dir[2]= cos(theta);
      (*thetaList)[k]=theta;
      (*phiList)[k]=phi;    
      (*probDensityList)[k]=2.*getODF(dir,bofa, bigBofa, false);
      (*facetList)[k]= surfFacet;
      probDensity+=(*probDensityList)[k]*((*facetList)[k]);
      surfTotal+=surfFacet;
      k+=1;
    }
  }
  printf("Nfacet: %d, Surface: %e, Probability: %e\n",totalFaces,surfTotal,probDensity);
  //for(i=0;i<totalFaces;i++)
  //{
  //  (*probDensityList)[i]/probDensity;
  //}
  free(lambdaA);
  freematrix(atmp,3);
  freematrix(eA,3);
  freematrix(Rot,3);
  free(facesPerTheta);
  return totalFaces;
}


#endif
