
//
// Description: Define damage law for cohesive layer
//
// Author:  <V.D. Nguyen>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "cohesiveDamageLaw.h"
#include <math.h>


BilinearCohesiveDamageLaw::BilinearCohesiveDamageLaw(const int num):cohesiveDamageLaw(num){};
			
BilinearCohesiveDamageLaw::BilinearCohesiveDamageLaw(const BilinearCohesiveDamageLaw& src):cohesiveDamageLaw(src){}
			
void BilinearCohesiveDamageLaw::computeDamage(const double pi, const double pc, 
															 const double p0, const double p1, 
															 const double D0, double& D, 
															 const bool stiff, double& dDdp) const{
	 D = D0;
	 dDdp = 0.;
	 
	 if ((p1> p0) and (p1 >pi)){
		if (p1 < pc){
			double fact = pc/(pc-pi);
			D = fact*(1. - pi/p1);
			if (stiff){
				dDdp = fact*pi/(p1*p1);
			}
		}
		else{
			D = 1.;
			dDdp = 0.;
		}
	}
};

ExponentialCohesiveDamageLaw::ExponentialCohesiveDamageLaw(const int num, const double a, const double b):
			cohesiveDamageLaw(num),_alpha(a),_beta(b){};
			
ExponentialCohesiveDamageLaw::ExponentialCohesiveDamageLaw(const ExponentialCohesiveDamageLaw& src):
			cohesiveDamageLaw(src),_alpha(src._alpha),_beta(src._beta){}

void ExponentialCohesiveDamageLaw::computeDamage(const double pi, const double pc, 
															 const double p0, const double p1, 
															 const double D0, double& D, 
															 const bool stiff, double& dDdp) const{
	D = D0;
	dDdp = 0.;
	
	if ((p1> p0) and (p1 >pi)){
		double refp = p1/pi;
		D = 1 - pow(refp,_alpha)*exp(-_beta*(refp-1.));
		
		if (stiff){
			dDdp = (-_alpha/refp+_beta)*(1.-D)/pi;
		}
	}
};