//
// Description: Define damage law for cohesive layer
//
// Author:  <V.D. Nguyen>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef COHESIVEDAMAGELAW_H_
#define COHESIVEDAMAGELAW_H_

class cohesiveDamageLaw{
	public:
		enum cohesiveDamageLawName{Bilinear=1, Exponential=2};
	protected:
		int _num;
 
	public:
		#ifndef SWIG
		cohesiveDamageLaw(const int num):_num(num) {};
		cohesiveDamageLaw(const cohesiveDamageLaw& src): _num(src._num){};
		virtual ~cohesiveDamageLaw(){};
		virtual int getNum() const{return _num;};
		virtual void computeDamage(const double pi, const double pc,  // initial and critical effective jump
															 const double p0, const double p1,  // previous and current maximal effective jump
															 const double D0, double& D,  // previous and currentd damage
															 const bool stiff, double& dDdp) const = 0; // derivative in respecto reffective jump
		virtual cohesiveDamageLaw::cohesiveDamageLawName getType() = 0;
		virtual cohesiveDamageLaw* clone() const = 0;
		#endif // SWIG
};


class BilinearCohesiveDamageLaw : public cohesiveDamageLaw{
	
	public:
		BilinearCohesiveDamageLaw(const int num);
		#ifndef SWIG
		BilinearCohesiveDamageLaw(const BilinearCohesiveDamageLaw& src);
		virtual ~BilinearCohesiveDamageLaw(){};
		virtual void computeDamage(const double pi, const double pc, 
															 const double p0, const double p1, 
															 const double D0, double& D, 
															 const bool stiff, double& dDdp) const;
		virtual cohesiveDamageLaw::cohesiveDamageLawName getType() {return cohesiveDamageLaw::Bilinear;};
		virtual cohesiveDamageLaw* clone() const {return new BilinearCohesiveDamageLaw(*this);};
		#endif // SWIG
};

class ExponentialCohesiveDamageLaw  : public cohesiveDamageLaw{
	protected:
		double _alpha, _beta;
	public:
		ExponentialCohesiveDamageLaw(const int num, const double a, const double b);
		#ifndef SWIG
		ExponentialCohesiveDamageLaw(const ExponentialCohesiveDamageLaw& src);
		virtual ~ExponentialCohesiveDamageLaw(){};
		virtual void computeDamage(const double pi, const double pc, 
															 const double p0, const double p1, 
															 const double D0, double& D, 
															 const bool stiff, double& dDdp) const;
		virtual cohesiveDamageLaw::cohesiveDamageLawName getType() {return cohesiveDamageLaw::Exponential;};
		virtual cohesiveDamageLaw* clone() const {return new ExponentialCohesiveDamageLaw(*this);};
		#endif // SWIG
};


#endif //COHESIVEDAMAGELAW_H_