#ifndef MATERIAL_CC
#define MATERIAL_CC 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#include "material.h"
#include "FiniteStrain.h"
#include "Weight.h"

#ifdef NONLOCALGMSH
#include "matrix_operations.h"
#include "lccfunctions.h"
#include "elasticlcc.h"
#include "j2plast.h"
#include "mlawNonLocalDamage.h"
#include "homogenization.h"
#include "homogenization_2rd.h"
using std::max;
using namespace MFH;
#include "rotations.h"
#endif
//****in all of the constbox**New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar  ****************
//*******c_g the characteristic length tensor, the value is returned by Clength_creat function***********


void Material::constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt)
{
  printf("Function constBoxSecant is not defined for this material\n");
}

void Material::constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt)
{
   constboxSecant(dstrn, strs_n,  strs, statev_n, statev, Calgo, Csd, dCsd, dCsdp_bar, dnu, dnudp_bar, alpha, dpdE, dstrsdp_bar,c_g, kinc, kstep, dt);
}

void Material::constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt)
{
  printf("Function constBoxLargD is not defined for this material\n");
}

void Material::constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev, double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt)
{
  printf("Function constBox_2order is not defined for this material\n");
}

void Material::constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt)
{
  printf("Function constbox_2ndNtr is not defined for this material\n");
}

int Material::get_pos_eqstrs() const
{
  printf("Function get_pos_eqstrs() is not defined for this material\n");
  return -1;
}

bool Material::eqstrs_exist() const
{
    return false;
}
          

#ifdef NONLOCALGMSH
int Material::get_pos_mtx_stress () const
{ 
   return -1;
}
int Material::get_pos_inc_stress () const
{
   return -1;
}
int Material::get_pos_mtx_strain () const
{
  return -1;
}
int Material::get_pos_inc_strain () const
{
  return -1;
}
int Material::get_pos_mtx_Ja() const
{
  return -1;
}
int Material::get_pos_inc_Ja() const
{
  return -1;
}
void Material::get_elOp_mtx(double** Cel)
{
	  printf("Function get_elOp_mtx is not defined for this material\n");
}

void Material::get_elOp_icl(double** Cel)
{
	  printf("Function get_elOp_icl is not defined for this material\n");

}

double Material::get_vfI()
{
	  printf("Function get_vfI is not defined for this material\n");

}
#endif




//remarks: For tangent M-T method, the state varibles keep the effective stress value, then this value is corrected by minus (D_stres_eff) in finit element
//         For first order secant M-T method, the state varibles keep the apparent stress value, then D_stres_eff is set zero as feedback for finite element. 

//**************************
//LINEAR ELASTICITY 
//PROPS: MODEL, iddam, E, nu
//STATEV: strn, strs, dstrn
//**************************
//constructor
EL_Material::EL_Material(double* props, int idmat):Material(props, idmat){
	
}

#ifdef NONLOCALGMSH
int EL_Material::get_pos_currentplasticstrainfornonlocal() const 
{
	if(dam)
	{
		return pos_p;
	}
	return -1;
}
int EL_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
	if(dam)
	{
		return pos_dam;
	}
	return -1;
}

int EL_Material::get_pos_damagefornonlocal() const
{
	if(dam) return pos_dam+2;
        return -1;
}
double EL_Material::getElasticEnergy (double* statev) 
{
    double eE = 0.;

    return eE;

}

double EL_Material::getPlasticEnergy (double* statev) 
{
    double eP = 0.;
    return eP;
}

#endif



//destructor
EL_Material::~EL_Material(){
	
	if(dam) delete dam;	
        if(clength) delete clength;

}

//print
void EL_Material::print(){

	printf("Linear elasticity\n");
	printf("Young modulus: %lf, Poisson ratio: %lf\n", E, nu);
	if(dam){ dam->print();}
        if(clength){clength->print();}
	
}


//constbox
void EL_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar,double *Sp_bar, double** c_g, int kinc, int kstep, double dt){


}

//costboxSecant
void EL_Material::constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){



}


void EL_Material::constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){
 
}

//constitutive box called by 2rd order method*******************
//for second-order method
//**************************************************************
void EL_Material::constbox_2order( int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev, 
double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt){


}





//reference operator from statev or material properties 
// = Hooke's operator in any case
void EL_Material::get_refOp(double* statev, double** C, int kinc, int kstep){

	get_elOp(C);
}

void EL_Material::get_elOp(double** Cel){

	compute_Hooke(E,nu,Cel);
}
void EL_Material::get_elOD(double* statev, double** Cel)
{
     get_elOp(Cel);
     if(dam)
     {
       double damage = statev[get_pos_dam()];
       for(int i=0; i < 6; i++)
         for(int j=0; j < 6; j++)
           Cel[i][j]*=1.-damage;
    }
}


void EL_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){

       printf("unload_step not implemented for EL material\n");
}

//*****************************************************
//RATE-INDEPENDENT ELASTO-PLASTICITY
//PROPS: MODEL, iddam, E, nu, sy0, htype, hmod1, (hmod2), hexp 
//STATEV: strn, strs, dstrn, pstrn, p, dp, 2*mu_iso
//*****************************************************
//constructor
EP_Material::EP_Material(double* props, int idmat):Material(props, idmat){

}


int EP_Material::get_pos_eqstrs() const 
{
	return pos_eqstrs;
}

bool EP_Material::eqstrs_exist() const
{
        return true;
}


#ifdef NONLOCALGMSH
int EP_Material::get_pos_currentplasticstrainfornonlocal() const 
{
	return pos_p;
}
int EP_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
}

int EP_Material::get_pos_damagefornonlocal() const
{
	if(dam) return pos_dam+2;
        return -1;
}
double EP_Material::getElasticEnergy (double* statev)
{
  double eE = 0.;

  return eE;
}

double EP_Material::getPlasticEnergy (double* statev) 
{
  double eP = 0.;
  return eP;
}
#endif

//destructor
EP_Material::~EP_Material(){
	if(dam) delete dam;
        if(clength) delete clength;
}

//print
void EP_Material::print(){

	printf("J2 elasto-plasticity\n");
	printf("Young modulus: %lf, Poisson ratio: %lf\n", E, nu);
	printf("Intial yield stress: %lf, hardening law: %d, hardening coefficients: %lf\t%lf\t%lf\n", sy0, htype,hmod1,hmod2,hexp);
	if(dam){ dam->print();}
        if(clength){clength->print();}
}

//constbox      //*****************New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar ****************
void EP_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar,double *Sp_bar, double** c_g, int kinc, int kstep, double dt){

}
//******************************************************
//costboxSecant
//****************************************************
void EP_Material::constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){


                
}
//end of costboxSecant

// constboxSecant with Cs computed from Zero stress
void EP_Material::constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){



}
//end of costboxSecant

//**************************************************************
// EP_Material::constbox for large deformation 
//*************************************************************************************************************
void EP_Material::constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){


}
//end of constbox large deformation

//****************************************************
// consbox called by MTSecNtr 2rd order method
//****************************************************

void EP_Material::constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt){

          
}
// end of constbox_2ndNtr()


//****************************************************
// consbox called by 2rd order method
//****************************************************

void EP_Material::constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev,  double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt){


}

//end of consbox_2order

//****************************************************
//****************************************************

void EP_Material::get_refOp(double* statev, double** C, int kinc, int kstep){

	

}

void EP_Material::get_elOD(double* statev, double** Cel)
{
}


void EP_Material::get_elOp(double** Cel){

	compute_Hooke(E,nu,Cel);
}

void EP_Material::unload_step(double* dstrn,double* strs_n, double* statev_n, int kinc, int kstep){


}



//*******************  Stochastic  EP  *************************
//Stochastic RATE-INDEPENDENT ELASTO-PLASTICITY
//PROPS: MODEL, iddam, E, nu, sy0, htype, hmod1, (hmod2), hexp 
//STATEV: strn, strs, dstrn, pstrn, p, dp, 2*mu_iso
//**************************************************************
//constructor
EP_Stoch_Material::EP_Stoch_Material(double* props, int idmat):Material(props, idmat){

}


int EP_Stoch_Material::get_pos_eqstrs() const 
{
	return pos_eqstrs;
}

bool EP_Stoch_Material::eqstrs_exist() const
{
        return true;
}


#ifdef NONLOCALGMSH
int EP_Stoch_Material::get_pos_currentplasticstrainfornonlocal() const 
{
	return pos_p;
}
int EP_Stoch_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
	if(dam)
	{
		return pos_dam;
	}
	return -1;
}

int EP_Stoch_Material::get_pos_damagefornonlocal() const
{
	if(dam) return pos_dam+2;
        return -1;
}
double EP_Stoch_Material::getElasticEnergy (double* statev)
{
  double eE = 0.;

  return eE;
}

double EP_Stoch_Material::getPlasticEnergy (double* statev) 
{
  double eP = 0.;
  return eP;
}
#endif

//destructor
EP_Stoch_Material::~EP_Stoch_Material(){
}

//print
void EP_Stoch_Material::print(){

}

//constbox      //*****************New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar ****************
void EP_Stoch_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar,double *Sp_bar, double** c_g, int kinc, int kstep, double dt){

}
//******************************************************
//costboxSecant
//****************************************************
void EP_Stoch_Material::constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){

                
}
//end of costboxSecant

// constboxSecant with Cs computed from Zero stress
void EP_Stoch_Material::constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){



}
//end of costboxSecant


//****************************************************
// consbox called by MTSecNtr 2rd order method
//****************************************************

void EP_Stoch_Material::constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt){

          
}
// end of constbox_2ndNtr()



//****************************************************
//****************************************************

void EP_Stoch_Material::get_refOp(double* statev, double** C, int kinc, int kstep){
	

}

void EP_Stoch_Material::get_elOD(double* statev, double** Cel)
{
}


void EP_Stoch_Material::get_elOp(double* statev, double** Cel){

}

void  EP_Stoch_Material::get_elOp(double** Cel) {
      printf("function get_elOp(double** Cel) is not defined for stochastic materials");
}

void EP_Stoch_Material::unload_step(double* dstrn,double* strs_n, double* statev_n, int kinc, int kstep){


}


//****************************************************************************************
//**** RATE-DEPENDENT elasto-viscoplastic material ************************************
//PROPS: MODEL, iddam, E, nu, sy0, htype, hmod1, (hmod2), hexp, viscmodel, vmod, vexp 
//STATEV: strn, strs, dstrn, pstrn, p, dp, 2*mu_iso
//****************************************************************************************
//constructor
EVP_Material::EVP_Material(double* props, int idmat):Material(props, idmat){

}


int EVP_Material::get_pos_eqstrs() const 
{
	return pos_eqstrs;
}

bool EVP_Material::eqstrs_exist() const
{
        return true;
}


#ifdef NONLOCALGMSH
int EVP_Material::get_pos_currentplasticstrainfornonlocal() const 
{
	return pos_p;
}
int EVP_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
	if(dam)
	{
		return pos_dam;
	}
	return -1;
}

int EVP_Material::get_pos_damagefornonlocal() const
{
	if(dam) return pos_dam+2;
        return -1;
}
double EVP_Material::getElasticEnergy (double* statev)
{
  double eE = 0.;
  return eE;
}

double EVP_Material::getPlasticEnergy (double* statev) 
{
  double eP = 0.;
  return eP;
}
#endif

//destructor
EVP_Material::~EVP_Material(){
	if(dam) delete dam;
        if(clength) delete clength;
}

//print
void EVP_Material::print(){

	printf("J2 elasto-ViscoPlasticity\n");
	printf("Young modulus: %lf, Poisson ratio: %lf\n", E, nu);
	printf("Intial yield stress: %lf, hardening law: %d, hardening coefficients: %lf\t%lf\t%lf\n", sy0, htype,hmod1,hmod2,hexp);
	printf("ViscoPlastic function: %d, ViscoPlastic coefficients: %lf\t%lf\n", viscmodel, vmod, vexp);
        if(dam){ dam->print();}
        if(clength){clength->print();}
}

//constbox      //*****************New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar ****************
void EVP_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar,double *Sp_bar, double** c_g, int kinc, int kstep, double dt){

}
//******************************************************
//costboxSecant
//****************************************************
void EVP_Material::constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){

                
}
//end of costboxSecant

// constboxSecant with Cs computed from Zero stress
void EVP_Material::constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){



}
//end of costboxSecant

//****************************************************
// consbox called by MTSecNtr 2rd order method
//****************************************************

void EVP_Material::constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt){

          
}
// end of constbox_2ndNtr()


//****************************************************
//    For large deformation
// consbox called by MTSecF 1rd order method
//****************************************************

void EVP_Material::constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){

                
}
// end of constbox_LargD()

//****************************************************
// consbox called by 2rd order method
//****************************************************

void EVP_Material::constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev,  double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt){


}

//end of consbox_2order


//****************************************************
//****************************************************

void EVP_Material::get_refOp(double* statev, double** C, int kinc, int kstep){

	

}

void EVP_Material::get_elOD(double* statev, double** Cel)
{
}


void EVP_Material::get_elOp(double** Cel){

	compute_Hooke(E,nu,Cel);
}

void EVP_Material::unload_step(double* dstrn,double* strs_n, double* statev_n, int kinc, int kstep){


}

//******End of elasto viscoplastic constitutive law***************************************

//********************************************************************************************
// COMPOSITE - VOIGT  
//PROPS: MODEL, IDDAM, Nph, IDMAT phase 1, IDTOP phase 1, ... , 
//	 	...	IDMAT phase N, IDTOP phase N
//  (material properties of each phase), (volume fraction of each phase)
//STATEV: strn, strs, dstrn, (statev of each phase)   
//********************************************************************************************
//constructor
VT_Material::VT_Material(double* props, int idmat):Material(props, idmat){


 
}
#ifdef NONLOCALGMSH
int VT_Material::get_pos_currentplasticstrainfornonlocal() const
{
	return -1;
}
int VT_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
	return -1;
}

int VT_Material::get_pos_damagefornonlocal() const
{
	return -1;
}
double VT_Material::getElasticEnergy (double* statev) 
{
    double eE = 0.;
    return eE;

}

double VT_Material::getPlasticEnergy (double* statev)
{
    double eP = 0.;
    return eP;
}

#endif

//destructor
VT_Material::~VT_Material(){

	int i;
	for (i=0;i<Nph;i++){
		if(mat[i]){
			delete mat[i];
		}
	}
	free(mat);
	free(vf);
	if(dam) delete(dam);
        if(clength) delete clength;
	
}

//print
void VT_Material::print(){

	int i;
	printf("Voigt\n");
	if(dam){ dam->print();}

	printf("Number of phases: %d\n", Nph);
	
	for(i=0;i<Nph;i++){
		printf("phase %d, volume fraction: %lf\n",i+1,vf[i]);
		mat[i]->print();
	}
}

//constbox
void VT_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){


}

//reference operator from statev
void VT_Material::get_refOp(double* statev, double** C, int kinc, int kstep){

	printf("get_refOp not implemented for Voigt material\n");

}

void VT_Material::get_elOp(double** Cel){

	printf("get_elOp not implemented for Voigt material\n");

}

void VT_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){

       printf("unload_step not implemented for Voigt material\n");
}

void VT_Material::get_elOD(double* statev, double** Cel)
{
     get_elOp(Cel);
     if(dam)
     {
       double damage = statev[get_pos_dam()];
       for(int i=0; i < 6; i++)
         for(int j=0; j < 6; j++)
           Cel[i][j]*=1.-damage;
    }
}
//**************************************************************************************************************************************************
// COMPOSITE - MORI-TANAKA  
//PROPS: MODEL, IDDAM, Nph=2, IDMAT matrix, IDMAT inclusion, IDTOP inclusion,
//  (material properties of each phase), topology of inclusion: vf, ar1, ar2, 3 euler angles
//STATEV: strn, strs, dstrn, MT macro operator (21 components - symmetric), MT Elastic operator (21 components - symmetric), (statev of each phase)   
//***************************************************************************************************************************************************
//constructor
MT_Material::MT_Material(double* props, int idmat):Material(props,idmat){

	
}
#ifdef NONLOCALGMSH
int MT_Material::get_pos_currentplasticstrainfornonlocal() const
{
        if(mtx_mat->get_pos_currentplasticstrainfornonlocal()>=0)	
	    return mtx_mat->get_pos_currentplasticstrainfornonlocal()+idsdv_m;
	return -1;

}
int MT_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
       if(mtx_mat->get_pos_effectiveplasticstrainfornonlocal()>=0)
	   return mtx_mat->get_pos_effectiveplasticstrainfornonlocal()+idsdv_m;
       return -1;
}

int MT_Material::get_pos_damagefornonlocal() const
{
       if(mtx_mat->get_pos_damagefornonlocal() >=0)
	   return mtx_mat->get_pos_damagefornonlocal()+idsdv_m;
       return -1;
}
int MT_Material::get_pos_mtx_stress () const
{ 
   return idsdv_m + mtx_mat->get_pos_strs();
}
int MT_Material::get_pos_inc_stress () const
{
   return idsdv_i + icl_mat->get_pos_strs();
}
int MT_Material::get_pos_mtx_strain () const
{
   return idsdv_m + mtx_mat->get_pos_strn();
}
int MT_Material::get_pos_inc_strain () const
{
   return idsdv_i + icl_mat->get_pos_strn();
}

double MT_Material::getElasticEnergy (double* statev) 
{
    double eEi = icl_mat->getElasticEnergy(&statev[idsdv_i]);
    double eE0 = mtx_mat->getElasticEnergy(&statev[idsdv_m]);
    double eE = vf_i*eEi+vf_m*eE0;

    return eE;

}

double MT_Material::getPlasticEnergy (double* statev) 
{
    double ePi = icl_mat->getPlasticEnergy(&statev[idsdv_i]);
    double eP0 = mtx_mat->getPlasticEnergy(&statev[idsdv_m]);
    double eP = vf_i*ePi+vf_m*eP0;

    return eP;
}
#endif

//destructor
MT_Material::~MT_Material(){

	
	if(mtx_mat) delete mtx_mat;
	if(icl_mat) delete icl_mat;

	free(ar);
	free(euler);

	if(dam) delete(dam);
        if(clength) delete clength;

}

//print
void MT_Material::print(){

	
	printf("Mori-Tanaka\n");
	if(dam) dam->print();
	printf("Number of phases: %d\n", Nph);
	
	printf("matrix, volume fraction: %lf\n",vf_m);
	mtx_mat->print();

	printf("inclusion, volume fraction: %lf\n",vf_i);
	printf("aspect ratio: %lf, %lf\n",ar[0],ar[1]);
	printf("euler angles: %lf, %lf, %lf\n",euler[0],euler[1],euler[2]);	
	icl_mat->print();
	
}

//constbox   ************* new tensor added by wu ling: double* dpdE, double* strs_dDdp_bar 13.04.2011
void MT_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	//alpha: for mid-point integration law to compute reference MT operator: C_MT = (1-alpha)*C_MT(tn) + alpha*C_MT(tn+1)
	

}

void MT_Material::get_refOp(double* statev, double** C, int kinc, int kstep){



}


void MT_Material::get_elOp(double** Cel){

}
// Cel with damaged matrix and inclusion, add by Ling WU June 2012 

void MT_Material::get_elOD(double* statev, double** Cel)
{
}


void MT_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){
   printf("Function unload_step is not defined for this MT material\n");
}
//**************************************************************************************************************************************************
// COMPOSITE - MORI-TANAKA  
//*******modified by Ling wu for secant mehod of first moment---  JUNE 2012
//PROPS: MODEL, IDDAM, Nph=2, IDMAT matrix, IDMAT inclusion, IDTOP inclusion,
//  (material properties of each phase), topology of inclusion: vf, ar1, ar2, 3 euler angles
//STATEV: strn, strs, dstrn, MT macro operator (21 components - symmetric), MT Elastic operator (21 components - symmetric), (statev of each phase)   
//***************************************************************************************************************************************************
//
//constructor
MTSecF_Material::MTSecF_Material(double* props, int idmat):MT_Material(props,idmat){

       malloc_lcc(&LCC, Nph, vf_i);
}

//destructor
MTSecF_Material::~MTSecF_Material(){
    
      free_lcc(&LCC);
}


//constbox   ************* new tensor added by wu ling: double* dpdE, double* strs_dDdp_bar 13.04.2011
//*********************************************
//unloading is carried out at each iteration 

void MTSecF_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* dstrsdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	//alpha: for mid-point integration law to compute reference MT operator: C_MT = (1-alpha)*C_MT(tn) + alpha*C_MT(tn+1)
	
        

}



// unloading step for MTSecF_Material material to reset statev_n in residual condition, by Ling Wu June 2012

void MTSecF_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){


}

//**************************************************************************************************************************************************
// COMPOSITE - MORI-TANAKA   
//*******modified by Ling wu for second moment for Ntr--  Feb 2014
//PROPS: MODEL, IDDAM, Nph=2, IDMAT matrix, IDMAT inclusion, IDTOP inclusion,
//  (material properties of each phase), topology of inclusion: vf, ar1, ar2, 3 euler angles
//STATEV: strn, strs, dstrn, MT macro operator (21 components - symmetric), MT Elastic operator (21 components - symmetric), (statev of each phase)   
//***************************************************************************************************************************************************
//
//constructor
MTSecNtr_Material::MTSecNtr_Material(double* props, int idmat):MT_Material(props,idmat){
     malloc_lcc(&LCC, Nph, vf_i);

}

//destructor
MTSecNtr_Material::~MTSecNtr_Material(){
    free_lcc(&LCC);
}


//constbox   ************* new tensor added by wu ling: double* dpdE, double* strs_dDdp_bar 13.04.2011
//*********************************************
//unloading is carried out at each iteration 

void MTSecNtr_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* dstrsdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	//alpha: for mid-point integration law to compute reference MT operator: C_MT = (1-alpha)*C_MT(tn) + alpha*C_MT(tn+1)
	

}




// unloading step for MTSecF_Material material to reset statev_n in residual condition, by Ling Wu June 2012

void MTSecNtr_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){


}
//**************************************************************************************************************************************************
// COMPOSITE - MORI-TANAKA  
//*******modified by Ling wu for secant mehod of Second moment---  AUG 2013
//PROPS: MODEL, IDDAM, Nph=2, IDMAT matrix, IDMAT inclusion, IDTOP inclusion,
//  (material properties of each phase), topology of inclusion: vf, ar1, ar2, 3 euler angles
//STATEV: strn, strs, dstrn, MT macro operator (21 components - symmetric), MT Elastic operator (21 components - symmetric), (statev of each phase)   
//***************************************************************************************************************************************************
//
//constructor
MTSecSd_Material::MTSecSd_Material(double* props, int idmat):MT_Material(props,idmat){

     malloc_lcc(&LCC, Nph, vf_i);

}

//destructor
MTSecSd_Material::~MTSecSd_Material(){

     free_lcc(&LCC);  

}


//constbox   ************* new tensor added by wu ling: double* dpdE, double* strs_dDdp_bar 13.04.2011
//*********************************************
//unloading is carried out at each iteration 

void MTSecSd_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* dstrsdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	//alpha: for mid-point integration law to compute reference MT operator: C_MT = (1-alpha)*C_MT(tn) + alpha*C_MT(tn+1)
	
        
}



// unloading step for MTSecSd_Material material to reset statev_n in residual condition, and the second monment von mise stress eqstrs in elastic-plastic phases, by Ling Wu AUG 2013
// the value of strain concentration tensor is also initialized with mu0 mu1 kappa0 kappa1

void MTSecSd_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){

}


//**************************************************************************
//By Ling Wu May,2016
//Composite material homogenized with Mori-Tanaka scheme with First-order Incremental-secant method
//when vfi > 1, volume fraction is a random variable depends on the location of Gaussian point.
//when euler > 180, euler angles are random variables depend on the location of Gaussian point.
//**************************************************************************
//constructor
MTSecF_Stoch_Material::MTSecF_Stoch_Material(double* props, int idmat):MT_Material(props,idmat){

	
}

//destructor
//destructor
MTSecF_Stoch_Material::~MTSecF_Stoch_Material(){

}

int MTSecF_Stoch_Material::get_pos_E()const 
{
   if(mtx_mat->get_pos_E() !=0) return idsdv_m + mtx_mat->get_pos_E();
   else return 0;
}
int MTSecF_Stoch_Material::get_pos_nu()const 
{
   if(mtx_mat->get_pos_nu() != 0) return idsdv_m + mtx_mat->get_pos_nu();
   else return 0;
}  
int MTSecF_Stoch_Material::get_pos_sy0()const 
{
   if(mtx_mat->get_pos_sy0()!= 0) return idsdv_m + mtx_mat->get_pos_sy0();
   else return 0;
}  
int MTSecF_Stoch_Material::get_pos_hmod1()const 
{
   if(mtx_mat->get_pos_hmod1()!= 0) return idsdv_m + mtx_mat->get_pos_hmod1();
   else return 0;
}  
int MTSecF_Stoch_Material::get_pos_hmod2()const 
{
   if(mtx_mat->get_pos_hmod2()!= 0) return idsdv_m + mtx_mat->get_pos_hmod2();
   else return 0;
}
int MTSecF_Stoch_Material::get_pos_hexp()const   
{
   if(mtx_mat->get_pos_hexp()!= 0) return idsdv_m + mtx_mat->get_pos_hexp();
   else return 0;
}
int MTSecF_Stoch_Material::get_pos_hp0()const 
{
   if(mtx_mat->get_pos_hp0()!= 0) return idsdv_m + mtx_mat->get_pos_hp0();
   else return 0;
}

int MTSecF_Stoch_Material::get_pos_DamParm1()const   
{
   if(mtx_mat->get_pos_DamParm1()!= 0) return idsdv_m + mtx_mat->get_pos_DamParm1();
   else return 0;
}
int MTSecF_Stoch_Material::get_pos_DamParm2()const   
{
   if(mtx_mat->get_pos_DamParm2()!= 0) return idsdv_m + mtx_mat->get_pos_DamParm2();
   else return 0;
}
//constbox
void MTSecF_Stoch_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* dstrsdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	//alpha: for mid-point integration law to compute reference MT operator: C_MT = (1-alpha)*C_MT(tn) + alpha*C_MT(tn+1)
	
}

// unloading step for MTSecF_Material material to reset statev_n in residual condition, by Ling Wu June 2012

void MTSecF_Stoch_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, double _Vfm, double _Vfi, 
                                                                          double* _euler, double* _ar, int kinc, int kstep){


}

#ifdef NONLOCALGMSH
double MTSecF_Stoch_Material::getElasticEnergy (double* statev) 
{
    double _Vfi, _Vfm;
    if(pos_vfi == 0){
        _Vfi = vf_i;     
    }
    else{
        _Vfi = statev[pos_vfi];
    }
    _Vfm = 1.0 - _Vfi;

    double eEi = icl_mat->getElasticEnergy(&statev[idsdv_i]);
    double eE0 = mtx_mat->getElasticEnergy(&statev[idsdv_m]);
    double eE = _Vfi*eEi+_Vfm*eE0;

    return eE;

}

double MTSecF_Stoch_Material::getPlasticEnergy(double* statev) 
{
    double _Vfi, _Vfm;
    if(pos_vfi == 0){
        _Vfi = vf_i;     
    }
    else{
        _Vfi = statev[pos_vfi];
    }
    _Vfm = 1.0 - _Vfi;


    double ePi = icl_mat->getPlasticEnergy(&statev[idsdv_i]);
    double eP0 = mtx_mat->getPlasticEnergy(&statev[idsdv_m]);
    double eP = _Vfi*ePi+_Vfm*eP0;

    return eP;
}
#endif

void MTSecF_Stoch_Material::get_refOp(double* statev, double** C, int kinc, int kstep){


	int i,j,k;

	if(kinc < 2 && kstep==1){
		get_elOp(statev, C);
	}
	else{
		k=pos_C;
		for(i=0;i<6;i++){
			for(j=i;j<6;j++){
				C[i][j] = statev[k++];
				C[j][i] = C[i][j];
			}
		}
	}		

}

void MTSecF_Stoch_Material::get_elOp(double* statev, double** Cel){

}

void MTSecF_Stoch_Material::get_elOD(double* statev, double** Cel)
{
}

#ifdef NONLOCALGMSH

void MTSecF_Stoch_Material:: get_elOp_mtx(double** Cel){
	       mtx_mat->get_elOp(Cel);
}

void MTSecF_Stoch_Material::get_elOp_icl(double** Cel){
	       icl_mat->get_elOp(Cel);
}
#endif

//****end of stochastic material


//**************************************************************************
//By Ling Wu Dec 2016
//Composite material homogenized with Mori-Tanaka scheme with Second-order Incremental-secant method
// "vfi"- volume fraction and "euler" are variables in the case of large deformation.
//**************************************************************************
//constructor
MTSecF_LargD::MTSecF_LargD(double* props, int idmat):MT_Material(props,idmat){

        pos_vfi = nsdv;
        nsdv += 1;
        pos_aspR = nsdv;
        nsdv += 2;
        pos_R66 = nsdv;
        nsdv += 36;
}

//destructor
//destructor
MTSecF_LargD::~MTSecF_LargD(){

}
//constbox
void MTSecF_LargD::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* dstrsdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	//alpha: for mid-point integration law to compute reference MT operator: C_MT = (1-alpha)*C_MT(tn) + alpha*C_MT(tn+1)
	
}

// unloading step for MTSecF_Material material to reset statev_n in residual condition, by Ling Wu June 2012

void MTSecF_LargD::unload_step(double* dstrn, double* strs_n, double* statev_n, double _Vfm, double _Vfi, 
                                       double** R66, int kinc, int kstep){

}

#ifdef NONLOCALGMSH
double MTSecF_LargD::getElasticEnergy (double* statev) 
{
    double _Vfi, _Vfm;
    _Vfi = statev[pos_vfi];
    _Vfm = 1.0 - _Vfi;

    double eEi = icl_mat->getElasticEnergy(&statev[idsdv_i]);
    double eE0 = mtx_mat->getElasticEnergy(&statev[idsdv_m]);
    double eE = _Vfi*eEi+_Vfm*eE0;

    return eE;
}

double MTSecF_LargD::getPlasticEnergy(double* statev) 
{
    double _Vfi, _Vfm;
    _Vfi = statev[pos_vfi];    
    _Vfm = 1.0 - _Vfi;

    double ePi = icl_mat->getPlasticEnergy(&statev[idsdv_i]);
    double eP0 = mtx_mat->getPlasticEnergy(&statev[idsdv_m]);
    double eP = _Vfi*ePi+_Vfm*eP0;

    return eP;
}
#endif

void MTSecF_LargD::get_refOp(double* statev, double** _R66, double** C, int kinc, int kstep){



}

void MTSecF_LargD::get_elOp(double* statev, double** _R66, double** Cel){

}

void MTSecF_LargD::get_elOD(double* statev, double** _R66, double** Cel)
{
}


//********************************************************************************************
// COMPOSITE - SELF-CONSISTENT
//PROPS: MODEL, IDDAM, mtx (1 or 0), Nicl, IDMAT matrix (if any), IDMAT inclusion, IDTOP inclusion,
//  (material properties of each phase), topology of inclusions: vf, ar1, ar2, 3 euler angles
//STATEV: strn, strs, dstrn, SC macro operator (36 components) , (statev of each phase)   
//********************************************************************************************
//constructor
SC_Material::SC_Material(double* props, int idmat):Material(props, idmat){

}
#ifdef NONLOCALGMSH
int SC_Material::get_pos_currentplasticstrainfornonlocal() const
{
	if(mtx_mat->get_pos_currentplasticstrainfornonlocal()>=0)
	     return mtx_mat->get_pos_currentplasticstrainfornonlocal()+idsdv_m;
        return -1;

}
int SC_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
	if(mtx_mat->get_pos_effectiveplasticstrainfornonlocal()>=0)
             return mtx_mat->get_pos_effectiveplasticstrainfornonlocal()+idsdv_m;
	return -1;
}

int SC_Material::get_pos_damagefornonlocal() const
{
	if(mtx_mat->get_pos_damagefornonlocal() >=0)
	     return mtx_mat->get_pos_damagefornonlocal();
	return -1;
}
int SC_Material::get_pos_mtx_stress () const
{ 
   return idsdv_m + mtx_mat->get_pos_strs();
}
int SC_Material::get_pos_inc_stress () const
{
   return idsdv_i[0] + icl_mat[0]->get_pos_strs();
}
int SC_Material::get_pos_mtx_strain () const
{
   return idsdv_m + mtx_mat->get_pos_strn();
}
int SC_Material::get_pos_inc_strain () const
{
   return idsdv_i[0] + icl_mat[0]->get_pos_strn();
}
double SC_Material::getElasticEnergy (double* statev) 
{
    int i=0;
    double eEi = 0.;
    double eE0 = mtx_mat->getElasticEnergy(&statev[idsdv_m]);
    for(i=0;i<Nicl;i++)
	eEi+=vf_i[i]*icl_mat[i]->getElasticEnergy(&statev[idsdv_i[i]]);
    double eE = eEi+vf_m*eE0;
    return eE;

}

double SC_Material::getPlasticEnergy (double* statev) 
{
    int i=0;
    double ePi = 0.;
    double eP0 = mtx_mat->getPlasticEnergy(&statev[idsdv_m]);
    for(i=0;i<Nicl;i++) ePi += vf_i[i]*icl_mat[i]->getPlasticEnergy(&statev[idsdv_i[i]]);
    double eP = ePi+vf_m*eP0;

    return eP;
}

#endif

//destructor
SC_Material::~SC_Material(){

	int i;
	if(mtx_mat) delete mtx_mat;
	for(i=0;i<Nicl;i++){
		if(icl_mat[i]) delete icl_mat[i];
	}
	free(icl_mat);

	freematrix(ar,Nicl);
	freematrix(euler,Nicl);
	free(idsdv_i);
	free(vf_i);

	if(dam) delete(dam);
        if(clength) delete clength;

}

//print
void SC_Material::print(){

	int i;
	printf("Self-Consistent\n");
	if(dam) dam->print();
	printf("Number of phases: %d\n", Nph);
	
	if(mtx){
		printf("matrix, volume fraction: %lf\n",vf_m);
		mtx_mat->print();
	}
	for(i=0;i<Nicl;i++){
		printf("inclusion %d, volume fraction: %lf\n",i+1,vf_i[i]);
		printf("aspect ratio: %lf, %lf\n",ar[i][0],ar[i][1]);
		printf("euler angles: %lf, %lf, %lf\n",euler[i][0],euler[i][1],euler[i][2]);	
		(icl_mat[i])->print();
	}
	
}

//constbox
void SC_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	
}

void SC_Material::get_refOp(double* statev_n, double** C, int kinc, int kstep){
}
void SC_Material::get_elOp(double** Cel){

	printf("get_elOp not implemented for Self-Consistent material\n");
}

void SC_Material::get_elOD(double* statev, double** Cel)
{
     printf("get_elOD not implemented for Self-Consistent material\n");
}

void SC_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){

       printf("unload_step not implemented for SC material\n");
}
//********************************************************************************************
// COMPOSITE - Two-step Mori-Tanaka/Voight 
//PROPS: MODEL, IDDAM, mtx (1 or 0), Nph, IDMAT matrix (if any), IDMAT inclusion, IDTOP inclusion,
//  (material properties of each phase), topology of inclusions: vf, ar1, ar2,  and Matrix orientation a
//STATEV: strn, strs, dstrn, MT macro operator (21 components) , (statev of each phase)   
//********************************************************************************************
//constructor

MT_VT_Material::MT_VT_Material(double* props, int idmat):Material(props, idmat){

}      

#ifdef NONLOCALGMSH
int MT_VT_Material::get_pos_currentplasticstrainfornonlocal() const
{
   if(pos_p>=0)
      return pos_p;
   return -1;

}
int MT_VT_Material::get_pos_effectiveplasticstrainfornonlocal() const
{
	if(pos_dam>=0)
             return pos_dam;
	return -1;
}

int MT_VT_Material::get_pos_damagefornonlocal() const
{
	if(pos_dam >=0)
	     return pos_dam+2;
	return -1;
}
int MT_VT_Material::get_pos_mtx_stress () const
{ 
   // at this point we take the value of the first phase
   return idsdv_i[0] + (ph_mat[0])->get_pos_mtx_stress ();
}
int MT_VT_Material::get_pos_inc_stress () const
{
   // at this point we take the value of the first phase
   return idsdv_i[0] + (ph_mat[0])->get_pos_inc_stress ();
}
int MT_VT_Material::get_pos_mtx_strain () const
{ 
   // at this point we take the value of the first phase
   return idsdv_i[0] + (ph_mat[0])->get_pos_mtx_strain ();
}
int MT_VT_Material::get_pos_inc_strain () const
{
   // at this point we take the value of the first phase
   return idsdv_i[0] + (ph_mat[0])->get_pos_inc_strain ();
}
double MT_VT_Material::getElasticEnergy (double* statev) 
{
    int i=0;
    double eE = 0.;
    for(i=0;i<Total_number_phase;i++)
	eE+=Wgt[i]*ph_mat[i]->getElasticEnergy(&statev[idsdv_i[i]]);;
    return eE;

}

double MT_VT_Material::getPlasticEnergy (double* statev) 
{
    int i=0;
    double eP = 0.;
    for(i=0;i<Total_number_phase;i++) eP += Wgt[i]*ph_mat[i]->getPlasticEnergy(&statev[idsdv_i[i]]);

    return eP;
}

#endif


//destructor
MT_VT_Material::~MT_VT_Material(){

	int i;
	//if(Total_number_phase) delete Total_number_phase;
	//for(i=0;i<s;i++){
	//	if(ph_mat[i]) delete ph_mat[i];
	//}
	free(ph_mat);
	free(idsdv_i);
        if(clength) delete clength;
          free(Wgt);
      //  if(dam) delete(dam);
}

void MT_VT_Material::print(){

	int i;
	printf("Two-step Mori-Tanaka/Voigt\n");
	if(dam) dam->print();
	printf("Number of phases: %d\n", Total_number_phase);

	for(i=0;i<Total_number_phase;i++){
			
		(ph_mat[i])->print();
	}
	
}

//constbox
void MT_VT_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt){
	

 
}     //*********add by MOHAMEDOU Mohamed

 
void MT_VT_Material::get_refOp(double* statev_n, double** C, int kinc, int kstep){

}

void MT_VT_Material::get_elOp(double** Cel){
double **Ctmp=NULL ;
if(Ctmp==NULL){	
 mallocmatrix(&Ctmp,6,6);
  }

cleartens4(Ctmp);
	
 for(int i=0;i<Total_number_phase;i++){
          ph_mat[i]->get_elOp(Ctmp);
 // C+=Ctmp*Wtg
          addtens4(1.,Cel,Wgt[i],Ctmp,Cel);
   }
  }
	
  


void MT_VT_Material::get_elOD(double* statev, double** Cel){
  
double **Ctmp=NULL ; 
 if(Ctmp==NULL){	
 mallocmatrix(&Ctmp,6,6);
  } 
 cleartens4(Ctmp);
	
 for(int i=0;i<Total_number_phase ;i++){

          ph_mat[i]->get_elOD(&(statev[idsdv_i[i]]),Ctmp);
 // C+=Ctmp*Wtg
          addtens4(1.,Cel,Wgt[i],Ctmp,Cel);
   }
  }

void MT_VT_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){

       printf("unload_step not implemented for Mori-Tanaka_Voight material\n");
}

//********************************************************************
//************  add by Ling wu 14/09/2011   *********************
//ANISOTROPIC LINEAR ELASTICITY 
//PROPS: MODEL, iddam, anpr(E1, E2,E3,nu12,nu13,nu23,G12,G13,G23) 9 material constants
//PROPS: three euler anagles from golbe coordinates to local coordinates 
//( in order to agree with the euler angle of inclusion)
//STATEV: strn, strs, dstrn in global coordinates
//*******************************************************************
//constructor
ANEL_Material::ANEL_Material(double* props, int idmat):Material(props, idmat){
	

}

//destructor
ANEL_Material::~ANEL_Material(){
	
	if(dam) delete dam;
        if(clength) delete clength;

        free(anpr);
        free(euler);	

}

//print
void ANEL_Material::print(){

	printf("Anisotropic Linear elasticity\n");
        printf("Young modulus E1,E2,E3: %lf,%lf,%lf\n", anpr[0],anpr[1],anpr[2]);
	printf("Poisson ratio nu12,nu13,nu23: %lf, %lf, %lf\n", anpr[3],anpr[4],anpr[5]);
        printf("Shear modulus G12,G13,G23: %lf, %lf, %lf\n", anpr[6],anpr[7],anpr[8]);
        printf("Euler angles: %lf, %lf, %lf\n", euler[0],euler[1],euler[2]);
	if(dam){ dam->print();}
	
}
#ifdef NONLOCALGMSH
int ANEL_Material::get_pos_currentplasticstrainfornonlocal() const {return -1;}
int ANEL_Material::get_pos_effectiveplasticstrainfornonlocal() const {return -1;}
int ANEL_Material::get_pos_damagefornonlocal() const
{
	if(dam) return pos_dam;
	return -1;
}
double ANEL_Material::getElasticEnergy (double* statev) 
{
    double eE = 0.;
    int i;
    static double* strn = NULL;
    static double* strs = NULL;
    if(strn == NULL)
    {
       mallocvector(&strn,6);
       mallocvector(&strs,6);
    }
    copyvect(&(statev[get_pos_strn()]), strn,6);
    copyvect(&statev[get_pos_strs()], strs, 6);   
    //enegry due to damage has been removed
    for (i =0; i< 6; i++) eE+=0.5*(strs[i]*strn[i]);

    return eE;

}

double ANEL_Material::getPlasticEnergy (double* statev) 
{
    double eP = 0.;
    return eP;
}
#endif

//constbox
void ANEL_Material::constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar,double *Sp_bar, double** c_g, int kinc, int kstep, double dt){

}

void ANEL_Material::constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt){


}


//****************************************************
// consbox called by 2rd order method
//****************************************************

void ANEL_Material::constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev,  double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt){


}

//end of consbox_2order


//reference operator from statev or material properties 
// = anisotropic elastic operator in any case
void ANEL_Material::get_refOp(double* statev, double** C, int kinc, int kstep){

	get_elOp(C);
}

void ANEL_Material::get_elOp(double** Cel){

	compute_AnisoOperator(anpr, euler,Cel);
}

void ANEL_Material::get_elOD(double* statev, double** Cel)
{
     get_elOp(Cel);
     if(dam)
     {
       double damage = statev[get_pos_dam()];
       for(int i=0; i < 6; i++)
         for(int j=0; j < 6; j++)
           Cel[i][j]*=1.-damage;
    }
}

void ANEL_Material::unload_step(double* dstrn, double* strs_n, double* statev_n, int kinc, int kstep){

       printf("unload_step not implemented for ANEL material\n");
}
#endif
