#ifndef MATERIAL_H
#define MATERIAL_H 1

#include "ID.h"
#include "damage.h" 
#include "clength.h"
#include "lccfunctions.h"
#include "elasticlcc.h"

#ifdef NONLOCALGMSH
namespace MFH {
#endif
//****in all of the constbox**New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar ****************
//Generic material
class Material{

	protected:
		int constmodel;
		int nsdv;
		int pos_strn, pos_strs, pos_dstrn; 		//position of strain, stress and dstrn in the statev vector
		Damage* dam;
		int pos_dam;	//position of damage state variables in statev vector

                int pos_p;

                Clength* clength;

                bool evaluateStiffness;

	public:
		
		
		Material(double* props, int idmat){
			constmodel = (int) props[idmat];
                        evaluateStiffness = true;
		}
		virtual ~Material(){ ; }
		inline int get_constmodel(){  return constmodel; }
		
		//give the number of state variables needed for the constitutive model
		inline int get_nsdv() const{  return nsdv; }
		
		//give the position of strn, strs and dstrn in the statev vector
		inline int get_pos_strn() const {	return pos_strn; }
		inline int get_pos_strs() const {	return pos_strs; }
		inline int get_pos_dstrn() const	{	return pos_dstrn; }
                virtual int get_pos_dR() const { return -1;}

                //give the position of damage in the statev vector === p_bar, dp_bar ,D, dD
                inline int get_pos_dam() const { if(dam)   return pos_dam; else return -1; }    
                inline int get_pos_p() const { if(dam)   return pos_p; else return -1; }  
                virtual bool isInLargeDeformation() const {return false;}
		virtual const Damage* getDamage() const {return dam;}
		//print the material properties
		virtual void print()=0;

                inline bool getEvaluateStiffness() const {return evaluateStiffness;}
                inline void setEvaluateStiffness(bool stiff) {evaluateStiffness = stiff;}

		//compute the constitutive response of the material
		//INPUT: dstrn, strs_n, statev_n, kinc, kstep
		//OUTPUT: strs, statev, Cref, dCref, Calgo
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt)=0;

		virtual void constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt );
		virtual void constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);

                virtual void constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);

                virtual void constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev, double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt);

                virtual void constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt);

		//compute a reference tangent operator over the time increment using a generalized mid-point rule
		virtual void get_refOp(double* statev, double** C, int kinc, int kstep)=0;

		//compute an elastic operator 
		virtual void get_elOp(double** Cel)=0;
		virtual void get_elOD(double* statev, double** Cel)=0;
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep)=0;

                //added function for second moment methed
                virtual int get_pos_eqstrs() const;
                virtual bool eqstrs_exist() const;
                //add function for random volume fraction and euler angle.
		virtual int get_pos_vfi() const {  return 0; }
		virtual int get_pos_euler() const { return 0; }
		virtual int get_pos_aspR() const { return 0; }
		virtual int get_pos_Base() const { return 0; }
		virtual int get_pos_F() const { return 0; }
		virtual int get_pos_Basem() const { return 0; }
		virtual int get_pos_Basei() const { return 0; }
		virtual int get_pos_Fm() const { return 0; }
		virtual int get_pos_Fi() const { return 0; }
		virtual int get_pos_Ja() const { return 0; }
		virtual int get_pos_Ri() const { return 0; }

		virtual int get_pos_E() const {  return 0; } 
		virtual int get_pos_nu() const {  return 0; } 
		virtual int get_pos_sy0() const {  return 0; }  
		virtual int get_pos_hmod1() const {  return 0; } 
		virtual int get_pos_hmod2() const {  return 0; } 
		virtual int get_pos_hp0() const {  return 0; } 
		virtual int get_pos_hexp() const {  return 0; } 
		virtual int get_pos_DamParm1() const {  return 0; } 
		virtual int get_pos_DamParm2() const {  return 0; } 
                

#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const = 0;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const = 0;
		virtual int get_pos_damagefornonlocal() const = 0;
                virtual int get_pos_mtx_stress () const;
                virtual int get_pos_inc_stress () const;
                virtual int get_pos_mtx_strain () const;
                virtual int get_pos_inc_strain () const;
		virtual int get_pos_mtx_Ja() const;
		virtual int get_pos_inc_Ja() const;
		virtual double getElasticEnergy (double* statev) =0;
		virtual double getPlasticEnergy (double* statev) =0;

		virtual void get_elOp_mtx(double** Cel);
		virtual void get_elOp_icl(double** Cel);
                virtual double get_vfI();
#endif
		

};

//Linear elastic material
class EL_Material : public Material{

	private:
		double E, nu;
                int pos_Ja;
                int pos_Ri;
                int pos_Base;
                int pos_F;
               
	public:
		
		EL_Material(double* props, int idmat);
		~EL_Material();
		
		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
		virtual void constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);

		virtual void constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);

                virtual void constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev, double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt);

               	virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);

		virtual int get_pos_Ja() const {  return pos_Ja; } 
		virtual int get_pos_Ri() const {  return pos_Ri; }  
		virtual int get_pos_Base() const {  return pos_Base; } 
		virtual int get_pos_F() const {  return pos_F; } 
#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif
		
}; 

//Elasto-plastic material
class EP_Material : public Material{

	private:
		double E,nu,sy0;
		int htype;
		double hmod1, hmod2, hp0, hexp;
		int pos_pstrn, pos_dp, pos_twomu, pos_eqstrs;
                int pos_Base, pos_F, pos_Ja;
                int pos_Ri;

	public:
	

		EP_Material(double* props, int idmat);
		~EP_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
//*****************New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar, double* dDdE,****************
		virtual void constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
//******************new constbox for large deformation
                virtual void constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
//******************new constbox for 2rd order method
                virtual void constbox_2order( int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev, double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt);

//******************new constbox for 2nd moment estimation of stress_trial and Ntrial
                virtual void constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt);


		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);

                virtual int get_pos_eqstrs() const;
                virtual bool eqstrs_exist() const;
		virtual int get_pos_Base() const {  return pos_Base; } 
		virtual int get_pos_F() const {  return pos_F; } 
		virtual int get_pos_Ja() const {  return pos_Ja; } 
		virtual int get_pos_Ri() const {  return pos_Ri; }  

#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif


};


// Elasto-plastic Stochastic material
class EP_Stoch_Material : public Material{

	private:
		double _E,_nu,_sy0;
		int htype;
		double _hmod1, _hmod2, _hp0, _hexp;
                int pos_E, pos_nu, pos_sy0;
                int pos_hmod1, pos_hmod2, pos_hp0, pos_hexp;
                int pos_DamParm1, pos_DamParm2;
		int pos_pstrn, pos_dp, pos_twomu,pos_eqstrs;
	public:
		EP_Stoch_Material(double* props, int idmat);
		~EP_Stoch_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
//*****************New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar, double* dDdE,****************
		virtual void constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
//******************new constbox for 2nd moment estimation of stress_trial and Ntrial
                virtual void constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt);

		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double* statev, double** Cel);
                virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);

                virtual int get_pos_eqstrs() const;
                virtual bool eqstrs_exist() const;

		virtual int get_pos_E() const {  return pos_E; } 
		virtual int get_pos_nu() const {  return pos_nu; } 
		virtual int get_pos_sy0() const {  return pos_sy0; } 
 
		virtual int get_pos_hmod1() const {  return pos_hmod1; } 
		virtual int get_pos_hmod2() const {  return pos_hmod2; } 
		virtual int get_pos_hp0() const {  return pos_hp0; } 
		virtual int get_pos_hexp() const {  return pos_hexp; }  
		virtual int get_pos_DamParm1() const {  return pos_DamParm1; } 
		virtual int get_pos_DamParm2() const {  return pos_DamParm2; } 

#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif
};



//Elasto-Viscoplastic material
class EVP_Material : public Material{

	private:
		double E,nu,sy0;
		int htype;
		double hmod1, hmod2, hp0, hexp;
		int viscmodel;
                double vmod, vexp, vstressy;
		int pos_pstrn, pos_dp, pos_twomu, pos_eqstrs;
                int pos_Base, pos_F, pos_Ja; 
                int pos_Ri;           

	public:
	
		EVP_Material(double* props, int idmat);
		~EVP_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
//*****************New tenor added by wu ling: double* dpdE, double* strs_dDdp_bar, double* dDdE,****************
		virtual void constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void constboxSecantZero(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void constbox_2ndNtr(double *DE, double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,  ELcc* LCC, EPR* epresult, double alpha, double** c_g, int kinc, int kstep, double dt);
                virtual void constboxLargD(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, 
double* dnu, double &dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev,  double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt);      

		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);

                virtual int get_pos_eqstrs() const;
                virtual bool eqstrs_exist() const;
		virtual int get_pos_Base() const {  return pos_Base; } 
		virtual int get_pos_F() const {  return pos_F; } 
		virtual int get_pos_Ja() const {  return pos_Ja; } 
		virtual int get_pos_Ri() const {  return pos_Ri; }  

#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif

};



//Composite material homogenized with Voigt scheme
class VT_Material : public Material{

	private:
		int Nph;					//number of phases in the composite
		Material** mat;		//array of pointers on Materials
		double* vf;				//array of volume fraction of each phase
			

	public:
		
		VT_Material(double* props, int idmat);
		~VT_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif

};  
// Composite material homogenized with Mori-Tanaka scheme
class MT_Material : public Material{

	protected:
		int Nph;
		Material* mtx_mat;
		Material* icl_mat;
		int idsdv_m, idsdv_i;
		double vf_i, vf_m;
		double* ar;
		double* euler;
		int pos_C;
       
	public:
		
		MT_Material(double* props, int idmat);
		~MT_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
		virtual int get_pos_C() const {return pos_C; } 
	
		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual int get_pos_mtx_stress () const;
                virtual int get_pos_inc_stress () const;
                virtual int get_pos_mtx_strain () const;
                virtual int get_pos_inc_strain () const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;


#endif
};


//Composite material homogenized with Mori-Tanaka scheme with fist-order Incremental-secant method
class MTSecF_Material : public MT_Material{

	private:
                 ELcc LCC;
	public:
		
		MTSecF_Material(double* props, int idmat);
		~MTSecF_Material();

		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
};

//Composite material homogenized with Mori-Tanaka scheme with incremental secant method, second moment for the elastic predictor
class MTSecNtr_Material : public MT_Material{

	private:
                  ELcc LCC;
	public:
		
		MTSecNtr_Material(double* props, int idmat);
		~MTSecNtr_Material();

		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
};

//By Ling Wu AUG 2013
//Composite material homogenized with Mori-Tanaka scheme with Second-order Incremental-secant method
class MTSecSd_Material : public MT_Material{

	private:
                  Lcc LCC;
	public:
		
		MTSecSd_Material(double* props, int idmat);
		~MTSecSd_Material();

		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
};

//By Ling Wu May 2016
//Composite material homogenized with Mori-Tanaka scheme with First-order Incremental-secant method
//when vfi > 1, volume fraction is a random variable depends on the location of Gaussian point.
//when euler > 180, euler angles are random variables depend on the location of Gaussian point.
class MTSecF_Stoch_Material : public MT_Material{

	private:
                 int pos_vfi;
                 int pos_euler;
                 int pos_aspR;      //aspect ratio  ar2/ar1, ar1=1.0, ar2=aspR
	public:
		
		MTSecF_Stoch_Material(double* props, int idmat);
		~MTSecF_Stoch_Material();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev_n, double _Vfm, double _Vfi, 
                                                                          double* _euler, double* _ar, int kinc, int kstep);
		virtual int get_pos_vfi() const {  return pos_vfi; } 
		virtual int get_pos_euler() const {  return pos_euler; } 
		virtual int get_pos_aspR() const {  return pos_aspR; } 

		virtual int get_pos_E()const; 
		virtual int get_pos_nu()const;  
		virtual int get_pos_sy0()const;   
		virtual int get_pos_hmod1()const;  
		virtual int get_pos_hmod2()const;  
		virtual int get_pos_hp0()const;  
		virtual int get_pos_hexp()const;  
		virtual int get_pos_DamParm1()const;  
		virtual int get_pos_DamParm2()const; 

#ifdef NONLOCALGMSH
                virtual double getElasticEnergy (double* statev);
		virtual double getPlasticEnergy (double* statev);
		virtual void get_elOp_mtx(double** Cel);
		virtual void get_elOp_icl(double** Cel);
                inline double get_vfI() {return vf_i;}
#endif
                virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
                virtual void get_elOp(double* statev, double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
};


//By Ling Wu Dec 2016
//Composite material homogenized with Mori-Tanaka scheme with Second-order Incremental-secant method
// "vfi"- volume fraction and "euler" are variables in the case of large deformation.
class MTSecF_LargD : public MT_Material{

	private:
                 int pos_vfi;
                 int pos_aspR;
                 int pos_R66;
                 int pos_dR;
                 
	public:
		
		MTSecF_LargD(double* props, int idmat);
		~MTSecF_LargD();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev_n, double _Vfm, double _Vfi, 
                                                                                       double** R66, int kinc, int kstep);
		virtual int get_pos_vfi() const {  return pos_vfi; } 
		virtual int get_pos_aspR() const {  return pos_aspR; }
		virtual int get_pos_R66() const {  return pos_R66; }
		virtual int get_pos_dR() const {  return pos_dR; }
                virtual int get_pos_Ri() const { return idsdv_i+icl_mat->get_pos_Ri(); }
		virtual int get_pos_Basem() const { return idsdv_m+mtx_mat->get_pos_Base(); }
		virtual int get_pos_Basei() const { return idsdv_i+icl_mat->get_pos_Base(); }
		virtual int get_pos_Fm() const { return idsdv_m+mtx_mat->get_pos_F(); }
		virtual int get_pos_Fi() const { return idsdv_i+icl_mat->get_pos_F(); }
		virtual int get_pos_mtx_Ja() const {return idsdv_m + mtx_mat->get_pos_Ja(); }
                virtual int get_pos_inc_Ja() const { return idsdv_i + icl_mat->get_pos_Ja(); }


                virtual bool isInLargeDeformation() const {return true;}
       

#ifdef NONLOCALGMSH
                virtual double getElasticEnergy (double* statev);
		virtual double getPlasticEnergy (double* statev);
#endif
                virtual void get_refOp(double* statev, double** _R66, double** C, int kinc, int kstep);
                virtual void get_elOp(double* statev, double** _R66, double** Cel);
		virtual void get_elOD(double* statev,  double** _R66, double** Cel);
};



// Composite material homogenized with Self-Consistent scheme  //la
class SC_Material : public Material{

	private:
		int Nph;
		int mtx;
		int Nicl;
		Material* mtx_mat;
		Material** icl_mat;
		int idsdv_m, *idsdv_i;
		double *vf_i, vf_m;
		double** ar;
		double** euler;
		int pos_C;

	public:
		
		SC_Material(double* props, int idmat);
		~SC_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);
		virtual int get_pos_C() const {	return pos_C; } 
		
		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual int get_pos_mtx_stress () const;
                virtual int get_pos_inc_stress () const;
                virtual int get_pos_mtx_strain () const;
                virtual int get_pos_inc_strain () const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif
};



// Composite material with multi-phase homogenized with Two-step shemas(Mori-Tnaka/Voight )  add by Mohamed Janv 2018
class MT_VT_Material : public Material{

	private:
		int N_theta_increment;
                int Total_number_phase;
 		Material** ph_mat; 
                int *idsdv_i;
		double* Wgt;//   vector of weight for all phase 
		int pos_C;
                bool withDamage;

	public:
		MT_VT_Material(double* props, int idmat);
		~MT_VT_Material();

		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev,double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);

		virtual int get_pos_C() const {	return pos_C; } 
		
		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);
		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);

#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual int get_pos_mtx_stress () const;
                virtual int get_pos_inc_stress () const;
                virtual int get_pos_mtx_strain () const;
                virtual int get_pos_inc_strain () const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif

};  //           add by Mohamed MOHAMEDOU Janv 2018

// Anisotropic Linear elastic material
class ANEL_Material : public Material{

	private:
		double* anpr;
                double* euler;

	public:
		
		ANEL_Material(double* props, int idmat);
		~ANEL_Material();
		
		virtual void print();
		virtual void constbox(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double** Cref, double*** dCref, double* tau, double** dtau, double** Calgo, double alpha, double* dpdE, double* strs_dDdp_bar, double *Sp_bar, double** c_g, int kinc, int kstep, double dt);

		virtual void constboxSecant(double* dstrn, double* strs_n,  double* strs, double* statev_n, double* statev, double **Calgo, double** Csd, double*** dCsd, double** dCsdp_bar, double* dnu, double& dnudp_bar, double alpha, double* dpdE, double* dstrsdp_bar, double** c_g, int kinc, int kstep, double dt);

//******************new constbox for 2rd order method
                virtual void constbox_2order(int mtx, double *DE, double* dstrn, double* strs, double* statev_n, double* statev, double **Calgo, Lcc* LCC, YieldF* YF, double** c_g, int kinc, int kstep, double dt);

		virtual void get_refOp(double* statev, double** C, int kinc, int kstep);
		virtual void get_elOp(double** Cel);

		virtual void get_elOD(double* statev, double** Cel);
                virtual void unload_step(double* dstrn, double* strs_n, double* statev, int kinc, int kstep);
#ifdef NONLOCALGMSH
		virtual int get_pos_currentplasticstrainfornonlocal() const;
		virtual int get_pos_effectiveplasticstrainfornonlocal() const;
		virtual int get_pos_damagefornonlocal() const;
                virtual double getElasticEnergy (double* statev) ;
		virtual double getPlasticEnergy (double* statev) ;
#endif
		
}; 

#ifdef NONLOCALGMSH
#else
Damage* init_damage(double* props, int iddam);
Clength* init_clength(double* props, int idclength);
#endif
#ifdef NONLOCALGMSH
}
#endif
#endif

