//
// C++ Interface: material law
//
// Author:  <Kevin Spilker>, (C) 2019
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef MLAWCLUSTERMATERIALLAWS_H_
#define MLAWCLUSTERMATERIALLAWS_H_

#include "mlaw.h"
#include "STensor3.h"
#include "STensor43.h"
#include "STensor63.h"
#include "ipCluster.h"

class mlawClusterMaterialLaws : public materialLaw{
  #ifndef SWIG
  protected:
    std::map<int, int> _clusternb; //give the cluster number for element i If we want per Gauss point we need define a class with two inputs: ele nb and Gp nb in the element and use it instead of the second int
    
    std::map<int, int> _matlawnb; //give the material law number for cluster i
    std::map<int, const materialLaw *> _matlawptr; //give the material law pointer for cluster i

    //std::map<int, IP3State*> _ipstates; //give the ipstate for cluster i

    std::map<int, STensor3> _eigen; //give the eigenstrain for cluster i

  #endif // SWIG

  public:
    mlawClusterMaterialLaws(const int num,const char *clusterName);
    void addClusterMaterialLaw(int clusterNb, int materialLawNb);
    void addClusterEigenStrain(int clusterNb, const STensor3 &Feig);
    #ifndef SWIG
    mlawClusterMaterialLaws(const mlawClusterMaterialLaws& src);
    mlawClusterMaterialLaws& operator=(const materialLaw &source);    
    virtual ~mlawClusterMaterialLaws();

    virtual matname getType() const{return materialLaw::cluster;}
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw); //need to fill  std::set<int, const materialLaw *> matlawptr

    virtual void constitutive(
            const STensor3& F0,         // initial deformation gradient (input @ time n)
            const STensor3& Fn,         // updated deformation gradient (input @ time n+1)
            STensor3 &P,                // updated 1st Piola-Kirchhoff stress tensor (output)
            const IPVariable *q0,       // array of initial internal variable
            IPVariable *q1,             // updated array of internal variable (in ipvcur on output),
            STensor43 &Tangent,         // constitutive tangents (output)
            const bool stiff,          // if true compute the tangents
            STensor43* elasticTangent = NULL) const;


    virtual materialLaw* clone() const {return new mlawClusterMaterialLaws(*this);};
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const{}; // do nothing

    virtual void createIPState(IPCluster *ivi, IPCluster *iv1, IPCluster *iv2) const;
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPCluster *&ipv,const MElement *ele,const int nbFF, const IntPt *GP, const int gpt) const;

    double soundSpeed() const{ Msg::Error("mlawClusterMaterialLaws does not have a soundspeed"); }
                 
    bool withEnergyDissipation() const { Msg::Error("mlawClusterMaterialLaws does not have withEnergyDissipation"); }

    #endif // SWIG
};

#endif //MLAWCLUSTERMATERIALLAWS
