//
// C++ Interface: J2 material in small strains
//
//
// Author:  <Van Dung NGUYEN>, (C) 2016
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef MLAWJ2VMSMALLSTRAIN_H_
#define MLAWJ2VMSMALLSTRAIN_H_

#include "mlaw.h"
#include "STensor3.h"
#include "STensor43.h"
#include "ipJ2linear.h"

class mlawJ2VMSmallStrain : public materialLaw {
  protected:
    // hardening at reference temperature
    J2IsotropicHardening *_j2IH; // isotropic hardening
    double  _K, _mu, _E, _nu, _lambda;
    double _tol; // tolerace of plastic corrector
    double _rho; // density
    double _perturbationfactor; // perturbation factor
    bool _tangentByPerturbation; // flag for tangent by perturbation
    //
    STensor43 _Cel, _I4dev, _I4; // elastic hook tensor

  public:
    mlawJ2VMSmallStrain(const int num, const double E, const double nu,const double rho,
                               const double sy0,const double h,const double tol=1.e-6,
                               const bool matrixbyPerturbation = false, const double pert = 1e-8,
                               const bool init =true);
    mlawJ2VMSmallStrain(const int num, const double E, const double nu,const double rho,
                               const J2IsotropicHardening& j2IH,const double tol=1.e-6,
                               const bool matrixbyPerturbation = false, const double pert = 1e-8,
                               const bool init =true);

    mlawJ2VMSmallStrain(const mlawJ2VMSmallStrain& src);
    virtual ~mlawJ2VMSmallStrain();

		virtual materialLaw* clone() const {
			return new mlawJ2VMSmallStrain(*this);
		}
		virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const{}; // do nothing
    virtual bool withEnergyDissipation() const {return true;};

    virtual const J2IsotropicHardening * getJ2IsotropicHardening() const {return _j2IH;}
    void setIsotropicHardeningLaw(const J2IsotropicHardening& isoHard){
      if (_j2IH != NULL) delete _j2IH;
      _j2IH = isoHard.clone();
    }
    virtual matname getType() const{return materialLaw::j2linear;}
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){};
    virtual double soundSpeed() const;
    virtual double density() const{return _rho;};
    virtual double shearModulus() const {return _mu;}
    virtual double bulkModulus() const {return _K;}
		virtual double scaleFactor() const{return _mu;};
    // predictor-corrector estimation
    virtual void constitutive(const STensor3& F0,         // previous deformation gradient (input @ time n)
                            const STensor3& Fn,         // current deformation gradient (input @ time n+1)
                            STensor3 &P,                // current 1st Piola-Kirchhoff stress tensor (output)
                            const IPVariable *q0,       // array of previous internal variables
                            IPVariable *q1,             // current array of internal variable (in ipvcur on output),
                            STensor43 &Tangent,         // tangents (output)
                            const bool stiff,            // if true compute the tangents
                           STensor43* elasticTangent = NULL) const;

  protected:
      double deformationEnergy(const STensor3 &Ee) const;
      void predictorCorector(const STensor3& F0,         // initial deformation gradient (input @ time n)
                            const STensor3& F,         // updated deformation gradient (input @ time n+1)
                            STensor3 &P,                // updated 1st Piola-Kirchhoff stress tensor (output)
                            const IPJ2linear *q0,       // array of initial internal variable
                            IPJ2linear *q,             // updated array of internal variable (in ipvcur on output),
                            STensor43& Tangent,         // mechanical tangents (output)
                            STensor43& depsEldF,
                            STensor3& dpdF,
                            const bool stiff,
                            STensor43* elasticTangent = NULL
                            ) const;

};


#endif // MLAWJ2VMSMALLSTRAIN_H_
