// C++ Interface: material law
// Linear Thermo Mechanical law
// Author:  <H. Gulasik, M. Pareja>, (C) 2017
// Copyright: See COPYING file that comes with this distribution

#ifndef MLAWPHENOMENOLOGICALSMP_H_
#define MLAWPHENOMENOLOGICALSMP_H_
#include "../internalPoints/ipPhenomenologicalSMP.h"
#include "mlaw.h"
#include "STensor3.h"
#include "STensor43.h"
#include "STensor63.h"
#include "ipSMP.h"
#include "scalarFunction.h"

class mlawPhenomenologicalSMP : public  materialLaw
{
protected:
    J2IsotropicHardening *_j2IH;
    double _t0;
    STensor3 _I2;
    STensor43 _I4,_I4dev;
    STensor3  _alphaDilatation, _Stiff_alphaDilatation;
    int _order;
    double _alpha, _beta, _gamma;
    double _rho;
    double _Kxg, _Kyg, _Kzg, _Kxr, _Kyr, _Kzr;
    STensor3 _k0, _kg, _kr;
    double _w, _w1, _w2, _c, _cp, _cg, _cr, _TaylorQuiney, _Tg, _alphag, _alphar, _alpha_trans, _Tc, _xi, _Tonsetr, _Tonsetg;
    double _Kg, _Kr, _Gg, _Gr, _nug, _nur;
    STensor43 _C0r, _C0g;

    scalarFunction* _temFunc_Sy0;
    scalarFunction* _temFunc_H;
    scalarFunction* _temFunc_Kg;
    scalarFunction* _temFunc_Kr;
    scalarFunction* _temFunc_Gg;
    scalarFunction* _temFunc_Gr;
    scalarFunction* _temFunc_alphag;
    scalarFunction* _temFunc_alphar;
    scalarFunction* _temFunc_ThermalConductivity_g;
    scalarFunction* _temFunc_ThermalConductivity_r;
    scalarFunction* _temFunc_cp_g;
    scalarFunction* _temFunc_cp_r;

public:
    mlawPhenomenologicalSMP(const int num, const double rho, const double alpha, const double beta, const double gamma,const double t0,
                            const double Kxg,const double Kyg, const double Kzg, const double Kxr, const double Kyr, const double Kzr,
                            const double TaylorQuiney, const double c, const double cp, const double cg, const double cr,const double Tg,
                            const double Tc,const double xi,const double w1,const double w2, const double Eg, const double Er,
                            const double nug, const double nur,const double alphag, const double alphar,const double alpha_trans,
                            const double tau_y0);
    mlawPhenomenologicalSMP(const int num,const double rho,const J2IsotropicHardening &j2IH,const double alpha, const double beta,
                            const double gamma,const double t0, const double Kxg,const double Kyg, const double Kzg, const double Kxr,
                            const double Kyr, const double Kzr, const double TaylorQuiney, const double c, const double cp,
                            const double cg, const double cr, const double Tg,const double Tc,const double xi, const double w1,
                            const double w2,const double Eg, const double Er, const double nug, const double nur, const double alphag,
                            const double alphar,const double alpha_trans,const double tau_y0);

#ifndef SWIG
    mlawPhenomenologicalSMP(const mlawPhenomenologicalSMP &source);
    mlawPhenomenologicalSMP& operator=(const materialLaw &source);
    virtual ~mlawPhenomenologicalSMP() {};
    virtual materialLaw* clone() const {return new mlawPhenomenologicalSMP(*this);};

    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const {}; // do nothing
    virtual bool withEnergyDissipation() const {return false;};
    virtual matname getType() const{return materialLaw::PhenomenologicalSMP;}// function of materialLaw

    virtual void createIPState(IPStateBase* &ips, const bool* state_=NULL, const MElement *ele=NULL, const int nbFF_=0,
                               const IntPt *GP=NULL, const int gpt = 0) const;

    virtual const J2IsotropicHardening * getJ2IsotropicHardening() const {return _j2IH;}
    void setIsotropicHardeningLaw(const J2IsotropicHardening& isoHard)
    {
        if (_j2IH != NULL) delete _j2IH;
        _j2IH = isoHard.clone();
    }

    virtual double getGg (double T, double *dGg=NULL) const
    {
        double GgT = _Gg*_temFunc_Gg->getVal(T);
        if(dGg!=NULL)
        {
            *dGg=_Gg*_temFunc_Gg->getDiff(T);
        }
        return GgT;
    }

    virtual double getGr (double T, double *dGr=NULL) const
    {
        double GrT = _Gr*_temFunc_Gr->getVal(T);
        if(dGr!=NULL)
        {
            *dGr=_Gr*_temFunc_Gr->getDiff(T);
        }
        return GrT;
    }

    virtual double getKg (double T, double *dKg=NULL) const
    {
        double KgT = _Kg*_temFunc_Kg->getVal(T);
        if(dKg!=NULL)
        {
            *dKg=_Kg*_temFunc_Kg->getDiff(T);
        }
        return KgT;
    }

    virtual double getKr (double T, double *dKr=NULL) const
    {
        double KrT = _Kr*_temFunc_Kr->getVal(T);
        if(dKr!=NULL)
        {
            *dKr=_Kr*_temFunc_Kr->getDiff(T);
        }
        return KrT;
    }

    virtual double getKg (double T, double GgT, double *dKg=NULL, double *dGgT=NULL) const
    {
        double KgT = GgT*(2./3.)*((1+_nug)/(1-2*_nug));
        if(dKg!=NULL)
        {
            *dKg=*dGgT*(2./3.)*((1+_nug)/(1-2*_nug));
        }
        return KgT;
    }

    virtual double getKr (double T, double GrT,double *dKr=NULL, double *dGrT=NULL) const
    {
        double KrT = GrT*(2./3.)*((1+_nur)/(1-2*_nur));
        if(dKr!=NULL)
        {
            *dKr=*dGrT*(2./3.)*((1+_nur)/(1-2*_nur));
        }
        return KrT;
    }

    virtual void getThermalConductivity_glassy (double T, STensor3& condg, STensor3* dcondg = NULL) const
    {
        double conductgT = _Kxg*_temFunc_ThermalConductivity_g->getVal(T);
        condg(0,0)  = -conductgT;
        condg(1,1)  = -conductgT;
        condg(2,2)  = -conductgT;
        if(dcondg != NULL)
        {
            double dconductgT = _Kxg*_temFunc_ThermalConductivity_g->getDiff(T);
            STensor3 A;
            A(0,0)  = -dconductgT;
            A(1,1)  = -dconductgT;
            A(2,2)  = -dconductgT;
            *dcondg = A;
        }
    }

    virtual void getThermalConductivity_rubbery (double T, STensor3& condr, STensor3* dcondr = NULL) const
    {
        double conductrT = _Kxr*_temFunc_ThermalConductivity_r->getVal(T);
        condr(0,0)  = -conductrT;
        condr(1,1)  = -conductrT;
        condr(2,2)  = -conductrT;
        if(dcondr != NULL)
        {
            double dconductrT = _Kxr*_temFunc_ThermalConductivity_r->getDiff(T);
            STensor3 A;
            A(0,0)  = -dconductrT;
            A(1,1)  = -dconductrT;
            A(2,2)  = -dconductrT;
            *dcondr = A;
        }
    }

    virtual double getalpha_glassy (double T, double *dalphag=NULL) const
    {
        double alphagT = _alphag*_temFunc_alphag->getVal(T);
        if(dalphag!=NULL)
        {
            *dalphag=_alphag*_temFunc_alphag->getDiff(T);
        }
        return alphagT;
    }

    virtual double getalpha_rubbery (double T, double *dalphar=NULL) const
    {
        double alpharT = _alphar*_temFunc_alphar->getVal(T);
        if(dalphar!=NULL)
        {
            *dalphar=_alphar*_temFunc_alphar->getDiff(T);
        }
        return alpharT;
    }

    virtual double getcp_glassy (double T, double *dcg=NULL) const
    {
        double cp_gT = _cg*_temFunc_cp_g->getVal(T);
        if(dcg!=NULL)
        {
            *dcg=_cg*_temFunc_cp_g->getDiff(T);
        }
        return cp_gT;
    }

    virtual double getcp_rubbery (double T, double *dcr=NULL) const
    {
        double cp_rT = _cr*_temFunc_cp_r->getVal(T);
        if(dcr!=NULL)
        {
            *dcr=_cr*_temFunc_cp_r->getDiff(T);
        }
        return cp_rT;
    }

    virtual void setTemperatureFunction_ThermalExpansionCoefficient_glassy(const scalarFunction& Tfunc)
    {
        if (_temFunc_alphag != NULL) delete _temFunc_alphag;
        _temFunc_alphag = Tfunc.clone();
    }

    virtual void setTemperatureFunction_ThermalExpansionCoefficient_rubbery(const scalarFunction& Tfunc)
    {
        if (_temFunc_alphar != NULL) delete _temFunc_alphar;
        _temFunc_alphar = Tfunc.clone();
    }

    virtual void setTemperatureFunction_ThermalConductivity_glassy(const scalarFunction& Tfunc)
    {
        if (_temFunc_ThermalConductivity_g != NULL) delete _temFunc_ThermalConductivity_g;
        _temFunc_ThermalConductivity_g = Tfunc.clone();
    }

    virtual void setTemperatureFunction_ThermalConductivity_rubbery(const scalarFunction& Tfunc)
    {
        if (_temFunc_ThermalConductivity_r != NULL) delete _temFunc_ThermalConductivity_r;
        _temFunc_ThermalConductivity_r = Tfunc.clone();
    }

    virtual void setTemperatureFunction_Cp_glassy(const scalarFunction& Tfunc)
    {
        if (_temFunc_cp_g != NULL) delete _temFunc_cp_g;
        _temFunc_cp_g = Tfunc.clone();
    }

    virtual void setTemperatureFunction_Cp_rubbery(const scalarFunction& Tfunc)
    {
        if (_temFunc_cp_r != NULL) delete _temFunc_cp_r;
        _temFunc_cp_r = Tfunc.clone();
    }

    virtual void setTemperatureFunction_Hardening(const scalarFunction& Tfunc)
    {
        if (_temFunc_H != NULL) delete _temFunc_H;
        _temFunc_H = Tfunc.clone();
    }

    virtual void setTemperatureFunction_InitialYieldStress(const scalarFunction& Tfunc)
    {
        if (_temFunc_Sy0 != NULL) delete _temFunc_Sy0;
        _temFunc_Sy0 = Tfunc.clone();
    }

    virtual void setTemperatureFunction_BulkModulus_glassy(const scalarFunction& Tfunc)
    {
        if (_temFunc_Kg != NULL) delete _temFunc_Kg;
        _temFunc_Kg = Tfunc.clone();
    }

    virtual void setTemperatureFunction_BulkModulus_rubbery(const scalarFunction& Tfunc)
    {
        if (_temFunc_Kr != NULL) delete _temFunc_Kr;
        _temFunc_Kr = Tfunc.clone();
    }

    virtual void setTemperatureFunction_ShearModulus_glassy(const scalarFunction& Tfunc)
    {
        if (_temFunc_Gg != NULL) delete _temFunc_Gg;
        _temFunc_Gg = Tfunc.clone();
    }

    virtual void setTemperatureFunction_ShearModulus_rubbery(const scalarFunction& Tfunc)
    {
        if (_temFunc_Gr != NULL) delete _temFunc_Gr;
        _temFunc_Gr = Tfunc.clone();
    }

    virtual double get_zg (double T, double w, double T_t, double *dzgdT=NULL, double *dTtdT=NULL) const
    {
        double zg=1./(1.+exp(2.*w*(T-T_t)));
        if(dzgdT!=NULL)
        {
            *dzgdT=0.;
            *dzgdT = -2.*w*exp(2.*w*(T-T_t));
            *dzgdT *= 1./pow((1.+2.*w*exp(2.*w*(T-T_t))),2);
            *dzgdT += -(*dzgdT)*(*dTtdT);
        }
        return zg;
    }

    virtual double get_Tt(double Tdot, double Tt0, double *T_t, double *dTtdT, double *w, double w0, double w1, double w2) const
    {
        if (_Tc != _Tg)
        {
            if (Tdot > 0.001)
            {
                *T_t = _Tg;
                *dTtdT = 0.;
                *w = w1;
            }
            else if (Tdot < -0.001)
            {
                *T_t = _Tc*exp(_xi*Tdot);
                *dTtdT =0.;// _xi*_Tc*exp(_xi*Tdot);
                *w = w2;
            }
            else
            {
                *T_t = Tt0;
                *w   = w0;
                *dTtdT = 0.;
            }
        }
        else
        {
            *T_t=_Tg;
            *dTtdT = 0.;
            *w = w1;
        }
    }


public:
    void   setStrainOrder(const int order) {_order = order;};
    double getInitialTemperature() const {return _t0;};
    double getTransitionTemperature() const {return _Tg;};
    double getCrystallizationTemperature() const {return _Tc;};
    double getAlphaTrans() const {return _alpha_trans;};
    double getTonsetg() const {return _Tonsetg;};
    double getTonsetr() const {return _Tonsetr;};
    double getExtraDofStoredEnergyPerUnitField(double zg) const;
    //virtual double getInitialExtraDofStoredEnergyPerUnitField() const {getExtraDofStoredEnergyPerUnitField(zg);}

    virtual void constitutive(
        const STensor3& F0,                 // initial deformation gradient (input @ time n)
        const STensor3& Fn,                 // updated deformation gradient (input @ time n+1)
        STensor3 &P,                        // updated 1st Piola-Kirchhoff stress tensor (output)
        const IPVariable *q0,               // array of initial internal variable
        IPVariable *q1,                     // updated array of internal variable (in ipvcur on output),
        STensor43 &Tangent,                 // constitutive tangents (output)
        const bool stiff,
         STensor43* elasticTangent=NULL
    ) const;

    virtual void constitutive(
        const STensor3& F0,                 // initial deformation gradient (input @ time n)
        const STensor3& Fn,                 // updated deformation gradient (input @ time n+1)
        STensor3 &P,                        // updated 1st Piola-Kirchhoff stress tensor (output)
                                            // contains the initial values on input
        const IPVariable *q0,               // array of initial internal variable
        IPVariable *q1,                     // updated array of internal variable (in ipvcur on output),
        STensor43 &Tangent,                 // constitutive tangents (output)
        const double T0,
        double T,
        const SVector3 &gradT,
        SVector3 &fluxT,
        STensor3 &dPdT,
        STensor3 &dqdgradT,
        SVector3 &dqdT,
        STensor33 &dqdF,
        const bool stiff,
        double &wth,
        double &dwthdt,
        STensor3 &dwthdF,
        double&  mechSource,
        double& dmechSourcedT,
        STensor3 & dmechSourcedF,
        STensor43* elasticTangent=NULL
    ) const;

    void constitutive1(
        const STensor3& F0,
        const STensor3& Fn,
        const STensor3& Fm,
        const IPPhenomenologicalSMP *q0,
        IPPhenomenologicalSMP *q1,
        STensor43& Tangentr,
        STensor3& dPrdT,
        const double T0,
        double T,
        bool stiff,
        double dh,
        STensor3& Pr,
        double& dgammataueq,
        double& dgammataueqdT,
        STensor3& dgammataueqdF
    ) const;

    void constitutive2(
        const STensor3& F0,
        const STensor3& Fn,
        const STensor3& Fm,
        const IPPhenomenologicalSMP *q0,
        IPPhenomenologicalSMP *q1,
        STensor43& Tangentr,
        STensor43& Tangentg,
        STensor3& dPrdT,
        STensor3& dPgdT,
        const double T0,
        double T,
        bool stiff,
        STensor3& Pr,
        STensor3& Pg,
        const double dh,
        double lambda_th,
        double& dgammataueq,
        double& dgammataueqdT,
        STensor3& dgammataueqdF
    ) const;

    void constitutive3(
        const STensor3& F0,
        const STensor3& Fn,
        const STensor3& Fm,
        const IPPhenomenologicalSMP *q0,
        IPPhenomenologicalSMP *q1,
        STensor43& Tangentg,
        STensor3& dPgdT,
        const double T0,
        double T,
        bool stiff,
        STensor3& Pg,
        const double dh,
        double lambda_th,
        double& dgammataueq,
        double& dgammataueqdT,
        STensor3& dgammataueqdF
    ) const;

    const STensor3&  getInitialConductivityTensor() const {return _k0;};

    virtual const STensor3& getStiff_alphaDilatation()const {return _Stiff_alphaDilatation;};

    virtual void initLaws(const std::map<int,materialLaw*> &maplaw) {}; // this law is initialized so nothing to do

    virtual double soundSpeed() const; // default but you can redefine it for your case

protected:
    void predictorCorrector(const STensor3& F0, const STensor3& Fn,const STensor3& Fm, const IPPhenomenologicalSMP *q0,
                            IPPhenomenologicalSMP *q1,STensor3& Pg, const double dh, const double T0, double T,const double Tdot,bool stiff,
                            const double lambda_th,double& dgammataueq, double& dgammataueqdT,STensor3& dgammataueqdF,STensor3& dPgdT,
                            STensor43& Tangentg)  const;

    void computeMechSource_rubbery(const IPPhenomenologicalSMP *q0, IPPhenomenologicalSMP *q1,const STensor3& Fer,const STensor3& tauer,const STensor43& dlnCer, double& dgammataueq)  const;
    void computeMechSource_glassy (const IPPhenomenologicalSMP *q0, IPPhenomenologicalSMP *q1,const STensor3& Feg,const STensor3& taueg,const STensor43& dlnCeg, double& dgammataueq)  const;

    bool checkDeltaEp(const STensor3& Fp0, const STensor3& Fp) const;

    void computePr(const double KrT, const double GrT,const STensor3& Fn,const STensor3& Fp,const STensor3& G,STensor3& Fer,STensor3& Eer,
                   STensor3& tauer,STensor43& dlnCer,STensor63& ddlnCer,const double zg,STensor3& Pr)const;

    void computePg(const double KgT, const double GgT, const STensor3& Fn,const STensor3& Ff,const STensor3& Fpg, const STensor3& G,
                   STensor3& Feg,STensor3& Eeg,STensor3& taueg,STensor43& dlnCeg,STensor63& ddlnCeg,const double zg,STensor3& Pg)const;

    virtual void HookeTensor(const double KT, const double GT, STensor43& C) const;
    virtual void invHookeTensor(const double KT, const double GT, STensor43& C) const;

    virtual void getUf(const double T, STensor3& Uf,const STensor3& Fm, const STensor3& tauer) const;

    void evaluate_reference_temperatures(const IPPhenomenologicalSMP *q0, IPPhenomenologicalSMP *q1, double zg, double T_t, const double T0,
                                         double T, double Tdot, double& lambdaT, double dzgdT=1., double *dlambdaTdT=NULL)  const;

    void tangentrubbery(const STensor3& Fn,const STensor3& Fer,const IPPhenomenologicalSMP *q0,IPPhenomenologicalSMP *q1,
                        const STensor3& tauer,const STensor43& dlnCer,const STensor63& ddlnCer,const STensor3& Eer, const double T0, const double T,
                        const double Tdot,const double dh,const double Tir,const double Tig,const double alphaig,const double zg,
                        const STensor3& G,double& dgammataueq, double& dgammataueqdT,STensor3& dgammataueqdF, STensor3& dPrdT,
                        STensor43& Tangentr) const;

    void tangentglassy(const STensor3& Fn,const STensor3& Fm,const IPPhenomenologicalSMP *q0,IPPhenomenologicalSMP *q1,
                       const STensor43& dexpDelta_gammaNpr,const STensor3& Fegpr, STensor43& dlnCegpr,const STensor3& Eegpr,
                       const STensor3& Npr,const double Eegpreq,const double Deltagamma,const double zg,const double H_gamma, const double T0,
                       const double T,const double Tdot,const double dh,const double Tir,const double Tig,const double alphaig,
                       const double taueq,const STensor3& Feg,const STensor3& Eeg,const STensor3& taueg,const STensor43& dlnCeg,
                       const STensor63& ddlnCeg, const STensor3& G,double& dgammataueq, double& dgammataueqdT,STensor3& dgammataueqdF,
                       STensor3& dPgdT, STensor43& Tangentg) const;

#endif // SWIG
};
#endif // MLAWPHENOMENOLOGICALSMP_H_
