#coding-Utf-8-*-
from gmshpy import*
#from dG3DpyDebug import*
from dG3Dpy import*
from math import*

# Script for plane notched specimen with Gurson


# Material law creation
# ===================================================================================
BulkLawNum1 = 1 
# material parameters - bulk law
rho = 7600.e-9 		# kg/m3:	density
young = 210.e3	# Pa: 	module de young
nu    = 0.3		# -: 	poisson coefficient
sy0   = 354.41 	#  	initial yield stress
h_hard  = 500. 
h_exp = 0.13		#	exponential coefficient

HardenLawNum1 = 2
Harden1 = SwiftJ2IsotropicHardening(HardenLawNum1, sy0, h_hard, h_exp)
# - gurson
q1    = 1.5
q2    = 1.
q3    = 1.5 
fVinitial = 1.5e-3
# - coalesence
fC     = 0.0#0.1#0.005
ff     = 1./q1#0.3#0.225
ffstar = 1./q1

NucleationLawNum1 = 3
Gdn1 = LinearNucleationLaw(NucleationLawNum1, 0.2, 0.005, 1.)

# - non-local law
l = 1.
cl = l*l
ClLawNum1 = 4	
Cl_Law1 = IsotropicCLengthLaw(ClLawNum1, cl)


TaylorQuineyFactor = 0.5 
#KK = 50.
#cp =1000.
KK = 50.
cp = 450.*rho
#alp = 0.
alp =12.e-6
Tinitial = 298.

# material law creation
BulkLaw1 = NonLocalPorousCoupledDG3DMaterialLaw(BulkLawNum1, young, nu, rho, q1,q2,q3, fVinitial, 2.,2., Harden1, Cl_Law1,TaylorQuineyFactor,KK,cp,alp,1e-8,False,1e-8)
BulkLaw1.setOrderForLogExp(9)
BulkLaw1.setYieldSurfaceExponent(20)
BulkLaw1.setNucleationLaw(Gdn1)
BulkLaw1.setSubStepping(True,3)
method =1
BulkLaw1.setNonLocalMethod(method)
if method==2:
	BulkLaw1.clearCLengthLaw()
	BulkLaw1.setCLengthLaw(Cl_Law1)
	BulkLaw1.setCLengthLaw(Cl_Law1)
	BulkLaw1.setCLengthLaw(Cl_Law1)

BulkLaw1.setReferenceT(Tinitial)

# Solver parameters - shift
# ===================================================================================
ftime =1.0 		# Final time
r_impl = 1.0

# Solver parameters - implicit
# ===================================================================================
soltype = 1 		# StaticLinear=0 (default, StaticNonLinear=1, Explicit=2, # Multi=3, Implicit=4, Eigen=5)
nstep = 500 		# Number of step
tol=1.e-6 		# Relative tolerance for NR scheme
tolAbs = 1.e-8		# Absolute tolerance for NR scheme
nstepArchIP=1		# Number of step between 2 archiving
nstepArchForce = 1	# Number of step between 2 force archiving
nstepArchEnergy = nstepArchForce # Number of step between 2 energy computation and archiving

MaxIter = 15		# Maximum number of iterations
StepIncrease = 2	# Number of successfull timestep before reincreasing it
StepReducFactor = 2.0 	# Timestep reduction factor
NumberReduction = 100	# Maximum number of timespep reduction: max reduction = pow(StepReducFactor,this)

fullDg = False        # O = CG, 1 = DG
dgnl = False		# DG for non-local variables inside a domain (only if fullDg)
eqRatio = 1.0e3		# Ratio between "elastic" and non-local equations 
space1 = 0 		# Function space (Lagrange=0)
beta1  = 30.0		# Penality parameter for DG

dgTher = False	# DG for thermal variables inside a domain (only if fullDg)
eqRatioTher = 1.0e3	# Ratio between "elastic" and thermal 
fieldSource = True     # account for cp(T-T0)
mecaSource  = True     # account for TaylorQuinley


# Domain creation
## ===================================================================================
numPhysVolume1 = 700 		# Number of a physical volume of the model in .geo
numDomain1 = 1001 		# Number of the domain
if method==2:
	numNonlocalVar = 3;
else:
	numNonlocalVar=1;
field1 = dG3DDomain(numDomain1,numPhysVolume1,space1,BulkLawNum1,fullDg,2,numNonlocalVar,1)
field1.stabilityParameters(beta1) 			# Adding stability parameters (for DG)
field1.setNonLocalStabilityParameters(beta1,dgnl) 		# Adding stability parameters (for DG)

#field1.matrixByPerturbation(1,1,1,1e-8) 		# Tangent computation analytically or by pertubation

field1.setNonLocalEqRatio(eqRatio)
field1.setConstitutiveExtraDofDiffusionStabilityParameters(beta1,dgTher)
field1.setConstitutiveExtraDofDiffusionEqRatio(eqRatioTher)
field1.setConstitutiveExtraDofDiffusionAccountSource(fieldSource, mecaSource)

# Solver creation
# ===================================================================================
mysolver = nonLinearMechSolver(1002) 		# Solver associated with numSolver1
meshfile= "plane_notch.msh" 	# name of mesh file
geofile= "plane_notch.geo" 	# name of geo file
#mysolver.loadModel(meshfile)		# add mesh
mysolver.createModel(geofile,meshfile,2,2)
mysolver.addDomain(field1) 		# add domain
mysolver.addMaterialLaw(BulkLaw1) 	# add material law
# solver parametrisation
mysolver.Scheme(soltype) 		# solver scheme
mysolver.Solver(2) 			# Library solver: Gmm=0 (default) Taucs=1 PETsc=2
mysolver.snlData(nstep,ftime*r_impl,tol,tolAbs) # solver parameters for imp. solving (numb. of step, final time and tolerance)
mysolver.snlManageTimeStep(MaxIter,StepIncrease,StepReducFactor,NumberReduction)
# solver archiving
mysolver.stepBetweenArchiving(nstepArchIP) 	# archiving frequency
mysolver.energyComputation(nstepArchEnergy)	# archiving frequency for energy
#mysolver.lineSearch(bool(0))			# lineSearch activation


# Boundary conditions
# ===============================
mysolver.displacementBC("Face",700,2,0.)
mysolver.displacementBC("Node",401,1,0.)
mysolver.displacementBC("Edge",1400,0,0.) 
if method==2:
	mysolver.initialBC("Face","Position",700,6,Tinitial)
elif method==1:
	mysolver.initialBC("Face","Position",700,4,Tinitial)
	mysolver.initialBC("Face","Position",700,3,fVinitial)
elif method==3:
	mysolver.initialBC("Face","Position",700,4,Tinitial)
	mysolver.initialBC("Face","Position",700,3,log(fVinitial))
mysolver.displacementBC("Edge",400,0,2)


# Variable storage
# ===============================
mysolver.internalPointBuildView("svm",IPField.SVM, 1, 1)
mysolver.internalPointBuildView("sig_xx",IPField.SIG_XX, 1, 1)
mysolver.internalPointBuildView("sig_yy",IPField.SIG_YY, 1, 1)
mysolver.internalPointBuildView("sig_zz",IPField.SIG_ZZ, 1, 1)
mysolver.internalPointBuildView("sig_xy",IPField.SIG_XY, 1, 1)
mysolver.internalPointBuildView("sig_yz",IPField.SIG_YZ, 1, 1)
mysolver.internalPointBuildView("sig_xz",IPField.SIG_XZ, 1, 1)
mysolver.internalPointBuildView("damage",IPField.DAMAGE,1,1)
mysolver.internalPointBuildView("local_fV",IPField.LOCAL_POROSITY,1,1)
mysolver.internalPointBuildView("local_fV_max",IPField.LOCAL_POROSITY,1,IPField.MAX_VALUE)
mysolver.internalPointBuildView("epl",IPField.PLASTICSTRAIN, 1, 1)
mysolver.internalPointBuildView("triaxiality",IPField.STRESS_TRIAXIALITY, 1, 1)

mysolver.internalPointBuildView("COALESCENCE",IPField.COALESCENCE, 1, 1)
mysolver.internalPointBuildView("chi",IPField.LIGAMENT_RATIO, 1, 1)
mysolver.internalPointBuildView("chi_max",IPField.LIGAMENT_RATIO, 1, IPField.MAX_VALUE)

mysolver.internalPointBuildView("LOCAL_0",IPField.LOCAL_0, 1, 1)
mysolver.internalPointBuildView("NONLOCAL_0",IPField.NONLOCAL_0, 1, 1)
mysolver.internalPointBuildView("LOCAL_1",IPField.LOCAL_1, 1, 1)
mysolver.internalPointBuildView("NONLOCAL_1",IPField.NONLOCAL_1, 1, 1)
mysolver.internalPointBuildView("LOCAL_2",IPField.LOCAL_2, 1, 1)
mysolver.internalPointBuildView("NONLOCAL_2",IPField.NONLOCAL_2, 1, 1)
mysolver.internalPointBuildView("LOCAL_POROSITY",IPField.LOCAL_POROSITY,1,1)
mysolver.internalPointBuildView("CORRECTED_POROSITY",IPField.CORRECTED_POROSITY,1,1)
mysolver.internalPointBuildView("YIELD_POROSITY",IPField.YIELD_POROSITY,1,1)

mysolver.internalPointBuildView("temperature",IPField.TEMPERATURE, 1, 1)
mysolver.internalPointBuildView("qx",IPField.THERMALFLUX_X, 1, 1)
mysolver.internalPointBuildView("qy",IPField.THERMALFLUX_Y, 1, 1)
mysolver.internalPointBuildView("qz",IPField.THERMALFLUX_Z, 1, 1)

mysolver.archivingForceOnPhysicalGroup("Edge", 1400, 0, nstepArchForce)
mysolver.archivingNodeDisplacement(401,0, nstepArchForce)
mysolver.archivingNodeDisplacementOnPhysical(0,100,1,nstepArchForce)
mysolver.archivingIPOnPhysicalGroup("Face", 700, IPField.DAMAGE,IPField.MAX_VALUE,nstepArchForce)
mysolver.archivingIPOnPhysicalGroup("Face", 700, IPField.LOCAL_POROSITY,IPField.MAX_VALUE,nstepArchForce)

mysolver.solve()

check = TestCheck()
check.equal(-1.912460e+03,mysolver.getArchivedForceOnPhysicalGroup("Edge", 1400, 0),1.e-6)
