#coding-Utf-8-*-
from gmshpy import *				

#from dG3DpyDebug import*			
from dG3Dpy import *				


# ===Problem parameters============================================================================
lawnum= 1 					
rho= 1020.					
alpha=beta=gamma= 0.				

Eg= 771.e6
Er= 0.9e6
nug= 0.29
nur= 0.49
Gg= Eg/2./(1+nug)
Gr= Er/2./(1+nur)
Mg= 7.4e4					
Mr= 0.168e4					
a= -Mg/Gg					
b= -Mr/Gr					
tau_y0= 10.e6	

t0= 400.
t1= 200.
Tg= 330.
Tc= 270.
alphar= 25.e-5
alphag= 13.e-5
cg= 1974210.		
cr= 1744200.
w1= 0.3			# temperature transition parameter for melting
w2= 0.2			# temperature transition parameter for crystalization
alpha_trans= -0.01      # transition strain

c= 0.			# shape fixing parameter related with Ff, c=1 complete shape fixing
cp= 0.			# shape recovery parameter related with Fp, cp=0 complete shape recovery

TaylorQuiney= 0.			# fraction of plastic energy converted to thermal energy	
xi= 0.					# temperature affecting Tt 				
Kxg=Kyg=Kzg=Kxr=Kyr=Kzr= 0.05		# conduction tensor				  

fieldSource= False  	#True #account for cp dT dt 	
mecaSource= False   	#True #account for Taylor Quincey	


# ===Material law creation=========================================================================
#GgT= linearScalarFunction(Tg, 1., a)			
#GrT= linearScalarFunction(Tg, 1., b)			

law1= PhenomenologicalSMPDG3DMaterialLaw(lawnum,rho,alpha,beta,gamma,t0,Kxg,Kyg,Kzg,			 Kxr,Kyr,Kzr,TaylorQuiney,c,cp,cg,cr,Tg,Tc,xi,w1,w2,Eg,Er,nug,nur,alphag,alphar,alpha_trans,tau_y0)		
law1.setStrainOrder(-1)												
#law1.setTemperatureFunction_ShearModulus_glassy(GgT)	#always normalized by the initial or default value	
#law1.setTemperatureFunction_ShearModulus_rubbery(GrT)	#always normalized by the initial or default value	


# ===Solver parameters=============================================================================
sol= 2          # Gmm=0 (default) Taucs=1 PETsc=2					
soltype= 1      # StaticLinear=0 (default) StaticNonLinear=1				
nstep= 200      # number of step (used only if soltype=1) #640				
ftime= 2.       # Final time (used only if soltype=1)22  t= 680				
tol= 1.e-6      # relative tolerance for NR scheme (used only if soltype=1)
nstepArch= 1    # Number of step between 2 archiving (used only if soltype=1)
fullDg= False   # O = CG, 1 = DG							
space1= 0       # function space (Lagrange=0)						
beta1= 40.	# DG parameter								


# ===Domain creation===============================================================================
# creation of ElasticField
nfield= 10				
myfield1= dG3DDomain(1000,nfield,space1,lawnum,fullDg,3,0,1)				
myfield1.setConstitutiveExtraDofDiffusionEqRatio(1.e6)					
myfield1.matrixByPerturbation(0,0,0,1e-8)						
#myfield1.matrixByPerturbation(1,1,1,1e-6)
myfield1.stabilityParameters(beta1)							
myfield1.setConstitutiveExtraDofDiffusionStabilityParameters(beta1,fullDg)		
myfield1.setConstitutiveExtraDofDiffusionAccountSource(fieldSource, mecaSource)		


# ===Solver creation===============================================================================
# geometry & mesh
geofile= "cube.geo"   
meshfile= "cube.msh" 

# creation of Solver
mysolver= nonLinearMechSolver(200)			
mysolver.createModel(geofile,meshfile,3,2)		
mysolver.addDomain(myfield1)				
mysolver.addMaterialLaw(law1)				
mysolver.Scheme(soltype)				
mysolver.Solver(sol)					
mysolver.snlData(nstep,ftime/1.,tol,tol)		
mysolver.snlManageTimeStep(50, 3, 2, 10)		
mysolver.stepBetweenArchiving(nstepArch)		


# ===Boundary conditions===========================================================================
# Boatti et al. [2016], 4.1. uniaxial
# mechanical BCs
#PresZZ= -0.2e6;
#cyclicFunctionPresZZ= cycleFunctionTime(0., 0., ftime/2., PresZZ, ftime/1., PresZZ);		
#mysolver.pressureOnPhysicalGroupBC(5678, cyclicFunctionPresZZ)
#ForceZZ= +0.2e6;
#cyclicFunctionForce= cycleFunctionTime(0., 0., ftime/2., ForceZZ, ftime/1., ForceZZ);		
#mysolver.forceBC("Face",5678,2,cyclicFunctionForce)	
#DispZZ= +0.25e-3;
#cyclicFunctionDisp= cycleFunctionTime(0., 0., ftime/2., DispZZ, ftime/1., DispZZ);		
#mysolver.displacementBC("Face",5678,2,cyclicFunctionDisp)	
mysolver.displacementBC("Face",1234,2,0.)			
mysolver.displacementBC("Face",4158,0,0.)			 
mysolver.displacementBC("Face",1265,1,0.)			


# thermal BCs
cyclicFunctionTemp1= cycleFunctionTime(0., t0, ftime/2., t0, ftime/1., t1);
mysolver.initialBC("Volume","Position",nfield,3,t0)		 
mysolver.displacementBC("Volume",nfield,3,cyclicFunctionTemp1);			


# ===Variable storage==============================================================================
mysolver.archivingNodeDisplacement(7,0,nstepArch);
mysolver.archivingNodeDisplacement(7,1,nstepArch);
mysolver.archivingNodeDisplacement(7,2,nstepArch);
mysolver.archivingNodeDisplacement(2,0,nstepArch);
mysolver.archivingNodeDisplacement(2,1,nstepArch);
mysolver.archivingNodeDisplacement(2,2,nstepArch);

mysolver.archivingForceOnPhysicalGroup("Face", 5678, 2)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 0)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 1)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 2)

mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SVM,IPField.MEAN_VALUE);

mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.PLASTICSTRAIN,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.TEMPERATURE,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_YZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_YZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.JACOBIAN,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_ZZ,IPField.MEAN_VALUE);

#mysolver.archivingIPOnPhysicalGroup(  "Node",	7, 	IPField.FF_ZZ,	1)
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FTH_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FTH_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FF_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FF_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FPG_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FPG_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FP_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FP_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FG_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FG_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.FR_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.FR_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.ZG,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TIG,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TIR,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TT,	IPField.MEAN_VALUE);

mysolver.internalPointBuildView("svm",IPField.SVM, 1, 1)
mysolver.internalPointBuildView("epl",IPField.PLASTICSTRAIN, 1, 1)
mysolver.internalPointBuildView("jacobian",IPField.JACOBIAN, 1, 1)
mysolver.internalPointBuildView("temperature",IPField.TEMPERATURE, 1, 1)
mysolver.internalPointBuildView("sig_xx",IPField.SIG_XX, 1, 1)
mysolver.internalPointBuildView("sig_yy",IPField.SIG_YY, 1, 1)
mysolver.internalPointBuildView("sig_zz",IPField.SIG_ZZ, 1, 1)
mysolver.internalPointBuildView("sig_xy",IPField.SIG_XY, 1, 1)
mysolver.internalPointBuildView("sig_yz",IPField.SIG_YZ, 1, 1)
mysolver.internalPointBuildView("sig_xz",IPField.SIG_XZ, 1, 1)
mysolver.internalPointBuildView("qx",IPField.THERMALFLUX_X, 1, 1)
mysolver.internalPointBuildView("qy",IPField.THERMALFLUX_Y, 1, 1)
mysolver.internalPointBuildView("qz",IPField.THERMALFLUX_Z, 1, 1)


# ===Solving=======================================================================================
mysolver.solve()    		


# ===Test==========================================================================================
check= TestCheck()		
check.equal(-5.160000e-05,mysolver.getArchivedNodalValue(2,0,mysolver.displacement),1e-5)



# ===[CONTINUE SIMULATION]=========================================================================
# ===Reset boundary conditions=====================================================================
LastF5678= mysolver.getArchivedForceOnPhysicalGroup("Face", 5678, 2)	
mysolver.resetBoundaryConditions(False)					


# ===Solver creation===============================================================================
# geometry					
geofile="cube.geo" # name of mesh file		# ??? recreate the geo amd msh
meshfile="cube.msh" # name of mesh file

# creation of Solver
nstep1=200;
mysolver.Scheme(soltype)
mysolver.Solver(sol)
mysolver.snlData(nstep1,ftime/1.,tol,tol)
mysolver.snlManageTimeStep(50, 3, 2, 10)
mysolver.stepBetweenArchiving(nstepArch)


# ===Boundary conditions===========================================================================
# mechanical BCs
#cyclicFunctionPresZZ2= cycleFunctionTime(0., PresZZ,   ftime/2., 0.,   ftime, 0.);		
#mysolver.pressureOnPhysicalGroupBC(5678, cyclicFunctionPresZZ2)
#cyclicFunctionF= cycleFunctionTime(0., ForceZZ,   ftime/2., 0.,   ftime, 0.);		
#mysolver.forceBC("Face",5678,2,cyclicFunctionF)	
#cyclicFunctionF= cycleFunctionTime(0., LastF5678/1.e-6,   ftime/2., 0.,   ftime, 0.);		
#mysolver.forceBC("Face",5678,2,cyclicFunctionF)	
mysolver.displacementBC("Face",1234,2,0.)			
mysolver.displacementBC("Face",4158,0,0.)			 
mysolver.displacementBC("Face",1265,1,0.)			

# thermal BCs
cyclicFunctionTemp1= cycleFunctionTime(0., t1, ftime/2., t1, ftime/1., t0);
#mysolver.initialBC("Volume","Position",nfield,3,t0)		 
mysolver.displacementBC("Volume",nfield,3,cyclicFunctionTemp1);	


# ===Variable storage==============================================================================
mysolver.archivingNodeDisplacement(7,0,nstepArch);
mysolver.archivingNodeDisplacement(7,1,nstepArch);
mysolver.archivingNodeDisplacement(7,2,nstepArch);
mysolver.archivingNodeDisplacement(8,0,nstepArch);
mysolver.archivingNodeDisplacement(8,1,nstepArch);
mysolver.archivingNodeDisplacement(8,2,nstepArch);

mysolver.archivingForceOnPhysicalGroup("Face", 5678, 2)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 0)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 1)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 2)
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.PLASTICSTRAIN,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.TEMPERATURE,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_YZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_YZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.JACOBIAN,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_ZZ,IPField.MEAN_VALUE);

mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SVM,IPField.MEAN_VALUE);

#mysolver.archivingIPOnPhysicalGroup(  "Node",	7, 	IPField.FF_ZZ,	1)
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FTH_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FTH_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FF_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FF_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FPG_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FPG_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FP_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FP_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FG_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FG_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.FR_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.FR_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.ZG,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TIG,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TIR,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TT,	IPField.MEAN_VALUE);

mysolver.internalPointBuildView("svm",IPField.SVM, 1, 1)
mysolver.internalPointBuildView("epl",IPField.PLASTICSTRAIN, 1, 1)
mysolver.internalPointBuildView("temperature",IPField.TEMPERATURE, 1, 1)
mysolver.internalPointBuildView("jacobian",IPField.JACOBIAN, 1, 1)
mysolver.internalPointBuildView("sig_xx",IPField.SIG_XX, 1, 1)
mysolver.internalPointBuildView("sig_yy",IPField.SIG_YY, 1, 1)
mysolver.internalPointBuildView("sig_zz",IPField.SIG_ZZ, 1, 1)
mysolver.internalPointBuildView("sig_xy",IPField.SIG_XY, 1, 1)
mysolver.internalPointBuildView("sig_yz",IPField.SIG_YZ, 1, 1)
mysolver.internalPointBuildView("sig_xz",IPField.SIG_XZ, 1, 1)
mysolver.internalPointBuildView("qx",IPField.THERMALFLUX_X, 1, 1)
mysolver.internalPointBuildView("qy",IPField.THERMALFLUX_Y, 1, 1)
mysolver.internalPointBuildView("qz",IPField.THERMALFLUX_Z, 1, 1)


# ===Solving=======================================================================================
mysolver.solve()


# ===Test==========================================================================================
check2 = TestCheck()
check2.equal(-1.912501e-16,mysolver.getArchivedNodalValue(8,1,mysolver.displacement),1e-1)	


