#coding-Utf-8-*-
from gmshpy import *

#from dG3DpyDebug import*
from dG3Dpy import *


# ===Problem parameters============================================================================
lawnum= 1 
rho   = 1020.
alpha = beta=gamma=0.

Eg=771.e6
Er=0.9e6
nug=0.29
nur=0.49
Gg=Eg/2./(1+nug)
Gr=Er/2./(1+nur)
Mg=7.4e4
Mr=0.168e4
a=-Mg/Gg
b=-Mr/Gr
tau_y0=10.e6

t0 = 400.
#t1=???
Tg=365.
Tc=335.3897
alphar=25.e-5
alphag=13.e-5
cg=1974210.
cr=1744200.
w1=0.3
w2=0.2
alpha_trans= -0.01

c=0.9
cp=0.

TaylorQuiney=0.
xi=0.00
Kxg=Kyg=Kzg=Kxr=Kyr=Kzr=0.5

fieldSource = True 	#account for cp d T dt 
mecaSource = True 	#account for Taylor Quincey


# ===Material law creation=========================================================================
GgT=linearScalarFunction(Tg,1.,a)
GrT=linearScalarFunction(Tg,1.,b)

law1   = PhenomenologicalSMPDG3DMaterialLaw(lawnum,rho,alpha,beta,gamma,t0,Kxg,Kyg,Kzg,
Kxr,Kyr,Kzr,TaylorQuiney,c,cp,cg,cr,Tg,Tc,xi,w1,w2,Eg,Er,nug,nur,alphag,alphar,alpha_trans,tau_y0)
law1.setStrainOrder(-1)
law1.setTemperatureFunction_ShearModulus_glassy(GgT) 	#always normalized by the initial or default value
law1.setTemperatureFunction_ShearModulus_rubbery(GrT) 	#always normalized by the initial or default value
law1.setUseBarF(True)

#===Solver parameters===================================================================================
sol 	= 2  		# Gmm=0 (default) Taucs=1 PETsc=2
soltype = 1 		# StaticLinear=0 (default) StaticNonLinear=1
nstep 	= 100		# number of step (used only if soltype=1)#640
ftime  	= 400.	   	# Final time (used only if soltype=1)22  t = 680
tol	= 1.e-6   	# relative tolerance for NR scheme (used only if soltype=1)
nstepArch= 1 		# Number of step between 2 archiving (used only if soltype=1)
fullDg 	=  False 	# O = CG, 1 = DG
space1 	= 0 		# function space (Lagrange=0)
beta1  	= 40.


#===Domain creation===================================================================================
# creation of ElasticField
nfield = 10 		
myfield1 = dG3DDomain(1000,nfield,space1,lawnum,fullDg,3,0,1)
myfield1.setConstitutiveExtraDofDiffusionEqRatio(1.e6)
#myfield1.matrixByPerturbation(0,0,0,1e-8)
#myfield1.matrixByPerturbation(1,1,1,1e-8)
myfield1.stabilityParameters(beta1)
myfield1.setConstitutiveExtraDofDiffusionStabilityParameters(beta1,fullDg)
myfield1.setConstitutiveExtraDofDiffusionAccountSource(fieldSource, mecaSource)


#===Solver creation===================================================================================
# geometry
geofile="cube.geo"	# name of mesh file
meshfile="cube.msh" 	# name of mesh file

# creation of Solver
mysolver = nonLinearMechSolver(1000)
mysolver.createModel(geofile,meshfile,3,1)
mysolver.addDomain(myfield1)
mysolver.addMaterialLaw(law1)
mysolver.Scheme(soltype)
mysolver.Solver(sol)
mysolver.snlData(nstep,ftime/1.,tol,tol)
mysolver.snlManageTimeStep(50, 3, 2, 10)
mysolver.stepBetweenArchiving(nstepArch)


#===Boundary conditions===================================================================================
#mechanical BC
#cyclicFunctionDisp=cycleFunctionTime(0., 0.,	ftime/2., 0.0003,	ftime/1.,0.0003);
#mysolver.displacementBC("Face",3487,1,cyclicFunctionDisp)

#mysolver.displacementBC("Face",1234,0,0.)
#mysolver.displacementBC("Face",1234,1,0.)
mysolver.displacementBC("Face",1234,2,0.)
mysolver.displacementBC("Face",1265,1,0.)
mysolver.displacementBC("Face",4158,0,0.)
#mysolver.displacementBC("Face",5678,1,0.)
#mysolver.displacementBC("Face",5678,0,0.)
#mysolver.displacementBC("Face",5678,2,0.)

#thermal BC
mysolver.initialBC("Volume","Position",nfield,3,t0)
mysolver.scalarFluxBC("Face",3487,3,5.19e-1)
#cyclicFunctionTemp1=cycleFunctionTime(0., t0,	ftime/4., t0-100.,	ftime/2., t0,	3.*ftime/4., t0/2.,	ftime, t0);
#mysolver.displacementBC("Face",3487,3,cyclicFunctionTemp1);
#mysolver.displacementBC("Volume",nfield,3,cyclicFunctionTemp1);


#===Variable storage===================================================================================
mysolver.internalPointBuildView("svm",IPField.SVM, 1, 1)
mysolver.internalPointBuildView("sig_xx",IPField.SIG_XX, 1, 1)
mysolver.internalPointBuildView("sig_yy",IPField.SIG_YY, 1, 1)
mysolver.internalPointBuildView("sig_zz",IPField.SIG_ZZ, 1, 1)
mysolver.internalPointBuildView("sig_xy",IPField.SIG_XY, 1, 1)
mysolver.internalPointBuildView("sig_yz",IPField.SIG_YZ, 1, 1)
mysolver.internalPointBuildView("sig_xz",IPField.SIG_XZ, 1, 1)
mysolver.internalPointBuildView("temperature",IPField.TEMPERATURE, 1, 1)
mysolver.internalPointBuildView("qx",IPField.THERMALFLUX_X, 1, 1)
mysolver.internalPointBuildView("qy",IPField.THERMALFLUX_Y, 1, 1)
mysolver.internalPointBuildView("qz",IPField.THERMALFLUX_Z, 1, 1)

mysolver.archivingNodeDisplacement(1,0,nstepArch);
mysolver.archivingNodeDisplacement(1,1,nstepArch);
mysolver.archivingNodeDisplacement(1,2,nstepArch);
mysolver.archivingNodeDisplacement(1,3,nstepArch);
mysolver.archivingNodeDisplacement(2,0,nstepArch);
mysolver.archivingNodeDisplacement(2,1,nstepArch);
mysolver.archivingNodeDisplacement(2,2,nstepArch);
mysolver.archivingNodeDisplacement(2,3,nstepArch);
mysolver.archivingNodeDisplacement(3,0,nstepArch);
mysolver.archivingNodeDisplacement(3,1,nstepArch);
mysolver.archivingNodeDisplacement(3,2,nstepArch);
mysolver.archivingNodeDisplacement(3,3,nstepArch);
mysolver.archivingNodeDisplacement(4,0,nstepArch);
mysolver.archivingNodeDisplacement(4,1,nstepArch);
mysolver.archivingNodeDisplacement(4,2,nstepArch);
mysolver.archivingNodeDisplacement(4,3,nstepArch);
mysolver.archivingNodeDisplacement(5,0,nstepArch);
mysolver.archivingNodeDisplacement(5,1,nstepArch);
mysolver.archivingNodeDisplacement(5,2,nstepArch);
mysolver.archivingNodeDisplacement(5,3,nstepArch);
mysolver.archivingNodeDisplacement(6,0,nstepArch);
mysolver.archivingNodeDisplacement(6,1,nstepArch);
mysolver.archivingNodeDisplacement(6,2,nstepArch);
mysolver.archivingNodeDisplacement(6,3,nstepArch);
mysolver.archivingNodeDisplacement(7,0,nstepArch);
mysolver.archivingNodeDisplacement(7,1,nstepArch);
mysolver.archivingNodeDisplacement(7,2,nstepArch);
mysolver.archivingNodeDisplacement(7,3,nstepArch);
mysolver.archivingNodeDisplacement(8,0,nstepArch);
mysolver.archivingNodeDisplacement(8,1,nstepArch);
mysolver.archivingNodeDisplacement(8,2,nstepArch);
mysolver.archivingNodeDisplacement(8,3,nstepArch);

mysolver.archivingForceOnPhysicalGroup("Face", 1234, 0)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 1)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 2)
mysolver.archivingForceOnPhysicalGroup("Face", 1234, 3)
mysolver.archivingForceOnPhysicalGroup("Face", 5678, 2)
mysolver.archivingForceOnPhysicalGroup("Face", 5678, 3)

mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SVM,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.PLASTICSTRAIN,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.TEMPERATURE,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_YZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.STRAIN_XZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_YZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.SIG_XZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.F_ZZ,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.JACOBIAN,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_XX,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_YY,IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",nfield, IPField.P_ZZ,IPField.MEAN_VALUE);

mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FTH_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FTH_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FF_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FF_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FPG_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FPG_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FP_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FP_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FG_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield, IPField.FG_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.FR_YY,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.FR_ZZ,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.ZG,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TIG,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TIR,	IPField.MEAN_VALUE);
mysolver.archivingIPOnPhysicalGroup("Volume",	nfield,	IPField.TT,	IPField.MEAN_VALUE);


#===Solving===================================================================================
mysolver.solve()


check = TestCheck()
check.equal(-5.260689e+01,mysolver.getArchivedForceOnPhysicalGroup("Face", 5678, 3),1.e-6)
check.equal( 4.241287e+02,mysolver.getArchivedNodalValue(1,3,mysolver.displacement),1.e-6)
check.equal(6.374043e-06,mysolver.getArchivedNodalValue(7,0,mysolver.displacement),1.e-6)

