//
// C++ Interface: fracture ipvariable
//
// Description: Class with definition of fracture cohesive ipvarible for dG3D
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef FRACTURECOHESIVEDG3DIPVARIABLE_H_
#define FRACTURECOHESIVEDG3DIPVARIABLE_H_

#include "dG3DIPVariable.h"
#include  "dG3DCohesiveIPVariable.h"

class FractureCohesive3DIPVariable : public IPVariable2ForFracture<dG3DIPVariableBase,Cohesive3DIPVariableBase>
{
 protected:
  double _facSigmac;
 public:
  FractureCohesive3DIPVariable(const double facSigmac=1.);
  FractureCohesive3DIPVariable(const FractureCohesive3DIPVariable &source);
  virtual FractureCohesive3DIPVariable& operator=(const IPVariable &source);
  virtual ~FractureCohesive3DIPVariable(){}
  virtual const double fractureStrengthFactor() const{return _facSigmac;}
  virtual void setFractureStrengthFactor(const double fsf){_facSigmac = fsf;}

  // but you can communicate values by filling these functions for your IPVariable
  virtual int numberValuesToCommunicateMPI()const{return 1;}
  virtual void fillValuesToCommunicateMPI(double *arrayMPI)const{arrayMPI[0]=_facSigmac;}
  virtual void getValuesFromMPI(const double *arrayMPI){_facSigmac=arrayMPI[0];}


  virtual IPVariable* getInternalData() {return _ipvbulk->getInternalData();};
  virtual const IPVariable* getInternalData()  const {return _ipvbulk->getInternalData();};

// access to bulk
  virtual const bool isInterface() const{return _ipvbulk->isInterface();}
  virtual double vonMises() const{return _ipvbulk->vonMises();}
  virtual double get(const int i) const;
  virtual double defoEnergy() const{return _ipvfrac->defoEnergy();}
  virtual double plasticEnergy() const{return _ipvfrac->plasticEnergy();}
  virtual double damageEnergy() const {return _ipvfrac->damageEnergy();};
  virtual int fractureEnergy(double* arrayEnergy) const{return _ipvfrac->fractureEnergy(arrayEnergy);}

  virtual void getCauchyStress(STensor3 &cauchy) const { return _ipvbulk->getCauchyStress(cauchy); }
  virtual double getJ() const { return _ipvbulk->getJ(); };

  virtual bool dissipationIsActive() const {return _ipvbulk->dissipationIsActive();};

  virtual void blockDissipation(const bool fl){_ipvbulk->blockDissipation(fl);};
  virtual bool dissipationIsBlocked() const {return _ipvbulk->dissipationIsBlocked();};

  virtual void brokenSolver(const bool fl){_ipvbulk->brokenSolver(fl);};
  virtual bool solverIsBroken() const {return _ipvbulk->solverIsBroken();};

  virtual double getBarJ() const {return _ipvbulk->getBarJ();}
  virtual double &getRefToBarJ() {return _ipvbulk->getRefToBarJ();}
  virtual double getLocalJ() const {return _ipvbulk->getLocalJ();}
  virtual double &getRefToLocalJ() {return _ipvbulk->getRefToLocalJ();}
  virtual void getBackLocalDeformationGradient(STensor3& localF) const ;

  virtual const STensor3 &getConstRefToDeformationGradient() const {return _ipvbulk->getConstRefToDeformationGradient();}
  virtual STensor3 &getRefToDeformationGradient() {return _ipvbulk->getRefToDeformationGradient();}

  virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStress() const {return _ipvbulk->getConstRefToFirstPiolaKirchhoffStress();}
  virtual STensor3 &getRefToFirstPiolaKirchhoffStress() {return _ipvbulk->getRefToFirstPiolaKirchhoffStress();}

  virtual const STensor43 &getConstRefToTangentModuli() const {return _ipvbulk->getConstRefToTangentModuli();}
  virtual STensor43 &getRefToTangentModuli() {return _ipvbulk->getRefToTangentModuli();}

  virtual const STensor43 &getConstRefToElasticTangentModuli() const {return _ipvbulk->getConstRefToElasticTangentModuli();};
  virtual STensor43 &getRefToElasticTangentModuli() {return _ipvbulk->getRefToElasticTangentModuli();};

  virtual const STensor43 &getConstRefToDGElasticTangentModuli() const {return _ipvbulk->getConstRefToDGElasticTangentModuli();}
  virtual void setRefToDGElasticTangentModuli(const STensor43 &eT) {return _ipvbulk->setRefToDGElasticTangentModuli(eT);}

  virtual const SVector3 &getConstRefToReferenceOutwardNormal() const {return _ipvbulk->getConstRefToReferenceOutwardNormal();}
  virtual SVector3 &getRefToReferenceOutwardNormal() {return _ipvbulk->getRefToReferenceOutwardNormal();}

  virtual const SVector3 &getConstRefToCurrentOutwardNormal() const {return _ipvbulk->getConstRefToCurrentOutwardNormal();}
  virtual SVector3 &getRefToCurrentOutwardNormal() {return _ipvbulk->getRefToCurrentOutwardNormal();}

  virtual const SVector3 &getConstRefToJump() const {return _ipvbulk->getConstRefToJump();}
  virtual SVector3 &getRefToJump() {return _ipvbulk->getRefToJump();}

  virtual const SVector3& getConstRefToIncompatibleJump() const {return _ipvbulk->getConstRefToIncompatibleJump();};
  virtual SVector3& getRefToIncompatibleJump() {return _ipvbulk->getRefToIncompatibleJump();};

   // non local
  virtual int getNumberNonLocalVariable() const {return _ipvbulk->getNumberNonLocalVariable();};
  //
  virtual double getConstRefToLocalVariable(const int idex) const {return _ipvbulk->getConstRefToLocalVariable(idex);};
  virtual double& getRefToLocalVariable(const int idex) {return _ipvbulk->getRefToLocalVariable(idex);};
  //
  virtual double getConstRefToNonLocalVariable(const int idex) const {return _ipvbulk->getConstRefToNonLocalVariable(idex);};
  virtual double& getRefToNonLocalVariable(const int idex) {return _ipvbulk->getRefToNonLocalVariable(idex);};

  virtual const std::vector<STensor3> &getConstRefToDLocalVariableDStrain() const {return _ipvbulk->getConstRefToDLocalVariableDStrain();};
  virtual std::vector<STensor3> &getRefToDLocalVariableDStrain() {return _ipvbulk->getRefToDLocalVariableDStrain();};

  virtual const std::vector<STensor3> &getConstRefToDStressDNonLocalVariable() const {return _ipvbulk->getConstRefToDStressDNonLocalVariable();};
  virtual std::vector<STensor3> &getRefToDStressDNonLocalVariable() {return _ipvbulk->getRefToDStressDNonLocalVariable();};

  virtual const fullMatrix<double> &getConstRefToDLocalVariableDNonLocalVariable() const {return _ipvbulk->getConstRefToDLocalVariableDNonLocalVariable();};
  virtual fullMatrix<double> &getRefToDLocalVariableDNonLocalVariable() {return _ipvbulk->getRefToDLocalVariableDNonLocalVariable();};

  virtual const fullVector<double> &getConstRefToNonLocalJump() const {return _ipvbulk->getConstRefToNonLocalJump(); }
  virtual fullVector<double> &getRefToNonLocalJump() {return _ipvbulk->getRefToNonLocalJump(); }

  virtual const std::vector<SVector3> &getConstRefToGradNonLocalVariable() const {return _ipvbulk->getConstRefToGradNonLocalVariable();}
  virtual std::vector<SVector3> &getRefToGradNonLocalVariable() {return _ipvbulk->getRefToGradNonLocalVariable();}
  //coupling non local with constitutiveExtraDof
  virtual const fullMatrix<double> & getConstRefTodLocalVariableDExtraDofDiffusionField() const {return _ipvbulk->getConstRefTodLocalVariableDExtraDofDiffusionField();}
  virtual fullMatrix<double> & getRefTodLocalVariableDExtraDofDiffusionField(){return _ipvbulk->getRefTodLocalVariableDExtraDofDiffusionField();}
  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idx) const {return _ipvbulk->getConstRefToCharacteristicLengthMatrix(idx);};


  // extradof
  virtual int getNumConstitutiveExtraDofDiffusionVariable() const {return _ipvbulk->getNumConstitutiveExtraDofDiffusionVariable();};
  virtual double getConstRefToField(const int idex) const {return _ipvbulk->getConstRefToField(idex);};
  virtual double& getRefToField(const int idex) {return _ipvbulk->getRefToField(idex);};
  virtual const std::vector<SVector3>  &getConstRefToGradField() const {return _ipvbulk->getConstRefToGradField();};
  virtual std::vector<SVector3>  &getRefToGradField() {return _ipvbulk->getRefToGradField();};
  virtual const std::vector<SVector3>  &getConstRefToFlux() const {return _ipvbulk->getConstRefToFlux();};
  virtual std::vector<SVector3>  &getRefToFlux() {return _ipvbulk->getRefToFlux();};
  virtual const std::vector<STensor3>  &getConstRefTodPdField() const {return _ipvbulk->getConstRefTodPdField();};
  virtual std::vector<STensor3>  &getRefTodPdField() {return _ipvbulk->getRefTodPdField();};
  virtual const std::vector<STensor33>  &getConstRefTodPdGradField() const {return _ipvbulk->getConstRefTodPdGradField();};
  virtual std::vector<STensor33>  &getRefTodPdGradField() {return _ipvbulk->getRefTodPdGradField();};
  virtual const std::vector<std::vector<STensor3> >  &getConstRefTodFluxdGradField() const {return _ipvbulk->getConstRefTodFluxdGradField();};
  virtual std::vector<std::vector<STensor3> >  &getRefTodFluxdGradField() {return _ipvbulk->getRefTodFluxdGradField();};
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFluxdField() const {return _ipvbulk->getConstRefTodFluxdField();};
  virtual std::vector<std::vector<SVector3> >  &getRefTodFluxdField() {return _ipvbulk->getRefTodFluxdField();};
  virtual const std::vector<STensor33>  &getConstRefTodFluxdF() const {return _ipvbulk->getConstRefTodFluxdF();};
  virtual std::vector<STensor33>   &getRefTodFluxdF() {return _ipvbulk->getRefTodFluxdF();};
  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToLinearK() const {return _ipvbulk->getConstRefToLinearK();};
  virtual void setConstRefToLinearK(const STensor3 &eT, int i, int j)  {_ipvbulk->setConstRefToLinearK(eT, i, j);};
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodLinearKdField() const {return _ipvbulk->getConstRefTodLinearKdField();}; 
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodLinearKdField() {return _ipvbulk->getRefTodLinearKdField();}; 
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodLinearKdF() const {return _ipvbulk->getConstRefTodLinearKdF();}; 
  virtual std::vector<std::vector<STensor43>>  &getRefTodLinearKdF() {return _ipvbulk->getRefTodLinearKdF();}; 
/*  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToEnergyK() const {return _ipvbulk->getConstRefToEnergyK();};
  virtual void setConstRefToEnergyK(const STensor3 &eT, int i, int j)  {_ipvbulk->setConstRefToEnergyK(eT, i, j);};
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodEnergyKdField() const {return _ipvbulk->getConstRefTodEnergyKdField();}; 
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodEnergyKdField() {return _ipvbulk->getRefTodEnergyKdField();}; 
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodEnergyKdF() const {return _ipvbulk->getConstRefTodEnergyKdF();}; 
  virtual std::vector<std::vector<STensor43>>  &getRefTodEnergyKdF() {return _ipvbulk->getRefTodEnergyKdF();}; */
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodFluxdGradFielddField() const {return _ipvbulk->getConstRefTodFluxdGradFielddField();}; 
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodFluxdGradFielddField() {return _ipvbulk->getRefTodFluxdGradFielddField();}; 
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodFluxdGradFielddF() const {return _ipvbulk->getConstRefTodFluxdGradFielddF();}; 
  virtual std::vector<std::vector<STensor43>>  &getRefTodFluxdGradFielddF() {return _ipvbulk->getRefTodFluxdGradFielddF();}; 
  virtual const fullVector<double>  &getConstRefToFieldSource() const {return _ipvbulk->getConstRefToFieldSource();};
  virtual fullVector<double>  &getRefToFieldSource() {return _ipvbulk->getRefToFieldSource();};
  virtual const fullMatrix<double>  &getConstRefTodFieldSourcedField() const {return _ipvbulk->getConstRefTodFieldSourcedField();};
  virtual fullMatrix<double>  &getRefTodFieldSourcedField() {return _ipvbulk->getRefTodFieldSourcedField();};
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFieldSourcedGradField() const {return _ipvbulk->getConstRefTodFieldSourcedGradField();};
  virtual std::vector<std::vector<SVector3> >  &getRefTodFieldSourcedGradField() {return _ipvbulk->getRefTodFieldSourcedGradField();};
  virtual const std::vector<STensor3>  &getConstRefTodFieldSourcedF() const {return _ipvbulk->getConstRefTodFieldSourcedF();};
  virtual std::vector<STensor3>  &getRefTodFieldSourcedF() {return _ipvbulk->getRefTodFieldSourcedF();};
  virtual const fullVector<double>  &getConstRefToMechanicalSource() const {return _ipvbulk->getConstRefToMechanicalSource();};
  virtual fullVector<double>  &getRefToMechanicalSource() {return _ipvbulk->getRefToMechanicalSource();};
  virtual const fullMatrix<double>  &getConstRefTodMechanicalSourcedField() const {return _ipvbulk->getConstRefTodMechanicalSourcedField();};
  virtual fullMatrix<double>  &getRefTodMechanicalSourcedField() {return _ipvbulk->getRefTodMechanicalSourcedField();};
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodMechanicalSourcedGradField() const {return _ipvbulk->getConstRefTodMechanicalSourcedGradField();};
  virtual std::vector<std::vector<SVector3> >  &getRefTodMechanicalSourcedGradField() {return _ipvbulk->getRefTodMechanicalSourcedGradField();};
  virtual const std::vector<STensor3>  &getConstRefTodMechanicalSourcedF() const {return _ipvbulk->getConstRefTodMechanicalSourcedF();};
  virtual std::vector<STensor3>  &getRefTodMechanicalSourcedF() {return _ipvbulk->getRefTodMechanicalSourcedF();};
  virtual const fullVector<double>  &getConstRefToExtraDofFieldCapacityPerUnitField() const {return _ipvbulk->getConstRefToExtraDofFieldCapacityPerUnitField();};
  virtual fullVector<double>  &getRefToExtraDofFieldCapacityPerUnitField() {return _ipvbulk->getRefToExtraDofFieldCapacityPerUnitField();};
  virtual const fullVector<double>  &getConstRefToFieldJump() const {return _ipvbulk->getConstRefToFieldJump();};
  virtual fullVector<double>  &getRefToFieldJump() {return _ipvbulk->getRefToFieldJump();};
  virtual const fullVector<double>  &getConstRefToOneOverFieldJump() const {return _ipvbulk->getConstRefToOneOverFieldJump();};
  virtual fullVector<double>  &getRefToOneOverFieldJump() {return _ipvbulk->getRefToOneOverFieldJump();};
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldm() const {return _ipvbulk->getConstRefTodOneOverFieldJumpdFieldm();};
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldm() {return _ipvbulk->getRefTodOneOverFieldJumpdFieldm() ;};
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldp() const {return _ipvbulk->getConstRefTodOneOverFieldJumpdFieldp();};
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldp() {return _ipvbulk->getRefTodOneOverFieldJumpdFieldp();};
  virtual const std::vector<SVector3>  &getConstRefToInterfaceFlux() const {return _ipvbulk->getConstRefToInterfaceFlux();};
  virtual std::vector<SVector3>  &getRefToInterfaceFlux() {return _ipvbulk->getRefToInterfaceFlux();};
 
  //Energy Conjugated Field (fT & fv)
  virtual double getConstRefToEnergyConjugatedField(const int idex) const {return _ipvbulk->getConstRefToEnergyConjugatedField(idex);};  
  virtual double  &getRefToEnergyConjugatedField(const int idex) {return _ipvbulk->getRefToEnergyConjugatedField(idex);};  
  virtual const std::vector<SVector3>  &getConstRefToGradEnergyConjugatedField() const {return _ipvbulk->getConstRefToGradEnergyConjugatedField();}; 
  virtual std::vector<SVector3>  &getRefToGradEnergyConjugatedField() {return _ipvbulk->getRefToGradEnergyConjugatedField();}; 
  virtual const  std::vector<std::vector<double>>  &getConstRefTodFielddEnergyConjugatedField() const {return _ipvbulk->getConstRefTodFielddEnergyConjugatedField();};
  virtual std::vector<std::vector<double>>  &getRefTodFielddEnergyConjugatedField() {return _ipvbulk->getRefTodFielddEnergyConjugatedField();};
  virtual const std::vector<std::vector<SVector3>>  &getConstRefTodGradFielddEnergyConjugatedField() const {return _ipvbulk->getConstRefTodGradFielddEnergyConjugatedField();}; 
  virtual std::vector<std::vector<SVector3>>  &getRefTodGradFielddEnergyConjugatedField() {return _ipvbulk->getRefTodGradFielddEnergyConjugatedField();};
  virtual const std::vector<std::vector<STensor3>>  &getConstRefTodGradFielddGradEnergyConjugatedField() const {return _ipvbulk->getConstRefTodGradFielddGradEnergyConjugatedField();}; 
  virtual std::vector<std::vector<STensor3>>  &getRefTodGradFielddGradEnergyConjugatedField() {return _ipvbulk->getRefTodGradFielddGradEnergyConjugatedField();}; 
  virtual const fullVector<double>  &getConstRefToEnergyConjugatedFieldJump() const {return _ipvbulk->getConstRefToEnergyConjugatedFieldJump();};  
  virtual fullVector<double>  &getRefToEnergyConjugatedFieldJump() {return _ipvbulk->getRefToEnergyConjugatedFieldJump();};
  virtual const  std::vector<std::vector<double>>  &getConstRefTodEnergyConjugatedFieldJumpdFieldm() const {return _ipvbulk->getConstRefTodEnergyConjugatedFieldJumpdFieldm();};
  virtual std::vector<std::vector<double>>  &getRefTodEnergyConjugatedFieldJumpdFieldm() {return _ipvbulk->getRefTodEnergyConjugatedFieldJumpdFieldm();};
  virtual const  std::vector<std::vector<double>>  &getConstRefTodEnergyConjugatedFieldJumpdFieldp() const {return _ipvbulk->getConstRefTodEnergyConjugatedFieldJumpdFieldp();};
  virtual std::vector<std::vector<double>>  &getRefTodEnergyConjugatedFieldJumpdFieldp() {return _ipvbulk->getRefTodEnergyConjugatedFieldJumpdFieldp();};

  //coupling constitutive extra dof with nonLocal
  virtual const std::vector<std::vector<SVector3> > & getConstRefTodFluxdNonLocalVariable() const {return _ipvbulk->getConstRefTodFluxdNonLocalVariable();}
  virtual std::vector<std::vector<SVector3> > & getRefTodFluxdNonLocalVariable(){return _ipvbulk->getRefTodFluxdNonLocalVariable();}
  virtual const fullMatrix<double> &getConstRefTodFieldSourcedNonLocalVariable() const{return _ipvbulk->getConstRefTodFieldSourcedNonLocalVariable();}
  virtual const fullMatrix<double> &getRefTodFieldSourcedNonLocalVariable(){return _ipvbulk->getRefTodFieldSourcedNonLocalVariable();}
  virtual const fullMatrix<double> &getConstRefTodMechanicalSourcedNonLocalVariable() const{return _ipvbulk->getConstRefTodMechanicalSourcedNonLocalVariable();}
  virtual fullMatrix<double> &getRefTodMechanicalSourcedNonLocalVariable(){return _ipvbulk->getRefTodMechanicalSourcedNonLocalVariable();}


  virtual const std::vector<const STensor3*> &getConstRefToLinearSymmetrizationCoupling() const  {return _ipvbulk->getConstRefToLinearSymmetrizationCoupling();};
  virtual void setConstRefToLinearSymmetrizationCoupling(const STensor3 &eT, int i)  {_ipvbulk->setConstRefToLinearSymmetrizationCoupling(eT,i);};


  //------------------get 1st piola b4 AV ---------------//

  virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStressb4AV() const {return _ipvbulk->getConstRefToFirstPiolaKirchhoffStressb4AV();}
  virtual STensor3 &getRefToFirstPiolaKirchhoffStressb4AV() {return _ipvbulk->getRefToFirstPiolaKirchhoffStressb4AV();}


  virtual void setBulkCriticalDamage(const double DT){_ipvbulk->setBulkCriticalDamage(DT);};
  virtual double getBulkCriticalDamage() const {return _ipvbulk->getBulkCriticalDamage();};

  virtual void setNonLocalToLocal(const bool fl){_ipvbulk->setNonLocalToLocal(fl);};
  virtual bool getNonLocalToLocal() const {return _ipvbulk->getNonLocalToLocal();};

  // interface
  virtual bool withDeformationGradient() const {return _ipvfrac->withDeformationGradient();}; // with deformation gradient
  virtual bool withJumpGradient() const {return _ipvfrac->withJumpGradient();}; // with jump gradient
  virtual bool withIncompatibleJump() const {return _ipvfrac->withIncompatibleJump();};

  virtual const STensor3 &getConstRefToJumpGradient() const {return _ipvfrac->getConstRefToJumpGradient();}
  virtual STensor3 &getRefToJumpGradient() {return _ipvfrac->getRefToJumpGradient();}

	 virtual const SVector3 &getConstRefToInterfaceForce() const {return _ipvfrac->getConstRefToInterfaceForce();}
  virtual SVector3 &getRefToInterfaceForce() {return _ipvfrac->getRefToInterfaceForce();}

  virtual const STensor3 &getConstRefToDInterfaceForceDjump() const {return _ipvfrac->getConstRefToDInterfaceForceDjump();}
  virtual STensor3 &getRefToDInterfaceForceDjump() {return _ipvfrac->getRefToDInterfaceForceDjump();}

  virtual const STensor33 &getConstRefToDInterfaceForceDjumpGradient() const {return _ipvfrac->getConstRefToDInterfaceForceDjumpGradient();}
  virtual STensor33 &getRefToDInterfaceForceDjumpGradient() {return _ipvfrac->getRefToDInterfaceForceDjumpGradient();}

  virtual const STensor33 &getConstRefToDInterfaceForceDDeformationGradient() const {return _ipvfrac->getConstRefToDInterfaceForceDDeformationGradient();};
  virtual STensor33 &getRefToDInterfaceForceDDeformationGradient() {return _ipvfrac->getRefToDInterfaceForceDDeformationGradient();};

  virtual const STensor3& getConstRefToDInterfaceForceDIncompatibleJump() const {return _ipvfrac->getConstRefToDInterfaceForceDIncompatibleJump();};
  virtual STensor3& getRefToDInterfaceForceDIncompatibleJump() {return _ipvfrac->getRefToDInterfaceForceDIncompatibleJump();};

	// for path following based on  irreversibe energt
	virtual double irreversibleEnergy() const  {return _ipvfrac->irreversibleEnergy();};
	virtual double& getRefToIrreversibleEnergy() {return _ipvfrac->getRefToIrreversibleEnergy();};
	// to avoid duplicate allocation
	virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _ipvfrac->getConstRefToDIrreversibleEnergyDDeformationGradient();};
	virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _ipvfrac->getRefToDIrreversibleEnergyDDeformationGradient();};

	// irreversible ennergy depend on nonlocal variale
	virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const {return _ipvbulk->getConstRefToDIrreversibleEnergyDNonLocalVariable(index);};
	virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) {return _ipvbulk->getRefToDIrreversibleEnergyDDNonLocalVariable(index);};

	virtual SVector3& getRefToDIrreversibleEnergyDJump() {return _ipvfrac->getRefToDIrreversibleEnergyDJump();};
	virtual const SVector3& getConstRefToDIrreversibleEnergyDJump() const {return _ipvfrac->getConstRefToDIrreversibleEnergyDJump();};

	virtual SVector3& getRefToDIrreversibleEnergyDIncompatibleJump() {return _ipvfrac->getRefToDIrreversibleEnergyDIncompatibleJump();};
	virtual const SVector3& getConstRefToDIrreversibleEnergyDIncompatibleJump() const {return _ipvfrac->getConstRefToDIrreversibleEnergyDIncompatibleJump();};
  // access to cohesive law

  virtual const SVector3& getConstRefToDGJump() const {return _ipvfrac->getConstRefToDGJump();};
  virtual SVector3& getRefToDGJump() {return _ipvfrac->getRefToDGJump();};

  virtual const SVector3& getConstRefToCohesiveJump() const {return _ipvfrac->getConstRefToCohesiveJump();};
  virtual SVector3& getRefToCohesiveJump() {return _ipvfrac->getRefToCohesiveJump();};

  virtual const STensor33& getConstRefToDCohesiveJumpDDeformationGradient() const {return _ipvfrac->getConstRefToDCohesiveJumpDDeformationGradient();};
  virtual STensor33& getRefToDCohesiveJumpDDeformationGradient() {return _ipvfrac->getRefToDCohesiveJumpDDeformationGradient();};

  virtual const STensor3& getConstRefToDCohesiveJumpDJump() const  {return _ipvfrac->getConstRefToDCohesiveJumpDJump();};
  virtual STensor3& getRefToDCohesiveJumpDJump() {return _ipvfrac->getRefToDCohesiveJumpDJump();};

  virtual const STensor3& getConstRefToDCohesiveJumpDIncompatibleJump() const{return _ipvfrac->getConstRefToDCohesiveJumpDIncompatibleJump();}
  virtual STensor3& getRefToDCohesiveJumpDIncompatibleJump() {return _ipvfrac->getRefToDCohesiveJumpDIncompatibleJump();}

  virtual const SVector3& getConstRefToInitialCohesiveJump() const {return _ipvfrac->getConstRefToInitialCohesiveJump();};
  virtual SVector3& getRefToInitialCohesiveJump() {return _ipvfrac->getRefToInitialCohesiveJump();};

  virtual bool isDeleted() const {return _ipvfrac->isDeleted();};
  virtual void Delete(const bool fl) {_ipvbulk->Delete(fl); _ipvfrac->Delete(fl);}
	virtual bool ifTension() const {return _ipvfrac->ifTension();};
	virtual void setTensionFlag(const bool tensionflag) {_ipvfrac->setTensionFlag(tensionflag);};

  virtual const STensor3& getConstRefToBulkDeformationGradientAtFailureOnset() const {return _ipvfrac->getConstRefToBulkDeformationGradientAtFailureOnset();};
  virtual STensor3& getRefToBulkDeformationGradientAtFailureOnset() {return _ipvfrac->getRefToBulkDeformationGradientAtFailureOnset();};

  virtual const SVector3& getConstRefToIncompatibleJumpAtFailureOnset() const {return _ipvfrac->getConstRefToIncompatibleJumpAtFailureOnset();};
  virtual SVector3& getRefToIncompatibleJumpAtFailureOnset() {return _ipvfrac->getRefToIncompatibleJumpAtFailureOnset();};

  // deformation gradient without otherparts relating to the displacement jump
  virtual const STensor3& getBulkDeformationGradient() const {return _ipvfrac->getBulkDeformationGradient();};
  virtual void setBulkDeformationGradient(const STensor3& Fb) {_ipvfrac->setBulkDeformationGradient(Fb);};

  virtual const STensor33& getConstRefToDInterfaceDeformationGradientDJump() const {return _ipvfrac->getConstRefToDInterfaceDeformationGradientDJump();};
  virtual STensor33& getRefToDInterfaceDeformationGradientDJump() {return _ipvfrac->getRefToDInterfaceDeformationGradientDJump();};

  virtual const STensor43& getConstRefToDInterfaceDeformationGradientDDeformationGradient() const {return _ipvfrac->getConstRefToDInterfaceDeformationGradientDDeformationGradient();};
  virtual STensor43& getRefToDInterfaceDeformationGradientDDeformationGradient() {return _ipvfrac->getRefToDInterfaceDeformationGradientDDeformationGradient();};

  virtual const STensor33& getConstRefToDInterfaceDeformationGradientDIncompatibleJump() const {return _ipvfrac->getConstRefToDInterfaceDeformationGradientDIncompatibleJump();};
  virtual STensor33& getRefToDInterfaceDeformationGradientDIncompatibleJump() {return _ipvfrac->getRefToDInterfaceDeformationGradientDIncompatibleJump();};

  //current basis
  virtual const SVector3& getConstRefToCohesiveNormal() const {return _ipvfrac->getConstRefToCohesiveNormal();};
  virtual SVector3& getRefToCohesiveNormal() {return _ipvfrac->getRefToCohesiveNormal();};
  virtual const SVector3& getConstRefToCohesiveTangent() const {return _ipvfrac->getConstRefToCohesiveTangent();};
  virtual SVector3& getRefToCohesiveTangent() {return _ipvfrac->getRefToCohesiveTangent();};
  virtual const SVector3& getConstRefToCohesiveBiTangent() const {return _ipvfrac->getConstRefToCohesiveBiTangent();};
  virtual SVector3& getRefToCohesiveBiTangent() {return _ipvfrac->getRefToCohesiveBiTangent();};


  //reference basis
  virtual const SVector3& getConstRefToCohesiveReferenceNormal() const {return _ipvfrac->getConstRefToCohesiveReferenceNormal();};
  virtual SVector3& getRefToCohesiveReferenceNormal() {return _ipvfrac->getRefToCohesiveReferenceNormal();};
  virtual const SVector3& getConstRefToCohesiveReferenceTangent() const {return _ipvfrac->getConstRefToCohesiveReferenceTangent();};
  virtual SVector3& getRefToCohesiveReferenceTangent() {return _ipvfrac->getRefToCohesiveReferenceTangent();};
  virtual const SVector3& getConstRefToCohesiveReferenceBiTangent() const {return _ipvfrac->getConstRefToCohesiveReferenceBiTangent();};
  virtual SVector3& getRefToCohesiveReferenceBiTangent() {return _ipvfrac->getRefToCohesiveReferenceBiTangent();};


  void initializeFracture(const SVector3 &ujump_,
                          const double smax, const double snor, const double tau, const double Gc,
                          const double beta_, const bool ift, const STensor3 &cauchy, const double K, const SVector3& norm, const SVector3& f0,
                          const IPVariable* bulkIPv)
  {  _ipvfrac->initializeFracture(ujump_, smax, snor, tau, Gc, beta_, ift, cauchy,K,norm,f0,bulkIPv);}


  virtual IPVariable* clone() const {return new FractureCohesive3DIPVariable(*this);};
  virtual void restart();
};


class MultiscaleFractureCohesive3DIPVariable : public FractureCohesive3DIPVariable{
	protected:

  public:
    MultiscaleFractureCohesive3DIPVariable(const double facSigmac=1.):FractureCohesive3DIPVariable(facSigmac){
    };
    MultiscaleFractureCohesive3DIPVariable(const MultiscaleFractureCohesive3DIPVariable &src):FractureCohesive3DIPVariable(src){};
    virtual MultiscaleFractureCohesive3DIPVariable& operator=(const IPVariable &source){
      FractureCohesive3DIPVariable::operator=(source);
			const MultiscaleFractureCohesive3DIPVariable* psrc = dynamic_cast<const MultiscaleFractureCohesive3DIPVariable*>(&source);
			if (psrc!=NULL){


			}
      return *this;
    };
    virtual ~MultiscaleFractureCohesive3DIPVariable(){}

		virtual void broken();
		virtual void nobroken();

    virtual IPVariable* clone() const {return new MultiscaleFractureCohesive3DIPVariable(*this);};
    virtual void restart(){
      FractureCohesive3DIPVariable::restart();
    };
};


#endif // FRACTURECOHESIVEIPVARIABLE_H_
