//
// C++ Interface: NonLocalDamagePowerYieldHyperDG3DMaterialLaw
//
// Description:
//
//
// Author: V.D. NGUYEN 2014
//
// Copyright: See COPYING file that comes with this distribution
//
#include "NonLocalDamageHyperelasticDG3DMaterialLaw.h"
#include "nonLocalDamageHyperelasticDG3DIPVariable.h"
#include "MInterfaceElement.h"
#include "FractureCohesiveDG3DIPVariable.h"

//
class NonLocalDamagePowerYieldHyperDG3DMaterialLawWithNonLocalBasedFailureCriterion;
NonLocalDamagePowerYieldHyperDG3DMaterialLaw::NonLocalDamagePowerYieldHyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol,
                   const bool matrixbyPerturbation , const double pert):
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldPowerYieldHyperlaw= new mlawNonLocalDamagePowerYieldHyper(num,E,nu,rho,tol,matrixbyPerturbation,pert);
  _nldPowerYieldHyperlaw->setCompressionHardening(comp);
  _nldPowerYieldHyperlaw->setTractionHardening(trac);
  _nldPowerYieldHyperlaw->setCLengthLaw(_cLLaw);
  _nldPowerYieldHyperlaw->setDamageLaw(_damLaw);

  fillElasticStiffness(E, nu, elasticStiffness);
};

//
NonLocalDamagePowerYieldHyperDG3DMaterialLaw::NonLocalDamagePowerYieldHyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac, const kinematicHardening& kin,
                   const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol,
                   const bool matrixbyPerturbation , const double pert):
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldPowerYieldHyperlaw= new mlawNonLocalDamagePowerYieldHyper(num,E,nu,rho,tol,matrixbyPerturbation,pert);
  _nldPowerYieldHyperlaw->setCompressionHardening(comp);
  _nldPowerYieldHyperlaw->setTractionHardening(trac);
  _nldPowerYieldHyperlaw->setKinematicHardening(kin);
  _nldPowerYieldHyperlaw->setCLengthLaw(_cLLaw);
  _nldPowerYieldHyperlaw->setDamageLaw(_damLaw);
  fillElasticStiffness(E, nu, elasticStiffness);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setYieldPowerFactor(const double n){
  _nldPowerYieldHyperlaw->setPowerFactor(n);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::nonAssociatedFlowRuleFactor(const double b){
  _nldPowerYieldHyperlaw->nonAssociatedFlowRuleFactor(b);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setPlasticPoissonRatio(const double nup){
  _nldPowerYieldHyperlaw->setPlasticPoissonRatio(nup);
};

  void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setNonAssociatedFlow(const bool flag){
    _nldPowerYieldHyperlaw->setNonAssociatedFlow(flag);
  };
void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setStrainOrder(const int o){
  _nldPowerYieldHyperlaw->setStrainOrder(o);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscosityEffect(const viscosityLaw& v,  const double p){
  _nldPowerYieldHyperlaw->setViscosityEffect(v,p);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscoelasticMethod(const int method){
  _nldPowerYieldHyperlaw->setViscoelasticMethod(method);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscoElasticNumberOfElement(const int N){
  _nldPowerYieldHyperlaw->setViscoElasticNumberOfElement(N);
};
void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscoElasticData(const int i, const double Ei, const double taui){
  _nldPowerYieldHyperlaw->setViscoElasticData(i,Ei,taui);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscoElasticData_Bulk(const int i, const double Ki, const double ki){
  _nldPowerYieldHyperlaw->setViscoElasticData_Bulk(i,Ki,ki);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscoElasticData_Shear(const int i, const double Gi, const double gi){
  _nldPowerYieldHyperlaw->setViscoElasticData_Shear(i,Gi,gi);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setViscoElasticData(const std::string filename){
  _nldPowerYieldHyperlaw->setViscoElasticData(filename);
}

NonLocalDamagePowerYieldHyperDG3DMaterialLaw::~NonLocalDamagePowerYieldHyperDG3DMaterialLaw(){
  if (_nldPowerYieldHyperlaw) delete _nldPowerYieldHyperlaw;
	_nldPowerYieldHyperlaw = NULL;
};

NonLocalDamagePowerYieldHyperDG3DMaterialLaw::NonLocalDamagePowerYieldHyperDG3DMaterialLaw(const NonLocalDamagePowerYieldHyperDG3DMaterialLaw &source):

    dG3DMaterialLaw(source)
{
	_nldPowerYieldHyperlaw = NULL;
	if (source._nldPowerYieldHyperlaw != NULL){
		_nldPowerYieldHyperlaw = new mlawNonLocalDamagePowerYieldHyper(*(source._nldPowerYieldHyperlaw));
	}
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::setTime(const double t,const double dtime){
  dG3DMaterialLaw::setTime(t,dtime);
  _nldPowerYieldHyperlaw->setTime(t,dtime);
}

materialLaw::matname NonLocalDamagePowerYieldHyperDG3DMaterialLaw::getType() const {return _nldPowerYieldHyperlaw->getType();}

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);

  if(iele==NULL) inter=false;
  IPVariable* ipvi = new  nonLocalDamageHyperelasticDG3DIPVariable(*_nldPowerYieldHyperlaw,inter);
  IPVariable* ipv1 = new  nonLocalDamageHyperelasticDG3DIPVariable(*_nldPowerYieldHyperlaw,inter);
  IPVariable* ipv2 = new  nonLocalDamageHyperelasticDG3DIPVariable(*_nldPowerYieldHyperlaw,inter);
  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
  _nldPowerYieldHyperlaw->createIPState((dynamic_cast <nonLocalDamageHyperelasticDG3DIPVariable*> (ipvi))->getIPHyperViscoElastoPlasticNonLocalDamage(),
                         (dynamic_cast <nonLocalDamageHyperelasticDG3DIPVariable*> (ipv1))->getIPHyperViscoElastoPlasticNonLocalDamage(),
             			 (dynamic_cast <nonLocalDamageHyperelasticDG3DIPVariable*> (ipv2))->getIPHyperViscoElastoPlasticNonLocalDamage());

}

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }

  ipv = new  nonLocalDamageHyperelasticDG3DIPVariable(*_nldPowerYieldHyperlaw,inter);
  IPHyperViscoElastoPlasticNonLocalDamage * ipvnl = static_cast <nonLocalDamageHyperelasticDG3DIPVariable*>(ipv)->getIPHyperViscoElastoPlasticNonLocalDamage();
  _nldPowerYieldHyperlaw->createIPVariable(ipvnl,ele,nbFF_,GP,gpt);
}

double NonLocalDamagePowerYieldHyperDG3DMaterialLaw::soundSpeed() const{return _nldPowerYieldHyperlaw->soundSpeed();} // or change value ??

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  nonLocalDamageHyperelasticDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageHyperelasticDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariable*>(ipvp);
  }

  _nldPowerYieldHyperlaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  nonLocalDamageHyperelasticDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageHyperelasticDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariable*>(ipvp);
  }

  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPHyperViscoElastoPlasticNonLocalDamage* q1 = ipvcur->getIPHyperViscoElastoPlasticNonLocalDamage();
  const IPHyperViscoElastoPlasticNonLocalDamage* q0 = ipvprev->getIPHyperViscoElastoPlasticNonLocalDamage();

  STensor43& elasticL = ipvcur->getRefToElasticTangentModuli();

  /* compute stress */
  _nldPowerYieldHyperlaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                        ipvcur->getRefToDLocalVariableDStrain()[0],ipvcur->getRefToDStressDNonLocalVariable()[0],
                        ipvcur->getRefToDLocalVariableDNonLocalVariable()(0,0),stiff,&elasticL);

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);

}

void NonLocalDamagePowerYieldHyperDG3DMaterialLaw::initialIPVariable(IPVariable* ipv) const{
	dG3DIPVariableBase* dgIPV = dynamic_cast<dG3DIPVariableBase*>(ipv);
	if (dgIPV!=NULL){
		dgIPV->setRefToDGElasticTangentModuli(this->elasticStiffness);
	}
	else{
		Msg::Error("dG3DIPVariableBase must be used dG3DLinearElasticMaterialLaw::initialIPVariable");
	}
};

//
LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2, const double tol,
                   const bool matrixbyPerturbation , const double pert):
                                                              dG3DMaterialLaw(num,rho,true)
{
  _localPowerYieldHyperlaw= new mlawLocalDamagePowerYieldHyperWithFailure(num,E,nu,rho,tol,matrixbyPerturbation,pert);
  _localPowerYieldHyperlaw->setCompressionHardening(comp);
  _localPowerYieldHyperlaw->setTractionHardening(trac);

  _localPowerYieldHyperlaw->clearAllDamageLaw();
  _localPowerYieldHyperlaw->setDamageLaw(_damLaw1);
  _localPowerYieldHyperlaw->setDamageLaw(_damLaw2);

  fillElasticStiffness(E, nu, elasticStiffness);
};

//
LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac, const kinematicHardening& kin,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2, const double tol,
                   const bool matrixbyPerturbation , const double pert):
                                                              dG3DMaterialLaw(num,rho,true)
{
  _localPowerYieldHyperlaw= new mlawLocalDamagePowerYieldHyperWithFailure(num,E,nu,rho,tol,matrixbyPerturbation,pert);
  _localPowerYieldHyperlaw->setCompressionHardening(comp);
  _localPowerYieldHyperlaw->setTractionHardening(trac);
  _localPowerYieldHyperlaw->setKinematicHardening(kin);

  _localPowerYieldHyperlaw->clearAllDamageLaw();
  _localPowerYieldHyperlaw->setDamageLaw(_damLaw1);
  _localPowerYieldHyperlaw->setDamageLaw(_damLaw2);


  fillElasticStiffness(E, nu, elasticStiffness);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setYieldPowerFactor(const double n){
  _localPowerYieldHyperlaw->setPowerFactor(n);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::nonAssociatedFlowRuleFactor(const double b){
  _localPowerYieldHyperlaw->nonAssociatedFlowRuleFactor(b);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setPlasticPoissonRatio(const double nup){
  _localPowerYieldHyperlaw->setPlasticPoissonRatio(nup);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setNonAssociatedFlow(const bool flag){
  _localPowerYieldHyperlaw->setNonAssociatedFlow(flag);
};
void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setStrainOrder(const int o){
  _localPowerYieldHyperlaw->setStrainOrder(o);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscosityEffect(const viscosityLaw& v,  const double p){
  _localPowerYieldHyperlaw->setViscosityEffect(v,p);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoelasticMethod(const int method){
  _localPowerYieldHyperlaw->setViscoelasticMethod(method);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticNumberOfElement(const int N){
  _localPowerYieldHyperlaw->setViscoElasticNumberOfElement(N);
};
void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData(const int i, const double Ei, const double taui){
  _localPowerYieldHyperlaw->setViscoElasticData(i,Ei,taui);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData_Bulk(const int i, const double Ki, const double ki){
  _localPowerYieldHyperlaw->setViscoElasticData_Bulk(i,Ki,ki);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData_Shear(const int i, const double Gi, const double gi){
  _localPowerYieldHyperlaw->setViscoElasticData_Shear(i,Gi,gi);
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData(const std::string filename){
  _localPowerYieldHyperlaw->setViscoElasticData(filename);
}

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setFailureCriterion(const mlawHyperelasticFailureCriterionBase& fCr){
  _localPowerYieldHyperlaw->setFailureCriterion(fCr);
};
LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::~LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(){
  if (_localPowerYieldHyperlaw) delete _localPowerYieldHyperlaw;
	_localPowerYieldHyperlaw = NULL;
};

LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure &source):

    dG3DMaterialLaw(source)
{
	_localPowerYieldHyperlaw = NULL;
	if (source._localPowerYieldHyperlaw){
		_localPowerYieldHyperlaw = new mlawLocalDamagePowerYieldHyperWithFailure(*(source._localPowerYieldHyperlaw));
	}
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setTime(const double t,const double dtime){
  dG3DMaterialLaw::setTime(t,dtime);
  _localPowerYieldHyperlaw->setTime(t,dtime);
}

materialLaw::matname LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::getType() const {return _localPowerYieldHyperlaw->getType();}

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);

  if(iele==NULL) inter=false;
  IPVariable* ipvi = new  localDamageHyperelasticDG3DIPVariableWithFailure(*_localPowerYieldHyperlaw,inter);
  IPVariable* ipv1 = new  localDamageHyperelasticDG3DIPVariableWithFailure(*_localPowerYieldHyperlaw,inter);
  IPVariable* ipv2 = new  localDamageHyperelasticDG3DIPVariableWithFailure(*_localPowerYieldHyperlaw,inter);
  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
}

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }
  ipv = new  localDamageHyperelasticDG3DIPVariableWithFailure(*_localPowerYieldHyperlaw,inter);
}

double LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::soundSpeed() const{return _localPowerYieldHyperlaw->soundSpeed();} // or change value ??

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
   /* get ipvariable */
  localDamageHyperelasticDG3DIPVariableWithFailure* ipvcur = NULL;
  const localDamageHyperelasticDG3DIPVariableWithFailure* ipvprev = NULL;

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<localDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const localDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<localDamageHyperelasticDG3DIPVariableWithFailure*>(ipv);
    ipvprev = static_cast<const localDamageHyperelasticDG3DIPVariableWithFailure*>(ipvp);
  }

  _localPowerYieldHyperlaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  localDamageHyperelasticDG3DIPVariableWithFailure* ipvcur = NULL;
  const localDamageHyperelasticDG3DIPVariableWithFailure* ipvprev = NULL;

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<localDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const localDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<localDamageHyperelasticDG3DIPVariableWithFailure*>(ipv);
    ipvprev = static_cast<const localDamageHyperelasticDG3DIPVariableWithFailure*>(ipvp);
  }

  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPHyperViscoElastoPlasticMultipleLocalDamage* q1 = ipvcur->getIPHyperViscoElastoPlasticMultipleLocalDamage();
  const IPHyperViscoElastoPlasticMultipleLocalDamage* q0 = ipvprev->getIPHyperViscoElastoPlasticMultipleLocalDamage();

  /* compute stress */
  _localPowerYieldHyperlaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),stiff);

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);
}

void LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::initialIPVariable(IPVariable* ipv) const{
	dG3DIPVariableBase* dgIPV = dynamic_cast<dG3DIPVariableBase*>(ipv);
	if (dgIPV!=NULL){
		dgIPV->setRefToDGElasticTangentModuli(this->elasticStiffness);
	}
	else{
		Msg::Error("dG3DIPVariableBase must be used dG3DLinearElasticMaterialLaw::initialIPVariable");
	}
};

//
NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const CLengthLaw &_cLLaw1, const CLengthLaw &_cLLaw2,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2, const double tol,
                   const bool matrixbyPerturbation , const double pert):
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldPowerYieldHyperlaw= new mlawNonLocalDamagePowerYieldHyperWithFailure(num,E,nu,rho,tol,matrixbyPerturbation,pert);
  _nldPowerYieldHyperlaw->setCompressionHardening(comp);
  _nldPowerYieldHyperlaw->setTractionHardening(trac);

  _nldPowerYieldHyperlaw->setCLengthLaw(_cLLaw1);
  _nldPowerYieldHyperlaw->setCLengthLaw(_cLLaw2);

  _nldPowerYieldHyperlaw->setDamageLaw(_damLaw1);
  _nldPowerYieldHyperlaw->setDamageLaw(_damLaw2);

  fillElasticStiffness(E, nu, elasticStiffness);
};

//
NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac, const kinematicHardening& kin,
                   const CLengthLaw &_cLLaw1, const CLengthLaw &_cLLaw2,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2, const double tol,
                   const bool matrixbyPerturbation , const double pert):
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldPowerYieldHyperlaw= new mlawNonLocalDamagePowerYieldHyperWithFailure(num,E,nu,rho,tol,matrixbyPerturbation,pert);
  _nldPowerYieldHyperlaw->setCompressionHardening(comp);
  _nldPowerYieldHyperlaw->setTractionHardening(trac);
  _nldPowerYieldHyperlaw->setKinematicHardening(kin);

  _nldPowerYieldHyperlaw->setCLengthLaw(_cLLaw1);
  _nldPowerYieldHyperlaw->setCLengthLaw(_cLLaw2);

  _nldPowerYieldHyperlaw->setDamageLaw(_damLaw1);
  _nldPowerYieldHyperlaw->setDamageLaw(_damLaw2);


  fillElasticStiffness(E, nu, elasticStiffness);
};


void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setYieldPowerFactor(const double n){
  _nldPowerYieldHyperlaw->setPowerFactor(n);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::nonAssociatedFlowRuleFactor(const double b){
  _nldPowerYieldHyperlaw->nonAssociatedFlowRuleFactor(b);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setPlasticPoissonRatio(const double nup){
  _nldPowerYieldHyperlaw->setPlasticPoissonRatio(nup);
};

  void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setNonAssociatedFlow(const bool flag){
    _nldPowerYieldHyperlaw->setNonAssociatedFlow(flag);
  };
void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setStrainOrder(const int o){
  _nldPowerYieldHyperlaw->setStrainOrder(o);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscosityEffect(const viscosityLaw& v,  const double p){
  _nldPowerYieldHyperlaw->setViscosityEffect(v,p);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoelasticMethod(const int method){
  _nldPowerYieldHyperlaw->setViscoelasticMethod(method);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticNumberOfElement(const int N){
  _nldPowerYieldHyperlaw->setViscoElasticNumberOfElement(N);
};
void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData(const int i, const double Ei, const double taui){
  _nldPowerYieldHyperlaw->setViscoElasticData(i,Ei,taui);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData_Bulk(const int i, const double Ki, const double ki){
  _nldPowerYieldHyperlaw->setViscoElasticData_Bulk(i,Ki,ki);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData_Shear(const int i, const double Gi, const double gi){
  _nldPowerYieldHyperlaw->setViscoElasticData_Shear(i,Gi,gi);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setViscoElasticData(const std::string filename){
  _nldPowerYieldHyperlaw->setViscoElasticData(filename);
}

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setFailureCriterion(const mlawHyperelasticFailureCriterionBase& fCr){
  _nldPowerYieldHyperlaw->setFailureCriterion(fCr);
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setCritialDamage(const double D){
  _nldPowerYieldHyperlaw->setCritialDamage(D);
};

NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::~NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(){
  if (_nldPowerYieldHyperlaw) delete _nldPowerYieldHyperlaw;
	_nldPowerYieldHyperlaw = NULL;
};

NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure &source):

    dG3DMaterialLaw(source)
{
	_nldPowerYieldHyperlaw = NULL;
	if (source._nldPowerYieldHyperlaw){
		_nldPowerYieldHyperlaw = new mlawNonLocalDamagePowerYieldHyperWithFailure(*(source._nldPowerYieldHyperlaw));
	}
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::setTime(const double t,const double dtime){
  dG3DMaterialLaw::setTime(t,dtime);
  _nldPowerYieldHyperlaw->setTime(t,dtime);
}

materialLaw::matname NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::getType() const {return _nldPowerYieldHyperlaw->getType();}

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);

  if(iele==NULL) inter=false;
  IPVariable* ipvi = new  nonLocalDamageHyperelasticDG3DIPVariableWithFailure(*_nldPowerYieldHyperlaw,inter);
  IPVariable* ipv1 = new  nonLocalDamageHyperelasticDG3DIPVariableWithFailure(*_nldPowerYieldHyperlaw,inter);
  IPVariable* ipv2 = new  nonLocalDamageHyperelasticDG3DIPVariableWithFailure(*_nldPowerYieldHyperlaw,inter);
  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
  _nldPowerYieldHyperlaw->createIPState((dynamic_cast <nonLocalDamageHyperelasticDG3DIPVariableWithFailure*> (ipvi))->getIPHyperViscoElastoPlasticMultipleNonLocalDamage(),
                         (dynamic_cast <nonLocalDamageHyperelasticDG3DIPVariableWithFailure*> (ipv1))->getIPHyperViscoElastoPlasticMultipleNonLocalDamage(),
             			 (dynamic_cast <nonLocalDamageHyperelasticDG3DIPVariableWithFailure*> (ipv2))->getIPHyperViscoElastoPlasticMultipleNonLocalDamage());

}

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }

  ipv = new  nonLocalDamageHyperelasticDG3DIPVariableWithFailure(*_nldPowerYieldHyperlaw,inter);
  IPHyperViscoElastoPlasticMultipleNonLocalDamage * ipvnl = static_cast <nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipv)->getIPHyperViscoElastoPlasticMultipleNonLocalDamage();
  _nldPowerYieldHyperlaw->createIPVariable(ipvnl,ele,nbFF_,GP,gpt);
}

double NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::soundSpeed() const{return _nldPowerYieldHyperlaw->soundSpeed();} // or change value ??

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  nonLocalDamageHyperelasticDG3DIPVariableWithFailure* ipvcur = NULL;
  const nonLocalDamageHyperelasticDG3DIPVariableWithFailure* ipvprev = NULL;

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipv);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipvp);
  }

  _nldPowerYieldHyperlaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  nonLocalDamageHyperelasticDG3DIPVariableWithFailure* ipvcur = NULL;
  const nonLocalDamageHyperelasticDG3DIPVariableWithFailure* ipvprev = NULL;

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipv);
    ipvprev = static_cast<const nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(ipvp);
  }

  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPHyperViscoElastoPlasticMultipleNonLocalDamage* q1 = ipvcur->getIPHyperViscoElastoPlasticMultipleNonLocalDamage();
  const IPHyperViscoElastoPlasticMultipleNonLocalDamage* q0 = ipvprev->getIPHyperViscoElastoPlasticMultipleNonLocalDamage();

  STensor43& elasticL = ipvcur->getRefToElasticTangentModuli();

  /* compute stress */
  _nldPowerYieldHyperlaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                        ipvcur->getRefToDLocalVariableDStrain(),ipvcur->getRefToDStressDNonLocalVariable(),
                        ipvcur->getRefToDLocalVariableDNonLocalVariable(),stiff,&elasticL);

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);
}

void NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure::initialIPVariable(IPVariable* ipv) const{
	dG3DIPVariableBase* dgIPV = dynamic_cast<dG3DIPVariableBase*>(ipv);
	if (dgIPV!=NULL){
		dgIPV->setRefToDGElasticTangentModuli(this->elasticStiffness);
	}
	else{
		Msg::Error("dG3DIPVariableBase must be used dG3DLinearElasticMaterialLaw::initialIPVariable");
	}
};

