//
// C++ Interface: NonLocalDamagePowerYieldHyperDG3DMaterialLaw
//
// Description:
//
//
// Author: V.D. NGUYEN 2014
//
// Copyright: See COPYING file that comes with this distribution
//
#ifndef NONLOCALDAMAGEHYPERELASTICDG3DMATERIALLAW_H_
#define NONLOCALDAMAGEHYPERELASTICDG3DMATERIALLAW_H_
#include "dG3DMaterialLaw.h"
#include "mlawNonLocalDamageHyperelastic.h"

class NonLocalDamagePowerYieldHyperDG3DMaterialLaw : public dG3DMaterialLaw
{

 protected:
  mlawNonLocalDamagePowerYieldHyper *_nldPowerYieldHyperlaw; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

 public:
  NonLocalDamagePowerYieldHyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol=1.e-6,
                   const bool matrixbyPerturbation = false, const double pert = 1e-8);
  NonLocalDamagePowerYieldHyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const kinematicHardening& kin,
                   const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol=1.e-6,
                   const bool matrixbyPerturbation = false, const double pert = 1e-8);

  void setYieldPowerFactor(const double n);
  void nonAssociatedFlowRuleFactor(const double b);
  void setPlasticPoissonRatio(const double nup);
  void setNonAssociatedFlow(const bool flag);
  void setStrainOrder(const int o);
  void setViscosityEffect(const viscosityLaw& v, const double p);
  void setViscoelasticMethod(const int method);
  void setViscoElasticNumberOfElement(const int N);
  void setViscoElasticData(const int i, const double Ei, const double taui);
  void setViscoElasticData_Bulk(const int i, const double Ki, const double ki);
  void setViscoElasticData_Shear(const int i, const double Gi, const double gi);
  void setViscoElasticData(const std::string filename);

#ifndef SWIG
  virtual ~NonLocalDamagePowerYieldHyperDG3DMaterialLaw();
  NonLocalDamagePowerYieldHyperDG3DMaterialLaw(const NonLocalDamagePowerYieldHyperDG3DMaterialLaw &source);
	virtual materialLaw* clone() const {return new NonLocalDamagePowerYieldHyperDG3DMaterialLaw(*this);}
   virtual bool withEnergyDissipation() const {return _nldPowerYieldHyperlaw->withEnergyDissipation();};
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime);
  virtual materialLaw::matname getType() const;
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const;
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual void initialIPVariable(IPVariable* ipv) const;

  virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_nldPowerYieldHyperlaw->setMacroSolver(sv);
	};

#endif
};

class LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure: public dG3DMaterialLaw
{

 protected:
  mlawLocalDamagePowerYieldHyperWithFailure *_localPowerYieldHyperlaw; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

 public:
  LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2,const double tol=1.e-6,
                   const bool matrixbyPerturbation = false, const double pert = 1e-8);
  LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const kinematicHardening& kin,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2, const double tol=1.e-6,
                   const bool matrixbyPerturbation = false, const double pert = 1e-8);

  void setYieldPowerFactor(const double n);
  void nonAssociatedFlowRuleFactor(const double b);
  void setPlasticPoissonRatio(const double nup);
  void setNonAssociatedFlow(const bool flag);
  void setStrainOrder(const int o);
  void setViscosityEffect(const viscosityLaw& v, const double p);
  void setViscoelasticMethod(const int method);
  void setViscoElasticNumberOfElement(const int N);
  void setViscoElasticData(const int i, const double Ei, const double taui);
  void setViscoElasticData_Bulk(const int i, const double Ki, const double ki);
  void setViscoElasticData_Shear(const int i, const double Gi, const double gi);
  void setViscoElasticData(const std::string filename);

  void setFailureCriterion(const mlawHyperelasticFailureCriterionBase& fCr);

#ifndef SWIG
  virtual ~LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure();
  LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure &source);
	virtual materialLaw* clone() const {return new LocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(*this);}
  virtual bool withEnergyDissipation() const {return _localPowerYieldHyperlaw->withEnergyDissipation();};
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime);
  virtual materialLaw::matname getType() const;
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const;
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual void initialIPVariable(IPVariable* ipv) const;
  virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_localPowerYieldHyperlaw->setMacroSolver(sv);
	};

#endif
};


class NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure: public dG3DMaterialLaw
{

 protected:
  mlawNonLocalDamagePowerYieldHyperWithFailure *_nldPowerYieldHyperlaw; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

 public:
  NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const CLengthLaw &_cLLaw1, const CLengthLaw &_cLLaw2,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2,const double tol=1.e-6,
                   const bool matrixbyPerturbation = false, const double pert = 1e-8);
  NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &comp,
                   const J2IsotropicHardening &trac,
                   const kinematicHardening& kin,
                   const CLengthLaw &_cLLaw1, const CLengthLaw &_cLLaw2,
                   const DamageLaw &_damLaw1, const DamageLaw &_damLaw2, const double tol=1.e-6,
                   const bool matrixbyPerturbation = false, const double pert = 1e-8);

  void setYieldPowerFactor(const double n);
  void nonAssociatedFlowRuleFactor(const double b);
  void setPlasticPoissonRatio(const double nup);
  void setNonAssociatedFlow(const bool flag);
  void setStrainOrder(const int o);
  void setViscosityEffect(const viscosityLaw& v, const double p);
  void setViscoelasticMethod(const int method);
  void setViscoElasticNumberOfElement(const int N);
  void setViscoElasticData(const int i, const double Ei, const double taui);
  void setViscoElasticData_Bulk(const int i, const double Ki, const double ki);
  void setViscoElasticData_Shear(const int i, const double Gi, const double gi);
  void setViscoElasticData(const std::string filename);

  void setFailureCriterion(const mlawHyperelasticFailureCriterionBase& fCr);
  void setCritialDamage(const double D);

#ifndef SWIG
  virtual ~NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure();
  NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(const NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure &source);
	virtual materialLaw* clone() const {return new NonLocalDamagePowerYieldHyperDG3DMaterialLawWithFailure(*this);}
  virtual bool withEnergyDissipation() const {return _nldPowerYieldHyperlaw->withEnergyDissipation();};
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime);
  virtual materialLaw::matname getType() const;
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const;
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual void initialIPVariable(IPVariable* ipv) const;
  virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_nldPowerYieldHyperlaw->setMacroSolver(sv);
	};

#endif
};

#endif // NONLOCALDAMAGEHYPERELASTICDG3DMATERIALLAW_H_
