//
// C++ Interface: ipvariable
//
// Description: Class with definition of ipvarible for dG3D
//
//
// Author:  <V.D. Nguyen>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "dG3DEnhancedStrainIPVariable.h"
#include "ipField.h"
#include "highOrderTensor.h"



#if defined(HAVE_MPI)

int EnhancedStrainIPVariableBase::getMacroNumberElementDataSendToMicroProblem() const{
	// send _enhancedF + _betaEe + Fbuck
	return 9+1+9;
};
	// get number of values obtained by microscopic analysis to send to macroscopic analysis
int EnhancedStrainIPVariableBase::getMicroNumberElementDataSendToMacroProblem() const{
	// send _DPDenhanceF _localEnhancedF _DLocalEnhancedFDF _DLocalEnhancedFDEnhancedF
	// bandwidth + normal + _DirreversibleEnergyDF
	return 81+9+81+81+1+3+9;
};
	// get macroscopic kinematic data to send to microscopic problem
void EnhancedStrainIPVariableBase::getMacroDataSendToMicroProblem(double* val) const{
	const STensor3& eF = this->getConstRefToEnhancedDeformationGradient();
	const STensor3& Fbulk = this->getConstRefToBulkDeformationGradient();
	int row = 0;
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		val[index] = eF(i,j);
	}
	row+= 9;
	
	val[row] = _betaElement;
	row += 1;
	
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		val[index] = Fbulk(i,j);
	}
	row+= 9;
};
	// get computed data obtaind by microscopic analysis to send to macroscopic analysis
void EnhancedStrainIPVariableBase::getMicroDataToMacroProblem(double* val) const{
	const STensor43& dPdeF = this->getConstRefToEnhancedTangentModuli();
	const STensor3& localEF = this->getConstRefToLocalEnhancedDeformationGradient();
	const STensor43& DlocalEFDF = this->getConstRefToDLocalEnhancedDeformationGradientDDeformationGradient();
	const STensor43& dLocalEFDeF = this->getConstRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient();
	const STensor3& dIrreversibleEnergDeF = this->getConstRefToDIrreversibleEnergyDEnhancedDeformationGradient();
	
	int row = 0;
	for (int index=row; index< row+81; index++){
		int i,j,k,l;
		Tensor43::getIntsFromIndex(index-row,i,j,k,l);
		val[index] = dPdeF(i,j,k,l);
	}
	row+= 81;
	
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		val[index] = localEF(i,j);
	}
	row+= 9;
	
	for (int index=row; index< row+81; index++){
		int i,j,k,l;
		Tensor43::getIntsFromIndex(index-row,i,j,k,l);
		val[index] = DlocalEFDF(i,j,k,l);
	}
	row+= 81;
	
	for (int index=row; index< row+81; index++){
		int i,j,k,l;
		Tensor43::getIntsFromIndex(index-row,i,j,k,l);
		val[index] = dLocalEFDeF(i,j,k,l);
	}
	row+= 81;
	
	val[row] = _bandWidth;
	row+= 1;
	
	for (int index = row; index < row+3; index++){
		int i;
		Tensor13::getIntsFromIndex(index-row,i);
		val[index] = _localizationNormal[i];
	}
	row+=3;
	
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		val[index] = dIrreversibleEnergDeF(i,j);
	}
	row+= 9;
	
	
};
	// set the received data from microscopic analysis to microscopic analysis
void EnhancedStrainIPVariableBase::setReceivedMacroDataToMicroProblem(const double* val){
	STensor3& eF = this->getRefToEnhancedDeformationGradient();
	STensor3& Fbulk = this->getRefToBulkDeformationGradient();
	int row = 0;
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		eF(i,j) = val[index];
	}
	row+= 9;
	
	_betaElement = val[row];
	row += 1;
	
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		Fbulk(i,j) = val[index];
	}
	row+= 9;
};
	// set the received data from microscopic analysis to macroscopic analysis
void EnhancedStrainIPVariableBase::setReceivedMicroDataToMacroProblem(const double* val){
	STensor43& dPdeF = this->getRefToEnhancedTangentModuli();
	STensor3& localEF = this->getRefToLocalEnhancedDeformationGradient();
	STensor43& DlocalEFDF = this->getRefToDLocalEnhancedDeformationGradientDDeformationGradient();
	STensor43& dLocalEFDeF = this->getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient();
	STensor3& dIrreversibleEnergDeF = this->getRefToDIrreversibleEnergyDEnhancedDeformationGradient();
	double& bandWidth = this->getRefToBandWidth();
	SVector3& normal = this->getRefToLocalizationNormal();
	
	int row = 0;
	for (int index=row; index< row+81; index++){
		int i,j,k,l;
		Tensor43::getIntsFromIndex(index-row,i,j,k,l);
		dPdeF(i,j,k,l)= val[index];
	}
	row+= 81;
	
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		localEF(i,j) = val[index];
	}
	row+= 9;
	
	for (int index=row; index< row+81; index++){
		int i,j,k,l;
		Tensor43::getIntsFromIndex(index-row,i,j,k,l);
		DlocalEFDF(i,j,k,l) = val[index];
	}
	row+= 81;
	
	for (int index=row; index< row+81; index++){
		int i,j,k,l;
		Tensor43::getIntsFromIndex(index-row,i,j,k,l);
		dLocalEFDeF(i,j,k,l) = val[index];
	}
	row+= 81;
	
	_bandWidth = val[row];
	row+= 1;
	
	for (int index = row; index < row+3; index++){
		int i;
		Tensor13::getIntsFromIndex(index-row,i);
		normal[i] = val[index];
	}
	row+=3;
	
	for (int index=row; index < row+9; index ++){
		int i,j;
		Tensor23::getIntsFromIndex(index-row,i,j);
		dIrreversibleEnergDeF(i,j) = val[index];
	}
	row+= 9;
};

#endif //HAVE_MPI

dG3DEnhancedStrainIPVariable::dG3DEnhancedStrainIPVariable() :IPVariable2Enhanced<dG3DIPVariableBase,EnhancedStrainIPVariableBase>(){}
dG3DEnhancedStrainIPVariable::dG3DEnhancedStrainIPVariable(const dG3DEnhancedStrainIPVariable &source) :
																					IPVariable2Enhanced<dG3DIPVariableBase,EnhancedStrainIPVariableBase>(source){}
dG3DEnhancedStrainIPVariable& dG3DEnhancedStrainIPVariable::operator=(const IPVariable &source)
{
  IPVariable2Enhanced<dG3DIPVariableBase,EnhancedStrainIPVariableBase>::operator=(source);
  const dG3DEnhancedStrainIPVariable *src = dynamic_cast<const dG3DEnhancedStrainIPVariable*>(&source);
  if(src != NULL){
	
	}
  return *this;
}

void dG3DEnhancedStrainIPVariable::dG3DEnhancedStrainIPVariable::restart()
{
  IPVariable2Enhanced<dG3DIPVariableBase,EnhancedStrainIPVariableBase>::restart();
  return;
}

double dG3DEnhancedStrainIPVariable::get(const int comp) const{
	if (comp == IPField::EXTRAFIELD_1){
		int i,j;
		Tensor23::getIntsFromIndex(0,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_2){
		int i,j;
		Tensor23::getIntsFromIndex(1,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_3){
		int i,j;
		Tensor23::getIntsFromIndex(2,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_4){
		int i,j;
		Tensor23::getIntsFromIndex(3,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_5){
		int i,j;
		Tensor23::getIntsFromIndex(4,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_6){
		int i,j;
		Tensor23::getIntsFromIndex(5,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_7){
		int i,j;
		Tensor23::getIntsFromIndex(6,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_8){
		int i,j;
		Tensor23::getIntsFromIndex(7,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_9){
		int i,j;
		Tensor23::getIntsFromIndex(8,i,j);
		return getConstRefToEnhancedDeformationGradient()(i,j);
	}
	else if (comp == IPField::EXTRAFIELD_NORM){
		const STensor3& extraF = this->getConstRefToEnhancedDeformationGradient();
		return dot(extraF,extraF);
	}
	else
		return _ipvbulk->get(comp);
}