//
// C++ Interface: ipvariable
//
// Description: Class with definition of ipvarible for dG3D
//
//
// Author:  <V.D. Nguyen>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef DG3DENHANCEDSTRAINIPVARIABLE_H_
#define DG3DENHANCEDSTRAINIPVARIABLE_H_

#include "dG3DIPVariable.h"

class EnhancedStrainIPVariableBase : public IPVariable{
	protected:
		STensor3 _enhancedF;
		STensor43 _DPDenhanceF;

		STensor3 _localEnhancedF;
		STensor43 _DLocalEnhancedFDF;
		STensor43 _DLocalEnhancedFDEnhancedF;

		STensor3 _DirreversibleEnergyDEnhancedF;

		STensor3 _Fbulk;

		double _betaElement; // from mesh
		double _bandWidth; // from microsolver
		SVector3 _localizationNormal;

public:
		EnhancedStrainIPVariableBase(): IPVariable(), _enhancedF(0.),_DPDenhanceF(0.),_localEnhancedF(0.),_DLocalEnhancedFDF(0.),_DLocalEnhancedFDEnhancedF(0.),
			_betaElement(0.),_bandWidth(0.),_localizationNormal(0.),_Fbulk(1.),_DirreversibleEnergyDEnhancedF(0.){}
		EnhancedStrainIPVariableBase(const EnhancedStrainIPVariableBase& src): IPVariable(src),_enhancedF(src._enhancedF),_DPDenhanceF(src._DPDenhanceF),
			_localEnhancedF(src._localEnhancedF),_DLocalEnhancedFDF(src._DLocalEnhancedFDF),_DLocalEnhancedFDEnhancedF(src._DLocalEnhancedFDEnhancedF),
			_betaElement(src._betaElement),_bandWidth(src._bandWidth),_localizationNormal(src._localizationNormal),_Fbulk(src._Fbulk),
			_DirreversibleEnergyDEnhancedF(src._DirreversibleEnergyDEnhancedF){}

		virtual EnhancedStrainIPVariableBase& operator=(const IPVariable& src){
			IPVariable::operator =(src);
			const EnhancedStrainIPVariableBase* psrc = dynamic_cast<const EnhancedStrainIPVariableBase*>(&src);
			if (psrc != NULL){
				_enhancedF = psrc->_enhancedF;
				_DPDenhanceF = psrc->_DPDenhanceF;
				_localEnhancedF = psrc->_localEnhancedF;
				_DLocalEnhancedFDF = psrc->_DLocalEnhancedFDF;
				_DLocalEnhancedFDEnhancedF = psrc->_DLocalEnhancedFDEnhancedF;
				_DirreversibleEnergyDEnhancedF = psrc->_DirreversibleEnergyDEnhancedF;
				_betaElement = psrc->_betaElement;
				_bandWidth = psrc->_bandWidth;
				_localizationNormal = psrc->_localizationNormal;
				_Fbulk = psrc->_Fbulk;
			}
			return *this;
		};
		virtual ~EnhancedStrainIPVariableBase(){}

		virtual const double& getConstRefToBetaElement() const {return _betaElement;};
		virtual double& getRefToBetaElement() {return _betaElement;};

		virtual const double& getConstRefToBandWidth() const {return _bandWidth;};
		virtual double& getRefToBandWidth() {return _bandWidth;};

		virtual const SVector3& getConstRefToLocalizationNormal() const {return _localizationNormal;};
		virtual SVector3& getRefToLocalizationNormal() {return _localizationNormal;};

		virtual const STensor3& getConstRefToBulkDeformationGradient() const {return _Fbulk;};
		virtual STensor3& getRefToBulkDeformationGradient() {return _Fbulk;};

		virtual const STensor3& getConstRefToEnhancedDeformationGradient() const {return _enhancedF;};
		virtual STensor3& getRefToEnhancedDeformationGradient() {return _enhancedF;};

		virtual const STensor43& getConstRefToEnhancedTangentModuli() const {return _DPDenhanceF;};
		virtual STensor43& getRefToEnhancedTangentModuli() {return _DPDenhanceF;};

		virtual const STensor3& getConstRefToLocalEnhancedDeformationGradient() const {return _localEnhancedF;};
		virtual STensor3& getRefToLocalEnhancedDeformationGradient() {return _localEnhancedF;};

		virtual const STensor43& getConstRefToDLocalEnhancedDeformationGradientDDeformationGradient() const {return _DLocalEnhancedFDF;};
		virtual STensor43& getRefToDLocalEnhancedDeformationGradientDDeformationGradient() {return _DLocalEnhancedFDF;};

		virtual const STensor43& getConstRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() const {return _DLocalEnhancedFDEnhancedF;};
		virtual STensor43& getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() {return _DLocalEnhancedFDEnhancedF;};

		virtual const STensor3& getConstRefToDIrreversibleEnergyDEnhancedDeformationGradient() const {return _DirreversibleEnergyDEnhancedF;};
		virtual STensor3& getRefToDIrreversibleEnergyDEnhancedDeformationGradient() {return _DirreversibleEnergyDEnhancedF;};

		virtual void restart(){
			restartManager::restart(_enhancedF);
			restartManager::restart(_DPDenhanceF);
			restartManager::restart(_localEnhancedF);
			restartManager::restart(_DLocalEnhancedFDF);
			restartManager::restart(_DLocalEnhancedFDEnhancedF);
			restartManager::restart(_DirreversibleEnergyDEnhancedF);
			restartManager::restart(_betaElement);
			restartManager::restart(_bandWidth);
			restartManager::restart(_localizationNormal);
			restartManager::restart(_Fbulk);
		}
		virtual IPVariable* clone() const {return new EnhancedStrainIPVariableBase(*this);};

		#if defined(HAVE_MPI)
    virtual int getMacroNumberElementDataSendToMicroProblem() const;
      // get number of values obtained by microscopic analysis to send to macroscopic analysis
    virtual int getMicroNumberElementDataSendToMacroProblem() const;
      // get macroscopic kinematic data to send to microscopic problem
    virtual void getMacroDataSendToMicroProblem(double* val) const;
      // get computed data obtaind by microscopic analysis to send to macroscopic analysis
    virtual void getMicroDataToMacroProblem(double* val) const;
      // set the received data from microscopic analysis to microscopic analysis
    virtual void setReceivedMacroDataToMicroProblem(const double* val);
      // set the received data from microscopic analysis to macroscopic analysis
    virtual void setReceivedMicroDataToMacroProblem(const double* val);
    #endif
};


class dG3DEnhancedStrainIPVariable : public IPVariable2Enhanced<dG3DIPVariableBase,EnhancedStrainIPVariableBase>{
	public:
		dG3DEnhancedStrainIPVariable();
		dG3DEnhancedStrainIPVariable(const dG3DEnhancedStrainIPVariable &source);
		virtual dG3DEnhancedStrainIPVariable& operator=(const IPVariable &source);
		virtual ~dG3DEnhancedStrainIPVariable(){}

		virtual double get(const int i) const;

		virtual IPVariable* getInternalData() {return _ipvbulk->getInternalData();};
    virtual const IPVariable* getInternalData()  const {return _ipvbulk->getInternalData();};

	// access to bulk
		virtual const bool isInterface() const{return _ipvbulk->isInterface();}
		virtual double vonMises() const{return _ipvbulk->vonMises();}
		virtual double defoEnergy() const{return _ipvbulk->defoEnergy();}
		virtual double plasticEnergy() const{return _ipvbulk->plasticEnergy();}
    virtual double damageEnergy() const {return _ipvbulk->damageEnergy();};
		virtual int fractureEnergy(double* arrayEnergy) const{return _ipvbulk->fractureEnergy(arrayEnergy);}

		virtual void getCauchyStress(STensor3 &cauchy) const { return _ipvbulk->getCauchyStress(cauchy); }
		virtual double getJ() const { return _ipvbulk->getJ(); };

		virtual bool dissipationIsActive() const {return _ipvbulk->dissipationIsActive();};

		virtual void blockDissipation(const bool fl){_ipvbulk->blockDissipation(fl);};
		virtual bool dissipationIsBlocked() const {return _ipvbulk->dissipationIsBlocked();};

		virtual const STensor3 &getConstRefToDeformationGradient() const {return _ipvbulk->getConstRefToDeformationGradient();}
		virtual STensor3 &getRefToDeformationGradient() {return _ipvbulk->getRefToDeformationGradient();}

		virtual double getBarJ() const {return _ipvbulk->getBarJ();}
		virtual double &getRefToBarJ() {return _ipvbulk->getRefToBarJ();}
		virtual double getLocalJ() const {return _ipvbulk->getLocalJ();}
		virtual double &getRefToLocalJ() {return _ipvbulk->getRefToLocalJ();}
                virtual void getBackLocalDeformationGradient(STensor3& localF) const 
                   {_ipvbulk->getBackLocalDeformationGradient(localF);};

		virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStress() const {return _ipvbulk->getConstRefToFirstPiolaKirchhoffStress();}
		virtual STensor3 &getRefToFirstPiolaKirchhoffStress() {return _ipvbulk->getRefToFirstPiolaKirchhoffStress();}

		virtual const STensor43 &getConstRefToTangentModuli() const {return _ipvbulk->getConstRefToTangentModuli();}
		virtual STensor43 &getRefToTangentModuli() {return _ipvbulk->getRefToTangentModuli();}

     virtual const STensor43 &getConstRefToElasticTangentModuli() const {return _ipvbulk->getConstRefToElasticTangentModuli();};
    virtual STensor43 &getRefToElasticTangentModuli() {return _ipvbulk->getRefToElasticTangentModuli();};

		virtual const STensor43 &getConstRefToDGElasticTangentModuli() const {return _ipvbulk->getConstRefToDGElasticTangentModuli();}
		virtual void setRefToDGElasticTangentModuli(const STensor43 &eT) {return _ipvbulk->setRefToDGElasticTangentModuli(eT);}

		virtual const SVector3 &getConstRefToReferenceOutwardNormal() const {return _ipvbulk->getConstRefToReferenceOutwardNormal();}
		virtual SVector3 &getRefToReferenceOutwardNormal() {return _ipvbulk->getRefToReferenceOutwardNormal();}

		virtual const SVector3 &getConstRefToCurrentOutwardNormal() const {return _ipvbulk->getConstRefToCurrentOutwardNormal();}
		virtual SVector3 &getRefToCurrentOutwardNormal() {return _ipvbulk->getRefToCurrentOutwardNormal();}

		virtual const SVector3 &getConstRefToJump() const {return _ipvbulk->getConstRefToJump();}
		virtual SVector3 &getRefToJump() {return _ipvbulk->getRefToJump();}

		virtual const SVector3& getConstRefToIncompatibleJump() const {return _ipvbulk->getConstRefToIncompatibleJump();};
		virtual SVector3& getRefToIncompatibleJump() {return _ipvbulk->getRefToIncompatibleJump();};

		 // non local
		 virtual int getNumberNonLocalVariable() const {return _ipvbulk->getNumberNonLocalVariable();};
    //
    virtual double getConstRefToLocalVariable(const int idex) const {return _ipvbulk->getConstRefToLocalVariable(idex);};
    virtual double& getRefToLocalVariable(const int idex) {return _ipvbulk->getRefToLocalVariable(idex);};
    //
    virtual double getConstRefToNonLocalVariable(const int idex) const {return _ipvbulk->getConstRefToNonLocalVariable(idex);};
    virtual double& getRefToNonLocalVariable(const int idex) {return _ipvbulk->getRefToNonLocalVariable(idex);};

    virtual const std::vector<STensor3> &getConstRefToDLocalVariableDStrain() const {return _ipvbulk->getConstRefToDLocalVariableDStrain();};
    virtual std::vector<STensor3> &getRefToDLocalVariableDStrain() {return _ipvbulk->getRefToDLocalVariableDStrain();};

    virtual const std::vector<STensor3> &getConstRefToDStressDNonLocalVariable() const {return _ipvbulk->getConstRefToDStressDNonLocalVariable();};
    virtual std::vector<STensor3> &getRefToDStressDNonLocalVariable() {return _ipvbulk->getRefToDStressDNonLocalVariable();};

    virtual const fullMatrix<double> &getConstRefToDLocalVariableDNonLocalVariable() const {return _ipvbulk->getConstRefToDLocalVariableDNonLocalVariable();};
    virtual fullMatrix<double> &getRefToDLocalVariableDNonLocalVariable() {return _ipvbulk->getRefToDLocalVariableDNonLocalVariable();};

    virtual const fullVector<double> &getConstRefToNonLocalJump() const {return _ipvbulk->getConstRefToNonLocalJump(); }
    virtual fullVector<double> &getRefToNonLocalJump() {return _ipvbulk->getRefToNonLocalJump(); }

    virtual const std::vector<SVector3> &getConstRefToGradNonLocalVariable() const {return _ipvbulk->getConstRefToGradNonLocalVariable();}
    virtual std::vector<SVector3> &getRefToGradNonLocalVariable() {return _ipvbulk->getRefToGradNonLocalVariable();}

    virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idx) const {return _ipvbulk->getConstRefToCharacteristicLengthMatrix(idx);};
    //coupling non local with constitutiveExtraDof
    virtual const fullMatrix<double> & getConstRefTodLocalVariableDExtraDofDiffusionField() const {return _ipvbulk->getConstRefTodLocalVariableDExtraDofDiffusionField();}
    virtual fullMatrix<double> & getRefTodLocalVariableDExtraDofDiffusionField(){return _ipvbulk->getRefTodLocalVariableDExtraDofDiffusionField();}

    // extra dofs
    virtual int getNumConstitutiveExtraDofDiffusionVariable() const {return _ipvbulk->getNumConstitutiveExtraDofDiffusionVariable();};
    virtual double getConstRefToField(const int idex) const {return _ipvbulk->getConstRefToField(idex);};
    virtual double&getRefToField(const int idex) {return _ipvbulk->getRefToField(idex);};
    virtual const std::vector<SVector3>  &getConstRefToGradField() const {return _ipvbulk->getConstRefToGradField();};
    virtual std::vector<SVector3>  &getRefToGradField() {return _ipvbulk->getRefToGradField();};
    virtual const std::vector<SVector3>  &getConstRefToFlux() const {return _ipvbulk->getConstRefToFlux();};
    virtual std::vector<SVector3>  &getRefToFlux() {return _ipvbulk->getRefToFlux();};
    virtual const std::vector<STensor3>  &getConstRefTodPdField() const {return _ipvbulk->getConstRefTodPdField();};
    virtual std::vector<STensor3>  &getRefTodPdField() {return _ipvbulk->getRefTodPdField();};
    virtual const std::vector<STensor33>  &getConstRefTodPdGradField() const {return _ipvbulk->getConstRefTodPdGradField();};
    virtual std::vector<STensor33>  &getRefTodPdGradField() {return _ipvbulk->getRefTodPdGradField();};
    virtual const std::vector<std::vector<STensor3> >  &getConstRefTodFluxdGradField() const {return _ipvbulk->getConstRefTodFluxdGradField();};
    virtual std::vector<std::vector<STensor3> >  &getRefTodFluxdGradField() {return _ipvbulk->getRefTodFluxdGradField();};
    virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFluxdField() const {return _ipvbulk->getConstRefTodFluxdField();};
    virtual std::vector<std::vector<SVector3> >  &getRefTodFluxdField() {return _ipvbulk->getRefTodFluxdField();};
    virtual const std::vector<STensor33>  &getConstRefTodFluxdF() const {return _ipvbulk->getConstRefTodFluxdF();};
    virtual std::vector<STensor33>   &getRefTodFluxdF() {return _ipvbulk->getRefTodFluxdF();};  
    virtual const std::vector<std::vector<const STensor3*> > &getConstRefToLinearK() const {return _ipvbulk->getConstRefToLinearK();};
    virtual void setConstRefToLinearK(const STensor3 &eT, int i, int j)  {_ipvbulk->setConstRefToLinearK(eT, i, j);};
    virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodLinearKdField() const {return _ipvbulk->getConstRefTodLinearKdField();}; 
    virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodLinearKdField() {return _ipvbulk->getRefTodLinearKdField();}; 
    virtual const std::vector<std::vector<STensor43>>  &getConstRefTodLinearKdF() const {return _ipvbulk->getConstRefTodLinearKdF();}; 
    virtual std::vector<std::vector<STensor43>>  &getRefTodLinearKdF() {return _ipvbulk->getRefTodLinearKdF();}; 
/*    virtual const std::vector<std::vector<const STensor3*> > &getConstRefToEnergyK() const {return _ipvbulk->getConstRefToEnergyK();};
    virtual void setConstRefToEnergyK(const STensor3 &eT, int i, int j)  {_ipvbulk->setConstRefToEnergyK(eT, i, j);};
    virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodEnergyKdField() const {return _ipvbulk->getConstRefTodEnergyKdField();}; 
    virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodEnergyKdField() {return _ipvbulk->getRefTodEnergyKdField();}; 
    virtual const std::vector<std::vector<STensor43>>  &getConstRefTodEnergyKdF() const {return _ipvbulk->getConstRefTodEnergyKdF();}; 
    virtual std::vector<std::vector<STensor43>>  &getRefTodEnergyKdF() {return _ipvbulk->getRefTodEnergyKdF();};  */
    virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodFluxdGradFielddField() const {return _ipvbulk->getConstRefTodFluxdGradFielddField();}; 
    virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodFluxdGradFielddField() {return _ipvbulk->getRefTodFluxdGradFielddField();}; 
    virtual const std::vector<std::vector<STensor43>>  &getConstRefTodFluxdGradFielddF() const {return _ipvbulk->getConstRefTodFluxdGradFielddF();}; 
    virtual std::vector<std::vector<STensor43>>  &getRefTodFluxdGradFielddF() {return _ipvbulk->getRefTodFluxdGradFielddF();};  
    virtual const fullVector<double>  &getConstRefToFieldSource() const {return _ipvbulk->getConstRefToFieldSource();};
    virtual fullVector<double>  &getRefToFieldSource() {return _ipvbulk->getRefToFieldSource();};
    virtual const fullMatrix<double>  &getConstRefTodFieldSourcedField() const {return _ipvbulk->getConstRefTodFieldSourcedField();};
    virtual fullMatrix<double>  &getRefTodFieldSourcedField() {return _ipvbulk->getRefTodFieldSourcedField();};
    virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFieldSourcedGradField() const {return _ipvbulk->getConstRefTodFieldSourcedGradField();};
    virtual std::vector<std::vector<SVector3> >  &getRefTodFieldSourcedGradField() {return _ipvbulk->getRefTodFieldSourcedGradField();};
    virtual const std::vector<STensor3>  &getConstRefTodFieldSourcedF() const {return _ipvbulk->getConstRefTodFieldSourcedF();};
    virtual std::vector<STensor3>  &getRefTodFieldSourcedF() {return _ipvbulk->getRefTodFieldSourcedF();};
    virtual const fullVector<double>  &getConstRefToMechanicalSource() const {return _ipvbulk->getConstRefToMechanicalSource();};
    virtual fullVector<double>  &getRefToMechanicalSource() {return _ipvbulk->getRefToMechanicalSource();};
    virtual const fullMatrix<double>  &getConstRefTodMechanicalSourcedField() const {return _ipvbulk->getConstRefTodMechanicalSourcedField();};
    virtual fullMatrix<double>  &getRefTodMechanicalSourcedField() {return _ipvbulk->getRefTodMechanicalSourcedField();};
    virtual const std::vector<std::vector<SVector3> >  &getConstRefTodMechanicalSourcedGradField() const {return _ipvbulk->getConstRefTodMechanicalSourcedGradField();};
    virtual std::vector<std::vector<SVector3> >  &getRefTodMechanicalSourcedGradField() {return _ipvbulk->getRefTodMechanicalSourcedGradField();};
    virtual const std::vector<STensor3>  &getConstRefTodMechanicalSourcedF() const {return _ipvbulk->getConstRefTodMechanicalSourcedF();};
    virtual std::vector<STensor3>  &getRefTodMechanicalSourcedF() {return _ipvbulk->getRefTodMechanicalSourcedF();};
    virtual const fullVector<double>  &getConstRefToExtraDofFieldCapacityPerUnitField() const {return _ipvbulk->getConstRefToExtraDofFieldCapacityPerUnitField();};
    virtual fullVector<double>  &getRefToExtraDofFieldCapacityPerUnitField() {return _ipvbulk->getRefToExtraDofFieldCapacityPerUnitField();};
    virtual const fullVector<double>  &getConstRefToFieldJump() const {return _ipvbulk->getConstRefToFieldJump();};
    virtual fullVector<double>  &getRefToFieldJump() {return _ipvbulk->getRefToFieldJump();};
    virtual const fullVector<double>  &getConstRefToOneOverFieldJump() const {return _ipvbulk->getConstRefToOneOverFieldJump();};
    virtual fullVector<double>  &getRefToOneOverFieldJump() {return _ipvbulk->getRefToOneOverFieldJump();};
    virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldm() const {return _ipvbulk->getConstRefTodOneOverFieldJumpdFieldm();};
    virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldm() {return _ipvbulk->getRefTodOneOverFieldJumpdFieldm() ;};
    virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldp() const {return _ipvbulk->getConstRefTodOneOverFieldJumpdFieldp();};
    virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldp() {return _ipvbulk->getRefTodOneOverFieldJumpdFieldp();};
    virtual const std::vector<SVector3>  &getConstRefToInterfaceFlux() const {return _ipvbulk->getConstRefToInterfaceFlux();};
    virtual std::vector<SVector3>  &getRefToInterfaceFlux() {return _ipvbulk->getRefToInterfaceFlux();};
    //coupling constitutive extra dof with nonLocal
    virtual const std::vector<std::vector<SVector3> > & getConstRefTodFluxdNonLocalVariable() const {return _ipvbulk->getConstRefTodFluxdNonLocalVariable();}
    virtual std::vector<std::vector<SVector3> > & getRefTodFluxdNonLocalVariable(){return _ipvbulk->getRefTodFluxdNonLocalVariable();}
    virtual const fullMatrix<double> &getConstRefTodFieldSourcedNonLocalVariable() const{return _ipvbulk->getConstRefTodFieldSourcedNonLocalVariable();}
    virtual const fullMatrix<double> &getRefTodFieldSourcedNonLocalVariable(){return _ipvbulk->getRefTodFieldSourcedNonLocalVariable();}
    virtual const fullMatrix<double> &getConstRefTodMechanicalSourcedNonLocalVariable() const{return _ipvbulk->getConstRefTodMechanicalSourcedNonLocalVariable();}
    virtual fullMatrix<double> &getRefTodMechanicalSourcedNonLocalVariable(){return _ipvbulk->getRefTodMechanicalSourcedNonLocalVariable();}
    virtual const std::vector<const STensor3*> &getConstRefToLinearSymmetrizationCoupling() const  {return _ipvbulk->getConstRefToLinearSymmetrizationCoupling();};
    virtual void setConstRefToLinearSymmetrizationCoupling(const STensor3 &eT, int i)  {_ipvbulk->setConstRefToLinearSymmetrizationCoupling(eT,i);};
  
  //Energy Conjugated Field (fT & fv)
    virtual double getConstRefToEnergyConjugatedField(const int idex) const {return _ipvbulk->getConstRefToEnergyConjugatedField(idex);};  
    virtual double  &getRefToEnergyConjugatedField(const int idex) {return _ipvbulk->getRefToEnergyConjugatedField(idex);};  
    virtual const std::vector<SVector3>  &getConstRefToGradEnergyConjugatedField() const {return _ipvbulk->getConstRefToGradEnergyConjugatedField();}; 
    virtual std::vector<SVector3>  &getRefToGradEnergyConjugatedField() {return _ipvbulk->getRefToGradEnergyConjugatedField();}; 
    virtual const  std::vector<std::vector<double>>  &getConstRefTodFielddEnergyConjugatedField() const {return _ipvbulk->getConstRefTodFielddEnergyConjugatedField();};
    virtual std::vector<std::vector<double>>  &getRefTodFielddEnergyConjugatedField() {return _ipvbulk->getRefTodFielddEnergyConjugatedField();};
    virtual const std::vector<std::vector<SVector3>>  &getConstRefTodGradFielddEnergyConjugatedField() const {return _ipvbulk->getConstRefTodGradFielddEnergyConjugatedField();}; 
    virtual std::vector<std::vector<SVector3>>  &getRefTodGradFielddEnergyConjugatedField() {return _ipvbulk->getRefTodGradFielddEnergyConjugatedField();};
    virtual const std::vector<std::vector<STensor3>>  &getConstRefTodGradFielddGradEnergyConjugatedField() const {return _ipvbulk->getConstRefTodGradFielddGradEnergyConjugatedField();}; 
    virtual std::vector<std::vector<STensor3>>  &getRefTodGradFielddGradEnergyConjugatedField() {return _ipvbulk->getRefTodGradFielddGradEnergyConjugatedField();}; 
    virtual const fullVector<double>  &getConstRefToEnergyConjugatedFieldJump() const {return _ipvbulk->getConstRefToEnergyConjugatedFieldJump();};  
    virtual fullVector<double>  &getRefToEnergyConjugatedFieldJump() {return _ipvbulk->getRefToEnergyConjugatedFieldJump();};
    virtual const  std::vector<std::vector<double>>  &getConstRefTodEnergyConjugatedFieldJumpdFieldm() const {return _ipvbulk->getConstRefTodEnergyConjugatedFieldJumpdFieldm();};
    virtual std::vector<std::vector<double>>  &getRefTodEnergyConjugatedFieldJumpdFieldm() {return _ipvbulk->getRefTodEnergyConjugatedFieldJumpdFieldm();};
    virtual const  std::vector<std::vector<double>>  &getConstRefTodEnergyConjugatedFieldJumpdFieldp() const {return _ipvbulk->getConstRefTodEnergyConjugatedFieldJumpdFieldp();};
    virtual std::vector<std::vector<double>>  &getRefTodEnergyConjugatedFieldJumpdFieldp() {return _ipvbulk->getRefTodEnergyConjugatedFieldJumpdFieldp();};


		//------------------get 1st piola b4 AV ---------------//

		virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStressb4AV() const {return _ipvbulk->getConstRefToFirstPiolaKirchhoffStressb4AV();}
		virtual STensor3 &getRefToFirstPiolaKirchhoffStressb4AV() {return _ipvbulk->getRefToFirstPiolaKirchhoffStressb4AV();}


		virtual void setBulkCriticalDamage(const double DT){_ipvbulk->setBulkCriticalDamage(DT);};
		virtual double getBulkCriticalDamage() const {return _ipvbulk->getBulkCriticalDamage();};

		virtual void setNonLocalToLocal(const bool fl){_ipvbulk->setNonLocalToLocal(fl);};
		virtual bool getNonLocalToLocal() const {return _ipvbulk->getNonLocalToLocal();};

		// for path following based on  irreversibe energt
		virtual double irreversibleEnergy() const  {return _ipvbulk->irreversibleEnergy();};
		virtual double& getRefToIrreversibleEnergy() {return _ipvbulk->getRefToIrreversibleEnergy();};
		// to avoid duplicate allocation
		virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _ipvbulk->getConstRefToDIrreversibleEnergyDDeformationGradient();};
		virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _ipvbulk->getRefToDIrreversibleEnergyDDeformationGradient();};

		// irreversible ennergy depend on nonlocal variale
		virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const {return _ipvbulk->getConstRefToDIrreversibleEnergyDNonLocalVariable(index);};
		virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) {return _ipvbulk->getRefToDIrreversibleEnergyDDNonLocalVariable(index);};



		// enhance function
		virtual const STensor3& getConstRefToEnhancedDeformationGradient() const {return _ipvfrac->getConstRefToEnhancedDeformationGradient();};
		virtual STensor3& getRefToEnhancedDeformationGradient() {return _ipvfrac->getRefToEnhancedDeformationGradient();};

		virtual const STensor43& getConstRefToEnhancedTangentModuli() const {return _ipvfrac->getConstRefToEnhancedTangentModuli();};
		virtual STensor43& getRefToEnhancedTangentModuli() {return _ipvfrac->getRefToEnhancedTangentModuli();};

		virtual const STensor3& getConstRefToLocalEnhancedDeformationGradient() const {return _ipvfrac->getConstRefToLocalEnhancedDeformationGradient();};
		virtual STensor3& getRefToLocalEnhancedDeformationGradient() {return _ipvfrac->getRefToLocalEnhancedDeformationGradient();};

		virtual const STensor43& getConstRefToDLocalEnhancedDeformationGradientDDeformationGradient() const {return _ipvfrac->getConstRefToDLocalEnhancedDeformationGradientDDeformationGradient();};
		virtual STensor43& getRefToDLocalEnhancedDeformationGradientDDeformationGradient() {return _ipvfrac->getRefToDLocalEnhancedDeformationGradientDDeformationGradient();};

		virtual const STensor43& getConstRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() const {return _ipvfrac->getConstRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient();};
		virtual STensor43& getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() {return _ipvfrac->getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient();};

		virtual const STensor3& getConstRefToDIrreversibleEnergyDEnhancedDeformationGradient() const {return _ipvfrac->getConstRefToDIrreversibleEnergyDEnhancedDeformationGradient();};
		virtual STensor3& getRefToDIrreversibleEnergyDEnhancedDeformationGradient() {return _ipvfrac->getRefToDIrreversibleEnergyDEnhancedDeformationGradient();};

		virtual const double& getConstRefToBetaElement() const {return _ipvfrac->getConstRefToBetaElement();};
		virtual double& getRefToBetaElement() {return _ipvfrac->getRefToBetaElement();};

		virtual const double& getConstRefToBandWidth() const {return _ipvfrac->getConstRefToBandWidth();};
		virtual double& getRefToBandWidth() {return _ipvfrac->getRefToBandWidth();};

		virtual const SVector3& getConstRefToLocalizationNormal() const {return _ipvfrac->getConstRefToLocalizationNormal();};
		virtual SVector3& getRefToLocalizationNormal() {return _ipvfrac->getRefToLocalizationNormal();};

		virtual const STensor3& getConstRefToBulkDeformationGradient() const {return _ipvfrac->getConstRefToBulkDeformationGradient();};
		virtual STensor3& getRefToBulkDeformationGradient() {return _ipvfrac->getRefToBulkDeformationGradient();};

		virtual IPVariable* clone() const {return new dG3DEnhancedStrainIPVariable(*this);};
		virtual void restart();
};





#endif // DG3DENHANCEDSTRAINIPVARIABLE_H_
