//
//
// Description: Class of materials for non linear dg
//
//
// Author:  <V.D. NGUYEN>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "dG3DEnhancedStrainMaterialLaw.h"
#include "dG3DEnhancedStrainIPVariable.h"
#include "embeddedLocalizationBand.h"
#include "dG3DMultiscaleIPVariable.h"
#include "nonLinearMechSolver.h"

enhancedStrainDG3DLinearElasticMaterialLaw::enhancedStrainDG3DLinearElasticMaterialLaw(const int num, const double rho):
	dG3DLinearElasticMaterialLaw(num,rho){}
enhancedStrainDG3DLinearElasticMaterialLaw::enhancedStrainDG3DLinearElasticMaterialLaw(const int num, const double rho, const double E, const double nu):
	dG3DLinearElasticMaterialLaw(num,rho,E,nu)
{
}
enhancedStrainDG3DLinearElasticMaterialLaw::enhancedStrainDG3DLinearElasticMaterialLaw(const enhancedStrainDG3DLinearElasticMaterialLaw& src):dG3DLinearElasticMaterialLaw(src){

};


void enhancedStrainDG3DLinearElasticMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;
  IPVariable* ipvi = new  dG3DEnhancedStrainIPVariable();
  IPVariable* ipv1 = new  dG3DEnhancedStrainIPVariable();
  IPVariable* ipv2 = new  dG3DEnhancedStrainIPVariable();

	dG3DEnhancedStrainIPVariable *ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipvi);
	IPVariable* ipvBulk = new LinearElasticDG3DIPVariable(inter);
	IPVariable* ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);

	ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipv1);
	ipvBulk = new LinearElasticDG3DIPVariable(inter);
	ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);

	ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipv2);
	ipvBulk = new LinearElasticDG3DIPVariable(inter);
	ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);

  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
}

void enhancedStrainDG3DLinearElasticMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }
  ipv = new  dG3DEnhancedStrainIPVariable();
	dG3DEnhancedStrainIPVariable *ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipv);
	IPVariable* ipvBulk = new LinearElasticDG3DIPVariable(inter);
	IPVariable* ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);
}


void enhancedStrainDG3DLinearElasticMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  dG3DEnhancedStrainIPVariable* ipvcur = static_cast<dG3DEnhancedStrainIPVariable*>(ipv);;
  const dG3DEnhancedStrainIPVariable* ipvprev = static_cast<const dG3DEnhancedStrainIPVariable*>(ipvp);;
  /* strain xyz */
  const STensor3& F = ipvcur->getConstRefToDeformationGradient();
  STensor3& P = ipvcur->getRefToFirstPiolaKirchhoffStress();

  static STensor3 Ftotal;
  Ftotal=(F);
  static STensor43 TangentTotal;
  TangentTotal*=(0.);
  Ftotal += ipvcur->getConstRefToEnhancedDeformationGradient();

  _elasticLaw->constitutive(Ftotal,P,stiff,&TangentTotal);


  ipvcur->getRefToTangentModuli() = TangentTotal;
  ipvcur->getRefToEnhancedTangentModuli() = TangentTotal;

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);
}


void enhancedStraindG3DMultiscaleMaterialLaw::fillElasticStiffness(double & rho, STensor43& L){
  printf("get elastic properties\n");
  this->assignMeshId(0,0);
  nonLinearMechSolver* solver = this->createMicroSolver(0,0);
	solver->tangentAveragingFlag(true);
	solver->setExtractCohesiveLawFromMicroDamage(false);
	solver->setExtractIrreversibleEnergyFlag(false);
	solver->initMicroSolver();

	L = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_F_F();
  rho = solver->getHomogenizedDensity();
  delete solver;
};

enhancedStraindG3DMultiscaleMaterialLaw::enhancedStraindG3DMultiscaleMaterialLaw(const int lnum, const int tag)
                            : dG3DMaterialLaw(lnum,0,false),numericalMaterial(tag),
														_lostSolutionUniquenssTolerance(0.),_RVELength(0.),_surfaceReductionRatio(0.){};

void enhancedStraindG3DMultiscaleMaterialLaw::initLaws(const std::map<int,materialLaw*> &maplaw){
  if (!_initialized){
    this->fillElasticStiffness(_rho,elasticStiffness);
    _initialized = true;
  }
};

void enhancedStraindG3DMultiscaleMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const{
  this->createIPState(true,ips,state_,ele,nbFF_,GP,gpt);
};

void enhancedStraindG3DMultiscaleMaterialLaw::createIPState(const bool isSolve, IPStateBase* &ips,const bool* state,
                                 const MElement *ele, const int nbFF, const IntPt *GP, const int gpt) const{

	IPVariable* ipvi = new  dG3DEnhancedStrainIPVariable();
  IPVariable* ipv1 = new  dG3DEnhancedStrainIPVariable();
  IPVariable* ipv2 = new  dG3DEnhancedStrainIPVariable();

	bool oninter = true;
	const MInterfaceElement* iele = dynamic_cast<const MInterfaceElement*>(ele);
	if (iele == NULL) oninter = false;

	dG3DEnhancedStrainIPVariable *ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipvi);
	IPVariable* ipvBulk = new dG3DMultiscaleIPVariable(oninter);
	IPVariable* ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);

	ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipv1);
	ipvBulk = new dG3DMultiscaleIPVariable(oninter);
	ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);

	ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipv2);
	ipvBulk = new dG3DMultiscaleIPVariable(oninter);
	ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);


	if(ips != NULL) delete ips;
	ips = new IP3State(state,ipvi,ipv1,ipv2);

  if (isSolve){
    int el = ele->getNum();
		nonLinearMechSolver* solver =  this->createMicroSolver(el,gpt);
		solver->setExtractCohesiveLawFromMicroDamage(true); // always extract cohesive law
		bool withNormal= false;
    solver->setCheckFailureOnset(true,withNormal);
    solver->setRVELengthInNormalDirection(_RVELength,_surfaceReductionRatio);
		solver->initMicroSolver();
		ips->setSolver(solver);
  }
};

void enhancedStraindG3DMultiscaleMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const{
  bool oninter = true;
  const MInterfaceElement* iele = dynamic_cast<const MInterfaceElement*>(ele);
  if (iele == NULL) oninter = false;
  ipv = new dG3DEnhancedStrainIPVariable();
	dG3DEnhancedStrainIPVariable *ipvf = static_cast<dG3DEnhancedStrainIPVariable*>(ipv);
	IPVariable* ipvBulk = new dG3DMultiscaleIPVariable(oninter);
	IPVariable* ipvExtra= new EnhancedStrainIPVariableBase();
	ipvf->setIPvBulk(ipvBulk);
	ipvf->setIPvEnhanced(ipvExtra);
};

void enhancedStraindG3DMultiscaleMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const{
  dG3DEnhancedStrainIPVariable* eipv = static_cast<dG3DEnhancedStrainIPVariable*>(ipv);
	const dG3DEnhancedStrainIPVariable* eipvprev = static_cast<const dG3DEnhancedStrainIPVariable*>(ipvprev);

	dG3DMultiscaleIPVariable* mipv = static_cast<dG3DMultiscaleIPVariable*>(eipv->getIPvBulk());
	nonLinearMechSolver* solver = mipv->getSolver();

	// check failure
	solver->setLostSolutionUniquenssTolerance(_lostSolutionUniquenssTolerance);
	solver->checkFailureOnset();
	mipv->brokenSolver(solver->solverIsBroken());
};

void enhancedStraindG3DMultiscaleMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff, const bool checkfrac){

	dG3DEnhancedStrainIPVariable* eipv = static_cast<dG3DEnhancedStrainIPVariable*>(ipv);
	const dG3DEnhancedStrainIPVariable* eipvprev = static_cast<const dG3DEnhancedStrainIPVariable*>(ipvprev);

	dG3DMultiscaleIPVariable* mipv = static_cast<dG3DMultiscaleIPVariable*>(eipv->getIPvBulk());
	nonLinearMechSolver* solver = mipv->getSolver();

	const STensor3& Fbulk = eipv->getConstRefToBulkDeformationGradient();
	const STensor3& Fbulkprev= eipvprev->getConstRefToBulkDeformationGradient();

	const double& betaE = eipvprev->getConstRefToBetaElement(); // avoid derivatives
	const STensor3& enhancF = eipv->getConstRefToEnhancedDeformationGradient();
	const STensor3& enhancFprev = eipvprev->getConstRefToEnhancedDeformationGradient();

	if ((!solver->solverIsBroken()) or mipv->isInterface()){
		// set kinematics input
		STensor3& F = eipv->getRefToDeformationGradient();
		F = Fbulk; // F bulk only
		F += enhancF;

		solver->getMicroBC()->setDeformationGradient(F);
		// set tangent averaging flag
		solver->tangentAveragingFlag(stiff);
		// solve micro problem
		double time = solver->microSolve();
		// get homogenized properties form solved micro problem
		eipv->getRefToFirstPiolaKirchhoffStress() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedStress();
		mipv->getRefToDefoEnergy() = solver->getHomogenizationState(IPStateBase::current)->getDeformationEnergy();
		mipv->getRefToPlasticEnergy() = solver->getHomogenizationState(IPStateBase::current)->getPlasticEnergy();
		eipv->getRefToIrreversibleEnergy() = solver->getHomogenizationState(IPStateBase::current)->getIrreversibleEnergy();


		eipv->getRefToLocalEnhancedDeformationGradient() *= 0.;

		if (stiff){
			eipv->getRefToTangentModuli() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_F_F();
			eipv->getRefToDIrreversibleEnergyDDeformationGradient() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_IrreversibleEnergy_F();
			eipv->getRefToDIrreversibleEnergyDEnhancedDeformationGradient() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_IrreversibleEnergy_F();

			eipv->getRefToEnhancedTangentModuli() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_F_F();
			eipv->getRefToDLocalEnhancedDeformationGradientDDeformationGradient() *= 0.;
			eipv->getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() *= 0.;
		};
	}
	else{
		double beta = solver->getHomogenizationState(IPStateBase::previous)->getActiveDissipationVolume()/solver->getRVEVolume();

		//printf("computing in failure state beta =  %e betE = %e  rve vomume = %e \n",beta,betaE,solver->getRVEVolume());

		STensor3& F = eipv->getRefToDeformationGradient();
		F = eipvprev->getConstRefToDeformationGradient();

		double factF = 0.;
		double factEnhanceF = 1.;

		if (betaE <= 0.999999){
			factF = (1.-beta)/(1.-betaE);
			factEnhanceF = (beta-betaE)/(1.-betaE);
		}
		else{
			printf(" element embeded in localization band beta = %e betaE = %e \n",beta,betaE);
			factF = 0.;
			factEnhanceF = 1.;
		}
		for (int i=0; i<3; i++){
			for (int j=0; j<3; j++){
				F(i,j) += factF*(Fbulk(i,j) - Fbulkprev(i,j)) + factEnhanceF*(enhancF(i,j) - enhancFprev(i,j));
			}
		}

		solver->getMicroBC()->setDeformationGradient(F);
		// set tangent averaging flag
		solver->tangentAveragingFlag(stiff);
		// solve micro problem
		double time = solver->microSolve();
		// get homogenized properties form solved micro problem
		eipv->getRefToFirstPiolaKirchhoffStress() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedStress();
		mipv->getRefToDefoEnergy() = solver->getHomogenizationState(IPStateBase::current)->getDeformationEnergy();
		mipv->getRefToPlasticEnergy() = solver->getHomogenizationState(IPStateBase::current)->getPlasticEnergy();
		eipv->getRefToIrreversibleEnergy() = solver->getHomogenizationState(IPStateBase::current)->getIrreversibleEnergy();

		eipv->getRefToLocalEnhancedDeformationGradient() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedActiveDissipationDeformationGradient();
		eipv->getRefToLocalEnhancedDeformationGradient() -= solver->getFdamOnset();
		if (stiff){
			eipv->getRefToTangentModuli() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_F_F();
			eipv->getRefToTangentModuli() *= factF;

			eipv->getRefToDIrreversibleEnergyDDeformationGradient() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_IrreversibleEnergy_F();
			eipv->getRefToDIrreversibleEnergyDDeformationGradient()*= factF;

			eipv->getRefToDIrreversibleEnergyDEnhancedDeformationGradient() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_IrreversibleEnergy_F();
			eipv->getRefToDIrreversibleEnergyDEnhancedDeformationGradient() *= factEnhanceF;

			eipv->getRefToEnhancedTangentModuli() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_F_F();
			eipv->getRefToEnhancedTangentModuli() *= factEnhanceF;

			eipv->getRefToDLocalEnhancedDeformationGradientDDeformationGradient() = solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_ActiveDissipationDeformationGradient_F();
			eipv->getRefToDLocalEnhancedDeformationGradientDDeformationGradient() *= factF;

			eipv->getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() =solver->getHomogenizationState(IPStateBase::current)->getHomogenizedTangentOperator_ActiveDissipationDeformationGradient_F();
			eipv->getRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient() *= factEnhanceF;
		};
	}


	eipv->getRefToBandWidth() = solver->getHomogenizationState(IPStateBase::current)->getAverageLocalizationBandWidth();
	eipv->getRefToLocalizationNormal() = solver->getLostSolutionUniquenssNormal();

	mipv->setRefToDGElasticTangentModuli(elasticStiffness);
};

void enhancedStraindG3DMultiscaleMaterialLaw::setElasticStiffness(IPVariable* ipv) const{
  dG3DIPVariableBase* dGipv =dynamic_cast<dG3DIPVariableBase*>(ipv);
  if (dGipv){
    dGipv->setRefToDGElasticTangentModuli(elasticStiffness);
	}
  else
    Msg::Error("dG3DIPVariableBase must be used");
};

void enhancedStraindG3DMultiscaleMaterialLaw::initialIPVariable(IPVariable* ipv, bool stiff){
	dG3DIPVariableBase* dGipv =dynamic_cast<dG3DIPVariableBase*>(ipv);
  if (dGipv){
		dGipv->setRefToDGElasticTangentModuli(elasticStiffness);
		dGipv->getRefToTangentModuli() = elasticStiffness;
	}
	else{
		Msg::Error("dG3DIPVariableBase must be used");
	};
};
