//
//
// Description: Class of materials for non linear dg
//
//
// Author:  <V.D. NGUYEN>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef _DG3DENHANCEDSTRAINMATERIALLAW_H_
#define _DG3DENHANCEDSTRAINMATERIALLAW_H_

#include "dG3DMaterialLaw.h"
#include "numericalMaterial.h"


class enhancedStrainDG3DLinearElasticMaterialLaw : public dG3DLinearElasticMaterialLaw{
	protected:

	public:
		enhancedStrainDG3DLinearElasticMaterialLaw(const int num, const double rho);
		enhancedStrainDG3DLinearElasticMaterialLaw(const int num, const double rho, const double E, const double nu);
		#ifndef SWIG
		enhancedStrainDG3DLinearElasticMaterialLaw(const enhancedStrainDG3DLinearElasticMaterialLaw& src);
		virtual ~enhancedStrainDG3DLinearElasticMaterialLaw(){};

		virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  // To allow initialization of bulk ip in case of fracture
		virtual void createIPVariable(IPVariable* &ipv, const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;

		virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);

		virtual materialLaw* clone() const {return new enhancedStrainDG3DLinearElasticMaterialLaw(*this);};
		#endif //SWIG
};

class enhancedStraindG3DMultiscaleMaterialLaw : public dG3DMaterialLaw, public numericalMaterial{
  protected:
		double _lostSolutionUniquenssTolerance;
    double _RVELength;
    double _surfaceReductionRatio;
  #ifndef SWIG
  protected:
    virtual void fillElasticStiffness(double & rho, STensor43& L);
  #endif // SWIG


  public:
    enhancedStraindG3DMultiscaleMaterialLaw(const int lnum, const int tag);
		void setLostSolutionUniquenssTolerance(const double tol){ _lostSolutionUniquenssTolerance = tol;};
    void setCharacteristicLength(const double L, const double surf) {_RVELength = L; _surfaceReductionRatio = surf;};
#ifndef SWIG
    enhancedStraindG3DMultiscaleMaterialLaw(const enhancedStraindG3DMultiscaleMaterialLaw& src)
                            : dG3DMaterialLaw(src),numericalMaterial(src),_RVELength(src._RVELength),_surfaceReductionRatio(src._surfaceReductionRatio){};
    virtual ~enhancedStraindG3DMultiscaleMaterialLaw(){};
    virtual matname getType() const{return materialLaw::numeric;};
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;

    virtual void createIPState(const bool isSolve, IPStateBase* &ips,const bool* state =NULL,
                                 const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt=0) const;
    // To allow initialization of bulk ip in case of fracture
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void calledBy2lawFracture(){}
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw);
    virtual double density() const {return _rho;};
    // As to be defined for explicit scheme. If law for implicit only
    // please define this function with return 0
    virtual double soundSpeed() const {return 0;};
    // If law enable fracture the bool checkfrac allow to detect fracture. This bool is unused otherwise
    // The bool is necessery for computation of matrix by perturbation in this case the check of fracture
    // has not to be performed.
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual void setElasticStiffness(IPVariable* ipv) const;
    virtual bool isNumeric() const{return true;}
		virtual void initialIPVariable(IPVariable* ipv, bool stiff);

		virtual materialLaw* clone() const {return new enhancedStraindG3DMultiscaleMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const  {
      for (int i=0; i< _allMaterialLaw.size(); i++){
        if (_allMaterialLaw[i]->withEnergyDissipation()){
          return true;
        }
      }
      return false;
    }
    #endif // SWIG
};


#endif // _DG3DENHANCEDSTRAINMATERIALLAW_H_
