//
// C++ Interface: ipvariable
//
// Description: Class with definition of ipvarible for dG3D
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "dG3DIPVariable.h"
#include "ipstate.h"
#include "ipField.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <string.h>
#include "restartManager.h"
#include "highOrderTensor.h"
dG3DIPVariableBase::dG3DIPVariableBase(const dG3DIPVariableBase &source) : ipFiniteStrain(source)
{}

dG3DIPVariableBase & dG3DIPVariableBase::operator = (const IPVariable &source)
{
  ipFiniteStrain::operator=(source);
  return *this;
}

nonlocalData::nonlocalData(const int num, const int numExtraDof): numNonLocalVariable(num), numConstitutiveExtraDofDiffusionVariable(numExtraDof) {
  if (num > 0){
    dLocalVariableDStrain.resize(num,STensor3(0.));
    dStressDNonLocalVariable.resize(num,STensor3(0.));
    gradNonLocalVariable.resize(num,SVector3(0.));
    dLocalVariableDNonLocalVariable.resize(num,num);
    dLocalVariableDNonLocalVariable.setAll(0.);
    nonLocalJump.resize(num);
    nonLocalJump.setAll(0.);
    if(numExtraDof>0)
    {
      dLocalVariableDExtraDofDiffusionField.resize(num,numExtraDof); dLocalVariableDExtraDofDiffusionField.setAll(0.);
    }
  }
};
nonlocalData::nonlocalData(const nonlocalData& source):
  numNonLocalVariable(source.numNonLocalVariable),
  numConstitutiveExtraDofDiffusionVariable(source.numConstitutiveExtraDofDiffusionVariable),
  dLocalVariableDStrain(source.dLocalVariableDStrain),
  dStressDNonLocalVariable(source.dStressDNonLocalVariable),
  dLocalVariableDNonLocalVariable(source.dLocalVariableDNonLocalVariable),
  nonLocalJump(source.nonLocalJump),
  gradNonLocalVariable(source.gradNonLocalVariable),
  dLocalVariableDExtraDofDiffusionField(source.dLocalVariableDExtraDofDiffusionField)
{
};
nonlocalData& nonlocalData::operator = (const nonlocalData& source){
  numNonLocalVariable             = source.numNonLocalVariable;
  numConstitutiveExtraDofDiffusionVariable = source.numConstitutiveExtraDofDiffusionVariable;
  dLocalVariableDStrain           = source.dLocalVariableDStrain;
  dStressDNonLocalVariable        = source.dStressDNonLocalVariable;
  dLocalVariableDNonLocalVariable = source.dLocalVariableDNonLocalVariable;
  nonLocalJump                    = source.nonLocalJump;
  gradNonLocalVariable            = source.gradNonLocalVariable;
  dLocalVariableDExtraDofDiffusionField= source.dLocalVariableDExtraDofDiffusionField;
  return *this;
};

void nonlocalData::restart(){
  restartManager::restart(numNonLocalVariable);
  restartManager::restart(numConstitutiveExtraDofDiffusionVariable);
  restartManager::restart(dLocalVariableDStrain);
  restartManager::restart(dStressDNonLocalVariable);
  restartManager::restart(dLocalVariableDNonLocalVariable.getDataPtr(),numNonLocalVariable*numNonLocalVariable);
  restartManager::restart(nonLocalJump.getDataPtr(),numNonLocalVariable);
  restartManager::restart(gradNonLocalVariable);
  restartManager::restart(dLocalVariableDExtraDofDiffusionField.getDataPtr(),numNonLocalVariable*numConstitutiveExtraDofDiffusionVariable);
};
//
constitutiveExtraDofDiffusionData::constitutiveExtraDofDiffusionData(const int num, const int numNonLocal, bool _useOfEnergy, bool _useOfEnergyConjugatedField): numConstitutiveExtraDofDiffusionVariable(num), numNonLocalVariable(numNonLocal),  useOfEnergy(_useOfEnergy), useOfEnergyConjugatedField(_useOfEnergyConjugatedField)
{
  if (num > 0){
    std::vector<STensor3>        StdSTens3Tmp; StdSTens3Tmp.resize(num,STensor3(0.));
    std::vector<std::vector<STensor3>>        StdSTens3Tmp2; StdSTens3Tmp2.resize(num,StdSTens3Tmp);
    std::vector<SVector3>        StdSVect3Tmp; StdSVect3Tmp.resize(num,SVector3(0.));
    STensor43		         StdSTens43Tmp; STensorOperation::zero(StdSTens43Tmp);
    std::vector<STensor43>	 StdSTens43Tmp2;StdSTens43Tmp2.resize(num,StdSTens43Tmp);
    std::vector<const STensor3*> StdSTens3PtTmp; StdSTens3PtTmp.resize(num,NULL);
    std::vector<double>          StdDoubleTmp; StdDoubleTmp.resize(num,0.);

    field.resize(num); field.setAll(0.);

    gradField.resize(num,SVector3(0.));
    flux.resize(num,SVector3(0.));
    dPdField.resize(num,STensor3(0.));
    dPdGradField.resize(num,STensor33(0.));
    dFluxdGradField.resize(num,StdSTens3Tmp);
    dFluxdField.resize(num,StdSVect3Tmp);
    dFluxdF.resize(num,STensor33(0.));

    linearK.resize(num,StdSTens3PtTmp);
    if(useOfEnergy)
    {
     /* EnergyK.resize(num,StdSTens3PtTmp);
      dEnergyKdField.resize(num,StdSTens3Tmp2);
      dEnergyKdF.resize(num,StdSTens43Tmp2);*/
      dlinearKdField.resize(num,StdSTens3Tmp2);
      dlinearKdF.resize(num,StdSTens43Tmp2);
      dFluxdGradFielddF.resize(num,StdSTens43Tmp2);
      dFluxdGradFielddField.resize(num,StdSTens3Tmp2);
    }
    fieldSource.resize(num); fieldSource.setAll(0.);
    dFieldSourcedField.resize(num,num); dFieldSourcedField.setAll(0.);
    dFieldSourcedGradField.resize(num,StdSVect3Tmp);
    dFieldSourcedF.resize(num,STensor3(0.));

    mechanicalSource.resize(num); mechanicalSource.setAll(0.);
    dMechanicalSourcedField.resize(num,num); dMechanicalSourcedField.setAll(0.);
    dMechanicalSourcedGradField.resize(num,StdSVect3Tmp);
    dMechanicalSourcedF.resize(num,STensor3(0.));

    fieldCapacityPerUnitField.resize(num); fieldCapacityPerUnitField.setAll(0.);

    fieldJump.resize(num); fieldJump.setAll(0.);

    if(useOfEnergyConjugatedField)
    {
      energyConjugatedField.resize(num); energyConjugatedField.setAll(0.);
      gradEnergyConjugatedField.resize(num,SVector3(0.));
      energyConjugatedFieldJump.resize(num); energyConjugatedFieldJump.setAll(0.);
      dFielddEnergyConjugatedField.resize(num,StdDoubleTmp);
      dGradFielddEnergyConjugatedField.resize(num,StdSVect3Tmp);
      dGradFielddGradEnergyConjugatedField.resize(num,StdSTens3Tmp);
      dEnergyConjugatedFieldJumpdFieldm.resize(num,StdDoubleTmp);
      dEnergyConjugatedFieldJumpdFieldp.resize(num,StdDoubleTmp);
    }

    oneOverFieldJump.resize(num); oneOverFieldJump.setAll(0.);
    dOneOverFieldJumpdFieldm.resize(num,num); dOneOverFieldJumpdFieldm.setAll(0.);
    dOneOverFieldJumpdFieldp.resize(num,num); dOneOverFieldJumpdFieldp.setAll(0.);
    interfaceFlux.resize(num,SVector3(0.));
    linearSymmetrizationCoupling.resize(num,NULL);
    if(numNonLocalVariable > 0) {
      std::vector<SVector3>        StdSVect3CouplingTmp; StdSVect3CouplingTmp.resize(numNonLocalVariable,SVector3(0.));
      dFluxdNonLocalVariable.resize(num,StdSVect3CouplingTmp);
      dFieldSourcedNonLocalVariable.resize(num,numNonLocalVariable); dFieldSourcedNonLocalVariable.setAll(0.);
      dMechanicalSourcedNonLocalVariable.resize(num,numNonLocalVariable); dMechanicalSourcedNonLocalVariable.setAll(0.);

    }
  }
};
constitutiveExtraDofDiffusionData::constitutiveExtraDofDiffusionData(const constitutiveExtraDofDiffusionData& source): numConstitutiveExtraDofDiffusionVariable(source.numConstitutiveExtraDofDiffusionVariable), numNonLocalVariable(source.numNonLocalVariable),
                                                                                   field(source.field), gradField(source.gradField), flux(source.flux),
                                                                                   dPdField(source.dPdField), dPdGradField(source.dPdGradField), dFluxdGradField(source.dFluxdGradField),
                                                                                   dFluxdField(source.dFluxdField),
                                                                                   dFluxdF(source.dFluxdF), linearK(source.linearK),dlinearKdField(source.dlinearKdField),
										   dlinearKdF(source.dlinearKdF),fieldSource(source.fieldSource),
                                                                                   dFieldSourcedField(source.dFieldSourcedField), dFieldSourcedGradField(source.dFieldSourcedGradField),
                                                                                   dFieldSourcedF(source.dFieldSourcedF),
	                                                                           mechanicalSource(source.mechanicalSource), dMechanicalSourcedField(source.dMechanicalSourcedField),
                                                                                   dMechanicalSourcedGradField(source.dMechanicalSourcedGradField), dMechanicalSourcedF(source.dMechanicalSourcedF),
                                                                                   fieldCapacityPerUnitField(source.fieldCapacityPerUnitField), fieldJump(source.fieldJump),
                                                                                   oneOverFieldJump(source.oneOverFieldJump),
                                                                                   dOneOverFieldJumpdFieldm(source.dOneOverFieldJumpdFieldm),
                                                                                   dOneOverFieldJumpdFieldp(source.dOneOverFieldJumpdFieldp), interfaceFlux(source.interfaceFlux),
                                                                                   linearSymmetrizationCoupling(source.linearSymmetrizationCoupling),
                                                                                   dFluxdNonLocalVariable(source.dFluxdNonLocalVariable),
                                                                                   dFieldSourcedNonLocalVariable(source.dFieldSourcedNonLocalVariable),
                                                                                   dMechanicalSourcedNonLocalVariable(source.dMechanicalSourcedNonLocalVariable),
                                                                                   useOfEnergyConjugatedField(source.useOfEnergyConjugatedField),useOfEnergy(source.useOfEnergy),
										   energyConjugatedField(source.energyConjugatedField), gradEnergyConjugatedField(source.gradEnergyConjugatedField),
										   dFielddEnergyConjugatedField(source.dFielddEnergyConjugatedField),
									           dGradFielddEnergyConjugatedField(source.dGradFielddEnergyConjugatedField),
										   dGradFielddGradEnergyConjugatedField(source.dGradFielddGradEnergyConjugatedField),
										   energyConjugatedFieldJump(source.energyConjugatedFieldJump),
										   dEnergyConjugatedFieldJumpdFieldm(source.dEnergyConjugatedFieldJumpdFieldm),
										   dEnergyConjugatedFieldJumpdFieldp(source.dEnergyConjugatedFieldJumpdFieldp), dFluxdGradFielddF(source.dFluxdGradFielddF),
										   dFluxdGradFielddField(source.dFluxdGradFielddField)
{

};
constitutiveExtraDofDiffusionData& constitutiveExtraDofDiffusionData::operator = (const constitutiveExtraDofDiffusionData& source){
  numConstitutiveExtraDofDiffusionVariable=source.numConstitutiveExtraDofDiffusionVariable;
  numNonLocalVariable=source.numNonLocalVariable;
  field=source.field;
  gradField=source.gradField;
  flux=source.flux;

  dPdField=source.dPdField;
  dPdGradField=source.dPdGradField;
  dFluxdGradField=source.dFluxdGradField;

  dFluxdField=source.dFluxdField;
  dFluxdF=source.dFluxdF;
  linearK=source.linearK;
  dlinearKdF=source.dlinearKdF;
  dlinearKdField=source.dlinearKdField;
 /* EnergyK=source.EnergyK;
  dEnergyKdF=source.dEnergyKdF;
  dEnergyKdField=source.dEnergyKdField;*/

  dFluxdGradFielddF=source.dFluxdGradFielddF;
  dFluxdGradFielddField=source.dFluxdGradFielddField;

  fieldSource=source.fieldSource;
  dFieldSourcedField=source.dFieldSourcedField;
  dFieldSourcedGradField=source.dFieldSourcedGradField;
  dFieldSourcedF=source.dFieldSourcedF;
  mechanicalSource=source.mechanicalSource;
  dMechanicalSourcedField=source.dMechanicalSourcedField;
  dMechanicalSourcedGradField=source.dMechanicalSourcedGradField;
  dMechanicalSourcedF=source.dMechanicalSourcedF;

  fieldCapacityPerUnitField=source.fieldCapacityPerUnitField;
  fieldJump=source.fieldJump;
  oneOverFieldJump=source.oneOverFieldJump;
  dOneOverFieldJumpdFieldm=source.dOneOverFieldJumpdFieldm;
  dOneOverFieldJumpdFieldp=source.dOneOverFieldJumpdFieldp;
  interfaceFlux=source.interfaceFlux;
  linearSymmetrizationCoupling=source.linearSymmetrizationCoupling;
  dFluxdNonLocalVariable=source.dFluxdNonLocalVariable;
  dFieldSourcedNonLocalVariable=source.dFieldSourcedNonLocalVariable;
  dMechanicalSourcedNonLocalVariable=source.dMechanicalSourcedNonLocalVariable;

  energyConjugatedField=source.energyConjugatedField;
  gradEnergyConjugatedField=source.gradEnergyConjugatedField;
  dFielddEnergyConjugatedField=source.dFielddEnergyConjugatedField;
  dGradFielddEnergyConjugatedField=source.dGradFielddEnergyConjugatedField;
  dGradFielddGradEnergyConjugatedField=source.dGradFielddGradEnergyConjugatedField;
  energyConjugatedFieldJump=source.energyConjugatedFieldJump;
  dEnergyConjugatedFieldJumpdFieldm=source.dEnergyConjugatedFieldJumpdFieldm;
  dEnergyConjugatedFieldJumpdFieldp=source.dEnergyConjugatedFieldJumpdFieldp;

  useOfEnergy=source.useOfEnergy;
  useOfEnergyConjugatedField=source.useOfEnergyConjugatedField;

  return *this;
};

void constitutiveExtraDofDiffusionData::restart(){

    restartManager::restart(numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(numNonLocalVariable);

    restartManager::restart(field.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);

    restartManager::restart(gradField);
    restartManager::restart(flux);
    restartManager::restart(dPdField);
    restartManager::restart(dPdGradField);
    restartManager::restart(dFluxdGradField);
    restartManager::restart(dFluxdField);
    restartManager::restart(dFluxdF);

    restartManager::restart(useOfEnergy);
    restartManager::restart(useOfEnergyConjugatedField);

    //restartManager::restart(linearK.resize(num,StdSTens3PtTmp);

    restartManager::restart(dlinearKdField);
    restartManager::restart(dlinearKdF);
    /*restartManager::restart(dEnergyKdField);
    restartManager::restart(dEnergyKdF);*/

    restartManager::restart(dFluxdGradFielddF);
    restartManager::restart(dFluxdGradFielddField);

    restartManager::restart(fieldSource.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dFieldSourcedField.getDataPtr(),numConstitutiveExtraDofDiffusionVariable*numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dFieldSourcedGradField);
    restartManager::restart(dFieldSourcedF);

    restartManager::restart(mechanicalSource.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dMechanicalSourcedField.getDataPtr(),numConstitutiveExtraDofDiffusionVariable*numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dMechanicalSourcedGradField);
    restartManager::restart(dMechanicalSourcedF);

    restartManager::restart(fieldCapacityPerUnitField.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);

    restartManager::restart(fieldJump.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(oneOverFieldJump.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dOneOverFieldJumpdFieldm.getDataPtr(),numConstitutiveExtraDofDiffusionVariable*numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dOneOverFieldJumpdFieldp.getDataPtr(),numConstitutiveExtraDofDiffusionVariable*numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(interfaceFlux);
    //restartManager::restart(linearSymmetrizationCoupling.resize(num,StdSTens3PtTmp);
    restartManager::restart(dFluxdNonLocalVariable);
    restartManager::restart(dFieldSourcedNonLocalVariable.getDataPtr(),numConstitutiveExtraDofDiffusionVariable*numNonLocalVariable);
    restartManager::restart(dMechanicalSourcedNonLocalVariable.getDataPtr(),numConstitutiveExtraDofDiffusionVariable*numNonLocalVariable);

    restartManager::restart(energyConjugatedField.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(gradEnergyConjugatedField);
    restartManager::restart(dFielddEnergyConjugatedField);
    restartManager::restart(dGradFielddEnergyConjugatedField);
    restartManager::restart(dGradFielddGradEnergyConjugatedField);
    restartManager::restart(energyConjugatedFieldJump.getDataPtr(),numConstitutiveExtraDofDiffusionVariable);
    restartManager::restart(dEnergyConjugatedFieldJumpdFieldm);
    restartManager::restart(dEnergyConjugatedFieldJumpdFieldp);

};

dG3DIPVariable::dG3DIPVariable(const bool oninter, int numNonLocal, int extraDofDiffusion, bool useOfEnergy, bool energyConjugatedField) : dG3DIPVariableBase(), _nonlocalData(NULL), _constitutiveExtraDofDiffusionData(NULL)
{
  _oninter = oninter;
  deformationGradient(0,0) = 1.;
  deformationGradient(1,1) = 1.;
  deformationGradient(2,2) = 1.;
  barJ = 1.;
  localJ = 1.;
  if(numNonLocal>0)
    _nonlocalData=new nonlocalData(numNonLocal,extraDofDiffusion);
  if(extraDofDiffusion>0)
    _constitutiveExtraDofDiffusionData = new constitutiveExtraDofDiffusionData(extraDofDiffusion,numNonLocal, useOfEnergy, energyConjugatedField);

}

dG3DIPVariable::dG3DIPVariable(const dG3DIPVariable &source) : dG3DIPVariableBase(source), _nonlocalData(NULL), _constitutiveExtraDofDiffusionData(NULL)
{
    _oninter                  = source._oninter;
    deformationGradient       = source.deformationGradient;
    barJ                      = source.barJ;
    localJ                    = source.localJ;
    firstPiolaKirchhoffStress = source.firstPiolaKirchhoffStress;
    tangentModuli             = source.tangentModuli;
    elasticTangentModuli      = source.elasticTangentModuli;
    DGElasticTangentModuli      = source.DGElasticTangentModuli;
    referenceOutwardNormal    = source.referenceOutwardNormal;
    currentOutwardNormal      = source.currentOutwardNormal;
    jump                      = source.jump;
    incompatibleJump               = source.incompatibleJump;
    firstPiolaKirchhoffStressb4AV = source.firstPiolaKirchhoffStressb4AV;
    if(source._nonlocalData!=NULL)
    {
      _nonlocalData = new nonlocalData(*(source._nonlocalData));
    }
    if(source._constitutiveExtraDofDiffusionData!=NULL)
    {
      _constitutiveExtraDofDiffusionData = new constitutiveExtraDofDiffusionData(*(source._constitutiveExtraDofDiffusionData));
    }

}

dG3DIPVariable & dG3DIPVariable::operator = (const IPVariable &_source)
{
    dG3DIPVariableBase::operator=(_source);
    const dG3DIPVariable *source=dynamic_cast<const dG3DIPVariable *> (&_source);
    if(source!=NULL)
    {
      _oninter                  = source->_oninter;
      barJ                      = source->barJ;
      localJ                    = source->localJ;
      deformationGradient       = source->deformationGradient;
      firstPiolaKirchhoffStress = source->firstPiolaKirchhoffStress;
      tangentModuli             = source->tangentModuli;
      elasticTangentModuli      = source->elasticTangentModuli;
      DGElasticTangentModuli      = source->DGElasticTangentModuli;
      referenceOutwardNormal    = source->referenceOutwardNormal;
      currentOutwardNormal      = source->currentOutwardNormal;
      jump                      = source->jump;
      incompatibleJump          = source->incompatibleJump;
      firstPiolaKirchhoffStressb4AV = source->firstPiolaKirchhoffStressb4AV;
      if(source->_nonlocalData!=NULL)
      {
        if(_nonlocalData==NULL)
          _nonlocalData = new nonlocalData(*(source->_nonlocalData));
        else
          _nonlocalData->operator=(*(source->_nonlocalData));
      }
      if(source->_constitutiveExtraDofDiffusionData!=NULL)
      {
        if(_constitutiveExtraDofDiffusionData==NULL)
          _constitutiveExtraDofDiffusionData = new constitutiveExtraDofDiffusionData(*(source->_constitutiveExtraDofDiffusionData));
        else
         _constitutiveExtraDofDiffusionData->operator=(*(source->_constitutiveExtraDofDiffusionData));
      }
    }
    return *this;

}

double dG3DIPVariable::getJ() const
{
  const STensor3& F = getConstRefToDeformationGradient();
  return STensorOperation::determinantSTensor3(F);
}
void dG3DIPVariable::getCauchyStress(STensor3 &cauchy) const
{
  double detJ = getJ();
  for (int i = 0; i< 3; i ++){
    for (int j = 0; j< 3; j ++){
      cauchy(i,j) = 0.;
      for(int k =0; k <3; k++){
        cauchy(i,j) += getConstRefToFirstPiolaKirchhoffStress()(i,k)*getConstRefToDeformationGradient()(j,k)/detJ;
      }
    }
  }
}

void dG3DIPVariable::getBackLocalDeformationGradient(STensor3& localF) const
{
  localF=getConstRefToDeformationGradient();
  double fact=getLocalJ()/getBarJ();
  localF*=pow(fact,1./3.);
};


double dG3DIPVariable::get(const int comp) const
{
  static STensor3 cauchy;
  getCauchyStress(cauchy);
  const STensor3& F = this->getConstRefToDeformationGradient();
  const STensor3& P = this->getConstRefToFirstPiolaKirchhoffStress();

  if(comp == IPField::SIG_XX)
    return cauchy(0,0);
  else if(comp == IPField::SIG_YY)
    return cauchy(1,1);
  else if(comp == IPField::SIG_ZZ)
    return cauchy(2,2);
  else if(comp == IPField::SIG_XY)
    return cauchy(0,1);
  else if(comp == IPField::SIG_YZ)
    return cauchy(1,2);
  else if(comp == IPField::SIG_XZ)
    return cauchy(0,2);
  else if(comp == IPField::SVM) // von Mises
  {
    return vonMises();
  }
  else if(comp == IPField::STRAIN_XX)
    return getConstRefToDeformationGradient()(0,0)-1.;
  else if(comp == IPField::STRAIN_YY)
    return getConstRefToDeformationGradient()(1,1)-1.;
  else if(comp == IPField::STRAIN_ZZ)
    return getConstRefToDeformationGradient()(2,2)-1.;
  else if(comp == IPField::STRAIN_XY)
    return (getConstRefToDeformationGradient()(0,1)+getConstRefToDeformationGradient()(1,0))/2.;
  else if(comp == IPField::STRAIN_YZ)
    return (getConstRefToDeformationGradient()(1,2)+getConstRefToDeformationGradient()(2,1))/2.;
  else if(comp == IPField::STRAIN_XZ)
    return (getConstRefToDeformationGradient()(0,2)+getConstRefToDeformationGradient()(2,0))/2.;
  else if (comp == IPField::F_XX){
    return F(0,0);
	}
	else if (comp == IPField::F_XY){
    return F(0,1);
	}
	else if (comp == IPField::F_XZ){
    return F(0,2);
	}
	else if (comp == IPField::F_YX){
    return F(1,0);
	}
	else if (comp == IPField::F_YY){
    return F(1,1);
	}
	else if (comp == IPField::F_YZ){
    return F(1,2);
	}
	else if (comp == IPField::F_ZX){
    return F(2,0);
	}
	else if (comp == IPField::F_ZY){
        return F(2,1);
	}
	else if (comp == IPField::F_ZZ){
    return F(2,2);
	}
	else if (comp == IPField::P_XX){
    return P(0,0);
	}
	else if (comp == IPField::P_XY){
    return P(0,1);
	}
	else if (comp == IPField::P_XZ){
    return P(0,2);
	}
	else if (comp == IPField::P_YX){
    return P(1,0);
	}
	else if (comp == IPField::P_YY){
    return P(1,1);
	}
	else if (comp == IPField::P_YZ){
    return P(1,2);
	}
	else if (comp == IPField::P_ZX){
    return P(2,0);
	}
	else if (comp == IPField::P_ZY){
    return P(2,1);
	}
	else if (comp == IPField::P_ZZ){
    return P(2,2);
	}
	else if (comp == IPField::JACOBIAN){
    return getJ();
	}
	else if (comp == IPField::PRESSION){
    return (cauchy(0,0)+cauchy(1,1)+cauchy(2,2))/3.;
	}
	else if (comp == IPField::FIRST_PRINCIPAL_STRESS){
    static fullMatrix<double> eigVec(3,3);
    static fullVector<double> eigVal(3);
    cauchy.eig(eigVec,eigVal,true);
    return eigVal(0);
	}
	else if (comp == IPField::SECOND_PRINCIPAL_STRESS){
    static fullMatrix<double> eigVec(3,3);
    static fullVector<double> eigVal(3);
    cauchy.eig(eigVec,eigVal,true);
    return eigVal(1);
	}
	else if (comp == IPField::THIRD_PRINCIPAL_STRESS){
    static fullMatrix<double> eigVec(3,3);
    static fullVector<double> eigVal(3);
    cauchy.eig(eigVec,eigVal,true);
    return eigVal(2);
	}
  else if (comp == IPField::LODE_PARAMETER){
    static fullMatrix<double> eigVec(3,3);
    static fullVector<double> eigVal(3);
    cauchy.eig(eigVec,eigVal,true);
    return (2*eigVal(1) - eigVal(0) - eigVal(2))/(eigVal(2)-eigVal(0));
  }
  else if (comp == IPField::LODE_ANGLE){
    double trSig;
    static STensor3 devSig;
    STensorOperation::decomposeDevTr(cauchy,devSig,trSig);
    double J1,J2,J3;
    STensorOperation::getThreeInvariantsSTensor3(devSig,J1,J2,J3);
    double sigEq = sqrt(-3.*J2);
    double xi = 27.*J3/(2.*sigEq*sigEq*sigEq);
    const double pi = 3.14159265358979323846;
    return (acos(xi))/3.*(180./pi);
  }
	else if (comp == IPField::DISP_JUMP_X){
    return getConstRefToJump()(0);
	}
	else if (comp == IPField::DISP_JUMP_Y){
    return getConstRefToJump()(1);
	}
	else if (comp == IPField::DISP_JUMP_Z){
    return getConstRefToJump()(2);
	}
	else if (comp == IPField::STRESS_TRIAXIALITY){
    double p = (cauchy(0,0)+cauchy(1,1)+cauchy(2,2))/3.;
    double svm = vonMises();
    if (p ==0.) return 0.;
    else return p/svm;
	}
  else if (comp == IPField::INCOMPATIBLE_STRAIN_X){
    const SVector3& istrain = this->getConstRefToIncompatibleJump();
    return istrain(0);
  }
  else if (comp == IPField::INCOMPATIBLE_STRAIN_Y){
    const SVector3& istrain = this->getConstRefToIncompatibleJump();
    return istrain(1);
  }
  else if (comp == IPField::INCOMPATIBLE_STRAIN_Z){
    const SVector3& istrain = this->getConstRefToIncompatibleJump();
    return istrain(2);
  }
	else if (comp == IPField::DAMAGE_IS_BLOCKED){
		return dissipationIsBlocked();
	}
  else if (comp == IPField::ACTIVE_DAMAGING_ZONE){
    return this->dissipationIsActive();
  }
	else if (comp == IPField::DEFO_ENERGY){
		return this->defoEnergy();
	}
	else if (comp == IPField::PLASTIC_ENERGY){
		return this->plasticEnergy();
	}
  else if (comp == IPField::DAMAGE_ENERGY){
    return this->damageEnergy();
  }
	else if (comp == IPField::IRREVERSIBLE_ENERGY){
		return this->irreversibleEnergy();
	}
  else if (comp == IPField::DELETED){
    if (this->isDeleted()) return 1.;
    else return 0.;
  }
  else if (comp == IPField::GL_EQUIVALENT_STRAIN){
    static STensor3 E, devE;
    static double trE;
    STensorOperation::diag(E,-1.);
    STensorOperation::multSTensor3FirstTransposeAdd(F,F,E);
    E *= (0.5);
    STensorOperation::decomposeDevTr(E,devE,trE);
    return sqrt(2.*devE.dotprod()/3.);
  }
  else if (comp == IPField::GL_XX){
    static STensor3 FTF;
    STensorOperation::multSTensor3FirstTranspose(F,F,FTF);
    return 0.5*(FTF(0,0)-1.);
  }
  else if (comp == IPField::GL_XY){
    static STensor3 FTF;
    STensorOperation::multSTensor3FirstTranspose(F,F,FTF);
    return 0.5*FTF(0,1);
  }
  else if (comp == IPField::GL_XZ){
    static STensor3 FTF;
    STensorOperation::multSTensor3FirstTranspose(F,F,FTF);
    return 0.5*FTF(0,2);
  }
  else if (comp == IPField::GL_YY){
    static STensor3 FTF;
    STensorOperation::multSTensor3FirstTranspose(F,F,FTF);
    return 0.5*(FTF(1,1)-1.);
  }
  else if (comp == IPField::GL_YZ){
    static STensor3 FTF;
    STensorOperation::multSTensor3FirstTranspose(F,F,FTF);
    return 0.5*FTF(1,2);
  }
  else if (comp == IPField::GL_ZZ){
    static STensor3 FTF;
    STensorOperation::multSTensor3FirstTranspose(F,F,FTF);
    return 0.5*(FTF(2,2)-1.);
  }
  else if (comp == IPField::LOCAL_0){
    if (getNumberNonLocalVariable()>0){
      return getConstRefToLocalVariable(0);
    }
    else return 0.;
  }
  else if (comp == IPField::LOCAL_1){
    if (getNumberNonLocalVariable()>1){
      return getConstRefToLocalVariable(1);
    }
    else return 0.;
  }
  else if (comp == IPField::LOCAL_2){
    if (getNumberNonLocalVariable() > 2){
      return getConstRefToLocalVariable(2);
    }
    else return 0.;
  }
  else if (comp == IPField::NONLOCAL_0){
    if (getNumberNonLocalVariable() > 0){
      return getConstRefToNonLocalVariable(0);
    }
    else return 0.;
  }
  else if (comp == IPField::NONLOCAL_1){
    if (getNumberNonLocalVariable()> 1){
      return getConstRefToNonLocalVariable(1);
    }
    else return 0.;
  }
  else if (comp == IPField::NONLOCAL_2){
    if (getNumberNonLocalVariable() > 2)
      return getConstRefToNonLocalVariable(2);
    else return 0.;
  }
  else if (comp == IPField::CL_XX){
    if (getNumberNonLocalVariable() > 0){
      const STensor3& CL = getConstRefToCharacteristicLengthMatrix(0);
      return CL(0,0);
    }
  }
  else if (comp == IPField::CL_YY){
    if (getNumberNonLocalVariable() > 0){
      const STensor3& CL = getConstRefToCharacteristicLengthMatrix(0);
      return CL(1,1);
    }
  }
  else if (comp == IPField::CL_ZZ){
    if (getNumberNonLocalVariable() > 0){
      const STensor3& CL = getConstRefToCharacteristicLengthMatrix(0);
      return CL(2,2);
    }
  }
  else
     return 0.;

}
double dG3DIPVariable::defoEnergy() const
{
  double ene=0;
  double detJ = getJ();
  if (detJ < 1.e-12) Msg::Error("Negative Jacobian");
  // default implementation for small deformations
  static STensor3 cauchy;
  getCauchyStress(cauchy);
  static STensor3 E;
  for (int i = 0; i< 3; i ++){
    for (int j = 0; j< 3; j ++){
     E(i,j)  = getConstRefToDeformationGradient()(i,j)/2.+getConstRefToDeformationGradient()(j,i)/2.;
     if(i == j) E(i,j)-=1.;
    }
  }
  for (int i = 0; i< 3; i ++)
  {
    for (int j = 0; j< 3; j ++)
    {
       ene += cauchy(i,j)*E(i,j)/2.;
    }
  }
  return ene;
}
double dG3DIPVariable::plasticEnergy() const
{
  return 0.;
}
double dG3DIPVariable::vonMises() const
{
  static STensor3 cauchy;
  getCauchyStress(cauchy);
  double svm= (cauchy(0,0)-cauchy(1,1))*(cauchy(0,0)-cauchy(1,1))+(cauchy(0,0)-cauchy(2,2))*(cauchy(0,0)-cauchy(2,2))+
               (cauchy(2,2)-cauchy(1,1))*(cauchy(2,2)-cauchy(1,1));
     svm += 6.*(cauchy(0,1)*cauchy(1,0)+cauchy(0,2)*cauchy(2,0)+cauchy(2,1)*cauchy(1,2));
     svm /= 2.;
     svm = sqrt(svm);
     return svm;
}

void dG3DIPVariable::restart()
{
  dG3DIPVariableBase::restart();
  restartManager::restart(localJ);
  restartManager::restart(barJ);
  restartManager::restart(deformationGradient);
  restartManager::restart(firstPiolaKirchhoffStress);
  restartManager::restart(tangentModuli);
  restartManager::restart(elasticTangentModuli);
  /// Ignore this pointer???const STensor43                   *elasticTangentModuli;
  restartManager::restart(_oninter);
  restartManager::restart(referenceOutwardNormal);
  restartManager::restart(currentOutwardNormal);
  restartManager::restart(jump);
  restartManager::restart(incompatibleJump);
  if(_nonlocalData!=NULL)
    _nonlocalData->restart();
  if(_constitutiveExtraDofDiffusionData!=NULL)
    _constitutiveExtraDofDiffusionData->restart();
  return;
}

J2SmallStrainDG3DIPVariable::J2SmallStrainDG3DIPVariable(const mlawJ2VMSmallStrain&_j2law, const bool oninter) : dG3DIPVariable(oninter)
{
  _j2ipv = new   IPJ2linear(_j2law.getJ2IsotropicHardening());
}
J2SmallStrainDG3DIPVariable::J2SmallStrainDG3DIPVariable(const J2SmallStrainDG3DIPVariable &source) : dG3DIPVariable(source)
{
 _j2ipv = NULL;
  if(source._j2ipv != NULL){
    _j2ipv = dynamic_cast<IPJ2linear*>(source._j2ipv->clone());
  }

}
J2SmallStrainDG3DIPVariable& J2SmallStrainDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const J2SmallStrainDG3DIPVariable* src = dynamic_cast<const J2SmallStrainDG3DIPVariable*>(&source);
  if(src != NULL){
    if (src->_j2ipv != NULL){
      if (_j2ipv != NULL) {
        _j2ipv->operator=(*dynamic_cast<IPVariable*>(src->_j2ipv));
      }
      else{
        _j2ipv = dynamic_cast<IPJ2linear*>(src->_j2ipv->clone());
      }
    }
    else{
      if (_j2ipv) {
        delete _j2ipv;
       _j2ipv = NULL;
      }
    }

  }
  return *this;
}
double J2SmallStrainDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return _j2ipv->_j2lepspbarre;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double  J2SmallStrainDG3DIPVariable::defoEnergy() const
{
  return getIPJ2linear()->defoEnergy();
}
double  J2SmallStrainDG3DIPVariable::plasticEnergy() const
{
  return getIPJ2linear()->plasticEnergy();
}

void J2SmallStrainDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_j2ipv);
  return;
}

J2LinearDG3DIPVariable::J2LinearDG3DIPVariable(const mlawJ2linear &_j2law, const bool oninter) : dG3DIPVariable(oninter)
{
  _j2ipv = new   IPJ2linear(_j2law.getJ2IsotropicHardening());
}
J2LinearDG3DIPVariable::J2LinearDG3DIPVariable(const J2LinearDG3DIPVariable &source) : dG3DIPVariable(source)
{
  _j2ipv = NULL;
  if(source._j2ipv != NULL){
    _j2ipv = dynamic_cast<IPJ2linear*>(source._j2ipv->clone());
  }
}
J2LinearDG3DIPVariable& J2LinearDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const J2LinearDG3DIPVariable* src = dynamic_cast<const J2LinearDG3DIPVariable*>(&source);
  if(src != NULL){
    if (src->_j2ipv != NULL){
      if (_j2ipv != NULL) {
        _j2ipv->operator=(*dynamic_cast<const IPVariable*>(src->_j2ipv));
      }
      else{
        _j2ipv = dynamic_cast<IPJ2linear*>(src->_j2ipv->clone());
      }
    }
    else{
      if (_j2ipv) {
        delete _j2ipv;
       _j2ipv = NULL;
      }
    }

  }
  return *this;
}
double J2LinearDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return _j2ipv->_j2lepspbarre;
  }
  else if(comp == IPField::FP_XX)
  {
    return _j2ipv->_j2lepsp(0,0);
  }
  else if(comp == IPField::FP_XY)
  {
    return _j2ipv->_j2lepsp(0,1);
  }
  else if(comp == IPField::FP_XZ)
  {
    return _j2ipv->_j2lepsp(0,2);
  }
  else if(comp == IPField::FP_YX)
  {
    return _j2ipv->_j2lepsp(1,0);
  }
  else if(comp == IPField::FP_YY)
  {
    return _j2ipv->_j2lepsp(1,1);
  }
  else if(comp == IPField::FP_YZ)
  {
    return _j2ipv->_j2lepsp(1,2);
  }
  else if(comp == IPField::FP_ZX)
  {
    return _j2ipv->_j2lepsp(2,0);
  }
  else if(comp == IPField::FP_ZY)
  {
    return _j2ipv->_j2lepsp(2,1);
  }
  else if(comp == IPField::FP_ZZ)
  {
    return _j2ipv->_j2lepsp(2,2);
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double  J2LinearDG3DIPVariable::defoEnergy() const
{
  return getIPJ2linear()->defoEnergy();
}
double  J2LinearDG3DIPVariable::plasticEnergy() const
{
  return getIPJ2linear()->plasticEnergy();
}

void J2LinearDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_j2ipv);
  return;
}

HyperViscoElasticdG3DIPVariable::HyperViscoElasticdG3DIPVariable(const mlawHyperViscoElastic& viscoLaw, const bool oninter): dG3DIPVariable(oninter){
  _ipViscoElastic = new IPHyperViscoElastic(viscoLaw.getViscoElasticNumberOfElement());
};
HyperViscoElasticdG3DIPVariable::HyperViscoElasticdG3DIPVariable(const HyperViscoElasticdG3DIPVariable& src): dG3DIPVariable(src){
  _ipViscoElastic = NULL;
  if (src._ipViscoElastic != NULL){
    _ipViscoElastic = dynamic_cast<IPHyperViscoElastic*>(src._ipViscoElastic->clone());
  }
};
HyperViscoElasticdG3DIPVariable& HyperViscoElasticdG3DIPVariable::operator = (const IPVariable& src){
  dG3DIPVariable::operator =(src);
  const HyperViscoElasticdG3DIPVariable* psrc = dynamic_cast<const HyperViscoElasticdG3DIPVariable*>(&src);
  if (psrc != NULL){
    if (psrc->_ipViscoElastic != NULL){
      if (_ipViscoElastic!= NULL){
        _ipViscoElastic->operator=(*dynamic_cast<const IPVariable*>(psrc->_ipViscoElastic));
      }
      else{
        _ipViscoElastic = dynamic_cast<IPHyperViscoElastic*>(psrc->_ipViscoElastic->clone());
      }
    }
  }
  return *this;
};
HyperViscoElasticdG3DIPVariable::~HyperViscoElasticdG3DIPVariable(){
  if (_ipViscoElastic){delete _ipViscoElastic; _ipViscoElastic=NULL;};
};

double  HyperViscoElasticdG3DIPVariable::defoEnergy() const
{
  return _ipViscoElastic->defoEnergy();
}

void HyperViscoElasticdG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_ipViscoElastic);
  return;
}

HyperViscoElastoPlasticdG3DIPVariable::HyperViscoElastoPlasticdG3DIPVariable(const mlawPowerYieldHyper& viscoEPLaw, const bool oninter):
      dG3DIPVariable(oninter){
  _ipViscoElastoPlastic = new IPHyperViscoElastoPlastic(viscoEPLaw.getConstRefToCompressionHardening(),
                                            viscoEPLaw.getConstRefToTractionHardening(),
                                            viscoEPLaw.getConstRefToShearHardening(),
                                            viscoEPLaw.getConstRefToKinematicHardening(),
                                            viscoEPLaw.getViscoElasticNumberOfElement());
};
HyperViscoElastoPlasticdG3DIPVariable::HyperViscoElastoPlasticdG3DIPVariable(const HyperViscoElastoPlasticdG3DIPVariable& src):dG3DIPVariable(src){
  _ipViscoElastoPlastic = NULL;
  if (src._ipViscoElastoPlastic != NULL){
    _ipViscoElastoPlastic = dynamic_cast<IPHyperViscoElastoPlastic*>(src._ipViscoElastoPlastic->clone());
  }
};
HyperViscoElastoPlasticdG3DIPVariable& HyperViscoElastoPlasticdG3DIPVariable::operator = (const IPVariable& src){
  dG3DIPVariable::operator =(src);
  const HyperViscoElastoPlasticdG3DIPVariable* psrc = dynamic_cast<const HyperViscoElastoPlasticdG3DIPVariable*>(&src);
  if (psrc != NULL){
    if (psrc->_ipViscoElastoPlastic != NULL){
      if (_ipViscoElastoPlastic!= NULL){
        _ipViscoElastoPlastic->operator=(*dynamic_cast<const IPVariable*>(psrc->_ipViscoElastoPlastic));
      }
      else{
        _ipViscoElastoPlastic = dynamic_cast<IPHyperViscoElastoPlastic*>(psrc->_ipViscoElastoPlastic->clone());
      }
    }
  }
  return *this;
};
HyperViscoElastoPlasticdG3DIPVariable::~HyperViscoElastoPlasticdG3DIPVariable(){
  if (_ipViscoElastoPlastic){delete _ipViscoElastoPlastic; _ipViscoElastoPlastic = NULL;}
};

double HyperViscoElastoPlasticdG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return _ipViscoElastoPlastic->getConstRefToEqPlasticStrain();
  }
  else if (comp == IPField::PLASTIC_POISSON_RATIO){
    return _ipViscoElastoPlastic->getConstRefToPlasticPoissonRatio();
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}

void HyperViscoElastoPlasticdG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_ipViscoElastoPlastic);
  return;
}


localDamageJ2HyperDG3DIPVariable::localDamageJ2HyperDG3DIPVariable(const mlawLocalDamageJ2Hyper &_j2law, const bool oninter) :
                                   dG3DIPVariable(oninter)
{
  _nldJ2Hyperipv = new IPLocalDamageJ2Hyper(_j2law.getJ2IsotropicHardening(),_j2law.getDamageLaw());
}

localDamageJ2HyperDG3DIPVariable::localDamageJ2HyperDG3DIPVariable(const localDamageJ2HyperDG3DIPVariable &source) : dG3DIPVariable(source)
{
  _nldJ2Hyperipv = NULL;
  if (source.getIPLocalDamageJ2Hyper() != NULL){
    _nldJ2Hyperipv = dynamic_cast<IPLocalDamageJ2Hyper*>(source.getIPLocalDamageJ2Hyper()->clone());
  }
}

localDamageJ2HyperDG3DIPVariable& localDamageJ2HyperDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const localDamageJ2HyperDG3DIPVariable* src = dynamic_cast<const localDamageJ2HyperDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    if (src->getIPLocalDamageJ2Hyper() != NULL){
      if (_nldJ2Hyperipv != NULL){
        _nldJ2Hyperipv->operator=(*(dynamic_cast<const IPVariable*>(src->getIPLocalDamageJ2Hyper())));
      }
      else
        _nldJ2Hyperipv = dynamic_cast<IPLocalDamageJ2Hyper*>(src->getIPLocalDamageJ2Hyper()->clone());
    }
    else{
      if (_nldJ2Hyperipv != NULL){
        delete _nldJ2Hyperipv; _nldJ2Hyperipv = NULL;
      }
    }

  }
  return *this;
}
double localDamageJ2HyperDG3DIPVariable::get(const int i) const
{
  if(i == IPField::PLASTICSTRAIN)
  {
    return getIPLocalDamageJ2Hyper()->getConstRefToEquivalentPlasticStrain();
  }
  else if(i == IPField::DAMAGE)
  {
    return getIPLocalDamageJ2Hyper()->getDamage();
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}
double localDamageJ2HyperDG3DIPVariable::defoEnergy() const
{
  return getIPLocalDamageJ2Hyper()->defoEnergy();
}
double localDamageJ2HyperDG3DIPVariable::plasticEnergy() const
{
  return getIPLocalDamageJ2Hyper()->plasticEnergy();
}

void localDamageJ2HyperDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  if (_nldJ2Hyperipv != NULL)
    restartManager::restart(_nldJ2Hyperipv);
  return;
}


localDamageIsotropicElasticityDG3DIPVariable::localDamageIsotropicElasticityDG3DIPVariable(const mlawLocalDamageHyperelastic & elasticLaw, const bool oninter) :
                                   dG3DIPVariable(oninter)
{
  _localIPV = new IPLocalDamageIsotropicElasticity(elasticLaw.getDamageLaw());
}

localDamageIsotropicElasticityDG3DIPVariable::localDamageIsotropicElasticityDG3DIPVariable(const localDamageIsotropicElasticityDG3DIPVariable &source) : dG3DIPVariable(source)
{
  _localIPV = NULL;
  if (source.getIPLocalDamageIsotropicElasticity() != NULL){
    _localIPV = dynamic_cast<IPLocalDamageIsotropicElasticity*>(source.getIPLocalDamageIsotropicElasticity()->clone());
  }
}

localDamageIsotropicElasticityDG3DIPVariable& localDamageIsotropicElasticityDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const localDamageIsotropicElasticityDG3DIPVariable* src = dynamic_cast<const localDamageIsotropicElasticityDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    if (src->getIPLocalDamageIsotropicElasticity() != NULL){
      if (_localIPV != NULL){
        _localIPV->operator=(*(dynamic_cast<const IPVariable*>(src->getIPLocalDamageIsotropicElasticity())));
      }
      else
        _localIPV = dynamic_cast<IPLocalDamageIsotropicElasticity*>(src->getIPLocalDamageIsotropicElasticity()->clone());
    }
    else{
      if (_localIPV != NULL){
        delete _localIPV; _localIPV = NULL;
      }
    }

  }
  return *this;
}
double localDamageIsotropicElasticityDG3DIPVariable::get(const int i) const
{
  if(i == IPField::DAMAGE)
  {
    return getIPLocalDamageIsotropicElasticity()->getDamage();
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}

void localDamageIsotropicElasticityDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  if (_localIPV!= NULL)
    restartManager::restart(_localIPV);
  return;
}

//====================================================ViscoelasticDG3DIPVariable==========================================================//

ViscoelasticDG3DIPVariable::ViscoelasticDG3DIPVariable(const mlawViscoelastic &_Vislaw, const bool oninter) : dG3DIPVariable(oninter)
{
  _Visipv = new   IPViscoelastic(_Vislaw.numInternalVariable());
}
ViscoelasticDG3DIPVariable::ViscoelasticDG3DIPVariable(const ViscoelasticDG3DIPVariable &source) : dG3DIPVariable(source)
{
  _Visipv = NULL;
  if (source._Visipv){
    _Visipv = new   IPViscoelastic(*source._Visipv);

  }
}
ViscoelasticDG3DIPVariable& ViscoelasticDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const ViscoelasticDG3DIPVariable* src = dynamic_cast<const ViscoelasticDG3DIPVariable*>(&source);
  if(src != NULL){
    if (src->_Visipv){
      if (_Visipv){
        _Visipv->operator=(*dynamic_cast<const IPVariable*>(src->_Visipv));
      }
      else{
        _Visipv = dynamic_cast<IPViscoelastic*>(src->_Visipv->clone());
      }

    }
    else{
      if (_Visipv) {
        delete _Visipv; _Visipv = NULL;
      }
    }
  }
  return *this;
}
double ViscoelasticDG3DIPVariable::get(const int comp) const
{

    return dG3DIPVariable::get(comp);
}

void ViscoelasticDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_Visipv);
  return;
}

//=========================================================EOSDG3DIPVariable======================================================================//

EOSDG3DIPVariable::EOSDG3DIPVariable(double elementSize, const bool oninter) : dG3DIPVariable(oninter)
{
  _eosipv = new   IPEOS(elementSize);
}
EOSDG3DIPVariable::EOSDG3DIPVariable(double elementSize,IPVariable *IPv, const bool oninter) : dG3DIPVariable(oninter)
{
    STensor3 ZEROS(0.);
    _PK1_vis = ZEROS;
    _eosipv = new   IPEOS(elementSize, IPv);
}

EOSDG3DIPVariable::EOSDG3DIPVariable(const EOSDG3DIPVariable &source) : dG3DIPVariable(source)
{
  _eosipv = NULL;
  if (source._eosipv!=NULL){
    _eosipv = new IPEOS(*source._eosipv);
  }
}
EOSDG3DIPVariable& EOSDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const EOSDG3DIPVariable* src = dynamic_cast<const EOSDG3DIPVariable*>(&source);
  if(src != NULL){
    if (src->_eosipv!=NULL){
      if (_eosipv!=NULL){
         _eosipv->operator=(*dynamic_cast<const IPVariable*>(src->_eosipv));
      }
      else{
        _eosipv = dynamic_cast<IPEOS*>(src->_eosipv->clone());
      }

    }
    else{
      if (_eosipv != NULL){
        delete _eosipv;
        _eosipv = NULL;
      }
    }
  }
  return *this;
}
double EOSDG3DIPVariable::get(const int comp) const
{
    return dG3DIPVariable::get(comp);
}

void EOSDG3DIPVariable::getCauchyStress(STensor3 &cauchy) const
{
  const STensor3& PK1_vis = _PK1_vis;
  cauchy=0.;
  double detJ = getJ();
  if (detJ < 1.e-12) Msg::Error("Negative Jacobian");
  for (int i = 0; i< 3; i ++)
  {
    for (int j = 0; j< 3; j ++)
    {
       for(int k =0; k <3; k++)
       {
           cauchy(i,j) += (getConstRefToFirstPiolaKirchhoffStress()(i,k) + (-1.*PK1_vis(i,k)))*getConstRefToDeformationGradient()(j,k)/detJ;
       }
    }
  }
}

void EOSDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_eosipv);
  restartManager::restart(_PK1_vis);
  return;
}


VUMATinterfaceDG3DIPVariable::VUMATinterfaceDG3DIPVariable(int _nsdv, double _size, const bool oninter) : dG3DIPVariable(oninter)
{
  _vumatipv = new IPVUMATinterface(_nsdv,_size);
}
VUMATinterfaceDG3DIPVariable::VUMATinterfaceDG3DIPVariable(const VUMATinterfaceDG3DIPVariable &source) : dG3DIPVariable(source)
{
    _vumatipv = NULL;
  if(source._vumatipv != NULL){
    _vumatipv = new IPVUMATinterface(*source._vumatipv);
  }
}
VUMATinterfaceDG3DIPVariable& VUMATinterfaceDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const VUMATinterfaceDG3DIPVariable* src = dynamic_cast<const VUMATinterfaceDG3DIPVariable*>(&source);
  if(src != NULL){
    if (src->_vumatipv != NULL){
      if (_vumatipv!= NULL){
        _vumatipv->operator= (*(dynamic_cast<const IPVariable*>(src->getIPVUMATinterface())));
      }
      else{
        _vumatipv = dynamic_cast<IPVUMATinterface*>(src->_vumatipv->clone());
      }

    }
    else{
      if (_vumatipv != NULL){
        delete _vumatipv;
        _vumatipv = NULL;
      }
    }

  }
  return *this;
}
double VUMATinterfaceDG3DIPVariable::get(const int comp) const
{
  if(comp >= 100) // Internal variables
  {
    return getIPVUMATinterface()->get(comp);
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}

double  VUMATinterfaceDG3DIPVariable::defoEnergy() const
{
  return getIPVUMATinterface()->defoEnergy();
}
double  VUMATinterfaceDG3DIPVariable::plasticEnergy() const
{
  return getIPVUMATinterface()->plasticEnergy();
}

void VUMATinterfaceDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_vumatipv);
  return;
}

//
TransverseIsotropicDG3DIPVariable::TransverseIsotropicDG3DIPVariable(const bool oninter) : dG3DIPVariable(oninter)
{

}
TransverseIsotropicDG3DIPVariable::TransverseIsotropicDG3DIPVariable(const TransverseIsotropicDG3DIPVariable &source) : dG3DIPVariable(source),
_tiipv(source._tiipv)
{

}
TransverseIsotropicDG3DIPVariable& TransverseIsotropicDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const TransverseIsotropicDG3DIPVariable* src = dynamic_cast<const TransverseIsotropicDG3DIPVariable*>(&source);
  if(src != NULL)
    _tiipv.operator=(*dynamic_cast<const IPVariable*>(&(src->_tiipv)));
  return *this;
}
double TransverseIsotropicDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return 0.;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double  TransverseIsotropicDG3DIPVariable::defoEnergy() const
{
  return getIPTransverseIsotropic()->defoEnergy();
}
double  TransverseIsotropicDG3DIPVariable::plasticEnergy() const
{
  return getIPTransverseIsotropic()->plasticEnergy();
}

void TransverseIsotropicDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(&_tiipv);
  return;
}




TransverseIsoCurvatureDG3DIPVariable::TransverseIsoCurvatureDG3DIPVariable(const SVector3 &GaussP, const SVector3 &Centroid,
                                          const SVector3 &PointStrart1, const SVector3 &PointStrart2, const double init_Angle, const bool oninter) :
                                                                           dG3DIPVariable(oninter), _tiipv(GaussP, Centroid, PointStrart1, PointStrart2, init_Angle)
{

}
TransverseIsoCurvatureDG3DIPVariable::TransverseIsoCurvatureDG3DIPVariable(const TransverseIsoCurvatureDG3DIPVariable &source) : dG3DIPVariable(source),  _tiipv(source._tiipv)
{

}
TransverseIsoCurvatureDG3DIPVariable& TransverseIsoCurvatureDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const TransverseIsoCurvatureDG3DIPVariable* src = dynamic_cast<const TransverseIsoCurvatureDG3DIPVariable*>(&source);
  if(src != NULL)
    _tiipv.operator=(*dynamic_cast<const IPVariable*>(&( src->_tiipv)));
  return *this;
}
double TransverseIsoCurvatureDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return 0.;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double  TransverseIsoCurvatureDG3DIPVariable::defoEnergy() const
{
  return getIPTransverseIsoCurvature()->defoEnergy();
}
double  TransverseIsoCurvatureDG3DIPVariable::plasticEnergy() const
{
  return getIPTransverseIsoCurvature()->plasticEnergy();
}

void TransverseIsoCurvatureDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(&_tiipv);
  return;
}


TransverseIsoYarnBDG3DIPVariable::TransverseIsoYarnBDG3DIPVariable(const SVector3 &GaussP, const Yarn YD, const bool oninter) :
                                                                   dG3DIPVariable(oninter), _tiipv(GaussP, YD)
{

}

TransverseIsoYarnBDG3DIPVariable::TransverseIsoYarnBDG3DIPVariable(const SVector3 &GaussP, const bool oninter):
                                                                   dG3DIPVariable(oninter), _tiipv(GaussP)
{

}


TransverseIsoYarnBDG3DIPVariable::TransverseIsoYarnBDG3DIPVariable(const SVector3 &GaussP, const vector<double> Dir, const bool oninter) :
                                                                   dG3DIPVariable(oninter), _tiipv(GaussP, Dir)
{

}

TransverseIsoYarnBDG3DIPVariable::TransverseIsoYarnBDG3DIPVariable(const TransverseIsoYarnBDG3DIPVariable &source) : dG3DIPVariable(source),  _tiipv(source._tiipv)
{

}

TransverseIsoYarnBDG3DIPVariable& TransverseIsoYarnBDG3DIPVariable::operator=(const IPVariable &source){
	dG3DIPVariable::operator =(source);
	const TransverseIsoYarnBDG3DIPVariable* psrc = dynamic_cast<const TransverseIsoYarnBDG3DIPVariable*>(&source);
	if (psrc != NULL){
		_tiipv.operator=(*dynamic_cast<const IPVariable*>(&psrc->_tiipv));
	}
	return *this;
};




/*
double TransverseIsoYarnBDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return 0.;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
*/

/*
double  TransverseIsoYarnBDG3DIPVariable::defoEnergy()
{
  return getIPTransverseIsoYarnB()->defoEnergy();
}
*/

/*
double  TransverseIsoYarnBDG3DIPVariable::plasticEnergy()
{
  return getIPTransverseIsoYarnB()->plasticEnergy();
}
*/

void TransverseIsoYarnBDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(&_tiipv);
  return;
}




AnisotropicDG3DIPVariable::AnisotropicDG3DIPVariable(const bool oninter) : dG3DIPVariable(oninter)
{

}
AnisotropicDG3DIPVariable::AnisotropicDG3DIPVariable(const AnisotropicDG3DIPVariable &source) : dG3DIPVariable(source),
_tiipv(source._tiipv)
{

}
AnisotropicDG3DIPVariable& AnisotropicDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const AnisotropicDG3DIPVariable* src = dynamic_cast<const AnisotropicDG3DIPVariable*>(&source);
  if(src != NULL)
    _tiipv.operator=(*dynamic_cast<const IPVariable*>(&( src->_tiipv)));
  return *this;
}
double AnisotropicDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return 0.;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double  AnisotropicDG3DIPVariable::defoEnergy() const
{
  return getIPAnisotropic()->defoEnergy();
}
double  AnisotropicDG3DIPVariable::plasticEnergy() const
{
  return getIPAnisotropic()->plasticEnergy();
}

void AnisotropicDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(&_tiipv);
  return;
}



AnisotropicStochDG3DIPVariable::AnisotropicStochDG3DIPVariable(const SVector3 &GaussP, const fullMatrix<double> &ExMat,
                                 const fullMatrix<double> &EyMat, const fullMatrix<double> &EzMat, const fullMatrix<double> &VxyMat,
                                 const fullMatrix<double> &VxzMat, const fullMatrix<double> &VyzMat, const fullMatrix<double> &MUxyMat,
                                 const fullMatrix<double> &MUxzMat, const fullMatrix<double> &MUyzMat, const double dx, const double dy,
                                 const double OrigX, const double OrigY, const int intpl, const bool oninter) : dG3DIPVariable(oninter), _tiipv(GaussP, ExMat,
                                 EyMat, EzMat, VxyMat, VxzMat, VyzMat, MUxyMat, MUxzMat, MUyzMat, dx, dy, OrigX, OrigY, intpl) {}

AnisotropicStochDG3DIPVariable::AnisotropicStochDG3DIPVariable(const AnisotropicStochDG3DIPVariable &source) : dG3DIPVariable(source),
_tiipv(source._tiipv) {}

AnisotropicStochDG3DIPVariable& AnisotropicStochDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const AnisotropicStochDG3DIPVariable* src = dynamic_cast<const AnisotropicStochDG3DIPVariable*>(&source);
  if(src != NULL)
    _tiipv.operator=(*dynamic_cast<const IPVariable*>(&( src->_tiipv)));
  return *this;
}
double AnisotropicStochDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return 0.;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double  AnisotropicStochDG3DIPVariable::defoEnergy() const
{
  return getIPAnisotropicStoch()->defoEnergy();
}
double  AnisotropicStochDG3DIPVariable::plasticEnergy() const
{
  return getIPAnisotropicStoch()->plasticEnergy();
}

void AnisotropicStochDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(&_tiipv);
  return;
}

//



ClusterDG3DIPVariable::ClusterDG3DIPVariable(IPCluster* ipc, const bool oninter) : dG3DIPVariable(oninter)
{
  _clusteripv=ipc;
}

ClusterDG3DIPVariable::ClusterDG3DIPVariable(const ClusterDG3DIPVariable &source) : dG3DIPVariable(source)
{
  _clusteripv = NULL;
  if (source.getIPCluster() != NULL){
    _clusteripv = new IPCluster(*(source.getIPCluster()));
  }
}

ClusterDG3DIPVariable& ClusterDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const ClusterDG3DIPVariable* src = dynamic_cast<const ClusterDG3DIPVariable*>(&source);
  if(src != NULL)
    _clusteripv->operator=(*(dynamic_cast<const IPVariable*>( src->_clusteripv )));
  return *this;
}
double ClusterDG3DIPVariable::get(const int comp) const
{
  if(comp == IPField::PLASTICSTRAIN)
  {
    return 0.;
  }
  else
  {
    return dG3DIPVariable::get(comp);
  }
}
double ClusterDG3DIPVariable::defoEnergy() const
{
  return getIPCluster()->defoEnergy();
}
double  ClusterDG3DIPVariable::plasticEnergy() const
{
  return getIPCluster()->plasticEnergy();
}

void ClusterDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  _clusteripv->restart();
  return;
}


/*
ExtraDofDiffusionDG3DIPVariableBase::ExtraDofDiffusionDG3DIPVariableBase( const bool oninter) : dG3DIPVariable(oninter),
{

}

ExtraDofDiffusionDG3DIPVariableBase::ExtraDofDiffusionDG3DIPVariableBase(const ExtraDofDiffusionDG3DIPVariableBase &src) :
                                                                        dG3DIPVariable(src)
{

}

ExtraDofDiffusionDG3DIPVariableBase& ExtraDofDiffusionDG3DIPVariableBase::operator=(const IPVariable &_src)
{
  dG3DIPVariable::operator=(_src);
  const ExtraDofDiffusionDG3DIPVariableBase* src = dynamic_cast<const ExtraDofDiffusionDG3DIPVariableBase*>(&_src);
  if(src != NULL)
  {

  }
  return *this;
}

void ExtraDofDiffusionDG3DIPVariableBase::restart()
{
  dG3DIPVariable::restart();
  return;
}*/

ThermoMechanicsDG3DIPVariableBase::ThermoMechanicsDG3DIPVariableBase( const bool oninter) : dG3DIPVariable(oninter,0,1), extraDofIPVariableBase()

{

}

ThermoMechanicsDG3DIPVariableBase::ThermoMechanicsDG3DIPVariableBase(const ThermoMechanicsDG3DIPVariableBase &source) :
                                                                        dG3DIPVariable(source), extraDofIPVariableBase()
{

}

ThermoMechanicsDG3DIPVariableBase& ThermoMechanicsDG3DIPVariableBase::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const ThermoMechanicsDG3DIPVariableBase* src = dynamic_cast<const ThermoMechanicsDG3DIPVariableBase*>(&source);
  if(src != NULL)
  {
  }
  return *this;
}

double ThermoMechanicsDG3DIPVariableBase::get(const int i) const
{
  if(i == IPField::TEMPERATURE)
  {
    return getConstRefToField(0);
  }
  else if(i == IPField::THERMALFLUX_X)
  {
    return getConstRefToFlux()[0](0);
  }
  else if(i == IPField::THERMALFLUX_Y)
  {
    return getConstRefToFlux()[0](1);
  }
  else if(i == IPField::THERMALFLUX_Z)
  {
    return getConstRefToFlux()[0](2);
  }
  else if (i == IPField::FIELD_SOURCE){
    return getConstRefToFieldSource()(0);
  }
  else if (i == IPField::MECHANICAL_SOURCE){
    return getConstRefToMechanicalSource()(0);
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}

void ThermoMechanicsDG3DIPVariableBase::restart()
{
  dG3DIPVariableBase::restart();
  return;
}

LinearThermoMechanicsDG3DIPVariable::LinearThermoMechanicsDG3DIPVariable(double tinitial, const bool oninter) :
                                   ThermoMechanicsDG3DIPVariableBase(oninter)
{
  this->getRefToTemperature()=tinitial;

}


LinearThermoMechanicsDG3DIPVariable::LinearThermoMechanicsDG3DIPVariable(const LinearThermoMechanicsDG3DIPVariable &source) :
                                                        ThermoMechanicsDG3DIPVariableBase(source),
																												_linearTMIP(source._linearTMIP)
{


}

LinearThermoMechanicsDG3DIPVariable& LinearThermoMechanicsDG3DIPVariable::operator=(const IPVariable &source)
{
  ThermoMechanicsDG3DIPVariableBase::operator=(source);
  const LinearThermoMechanicsDG3DIPVariable* src = dynamic_cast<const LinearThermoMechanicsDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    _linearTMIP.operator=(*(dynamic_cast<const IPVariable*>(&src->_linearTMIP)));
  }
  return *this;
}
double LinearThermoMechanicsDG3DIPVariable::get(const int i) const
{
  return ThermoMechanicsDG3DIPVariableBase::get(i);
}
double LinearThermoMechanicsDG3DIPVariable::defoEnergy() const
{
  return getIPLinearThermoMechanics()->defoEnergy();
}
double LinearThermoMechanicsDG3DIPVariable::plasticEnergy() const
{
  return getIPLinearThermoMechanics()->plasticEnergy();
}

void LinearThermoMechanicsDG3DIPVariable::restart()
{
  ThermoMechanicsDG3DIPVariableBase::restart();
  restartManager::restart(&_linearTMIP);
  return;
}



AnIsotropicTherMechDG3DIPVariable::AnIsotropicTherMechDG3DIPVariable(double tinitial, const bool oninter) :
                                   ThermoMechanicsDG3DIPVariableBase(oninter)
{
  this->getRefToTemperature()=tinitial;

}


AnIsotropicTherMechDG3DIPVariable::AnIsotropicTherMechDG3DIPVariable(const AnIsotropicTherMechDG3DIPVariable &source) :
                                                        ThermoMechanicsDG3DIPVariableBase(source),
																												_AnIsotropicTMIP(source._AnIsotropicTMIP)
{


}
AnIsotropicTherMechDG3DIPVariable& AnIsotropicTherMechDG3DIPVariable::operator=(const IPVariable &source)
{
  ThermoMechanicsDG3DIPVariableBase::operator=(source);
  const AnIsotropicTherMechDG3DIPVariable* src = dynamic_cast<const AnIsotropicTherMechDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    _AnIsotropicTMIP.operator=(*(dynamic_cast<const IPVariable*>(&src->_AnIsotropicTMIP)));
  }
  return *this;
}
double AnIsotropicTherMechDG3DIPVariable::get(const int i) const
{
  return ThermoMechanicsDG3DIPVariableBase::get(i);
}
double AnIsotropicTherMechDG3DIPVariable::defoEnergy() const
{
  return getIPAnIsotropicTherMech()->defoEnergy();
}
double AnIsotropicTherMechDG3DIPVariable::plasticEnergy() const
{
  return getIPAnIsotropicTherMech()->plasticEnergy();
}

void AnIsotropicTherMechDG3DIPVariable::restart()
{
  ThermoMechanicsDG3DIPVariableBase::restart();
  restartManager::restart(&_AnIsotropicTMIP);
  return;
}


J2ThermoMechanicsDG3DIPVariable::J2ThermoMechanicsDG3DIPVariable(double tinitial ,const mlawJ2ThermoMechanics &_j2law, const bool oninter) : ThermoMechanicsDG3DIPVariableBase(oninter)
{
  _j2ipv = new   IPJ2ThermoMechanics(_j2law.getJ2IsotropicHardening());
    this->getRefToTemperature()=tinitial;
}
J2ThermoMechanicsDG3DIPVariable::J2ThermoMechanicsDG3DIPVariable(const J2ThermoMechanicsDG3DIPVariable &source) : ThermoMechanicsDG3DIPVariableBase(source)
{
  _j2ipv = NULL;
  if (source._j2ipv != NULL){
    _j2ipv = dynamic_cast<IPJ2ThermoMechanics*>(source._j2ipv->clone());
  }
}
J2ThermoMechanicsDG3DIPVariable& J2ThermoMechanicsDG3DIPVariable::operator=(const IPVariable &source)
{
  ThermoMechanicsDG3DIPVariableBase::operator=(source);
  const J2ThermoMechanicsDG3DIPVariable* src = dynamic_cast<const J2ThermoMechanicsDG3DIPVariable*>(&source);
  if(src != NULL){
    if (src->_j2ipv != NULL){
			if (_j2ipv!= NULL){
				_j2ipv->operator=(*dynamic_cast<const IPVariable*>(src->_j2ipv->clone()));
			}
			else
				_j2ipv = dynamic_cast<IPJ2ThermoMechanics*>(src->_j2ipv->clone());
    }
		else{
			if (_j2ipv!=NULL){
				delete _j2ipv;
				_j2ipv=NULL;
			}
		}
  }

  return *this;
}
double J2ThermoMechanicsDG3DIPVariable::get(const int i) const
{
  if(i== IPField::PLASTICSTRAIN)
  {
    return _j2ipv->_j2lepspbarre;
  }
  else
  {
    return ThermoMechanicsDG3DIPVariableBase::get(i);
  }
}
double  J2ThermoMechanicsDG3DIPVariable::defoEnergy() const
{
  return getIPJ2ThermoMechanics()->defoEnergy();
}
double  J2ThermoMechanicsDG3DIPVariable::plasticEnergy() const
{
  return getIPJ2ThermoMechanics()->plasticEnergy();
}

void J2ThermoMechanicsDG3DIPVariable::restart()
{
  ThermoMechanicsDG3DIPVariableBase::restart();
  if (_j2ipv != NULL)
    restartManager::restart(_j2ipv);
  return;
}

J2FullThermoMechanicsDG3DIPVariable::J2FullThermoMechanicsDG3DIPVariable(const J2IsotropicHardening* j2harden, const bool oninter):
			ThermoMechanicsDG3DIPVariableBase(oninter),_j2Thermo(j2harden){};
J2FullThermoMechanicsDG3DIPVariable::J2FullThermoMechanicsDG3DIPVariable(const J2FullThermoMechanicsDG3DIPVariable& src):
			ThermoMechanicsDG3DIPVariableBase(src),_j2Thermo(src._j2Thermo){};
J2FullThermoMechanicsDG3DIPVariable& J2FullThermoMechanicsDG3DIPVariable::operator =(const IPVariable& src){
	ThermoMechanicsDG3DIPVariableBase::operator =(src);
	const J2FullThermoMechanicsDG3DIPVariable *psrc = dynamic_cast<const J2FullThermoMechanicsDG3DIPVariable*>(&src);
	if (psrc != NULL){
		_j2Thermo.operator=(*(dynamic_cast<const IPVariable*>(&(psrc->_j2Thermo))));
	}
	return *this;
};;

double J2FullThermoMechanicsDG3DIPVariable::get(const int i) const{
  if ( i== IPField::PLASTICSTRAIN){
    return _j2Thermo.getConstRefToEquivalentPlasticStrain();
  }
  else
  {
    return ThermoMechanicsDG3DIPVariableBase::get(i);
  }
};

void J2FullThermoMechanicsDG3DIPVariable::restart(){
	ThermoMechanicsDG3DIPVariableBase::restart();
	_j2Thermo.restart();
};


SMPDG3DIPVariable::SMPDG3DIPVariable( double tinitial,const mlawSMP &_lawSMP,  const bool oninter) :
                                   ThermoMechanicsDG3DIPVariableBase(oninter)
{
  _TMIPSMP = new IPSMP(_lawSMP.getSa0(),_lawSMP.getPhia0(),_lawSMP.getSastar0());
  this->getRefToTemperature()=tinitial;
}

SMPDG3DIPVariable::SMPDG3DIPVariable(const SMPDG3DIPVariable &source) :
                                                        ThermoMechanicsDG3DIPVariableBase(source)
{
  _TMIPSMP = NULL;
  if (source._TMIPSMP != NULL){
    _TMIPSMP = dynamic_cast<IPSMP*>(source._TMIPSMP->clone());
  }

}

SMPDG3DIPVariable& SMPDG3DIPVariable::operator=(const IPVariable &source)
{
  ThermoMechanicsDG3DIPVariableBase::operator=(source);
  const SMPDG3DIPVariable* src = dynamic_cast<const SMPDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    if (src->_TMIPSMP != NULL) {
			if (_TMIPSMP!=NULL){
				_TMIPSMP->operator=(*dynamic_cast<const IPVariable*>(src->_TMIPSMP));
			}
			else
				_TMIPSMP = dynamic_cast<IPSMP*>(src->_TMIPSMP->clone());
    }
		else{
			if (_TMIPSMP!=NULL){
				delete _TMIPSMP;
				_TMIPSMP = NULL;
			}
		}
  }
  return *this;
}
double SMPDG3DIPVariable::get(const int i) const
{
  if(i == IPField::PLASTICSTRAIN)
  {
   // return getInternalEnergyExtraDofDiffusion();
    return getRefToEquivalentPlasticDefo1();
  }
 /*   if(i == IPField::Plasric_Defo_mec1)
  {
    return getConstRefToEquivalentPlasticDefo1();
  }*/
  else
  {
    return ThermoMechanicsDG3DIPVariableBase::get(i);
  }
}
 // return ThermoMechanicsDG3DIPVariableBase::get(i);
//}
double SMPDG3DIPVariable::defoEnergy() const
{
  return getIPSMP()->defoEnergy();
}
double SMPDG3DIPVariable::plasticEnergy() const
{
  return getIPSMP()->plasticEnergy();
}

void SMPDG3DIPVariable::restart()
{
  ThermoMechanicsDG3DIPVariableBase::restart();
  if (_TMIPSMP!= NULL)
    restartManager::restart(_TMIPSMP);
  return;
}
//




//PhenomenologicalSMP
PhenomenologicalSMPDG3DIPVariable::PhenomenologicalSMPDG3DIPVariable(double tinitial, const mlawPhenomenologicalSMP &_lawSMP, const bool oninter)
                                  :ThermoMechanicsDG3DIPVariableBase(oninter)
{   _TMIPSMP = new IPPhenomenologicalSMP(tinitial, _lawSMP.getTransitionTemperature(), _lawSMP.getTonsetg(), _lawSMP.getTonsetr(),
                                         _lawSMP.getAlphaTrans(), _lawSMP.getJ2IsotropicHardening());
    this->getRefToTemperature()=tinitial;
}

PhenomenologicalSMPDG3DIPVariable::PhenomenologicalSMPDG3DIPVariable(const PhenomenologicalSMPDG3DIPVariable &source)
                                  :ThermoMechanicsDG3DIPVariableBase(source)
{_TMIPSMP = NULL;
    if (source._TMIPSMP != NULL)
    {_TMIPSMP = dynamic_cast<IPPhenomenologicalSMP*>(source._TMIPSMP->clone());
    }
}

PhenomenologicalSMPDG3DIPVariable& PhenomenologicalSMPDG3DIPVariable::operator=(const IPVariable &source)
{
    ThermoMechanicsDG3DIPVariableBase::operator=(source);
    const PhenomenologicalSMPDG3DIPVariable* src = dynamic_cast<const PhenomenologicalSMPDG3DIPVariable*>(&source);
    if(src != NULL)
    {
        if (src->_TMIPSMP != NULL)
        {
            if (_TMIPSMP!=NULL)
            {
            _TMIPSMP->operator=(*dynamic_cast<const IPVariable*>(src->_TMIPSMP));}
            else
            _TMIPSMP = dynamic_cast<IPPhenomenologicalSMP*>(src->_TMIPSMP->clone());
        }
        else
        {
            if (_TMIPSMP!=NULL)
            {
                delete _TMIPSMP;
                _TMIPSMP = NULL;
            }
        }
    }
    return *this;
}

double PhenomenologicalSMPDG3DIPVariable::get(const int i) const
{   if(i==IPField::PLASTICSTRAIN)   {return _TMIPSMP->getRefToEquivalentPlasticDefo();}

    else if(i==IPField::FTH_YY)     {return _TMIPSMP->getConstRefToFth()(1,1);}
    else if(i==IPField::FTH_ZZ)     {return _TMIPSMP->getConstRefToFth()(2,2);}

    else if(i==IPField::FF_YY)      {return _TMIPSMP->getConstRefToFf()(1,1);}
    else if(i==IPField::FF_ZZ)      {return _TMIPSMP->getConstRefToFf()(2,2);}

    else if(i==IPField::FPG_YY)     {return _TMIPSMP->getConstRefToFpg()(1,1);}
    else if(i==IPField::FPG_ZZ)     {return _TMIPSMP->getConstRefToFpg()(2,2);}

    else if(i==IPField::FG_YY)      {return _TMIPSMP->getConstRefToFg()(1,1);}
    else if(i==IPField::FG_ZZ)      {return _TMIPSMP->getConstRefToFg()(2,2);}

    else if(i==IPField::FP_YY)      {return _TMIPSMP->getConstRefToFp()(1,1);}
    else if(i==IPField::FP_ZZ)      {return _TMIPSMP->getConstRefToFp()(2,2);}

    else if(i==IPField::FR_YY)      {return _TMIPSMP->getConstRefToFr()(1,1);}
    else if(i==IPField::FR_ZZ)      {return _TMIPSMP->getConstRefToFr()(2,2);}

    else if(i==IPField::ZG)         {return _TMIPSMP->getzg();}
    else if(i==IPField::TIG)        {return _TMIPSMP->getConstRefToTig();}
    else if(i==IPField::TIR)        {return _TMIPSMP->getConstRefToTir();}
    else if(i==IPField::TT)         {return _TMIPSMP->getConstRefToTt();}

    else                            {return ThermoMechanicsDG3DIPVariableBase::get(i);}
}

double PhenomenologicalSMPDG3DIPVariable::defoEnergy() const
{return getIPPhenomenologicalSMP()->defoEnergy();
}

double PhenomenologicalSMPDG3DIPVariable::plasticEnergy() const
{return getIPPhenomenologicalSMP()->plasticEnergy();
}

void PhenomenologicalSMPDG3DIPVariable::restart()
{   ThermoMechanicsDG3DIPVariableBase::restart();
    if (_TMIPSMP!= NULL)
        restartManager::restart(_TMIPSMP);
    return;
}
//PhenomenologicalSMP

//
crystalPlasticityDG3DIPVariable::crystalPlasticityDG3DIPVariable(int _nsdv, double size, const bool oninter) : dG3DIPVariable(oninter,0)
{
  _ipv = new IPCrystalPlasticity(_nsdv,size);
}

crystalPlasticityDG3DIPVariable::crystalPlasticityDG3DIPVariable(const crystalPlasticityDG3DIPVariable &source) :
                                                                   dG3DIPVariable(source)
{
  _ipv = NULL;
  if (source.getIPCrystalPlasticity() != NULL){
    _ipv = new IPCrystalPlasticity((int)source.getIPCrystalPlasticity()->getNsdv(),(double)source.getIPCrystalPlasticity()->elementSize());
    _ipv->operator= (*dynamic_cast<const IPVariable*>(source.getIPCrystalPlasticity()));
  }

}

crystalPlasticityDG3DIPVariable& crystalPlasticityDG3DIPVariable::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const crystalPlasticityDG3DIPVariable* src = dynamic_cast<const crystalPlasticityDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    if (src->getIPCrystalPlasticity() != NULL){
      if (_ipv != NULL){
        _ipv->operator=(*dynamic_cast<const IPVariable*>(src->getIPCrystalPlasticity()));
      }
      else{
        _ipv = dynamic_cast<IPCrystalPlasticity*>(src->getIPCrystalPlasticity()->clone());
      }
    }
    else{
      if (_ipv != NULL){
        delete _ipv;
        _ipv = NULL;
      }
    }
  }
  return *this;
}
double crystalPlasticityDG3DIPVariable::get(const int i) const
{
  double val = dG3DIPVariable::get(i);
  if (val == 0)
    val = getIPCrystalPlasticity()->get(i);
  return val;
}
double crystalPlasticityDG3DIPVariable::defoEnergy() const
{
  return getIPCrystalPlasticity()->defoEnergy();
}
double crystalPlasticityDG3DIPVariable::plasticEnergy() const
{
  return getIPCrystalPlasticity()->plasticEnergy();
}


void crystalPlasticityDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  if (_ipv != NULL)
    restartManager::restart(_ipv);
  return;
}

//

ElecTherMechDG3DIPVariableBase::ElecTherMechDG3DIPVariableBase( const bool oninter) : dG3DIPVariable(oninter,0,2,true,true)

{

}




ElecTherMechDG3DIPVariableBase::ElecTherMechDG3DIPVariableBase(const ElecTherMechDG3DIPVariableBase &source) :
                                                                        dG3DIPVariable(source)
{
  //Temperature    = source.Temperature;
  //gradT          = source.gradT;
  //fluxT          = source.fluxT;
  //dPdT           = source.dPdT;
  //dFluxTdGradT   = source.dFluxTdGradT ;
  //dFluxTdT       = source.dFluxTdT;
  //dFluxTdGradV   = source.dFluxTdGradV ;
  //dFluxTdV       = source.dFluxTdV;
  //dFluxTdF       = source.dFluxTdF;
  //TemperatureJump= source.TemperatureJump;
  //interfaceFluxT = source.interfaceFluxT;

  //Voltage        = source.Voltage;
  //gradV          = source.gradV;
  //fluxV          = source.fluxV;
  //dPdV           = source.dPdV;
  //dFluxVdGradV   = source.dFluxVdGradV ;
  //dFluxVdV       = source.dFluxVdV;
  //dFluxVdGradT   = source.dFluxVdGradT ;
  //dFluxVdT       = source.dFluxVdT;
  //dFluxVdF       = source.dFluxVdF;
  //VoltageJump    = source.VoltageJump;
  //interfaceFluxV = source.interfaceFluxV;
 //thermalSource  = source.thermalSource;
  //dThermalSourcedF = source.dThermalSourcedF;
  //dThermalSourcedT = source.dThermalSourcedT;


  //l10		= source.l10;
  //l20		= source.l20;
  //k10		= source.k10;
  //k20		= source.k20;

  //dl10dT	= source.dl10dT;
  //dl20dT	= source.dl20dT;
  //dk10dT	= source.dk10dT;
  //dk20dT	= source.dk20dT;

  //dl20dv	= source.dl20dv;
  //dk10dv	= source.dk10dv;

  //dl10dF	= source.dl10dF;
  //dl20dF	= source.dl20dF;
  //dk10dF	= source.dk10dF;
  //dk20dF	= source.dk20dF;

  //fvJump	= source.fvJump;
 // fTJump	= source.fTJump;
 // dfvjumpdvm	= source.dfvjumpdvm;
  //dfvjumpdvp	= source.dfvjumpdvp;
  //dfvjumpdTm	= source.dfvjumpdTm;
  //dfvjumpdTp	= source.dfvjumpdTp;
  //dfTjumpdTm	= source.dfTjumpdTm;
 // dfTjumpdTp	= source.dfTjumpdTp;


  //fT             = source.fT;
  //gradfT         = source.gradfT;
  //fV             = source.fV;
  //gradfV         = source.gradfV;

  //dgradVdgradfV	= source.dgradVdgradfV;
  //dgradVdgradfT	= source.dgradVdgradfT;
  //dgradTdgradfT	= source.dgradTdgradfT;
  //dgradTdgradfV	= source.dgradTdgradfV;
  //dVdfV		= source. dVdfV;
  //dVdfT		= source. dVdfT;
  //dTdfT		= source. dTdfT;
  //dTdfV		= source. dTdfV;
  //dgradVdfV	= source.dgradVdfV;
  //dgradVdfT	= source.dgradVdfT;
  //dgradTdfT	= source.dgradTdfT;
  //dgradTdfV	= source.dgradTdfV;


  //fluxjy	= source.fluxjy;
  //djydV 	= source.djydV;
  //djydgradV	= source.djydgradV;
  //djydT		= source.djydT;
  //djydgradT	= source.djydgradT;
  //djydF		= source.djydF;
  //jy1		= source.jy1;
  //djy1dT	= source.djy1dT;
  //djy1dV	= source.djy1dV;
  //djy1dF	= source.djy1dF;



}
ElecTherMechDG3DIPVariableBase& ElecTherMechDG3DIPVariableBase::operator=(const IPVariable &source)
{
  dG3DIPVariable::operator=(source);
  const ElecTherMechDG3DIPVariableBase* src = dynamic_cast<const ElecTherMechDG3DIPVariableBase*>(&source);
  if(src != NULL)
  {
  /*  lineark10      = src->lineark10;
    lineark20      = src->lineark20;*/
    //Temperature    = src->Temperature;
    //gradT          = src->gradT;
    //fluxT          = src->fluxT;
    //dPdT           = src-> dPdT;
    //dFluxTdGradT   = src->dFluxTdGradT;
    //dFluxTdT       = src->dFluxTdT;
    //dFluxTdGradV   = src->dFluxTdGradV;
    //dFluxTdV       = src->dFluxTdV;
    //dFluxTdF       = src->dFluxTdF;
    //TemperatureJump= src->TemperatureJump;
    //interfaceFluxT = src-> interfaceFluxT;

   /* linearl10      = src->linearl10;
    linearl20      = src->linearl20;*/
    //Voltage        = src->Voltage;
    //gradV          = src->gradV;
    //fluxV          = src->fluxV;
    //dPdV           = src->dPdV;
    //dFluxVdGradV   = src->dFluxVdGradV;
    //dFluxVdV       = src->dFluxVdV;
    //dFluxVdGradT   = src->dFluxVdGradT;
    //dFluxVdT       = src->dFluxVdT;
    //dFluxVdF       = src->dFluxVdF;
    //VoltageJump    = src->VoltageJump;
    //interfaceFluxV = src-> interfaceFluxV;
    //thermalSource  = src-> thermalSource;
    //dThermalSourcedF = src-> dThermalSourcedF;
    //dThermalSourcedT = src-> dThermalSourcedT;

    //l10 	= src->l10;
    //l20 	= src->l20;
    //k10 	= src->k10;
    //k20 	= src->k20;

    //dl10dT 	= src->dl10dT;
    //dl20dT 	= src->dl20dT;
    //dk10dT 	= src->dk10dT;
    //dk20dT 	= src->dk20dT;

    //dl20dv 	= src->dl20dv;
    //dk10dv 	= src->dk10dv;

    //dl10dF 	= src->dl10dF;
    //dl20dF 	= src->dl20dF;
    //dk10dF 	= src->dk10dF;
    //dk20dF 	= src->dk20dF;

    //fvJump	= src->fvJump;
    //fTJump	= src->fTJump;
    //dfvjumpdvm	= src->dfvjumpdvm;
    //dfvjumpdvp	= src->dfvjumpdvp;
    //dfvjumpdTm	= src->dfvjumpdTm;
    //dfvjumpdTp	= src->dfvjumpdTp;
    //dfTjumpdTm	= src->dfTjumpdTm;
    //dfTjumpdTp	= src->dfTjumpdTp;


    //fT             = src->fT;
    //gradfT         = src->gradfT;
    //fV             = src->fV;
    //gradfV         = src->gradfV;

    //dgradVdgradfV= src->dgradVdgradfV;
    //dgradVdgradfT= src->dgradVdgradfT;
    //dgradTdgradfT= src->dgradTdgradfT;
    //dgradTdgradfV= src->dgradTdgradfV;
    //dVdfV	= src-> dVdfV;
    //dVdfT	= src-> dVdfT;
    //dTdfT	= src-> dTdfT;
    //dTdfV	= src-> dTdfV;
    //dgradVdfV	= src->dgradVdfV;
    //dgradVdfT	= src->dgradVdfT;
    //dgradTdfT	= src->dgradTdfT;
    //dgradTdfV	= src->dgradTdfV;

    //fluxjy	= src->fluxjy;
    //djydV	=src->djydV;
    //djydgradV	= src->djydgradV;
    //djydT	= src->djydT;
    //djydgradT	= src->djydgradT;
    //djydF	= src->djydF;
    //jy1		= src->jy1;
    //djy1dT	=src->djy1dT;
    //djy1dV	=src->djy1dV;
    //djy1dF	=src->djy1dF;


  }
  return *this;

}
double ElecTherMechDG3DIPVariableBase::get(const int i) const
{
  if(i == IPField::TEMPERATURE)
  {
    return getConstRefToTemperature();
  }
  else if(i == IPField::THERMALFLUX_X)
  {
    return getConstRefToFlux()[0](0);
  }
  else if(i == IPField::THERMALFLUX_Y)
  {
    return getConstRefToFlux()[0](1);
  }
  else if(i == IPField::THERMALFLUX_Z)
  {
    return getConstRefToFlux()[0](2);
  }
  else if(i == IPField::VOLTAGE)
  {
    return getConstRefToVoltage();
  }
  else if(i == IPField::ELECTRICALFLUX_X)
  {
    return getConstRefToFlux()[1](0);
  }
  else if(i == IPField::ELECTRICALFLUX_Y)
  {
    return getConstRefToFlux()[1](1);
  }
  else if(i == IPField::ELECTRICALFLUX_Z)
  {
    return getConstRefToFlux()[1](2);
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}


void ElecTherMechDG3DIPVariableBase::restart()
{
  dG3DIPVariable::restart();
  //restartManager::restart(Temperature);
  //restartManager::restart(gradT);
  //restartManager::restart(fluxT);
  //restartManager::restart(dPdT);
  //restartManager::restart(dFluxTdGradT);
  //restartManager::restart(dFluxTdT);
  //restartManager::restart(dFluxTdGradV);
  //restartManager::restart(dFluxTdV);
  //restartManager::restart(dFluxTdF);
  ///const STensor3               *lineark10; // Nor save nor load
  ///const STensor3               *lineark20; // nor save nor load
  //restartManager::restart(TemperatureJump);
  //restartManager::restart(interfaceFluxT);

  //restartManager::restart(Voltage);
  //restartManager::restart(gradV);
  //restartManager::restart(fluxV);
  //restartManager::restart(dPdV);
  //restartManager::restart(dFluxVdGradV);
  //restartManager::restart(dFluxVdV);
  //restartManager::restart(dFluxVdGradV);
  //restartManager::restart(dFluxVdV);
  //restartManager::restart(dFluxVdGradT);
  //restartManager::restart(dFluxVdT);
  //restartManager::restart(dFluxVdF);
  ///const STensor3               *linearl10;  Nor save nor load
  ///const STensor3               *linearl20;  Nor save nor load
  //restartManager::restart(VoltageJump);
  //restartManager::restart(interfaceFluxV);
  //restartManager::restart(thermalSource);
  //restartManager::restart(dThermalSourcedT);
  //restartManager::restart(dThermalSourcedF);

  //modified
  //restartManager::restart(l10);
  //restartManager::restart(l20);
  //restartManager::restart(k10);
  //restartManager::restart(k20);
  //restartManager::restart(dl10dT);
  //restartManager::restart(dl20dT);
  //restartManager::restart(dk10dT);
  //restartManager::restart(dk20dT);
  //restartManager::restart(dk10dv);
  //restartManager::restart(dl20dv);

  //restartManager::restart(dl10dF);
  //restartManager::restart(dl20dF);
  //restartManager::restart(dk10dF);
  //restartManager::restart(dk20dF);

  //restartManager::restart(fvJump);
 //restartManager::restart(fTJump);
  //restartManager::restart(dfvjumpdvm);
  //restartManager::restart(dfvjumpdvp);
  //restartManager::restart(dfvjumpdTm);
  //restartManager::restart(dfvjumpdTp);
  //restartManager::restart(dfTjumpdTm);
  //restartManager::restart(dfTjumpdTp);

  //restartManager::restart(fT);
  //restartManager::restart(gradfT);
  //restartManager::restart(fV);
  //restartManager::restart(gradfV);

  //restartManager::restart(dgradVdgradfV);
  //restartManager::restart(dgradVdgradfT);
  //restartManager::restart(dgradTdgradfT);
  //restartManager::restart(dgradTdgradfV);
  //restartManager::restart(dVdfV);
  //restartManager::restart(dVdfT);
  //restartManager::restart(dTdfT);
  //restartManager::restart(dTdfV);
  //restartManager::restart(dgradVdfV);
  //restartManager::restart(dgradVdfT);
  //restartManager::restart(dgradTdfT);
  //restartManager::restart(dgradTdfV);

  //restartManager::restart(fluxjy);
  //restartManager::restart(djydV);
  //restartManager::restart(djydgradV);
  //restartManager::restart(djydT);
  //restartManager::restart(djydgradT);
  //restartManager::restart(djydF);
  //restartManager::restart(jy1);
  //restartManager::restart(djy1dT);
  //restartManager::restart(djy1dV);
  //restartManager::restart(djy1dF);


  return;
}

LinearElecTherMechDG3DIPVariable::LinearElecTherMechDG3DIPVariable( double tinitial,double vinitial,const bool oninter) :
                                   ElecTherMechDG3DIPVariableBase(oninter)
{

this->getRefToTemperature()=tinitial;
this->getRefToVoltage()=vinitial;
this->getRefTofT()         =1./tinitial;
this->getRefTofV()         =-vinitial/tinitial;
}


LinearElecTherMechDG3DIPVariable::LinearElecTherMechDG3DIPVariable(const LinearElecTherMechDG3DIPVariable &source) :
                                                        ElecTherMechDG3DIPVariableBase(source),
																												_linearETMIP(source._linearETMIP)
{

}

LinearElecTherMechDG3DIPVariable& LinearElecTherMechDG3DIPVariable::operator=(const IPVariable &source)
{
  ElecTherMechDG3DIPVariableBase::operator=(source);
  const LinearElecTherMechDG3DIPVariable* src = dynamic_cast<const LinearElecTherMechDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    _linearETMIP.operator=(*dynamic_cast<const IPVariable*>(&src->_linearETMIP));
  }
  return *this;
}
double LinearElecTherMechDG3DIPVariable::get(const int i) const
{

  return ElecTherMechDG3DIPVariableBase::get(i);
}

//should i add to ipField Voltage and  Vflux_X ,Y ,Z ????


double LinearElecTherMechDG3DIPVariable::defoEnergy() const
{
  return getIPLinearElecTherMech()->defoEnergy();
}
double LinearElecTherMechDG3DIPVariable::plasticEnergy() const
{
  return getIPLinearElecTherMech()->plasticEnergy();
}

void LinearElecTherMechDG3DIPVariable::restart()
{
  ElecTherMechDG3DIPVariableBase::restart();
  restartManager::restart(&_linearETMIP);
  return;
}

AnIsotropicElecTherMechDG3DIPVariable::AnIsotropicElecTherMechDG3DIPVariable(double tinitial,double vinitial,const mlawAnIsotropicElecTherMech &_lawETM,  const bool oninter) :
                                  ElecTherMechDG3DIPVariableBase(oninter)
{
  _ETMIP = new IPAnIsotropicElecTherMech();//_lawETM.getSa0(),_lawETM.getPhia0(),_lawETM.getSastar0());
  this->getRefToTemperature()=tinitial;
  this->getRefToVoltage()=vinitial;
  this->getRefTofT()         =1./tinitial;
  this->getRefTofV()         =-vinitial/tinitial;

}

AnIsotropicElecTherMechDG3DIPVariable::AnIsotropicElecTherMechDG3DIPVariable(const AnIsotropicElecTherMechDG3DIPVariable &source) :
                                                        ElecTherMechDG3DIPVariableBase(source)
{
  _ETMIP = NULL;
  if(source._ETMIP != NULL)
    _ETMIP = new IPAnIsotropicElecTherMech(*source._ETMIP);
}

AnIsotropicElecTherMechDG3DIPVariable& AnIsotropicElecTherMechDG3DIPVariable::operator=(const IPVariable &source)
{
  ElecTherMechDG3DIPVariableBase::operator=(source);
  const AnIsotropicElecTherMechDG3DIPVariable* src = dynamic_cast<const AnIsotropicElecTherMechDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    if (src->_ETMIP != NULL){
      if (_ETMIP != NULL){
        _ETMIP->operator=(*(dynamic_cast<const IPVariable*>(src->_ETMIP)));
      }
      else{
        _ETMIP = dynamic_cast<IPAnIsotropicElecTherMech*>(src->_ETMIP->clone());
      }
    }
    else{
      if (_ETMIP!=NULL){
        delete _ETMIP; _ETMIP = NULL;
      }
    }
  }
  return *this;
}
double AnIsotropicElecTherMechDG3DIPVariable::get(const int i) const
{
  return ElecTherMechDG3DIPVariableBase::get(i);
}

double AnIsotropicElecTherMechDG3DIPVariable::defoEnergy() const
{
  return getIPAnIsotropicElecTherMech()->defoEnergy();
}
double AnIsotropicElecTherMechDG3DIPVariable::plasticEnergy() const
{
  return getIPAnIsotropicElecTherMech()->plasticEnergy();
}

void AnIsotropicElecTherMechDG3DIPVariable::restart()
{
  ElecTherMechDG3DIPVariableBase::restart();
  restartManager::restart(_ETMIP);
  return;
}


ElecSMPDG3DIPVariable::ElecSMPDG3DIPVariable(double tinitial,double vinitial,const mlawElecSMP &_lawESMP,  const bool oninter) :
                                  ElecTherMechDG3DIPVariableBase(oninter)
{
  _ESMPIP = new IPElecSMP(_lawESMP.getSa0(),_lawESMP.getPhia0(),_lawESMP.getSastar0());
  this->getRefToTemperature()=tinitial;
  this->getRefToVoltage()=vinitial;
  this->getRefTofT()=1./tinitial;
  this->getRefTofV()=-vinitial/tinitial;

}

ElecSMPDG3DIPVariable::ElecSMPDG3DIPVariable(const ElecSMPDG3DIPVariable &source) :
                                                        ElecTherMechDG3DIPVariableBase(source)
{
  _ESMPIP = NULL;
  if(source._ESMPIP != NULL){
    _ESMPIP = new IPElecSMP(*source._ESMPIP);
  }
}

ElecSMPDG3DIPVariable& ElecSMPDG3DIPVariable::operator=(const IPVariable &source)
{
  ElecTherMechDG3DIPVariableBase::operator=(source);
  const ElecSMPDG3DIPVariable* src = dynamic_cast<const ElecSMPDG3DIPVariable*>(&source);
  if(src != NULL)
  {

    if (src->_ESMPIP != NULL){
      if (_ESMPIP != NULL){
        _ESMPIP->operator=(*(dynamic_cast<const IPVariable*>(src->_ESMPIP)));
      }
      else{
        _ESMPIP = dynamic_cast<IPElecSMP*>(src->_ESMPIP->clone());
      }
    }
    else{
      if (_ESMPIP != NULL){
        delete _ESMPIP;
        _ESMPIP = NULL;
      }
    }

  }
  return *this;
}
double ElecSMPDG3DIPVariable::get(const int i) const
{
  if(i == IPField::PLASTICSTRAIN)
  {
   // return getInternalEnergyExtraDofDiffusion();
    return getRefToEquivalentPlasticDefo1();
  }
  return ElecTherMechDG3DIPVariableBase::get(i);
}

double ElecSMPDG3DIPVariable::defoEnergy() const
{
  return getIPElecSMP()->defoEnergy();
}
double ElecSMPDG3DIPVariable::plasticEnergy() const
{
  return getIPElecSMP()->plasticEnergy();
}

void ElecSMPDG3DIPVariable::restart()
{
  ElecTherMechDG3DIPVariableBase::restart();
  restartManager::restart(_ESMPIP);
  return;
}
