//
// C++ Interface: ipvariable
//
// Description: Class with definition of ipvarible for dG3D
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef DG3DIPVARIABLE_H_
#define DG3DIPVARIABLE_H_
#include "ipvariable.h"
#include "STensor43.h"
#include "STensor33.h"
#include "ipJ2linear.h"
#include "ipVUMATinterface.h"
#include "ipTransverseIsotropic.h"
#include "ipTransverseIsoCurvature.h"
#include "ipTransverseIsoYarnB.h"
#include "ipAnisotropic.h"
#include "ipAnisotropicStoch.h"
#include "ipCluster.h"
#include "ipFiniteStrain.h"
#include "ipJ2ThermoMechanics.h"
#include "ipLinearThermoMechanics.h"
#include "ipSMP.h"
#include "ipEOS.h"
#include "ipCrystalPlasticity.h"
#include "ipViscoelastic.h"
#include "ipAnIsotropicElecTherMech.h"
#include "ipLinearElecTherMech.h"
#include "mlawJ2linear.h"
#include "mlawJ2VMSmallStrain.h"
#include "mlawJ2ThermoMechanics.h"
#include "mlawLocalDamageJ2Hyper.h"
#include "mlawLinearThermoMechanics.h"
#include "mlawSMP.h"
#include "mlawPhenomenologicalSMP.h"
#include "mlawViscoelastic.h"
#include "mlawLinearElecTherMech.h"
#include "mlawAnIsotropicElecTherMech.h"
#include "mlawElecSMP.h"
#include "ipElecSMP.h"
#include "ipCohesive.h"
#include "ipAnIsotropicTherMech.h"
#include "extraDofIPVariable.h"
#include "mlawHyperelasticDamage.h"
#include "ipHyperelastic.h"
#include "mlawHyperelastic.h"


class dG3DIPVariableBase : public ipFiniteStrain
{
 public:
  dG3DIPVariableBase() : ipFiniteStrain(){}
  dG3DIPVariableBase(const dG3DIPVariableBase &source);
  virtual dG3DIPVariableBase& operator=(const IPVariable &source);
  virtual ~dG3DIPVariableBase(){}

  virtual const bool isInterface() const=0;

  virtual IPVariable* getInternalData() =0;
  virtual const IPVariable* getInternalData()  const =0;

  // Archiving data
  virtual double get(const int i) const=0;
  virtual double defoEnergy() const=0;
  virtual double plasticEnergy() const=0;
  virtual double damageEnergy() const=0;
  virtual double vonMises() const=0;
  virtual void getCauchyStress(STensor3 &cauchy) const=0;
  virtual double getJ() const=0;
  virtual double getBarJ() const=0;
  virtual double &getRefToBarJ()=0;
  virtual double getLocalJ() const=0;
  virtual double &getRefToLocalJ()=0;
  virtual void getBackLocalDeformationGradient(STensor3& localF) const=0;

  virtual const STensor3 &getConstRefToDeformationGradient() const=0;
  virtual STensor3 &getRefToDeformationGradient()=0;

  virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStress() const=0;
  virtual STensor3 &getRefToFirstPiolaKirchhoffStress()=0;

  virtual const STensor43 &getConstRefToTangentModuli() const=0;
  virtual STensor43 &getRefToTangentModuli()=0;

  virtual const STensor43 &getConstRefToElasticTangentModuli() const = 0;
  virtual STensor43 &getRefToElasticTangentModuli() = 0;


  virtual const STensor43 &getConstRefToDGElasticTangentModuli() const=0;
  virtual void setRefToDGElasticTangentModuli(const STensor43 &eT)=0;

  virtual const SVector3 &getConstRefToReferenceOutwardNormal() const=0;
  virtual SVector3 &getRefToReferenceOutwardNormal()=0;

  virtual const SVector3 &getConstRefToCurrentOutwardNormal() const=0;
  virtual SVector3 &getRefToCurrentOutwardNormal()=0;

  virtual const SVector3 &getConstRefToJump() const=0;
  virtual SVector3 &getRefToJump()=0;

  // non local
  virtual int getNumberNonLocalVariable() const =0;
  //
  virtual double getConstRefToLocalVariable(const int idex) const = 0;
  virtual double& getRefToLocalVariable(const int idex) = 0;
  //
  virtual double getConstRefToNonLocalVariable(const int idex) const = 0;
  virtual double& getRefToNonLocalVariable(const int idex) = 0;

  virtual const std::vector<STensor3> &getConstRefToDLocalVariableDStrain() const = 0;
  virtual std::vector<STensor3> &getRefToDLocalVariableDStrain() = 0;

  virtual const std::vector<STensor3> &getConstRefToDStressDNonLocalVariable() const = 0;
  virtual std::vector<STensor3> &getRefToDStressDNonLocalVariable() = 0;

  virtual const fullMatrix<double> &getConstRefToDLocalVariableDNonLocalVariable() const = 0;
  virtual fullMatrix<double> &getRefToDLocalVariableDNonLocalVariable() = 0;

  virtual const fullVector<double> &getConstRefToNonLocalJump() const = 0;
  virtual fullVector<double> &getRefToNonLocalJump() = 0;

  virtual const std::vector<SVector3> &getConstRefToGradNonLocalVariable() const = 0;
  virtual std::vector<SVector3> &getRefToGradNonLocalVariable() = 0;
  virtual const fullMatrix<double> & getConstRefTodLocalVariableDExtraDofDiffusionField() const=0;
  virtual fullMatrix<double> & getRefTodLocalVariableDExtraDofDiffusionField()=0;

  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idx) const = 0;

  //-----------------get 1st piola before AV ------------//

  virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStressb4AV() const=0;
  virtual STensor3 &getRefToFirstPiolaKirchhoffStressb4AV()=0;

  virtual void setBulkCriticalDamage(const double DT){}; // set critical damage value
  virtual double getBulkCriticalDamage() const {return 1.;};

  virtual void setNonLocalToLocal(const bool fl){};
  virtual bool getNonLocalToLocal() const {return false;};

  // for multiple field formulation
  // incompatible jump
  virtual const SVector3& getConstRefToIncompatibleJump() const = 0;
  virtual SVector3& getRefToIncompatibleJump() = 0;

  // for path following based on  irreversibe energt
  virtual double irreversibleEnergy() const = 0;
  virtual double& getRefToIrreversibleEnergy() = 0;

  virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const = 0;
  virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() = 0;

  // irreversible ennergy depend on nonlocal variale
  virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const =0;
  virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) =0;



  //extraDof
  virtual int getNumConstitutiveExtraDofDiffusionVariable() const=0;
  virtual double getConstRefToField(const int idex) const=0;
  virtual double  &getRefToField(const int idex)=0;
  virtual const std::vector<SVector3>  &getConstRefToGradField() const=0;
  virtual std::vector<SVector3>  &getRefToGradField()=0;
  virtual const std::vector<SVector3>  &getConstRefToFlux() const=0;
  virtual std::vector<SVector3>  &getRefToFlux()=0;
  virtual const std::vector<STensor3>  &getConstRefTodPdField() const=0;
  virtual std::vector<STensor3>  &getRefTodPdField()=0;
  virtual const std::vector<STensor33>  &getConstRefTodPdGradField() const=0;
  virtual std::vector<STensor33>  &getRefTodPdGradField()=0;
  virtual const std::vector<std::vector<STensor3> >  &getConstRefTodFluxdGradField() const=0;
  virtual std::vector<std::vector<STensor3> >  &getRefTodFluxdGradField()=0;
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFluxdField() const=0;
  virtual std::vector<std::vector<SVector3> >  &getRefTodFluxdField()=0;
  virtual const std::vector<STensor33>  &getConstRefTodFluxdF() const=0;
  virtual std::vector<STensor33>   &getRefTodFluxdF()=0;
  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToLinearK() const =0;
  virtual void setConstRefToLinearK(const STensor3 &eT, int i, int j) =0;
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodLinearKdField() const=0;
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodLinearKdField()=0;
  virtual const std::vector< std::vector<STensor43> >  &getConstRefTodLinearKdF() const=0;
  virtual std::vector< std::vector<STensor43> >  &getRefTodLinearKdF()=0;
/*  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToEnergyK() const =0;
  virtual void setConstRefToEnergyK(const STensor3 &eT, int i, int j) =0;
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodEnergyKdField() const=0;
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodEnergyKdField()=0;
  virtual const std::vector< std::vector<STensor43> >  &getConstRefTodEnergyKdF() const=0;
  virtual std::vector<std::vector<STensor43>>  &getRefTodEnergyKdF()=0; */
  virtual const std::vector< std::vector<STensor43> >  &getConstRefTodFluxdGradFielddF() const=0;
  virtual std::vector<std::vector<STensor43> >  &getRefTodFluxdGradFielddF()=0;
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodFluxdGradFielddField() const=0;
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodFluxdGradFielddField()=0;
  virtual const fullVector<double>  &getConstRefToFieldSource() const=0;
  virtual fullVector<double>  &getRefToFieldSource()=0;
  virtual const fullMatrix<double>  &getConstRefTodFieldSourcedField() const=0;
  virtual fullMatrix<double>  &getRefTodFieldSourcedField()=0;
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFieldSourcedGradField() const=0;
  virtual std::vector<std::vector<SVector3> >  &getRefTodFieldSourcedGradField()=0;
  virtual const std::vector<STensor3>  &getConstRefTodFieldSourcedF() const=0;
  virtual std::vector<STensor3>  &getRefTodFieldSourcedF()=0;
  virtual const fullVector<double>  &getConstRefToMechanicalSource() const=0;
  virtual fullVector<double>  &getRefToMechanicalSource()=0;
  virtual const fullMatrix<double>  &getConstRefTodMechanicalSourcedField() const=0;
  virtual fullMatrix<double>  &getRefTodMechanicalSourcedField()=0;
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodMechanicalSourcedGradField() const=0;
  virtual std::vector<std::vector<SVector3> >  &getRefTodMechanicalSourcedGradField()=0;
  virtual const std::vector<STensor3>  &getConstRefTodMechanicalSourcedF() const=0;
  virtual std::vector<STensor3>  &getRefTodMechanicalSourcedF()=0;
  virtual const fullVector<double>  &getConstRefToExtraDofFieldCapacityPerUnitField() const=0;
  virtual fullVector<double>  &getRefToExtraDofFieldCapacityPerUnitField()=0;
  virtual const fullVector<double>  &getConstRefToFieldJump() const=0;
  virtual fullVector<double>  &getRefToFieldJump()=0;
  virtual const fullVector<double>  &getConstRefToOneOverFieldJump() const=0;
  virtual fullVector<double>  &getRefToOneOverFieldJump()=0;
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldm() const=0;
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldm()=0;
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldp() const=0;
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldp()=0;
  virtual const std::vector<SVector3>  &getConstRefToInterfaceFlux() const=0;
  virtual std::vector<SVector3>  &getRefToInterfaceFlux()=0;
  virtual const std::vector<const STensor3*> &getConstRefToLinearSymmetrizationCoupling() const =0;
  virtual void setConstRefToLinearSymmetrizationCoupling(const STensor3 &eT, int i) =0;

  virtual const std::vector<std::vector<SVector3> > & getConstRefTodFluxdNonLocalVariable() const=0;
  virtual std::vector<std::vector<SVector3> > & getRefTodFluxdNonLocalVariable()=0;
  virtual const fullMatrix<double> &getConstRefTodFieldSourcedNonLocalVariable() const=0;
  virtual const fullMatrix<double> &getRefTodFieldSourcedNonLocalVariable()=0;
  virtual const fullMatrix<double> &getConstRefTodMechanicalSourcedNonLocalVariable() const=0;
  virtual fullMatrix<double> &getRefTodMechanicalSourcedNonLocalVariable()=0;

  //Energy Conjugated Field (fT & fv)

  virtual double getConstRefToEnergyConjugatedField(const int idex) const=0;
  virtual double  &getRefToEnergyConjugatedField(const int idex)=0;
  virtual const std::vector<SVector3>  &getConstRefToGradEnergyConjugatedField() const=0;
  virtual std::vector<SVector3>  &getRefToGradEnergyConjugatedField()=0;
  virtual const  std::vector<std::vector<double> >  &getConstRefTodFielddEnergyConjugatedField() const=0;
  virtual std::vector<std::vector<double> >  &getRefTodFielddEnergyConjugatedField()=0;
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodGradFielddEnergyConjugatedField() const=0;
  virtual std::vector<std::vector<SVector3> >  &getRefTodGradFielddEnergyConjugatedField()=0;
  virtual const std::vector<std::vector<STensor3> >  &getConstRefTodGradFielddGradEnergyConjugatedField() const=0;
  virtual std::vector<std::vector<STensor3> >  &getRefTodGradFielddGradEnergyConjugatedField()=0;
  virtual const fullVector<double>  &getConstRefToEnergyConjugatedFieldJump() const=0;
  virtual fullVector<double>  &getRefToEnergyConjugatedFieldJump()=0;
  virtual const  std::vector<std::vector<double> >  &getConstRefTodEnergyConjugatedFieldJumpdFieldm() const=0;
  virtual std::vector<std::vector<double> >  &getRefTodEnergyConjugatedFieldJumpdFieldm()=0;
  virtual const  std::vector<std::vector<double> >  &getConstRefTodEnergyConjugatedFieldJumpdFieldp() const=0;
  virtual std::vector<std::vector<double> >  &getRefTodEnergyConjugatedFieldJumpdFieldp()=0;

  virtual void restart(){}
};

class nonlocalData{
  public:
    int numNonLocalVariable; // number of nonlocal variables
    int numConstitutiveExtraDofDiffusionVariable; //number of extra dof variables
    // for coupling
    std::vector<STensor3> dLocalVariableDStrain;
    std::vector<STensor3> dStressDNonLocalVariable;
    fullMatrix<double> dLocalVariableDNonLocalVariable;

    // for dg: jump in non local epl
    fullVector<double> nonLocalJump;
    std::vector<SVector3> gradNonLocalVariable;

    //coupling with constitutiveExtraDof
    fullMatrix<double> dLocalVariableDExtraDofDiffusionField;


  public:
    nonlocalData(const int numVer, const int numExtraDof=0);
    nonlocalData(const nonlocalData& src);
    nonlocalData& operator = (const nonlocalData& src);
    virtual ~nonlocalData(){};
    void restart();
    virtual int getNumNonLocalVariable() const {return numNonLocalVariable;}
    virtual int getConstitutiveExtraDofDiffusionVariable() const {return numConstitutiveExtraDofDiffusionVariable;}

};


class constitutiveExtraDofDiffusionData{
 public:
  int numConstitutiveExtraDofDiffusionVariable; //number of extra dof variables
  int numNonLocalVariable; //number of extra dof variables
  bool useOfEnergy;
  bool useOfEnergyConjugatedField;

  fullVector<double>           field;
  std::vector<SVector3>        gradField;
  std::vector<SVector3>        flux;
  std::vector<STensor3>        dPdField;
  std::vector<STensor33>       dPdGradField;
  std::vector<std::vector<STensor3> >        dFluxdGradField;
  std::vector<std::vector<SVector3> >        dFluxdField;
  std::vector<STensor33>       dFluxdF;
  std::vector<std::vector<const STensor3*> > linearK;
  fullVector<double>           fieldSource;  //related to cp*dT/dt
  fullMatrix<double>           dFieldSourcedField;
  std::vector<std::vector<SVector3> >        dFieldSourcedGradField;
  std::vector<STensor3>        dFieldSourcedF;

  fullVector<double>           mechanicalSource; // related to convert mechanical energy to heat
  fullMatrix<double>           dMechanicalSourcedField;
  std::vector<std::vector<SVector3> >        dMechanicalSourcedGradField;
  std::vector<STensor3>        dMechanicalSourcedF;

  fullVector<double>           fieldCapacityPerUnitField; // field capacity

  // for dg: jump in non local epl
  fullVector<double>           fieldJump;
  fullVector<double>           oneOverFieldJump;
  fullMatrix<double>           dOneOverFieldJumpdFieldm;
  fullMatrix<double>           dOneOverFieldJumpdFieldp;
  std::vector<SVector3>        interfaceFlux;
  std::vector< const STensor3* > linearSymmetrizationCoupling;

  //coupling with nonLocal
  std::vector<std::vector<SVector3> >       dFluxdNonLocalVariable;
  fullMatrix<double>          dFieldSourcedNonLocalVariable;
  fullMatrix<double>          dMechanicalSourcedNonLocalVariable;


  // Use of energy

 /* std::vector<std::vector<const STensor3*> > EnergyK;
  std::vector<std::vector<std::vector<STensor3>>> dEnergyKdField;
  std::vector<std::vector<STensor43>> dEnergyKdF;*/
  std::vector<std::vector<STensor43> > dlinearKdF;
  std::vector<std::vector<std::vector<STensor3> > > dlinearKdField;
  std::vector<std::vector<std::vector<STensor3> > > dFluxdGradFielddField;
  std::vector<std::vector<STensor43> > dFluxdGradFielddF;


  //use of conjugated fields (fT & fv)

  fullVector<double>           energyConjugatedField;
  std::vector<SVector3>        gradEnergyConjugatedField;
  std::vector<std::vector<double> >          dFielddEnergyConjugatedField;
  std::vector<std::vector<SVector3> >        dGradFielddEnergyConjugatedField;
  std::vector<std::vector<STensor3> >        dGradFielddGradEnergyConjugatedField;
  fullVector<double>           energyConjugatedFieldJump;
  std::vector<std::vector<double> >          dEnergyConjugatedFieldJumpdFieldm;
  std::vector<std::vector<double> >          dEnergyConjugatedFieldJumpdFieldp;




 public:
  constitutiveExtraDofDiffusionData(const int numVer, const int numNonLocal=0,   bool useOfEnergy = false, bool useOfEnergyConjugatedField=false);
  constitutiveExtraDofDiffusionData(const constitutiveExtraDofDiffusionData& src);
  constitutiveExtraDofDiffusionData& operator = (const constitutiveExtraDofDiffusionData& src);
  virtual ~constitutiveExtraDofDiffusionData(){};
  void restart();
  virtual int getConstitutiveExtraDofDiffusionVariable() const {return numConstitutiveExtraDofDiffusionVariable;}
  virtual int getNumNonLocalVariable() const {return numNonLocalVariable;}
  virtual bool useConjugatedField() const {return useOfEnergyConjugatedField;}

};



class dG3DIPVariable : public dG3DIPVariableBase
{
 private:
  double                       barJ;
  double                       localJ;
  STensor3                     deformationGradient; //we actually store (barJ/J)^1/3 F when using barF method
  STensor3                     firstPiolaKirchhoffStress;
  STensor43                    tangentModuli; //include geometric terms
  STensor43                    elasticTangentModuli; //elastic tangent operators by blocking all dissipation mechanism
  const STensor43              *DGElasticTangentModuli;

  // for dG
  bool _oninter;
  SVector3                     referenceOutwardNormal;
  SVector3                     currentOutwardNormal;

  SVector3                     jump;
  SVector3                     incompatibleJump;

  STensor3 firstPiolaKirchhoffStressb4AV;

protected:
    nonlocalData *_nonlocalData;
    constitutiveExtraDofDiffusionData *_constitutiveExtraDofDiffusionData;

 public:
  dG3DIPVariable(const bool oninter=false, int numNonLocal=0, int extraDofDiffusion=0, bool useOfEnergy=false, bool useOfEnergyConjugatedField=false); // A empty constructor is mandatory to build FractureCohesive3DIPVariable OK like this as the fracture is on interface??
  dG3DIPVariable(const dG3DIPVariable &source);
  virtual dG3DIPVariable &operator = (const IPVariable &_source);
  virtual ~dG3DIPVariable()
  {
    if(_nonlocalData!=NULL) delete _nonlocalData;
    if(_constitutiveExtraDofDiffusionData!=NULL) delete _constitutiveExtraDofDiffusionData;
  }
  virtual const bool isInterface() const {return _oninter;}

  virtual IPVariable* getInternalData() =0;
  virtual const IPVariable* getInternalData()  const =0;

  // Archiving data
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const{return 0.;};
  virtual double vonMises() const;
  virtual void getCauchyStress(STensor3 &cauchy) const;
  virtual double getJ() const;

  virtual double getBarJ() const {return barJ;};
  virtual double &getRefToBarJ() {return barJ;};
  virtual double getLocalJ() const {return localJ;};
  virtual double &getRefToLocalJ() {return localJ;};
  virtual void getBackLocalDeformationGradient(STensor3& localF) const;

  virtual const STensor3 &getConstRefToDeformationGradient() const {return deformationGradient;}
  virtual STensor3 &getRefToDeformationGradient() {return deformationGradient;}

  virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStress() const {return firstPiolaKirchhoffStress;}
  virtual STensor3 &getRefToFirstPiolaKirchhoffStress() {return firstPiolaKirchhoffStress;}

  virtual const STensor43 &getConstRefToTangentModuli() const {return tangentModuli;}
  virtual STensor43 &getRefToTangentModuli() {return tangentModuli;}

  virtual const STensor43 &getConstRefToElasticTangentModuli() const {return elasticTangentModuli;}
  virtual STensor43 &getRefToElasticTangentModuli() {return elasticTangentModuli;}


  virtual const STensor43 &getConstRefToDGElasticTangentModuli() const {return *DGElasticTangentModuli;}
  virtual void setRefToDGElasticTangentModuli(const STensor43 &eT) {
                                                                  DGElasticTangentModuli=&eT;
                                                                 }

  virtual const SVector3 &getConstRefToReferenceOutwardNormal() const {return referenceOutwardNormal;}
  virtual SVector3 &getRefToReferenceOutwardNormal() {return referenceOutwardNormal;}

  virtual const SVector3 &getConstRefToCurrentOutwardNormal() const {return currentOutwardNormal;}
  virtual SVector3 &getRefToCurrentOutwardNormal() {return currentOutwardNormal;}

  virtual const SVector3 &getConstRefToJump() const {return jump;}
  virtual SVector3 &getRefToJump() {return jump;}

  virtual const SVector3& getConstRefToIncompatibleJump() const {return incompatibleJump;};
  virtual SVector3& getRefToIncompatibleJump() {return incompatibleJump;};

  // non local

  virtual int getNumberNonLocalVariable() const
  {
    if(_nonlocalData==NULL)
      return 0;
    else
      return _nonlocalData->getNumNonLocalVariable();
  };
  //

  virtual double getConstRefToLocalVariable(const int idex) const {Msg::Error("getConstRefToLocalVariable not defined");};
  virtual double& getRefToLocalVariable(const int idex) {Msg::Error("getRefToLocalVariable not defined");};
  //
  virtual double getConstRefToNonLocalVariable(const int idex) const {Msg::Error("getConstRefToNonLocalVariable not defined");};
  virtual double& getRefToNonLocalVariable(const int idex) {Msg::Error("getRefToNonLocalVariable not defined");};

  virtual const std::vector<STensor3> &getConstRefToDLocalVariableDStrain() const
  {
    if(_nonlocalData==NULL)
      Msg::Error("getConstRefToDLocalVariableDStrain(): _nonlocalData not created");
    else
      return _nonlocalData->dLocalVariableDStrain;
  }
  virtual std::vector<STensor3> &getRefToDLocalVariableDStrain()
  {
    if(_nonlocalData==NULL)
      Msg::Error("getRefToDLocalVariableDStrain(): _nonlocalData not created");
    else
      return _nonlocalData->dLocalVariableDStrain;
  }

  virtual const std::vector<STensor3> &getConstRefToDStressDNonLocalVariable() const
  {
    if(_nonlocalData==NULL)
      Msg::Error("getConstRefToDStressDNonLocalVariable(): _nonlocalData not created");
    else
      return _nonlocalData->dStressDNonLocalVariable;
  }
  virtual std::vector<STensor3> &getRefToDStressDNonLocalVariable()
  {
    if(_nonlocalData==NULL)
      Msg::Error("getRefToDStressDNonLocalVariable(): _nonlocalData not created");
    else
      return _nonlocalData->dStressDNonLocalVariable;
  }

  virtual const fullMatrix<double> &getConstRefToDLocalVariableDNonLocalVariable() const
  {
    if(_nonlocalData==NULL)
      Msg::Error("getConstRefToDLocalVariableDNonLocalVariable(): _nonlocalData not created");
    else
      return _nonlocalData->dLocalVariableDNonLocalVariable;
  }
  virtual fullMatrix<double> &getRefToDLocalVariableDNonLocalVariable()
  {
    if(_nonlocalData==NULL)
      Msg::Error("getRefToDLocalVariableDNonLocalVariable(): _nonlocalData not created");
    else
      return _nonlocalData->dLocalVariableDNonLocalVariable;
  }

  virtual const fullVector<double> &getConstRefToNonLocalJump() const
  {
    if(_nonlocalData==NULL)
      Msg::Error("getConstRefToNonLocalJump(): _nonlocalData not created");
    else
      return _nonlocalData->nonLocalJump;
  }
  virtual fullVector<double> &getRefToNonLocalJump()
  {
    if(_nonlocalData==NULL)
      Msg::Error("getRefToNonLocalJump(): _nonlocalData not created");
    else
      return _nonlocalData->nonLocalJump;
  }

  virtual const std::vector<SVector3> &getConstRefToGradNonLocalVariable() const
  {
    if(_nonlocalData==NULL)
      Msg::Error("getConstRefToGradNonLocalVariable(): _nonlocalData not created");
    else
      return _nonlocalData->gradNonLocalVariable;
  }
  virtual std::vector<SVector3> &getRefToGradNonLocalVariable()
  {
    if(_nonlocalData==NULL)
      Msg::Error("getRefToGradNonLocalVariable(): _nonlocalData not created");
    else
      return _nonlocalData->gradNonLocalVariable;
  }
  //coupling non local with constitutiveExtraDof
  virtual const fullMatrix<double> & getConstRefTodLocalVariableDExtraDofDiffusionField() const
  {
    if(_nonlocalData==NULL)
      Msg::Error("getConstRefTodLocalVariableDExtraDofDiffusionField(): _nonlocalData not created");
    else
      return _nonlocalData->dLocalVariableDExtraDofDiffusionField;
  }
  virtual fullMatrix<double> & getRefTodLocalVariableDExtraDofDiffusionField()
  {
    if(_nonlocalData==NULL)
      Msg::Error("getRefTodLocalVariableDExtraDofDiffusionField(): _nonlocalData not created");
    else
      return _nonlocalData->dLocalVariableDExtraDofDiffusionField;
  }

  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idx) const {Msg::Error("getConstRefToCharacteristicLengthMatrix not defined"); };

  //-----------------get 1st piola before AV ------------//
  virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStressb4AV() const {return firstPiolaKirchhoffStressb4AV;}
  virtual STensor3 &getRefToFirstPiolaKirchhoffStressb4AV() {return firstPiolaKirchhoffStressb4AV;}

  // for path following based on  irreversibe energt
  virtual double irreversibleEnergy() const {return 0.;};
  virtual double& getRefToIrreversibleEnergy() {
    Msg::Error("dG3DIPVariable::getRefToIrreversibleEnergy has not been implemented");
  };

  virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {
    Msg::Error("dG3DIPVariable::getConstRefToDIrreversibleEnergyDDeformationGradient has not been implemented");
  };
  virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {
    Msg::Error("dG3DIPVariable::getConstRefToDIrreversibleEnergyDDeformationGradient has not been implemented");
  };

  // irreversible ennergy depend on nonlocal variale
  virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const {
    Msg::Error("dG3DIPVariable::getConstRefToDIrreversibleEnergyDNonLocalVariable has not been implemented");
  };
  virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) {
    Msg::Error("dG3DIPVariable::getRefToDIrreversibleEnergyDDNonLocalVariable has not been implemented");
  };
  //
  // Extra Dof

  virtual int getNumConstitutiveExtraDofDiffusionVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      return 0;
    else
      return _constitutiveExtraDofDiffusionData->getConstitutiveExtraDofDiffusionVariable();
  }

  virtual double getConstRefToField(const int idex) const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->field(idex);
  }
  virtual double  &getRefToField(const int idex)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->field(idex);
  }
  virtual const std::vector<SVector3>  &getConstRefToGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradField;
  }
  virtual std::vector<SVector3>  &getRefToGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradField;
  }
  virtual const std::vector<SVector3>  &getConstRefToFlux() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToFlux() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->flux;
  }
  virtual std::vector<SVector3>  &getRefToFlux()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToFlux() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->flux;
  }
  virtual const std::vector<STensor3>  &getConstRefTodPdField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodPdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdField;
  }
  virtual std::vector<STensor3>  &getRefTodPdField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodPdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdField;
  }
  virtual const std::vector<STensor33>  &getConstRefTodPdGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodPdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdGradField;
  }
  virtual std::vector<STensor33>  &getRefTodPdGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodPdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdGradField;
  }
  virtual const std::vector< std::vector<STensor3> >  &getConstRefTodFluxdGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradField;
  }
  virtual std::vector<std::vector<STensor3> >  &getRefTodFluxdGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradField;
  }
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFluxdField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdField;
  }
  virtual std::vector<std::vector<SVector3> >  &getRefTodFluxdField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdField;
  }
  virtual const std::vector<STensor33>   &getConstRefTodFluxdF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdF;
  }
  virtual std::vector<STensor33>   &getRefTodFluxdF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdF;
  }
  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToLinearK() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTolinearK() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->linearK;
  }
  virtual void setConstRefToLinearK(const STensor3 &eT, int i, int j)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("setConstRefTolinearK() : _constitutiveExtraDofDiffusionData not created");
    else
    {
      if(i < getNumConstitutiveExtraDofDiffusionVariable() && j< getNumConstitutiveExtraDofDiffusionVariable())
         _constitutiveExtraDofDiffusionData->linearK[i][j] = &eT;
      else
         Msg::Error("setConstRefTolinearK() : index out of range");
    }
  }
/*  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToEnergyK() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyK() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->EnergyK;
  }
  virtual void setConstRefToEnergyK(const STensor3 &eT, int i, int j)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("setConstRefToEnergyK() : _constitutiveExtraDofDiffusionData not created");
    else
    {
      if(i < getNumConstitutiveExtraDofDiffusionVariable() && j< getNumConstitutiveExtraDofDiffusionVariable())
         _constitutiveExtraDofDiffusionData->EnergyK[i][j] = &eT;
      else
         Msg::Error("setConstRefToEnergyK() : index out of range");
    }
  }
  virtual const std::vector< std::vector<std::vector<STensor3> > >  &getConstRefTodEnergyKdField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodEnergyKdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyKdField;
  }
  virtual std::vector< std::vector<std::vector<STensor3> > >  &getRefTodEnergyKdField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodEnergyKdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyKdField;
  }
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodEnergyKdF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodEnergyKdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyKdF;
  }
  virtual std::vector<std::vector<STensor43>>  &getRefTodEnergyKdF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodEnergyKdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyKdF;
  }*/
  virtual const std::vector< std::vector<std::vector<STensor3> > >  &getConstRefTodFluxdGradFielddField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdGradFielddField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradFielddField;
  }
  virtual std::vector< std::vector<std::vector<STensor3> > >  &getRefTodFluxdGradFielddField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdGradFielddField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradFielddField;
  }
  virtual const std::vector<std::vector<STensor43> >  &getConstRefTodFluxdGradFielddF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdGradFielddF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradFielddF;
  }
  virtual std::vector<std::vector<STensor43> >  &getRefTodFluxdGradFielddF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdGradFielddF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradFielddF;
  }
  virtual const std::vector< std::vector< std::vector<STensor3> > >  &getConstRefTodLinearKdField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodLinearKdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dlinearKdField;
  }
  virtual std::vector< std::vector< std::vector<STensor3> > >  &getRefTodLinearKdField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodLinearKdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dlinearKdField;
  }
  virtual const std::vector< std::vector<STensor43> >  &getConstRefTodLinearKdF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodLinearKdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dlinearKdF;
  }
  virtual std::vector< std::vector<STensor43> >  &getRefTodLinearKdF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodLinearKdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dlinearKdF;
  }
  virtual const fullVector<double>  &getConstRefToFieldSource() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToFieldSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldSource;
  }
  virtual fullVector<double>  &getRefToFieldSource()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToFieldSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldSource;
  }
  virtual const fullMatrix<double>  &getConstRefTodFieldSourcedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFieldSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedField;
  }
  virtual fullMatrix<double>  &getRefTodFieldSourcedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedField;
  }
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFieldSourcedGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedGradField;
  }
  virtual std::vector<std::vector<SVector3> >  &getRefTodFieldSourcedGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedGradField;
  }
  virtual const std::vector<STensor3>  &getConstRefTodFieldSourcedF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFieldSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedF;
  }
  virtual std::vector<STensor3>  &getRefTodFieldSourcedF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedF;
  }
  virtual const fullVector<double>  &getConstRefToMechanicalSource() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToMechanicalSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->mechanicalSource;
  }
  virtual fullVector<double>  &getRefToMechanicalSource()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToMechanicalSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->mechanicalSource;
  }
  virtual const fullMatrix<double>  &getConstRefTodMechanicalSourcedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedField;
  }
  virtual fullMatrix<double>  &getRefTodMechanicalSourcedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedField;
  }
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodMechanicalSourcedGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedGradField;
  }
  virtual std::vector<std::vector<SVector3> >  &getRefTodMechanicalSourcedGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedGradField;
  }
  virtual const std::vector<STensor3>  &getConstRefTodMechanicalSourcedF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedF;
  }
  virtual std::vector<STensor3>  &getRefTodMechanicalSourcedF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedF;
  }
  virtual const fullVector<double>  &getConstRefToExtraDofFieldCapacityPerUnitField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToExtraDofFieldCapacityPerUnitField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldCapacityPerUnitField;
  }
  virtual fullVector<double>  &getRefToExtraDofFieldCapacityPerUnitField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToExtraDofFieldCapacityPerUnitField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldCapacityPerUnitField;
  }
  virtual const fullVector<double>  &getConstRefToFieldJump() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldJump;
  }
  virtual fullVector<double>  &getRefToFieldJump()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldJump;
  }
  virtual const fullVector<double>  &getConstRefToOneOverFieldJump() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToOneOverFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->oneOverFieldJump;
  }
  virtual fullVector<double>  &getRefToOneOverFieldJump()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToOneOverFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->oneOverFieldJump;
  }
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldm() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodOneOverFieldJumpdFieldm : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldm;
  }
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldm()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodOneOverFieldJumpdFieldm() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldm;
  }
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldp() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodOneOverFieldJumpdFieldp : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldp;
  }
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldp()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodOneOverFieldJumpdFieldp() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldp;
  }
  virtual const std::vector<SVector3>  &getConstRefToInterfaceFlux() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToInterfaceFlux : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->interfaceFlux;
  }
  virtual std::vector<SVector3>  &getRefToInterfaceFlux()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToInterfaceFlux() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->interfaceFlux;
  }
  virtual const std::vector<const STensor3*> &getConstRefToLinearSymmetrizationCoupling() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToLinearSymmetrizationCoupling() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->linearSymmetrizationCoupling;
  }
  virtual void setConstRefToLinearSymmetrizationCoupling(const STensor3 &eT, int i)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("setConstRefToLinearSymmetrizationCoupling() : _constitutiveExtraDofDiffusionData not created");
    else
    {
      if(i < getNumConstitutiveExtraDofDiffusionVariable())
         _constitutiveExtraDofDiffusionData->linearSymmetrizationCoupling[i] = &eT;
      else
         Msg::Error("setConstRefToLinearSymmetrizationCoupling() : index out of range");
    }
  }
  //coupling constitutive extra dof with nonLocal
  virtual const std::vector<std::vector<SVector3> > & getConstRefTodFluxdNonLocalVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdNonLocalVariable;
  }
  virtual std::vector<std::vector<SVector3> > & getRefTodFluxdNonLocalVariable()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdNonLocalVariable;
  }
  virtual const fullMatrix<double> &getConstRefTodFieldSourcedNonLocalVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFieldSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedNonLocalVariable;
  }
  virtual fullMatrix<double> &getRefTodFieldSourcedNonLocalVariable()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedNonLocalVariable;
  }
  virtual const fullMatrix<double> &getConstRefTodMechanicalSourcedNonLocalVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedNonLocalVariable;
  }
  virtual fullMatrix<double> &getRefTodMechanicalSourcedNonLocalVariable()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedNonLocalVariable;
  }

  //Energy Conjugated Fiedl (fT & fv)

  virtual double getConstRefToEnergyConjugatedField(const int idex) const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedField(idex);
  }
  virtual double   &getRefToEnergyConjugatedField(const int idex)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedField(idex);
  }
  virtual const std::vector<SVector3>  &getConstRefToGradEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradEnergyConjugatedField;
  }
  virtual std::vector<SVector3>  &getRefToGradEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradEnergyConjugatedField;
  }
  virtual const  std::vector< std::vector<double> >  &getConstRefTodFielddEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFielddEnergyConjugatedField;
  }
  virtual std::vector< std::vector<double> >  &getRefTodFielddEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFielddEnergyConjugatedField;
  }
  virtual const std::vector< std::vector<SVector3> >  &getConstRefTodGradFielddEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodGradFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddEnergyConjugatedField;
  }
  virtual std::vector< std::vector<SVector3> >  &getRefTodGradFielddEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodGradFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddEnergyConjugatedField;
  }
  virtual const std::vector< std::vector<STensor3> >  &getConstRefTodGradFielddGradEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodGradFielddGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddGradEnergyConjugatedField;
  }
  virtual std::vector< std::vector<STensor3> >  &getRefTodGradFielddGradEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodGradFielddGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddGradEnergyConjugatedField;
  }
  virtual const fullVector<double>  &getConstRefToEnergyConjugatedFieldJump() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyConjugatedFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedFieldJump;
  }
  virtual fullVector<double>  &getRefToEnergyConjugatedFieldJump()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToEnergyConjugatedFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedFieldJump;
  }
  virtual const  std::vector< std::vector<double> >  &getConstRefTodEnergyConjugatedFieldJumpdFieldm() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodEnergyConjugatedFieldJumpdFieldm() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldm;
  }
  virtual std::vector< std::vector<double> >  &getRefTodEnergyConjugatedFieldJumpdFieldm()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodEnergyConjugatedFieldJumpdFieldm() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldm;
  }
  virtual const  std::vector< std::vector<double> >  &getConstRefTodEnergyConjugatedFieldJumpdFieldp() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodEnergyConjugatedFieldJumpdFieldp() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldp;
  }
  virtual std::vector< std::vector<double> >  &getRefTodEnergyConjugatedFieldJumpdFieldp()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodEnergyConjugatedFieldJumpdFieldp() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldp;
  }


  virtual IPVariable* clone() const =0;
  virtual void restart();
};

class LinearElasticDG3DIPVariable :public dG3DIPVariable{
  protected:

  public:
    LinearElasticDG3DIPVariable(const bool oninter=false):dG3DIPVariable(oninter){}; // A empty constructor is mandatory to build FractureCohesive3DIPVariable OK like this as the fracture is on interface??
    LinearElasticDG3DIPVariable(const LinearElasticDG3DIPVariable &source):dG3DIPVariable(source){};
    virtual LinearElasticDG3DIPVariable &operator = (const IPVariable &_source){
      dG3DIPVariable::operator =(_source);
    };
    virtual ~LinearElasticDG3DIPVariable()
    {

    }

    virtual IPVariable* getInternalData() {return NULL;};
    virtual const IPVariable* getInternalData()  const {return NULL;};

    virtual void getCauchyStress(STensor3 &cauchy) const{
      cauchy= this->getConstRefToFirstPiolaKirchhoffStress();
    };
    virtual IPVariable* clone() const {return new LinearElasticDG3DIPVariable(*this);};
};

class J2SmallStrainDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPJ2linear *_j2ipv;
 public:
  J2SmallStrainDG3DIPVariable(const mlawJ2VMSmallStrain &_j2law, const bool oninter=false);
  J2SmallStrainDG3DIPVariable(const J2SmallStrainDG3DIPVariable &source);
  virtual J2SmallStrainDG3DIPVariable& operator=(const IPVariable &source);
  virtual ~J2SmallStrainDG3DIPVariable(){if(_j2ipv!=NULL) {delete _j2ipv; _j2ipv=NULL;}};

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_j2ipv)
      _j2ipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _j2ipv;};
  virtual const IPVariable* getInternalData()  const {return _j2ipv;};

 /* specific function */
  IPJ2linear* getIPJ2linear(){return _j2ipv;}
  const IPJ2linear* getIPJ2linear() const{return _j2ipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _j2ipv->damageEnergy();};

  virtual bool dissipationIsActive() const {return _j2ipv->dissipationIsActive();};
  virtual void blockDissipation(const bool fl){   _j2ipv->blockDissipation(fl);}
  virtual bool dissipationIsBlocked() const { return _j2ipv->dissipationIsBlocked();}

  // for path following based on  irreversibe energt
  virtual double irreversibleEnergy() const {return _j2ipv->irreversibleEnergy();};
  virtual double& getRefToIrreversibleEnergy() {return _j2ipv->getRefToIrreversibleEnergy();};

  virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _j2ipv->getConstRefToDIrreversibleEnergyDF();};
  virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _j2ipv->getRefToDIrreversibleEnergyDF();};

  virtual IPVariable* clone() const {return new J2SmallStrainDG3DIPVariable(*this);};
  virtual void restart();
};

class J2LinearDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPJ2linear *_j2ipv;
 public:
  J2LinearDG3DIPVariable(const mlawJ2linear &_j2law, const bool oninter=false);
  J2LinearDG3DIPVariable(const J2LinearDG3DIPVariable &source);
  virtual J2LinearDG3DIPVariable& operator=(const IPVariable &source);
  virtual ~J2LinearDG3DIPVariable(){if(_j2ipv!=NULL) {delete _j2ipv; _j2ipv=NULL;}};

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_j2ipv)
      _j2ipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _j2ipv;};
  virtual const IPVariable* getInternalData()  const {return _j2ipv;};

 /* specific function */
  IPJ2linear* getIPJ2linear(){return _j2ipv;}
  const IPJ2linear* getIPJ2linear() const{return _j2ipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _j2ipv->damageEnergy();};

  virtual bool dissipationIsActive() const {return _j2ipv->dissipationIsActive();};
  virtual void blockDissipation(const bool fl){   _j2ipv->blockDissipation(fl);}
  virtual bool dissipationIsBlocked() const { return _j2ipv->dissipationIsBlocked();}

  // for path following based on  irreversibe energt
  virtual double irreversibleEnergy() const {return _j2ipv->irreversibleEnergy();};
  virtual double& getRefToIrreversibleEnergy() {return _j2ipv->getRefToIrreversibleEnergy();};

  virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _j2ipv->getConstRefToDIrreversibleEnergyDF();};
  virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _j2ipv->getRefToDIrreversibleEnergyDF();};

  virtual IPVariable* clone() const {return new J2LinearDG3DIPVariable(*this);};
  virtual void restart();
};

class HyperViscoElasticdG3DIPVariable : public dG3DIPVariable{
  protected:
    IPHyperViscoElastic * _ipViscoElastic;

public:
    HyperViscoElasticdG3DIPVariable(const mlawHyperViscoElastic& viscoLaw, const bool oninter=false);
    HyperViscoElasticdG3DIPVariable(const HyperViscoElasticdG3DIPVariable& src);
    virtual HyperViscoElasticdG3DIPVariable& operator = (const IPVariable& src);
    virtual ~HyperViscoElasticdG3DIPVariable();

    virtual IPVariable* getInternalData() {return _ipViscoElastic;};
    virtual const IPVariable* getInternalData()  const {return _ipViscoElastic;};

    IPHyperViscoElastic* getIPHyperViscoElastic() {return _ipViscoElastic;};
    const IPHyperViscoElastic* getIPHyperViscoElastic() const {return _ipViscoElastic;};

    virtual void setLocation(const IPVariable::LOCATION loc) {
      dG3DIPVariable::setLocation(loc);
      if (_ipViscoElastic)
        _ipViscoElastic->setLocation(loc);
    };

    // for path following based on  irreversibe energt
    virtual double irreversibleEnergy() const {return _ipViscoElastic->irreversibleEnergy();};
    virtual double& getRefToIrreversibleEnergy() {return _ipViscoElastic->getRefToIrreversibleEnergy();};

    virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _ipViscoElastic->getConstRefToDIrreversibleEnergyDF();};
    virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _ipViscoElastic->getRefToDIrreversibleEnergyDF();};

    virtual double defoEnergy() const;

    virtual IPVariable* clone() const {return new HyperViscoElasticdG3DIPVariable(*this);};
    virtual void restart();
};

class HyperViscoElastoPlasticdG3DIPVariable : public dG3DIPVariable{
  protected:
    IPHyperViscoElastoPlastic* _ipViscoElastoPlastic;

  public:
    HyperViscoElastoPlasticdG3DIPVariable(const mlawPowerYieldHyper& viscoEPLaw, const bool oninter);
    HyperViscoElastoPlasticdG3DIPVariable(const HyperViscoElastoPlasticdG3DIPVariable& src);
    virtual HyperViscoElastoPlasticdG3DIPVariable& operator = (const IPVariable& src);
    virtual ~HyperViscoElastoPlasticdG3DIPVariable();

    virtual IPVariable* getInternalData() {return _ipViscoElastoPlastic;};
    virtual const IPVariable* getInternalData()  const {return _ipViscoElastoPlastic;};

    IPHyperViscoElastoPlastic* getIPHyperViscoElastoPlastic() {return _ipViscoElastoPlastic;}
    const IPHyperViscoElastoPlastic* getIPHyperViscoElastoPlastic() const {return _ipViscoElastoPlastic;}

    virtual void setLocation(const IPVariable::LOCATION loc) {
      dG3DIPVariable::setLocation(loc);
      if (_ipViscoElastoPlastic)
        _ipViscoElastoPlastic->setLocation(loc);
    };

    virtual double get(const int i) const;

    virtual bool dissipationIsActive() const {return _ipViscoElastoPlastic->dissipationIsActive();};
    virtual void blockDissipation(const bool fl){   _ipViscoElastoPlastic->blockDissipation(fl);}
    virtual bool dissipationIsBlocked() const { return _ipViscoElastoPlastic->dissipationIsBlocked();}

    // for path following based on  irreversibe energt
    virtual double irreversibleEnergy() const {return _ipViscoElastoPlastic->irreversibleEnergy();};
    virtual double& getRefToIrreversibleEnergy() {return _ipViscoElastoPlastic->getRefToIrreversibleEnergy();};

    virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _ipViscoElastoPlastic->getConstRefToDIrreversibleEnergyDF();};
    virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _ipViscoElastoPlastic->getRefToDIrreversibleEnergyDF();};

    virtual double defoEnergy() const {return _ipViscoElastoPlastic->defoEnergy();};
    virtual double plasticEnergy() const {return _ipViscoElastoPlastic->plasticEnergy();};
    virtual double damageEnergy() const {return _ipViscoElastoPlastic->damageEnergy();};

    virtual IPVariable* clone() const {return new HyperViscoElastoPlasticdG3DIPVariable(*this);};
    virtual void restart();
};

class localDamageJ2HyperDG3DIPVariable : public dG3DIPVariable{

 protected:
  IPLocalDamageJ2Hyper *_nldJ2Hyperipv;

 public:
  localDamageJ2HyperDG3DIPVariable(const mlawLocalDamageJ2Hyper &_j2law, const bool oninter=false);
  localDamageJ2HyperDG3DIPVariable(const localDamageJ2HyperDG3DIPVariable &source);
  virtual localDamageJ2HyperDG3DIPVariable& operator=(const IPVariable &source);

  virtual IPVariable* getInternalData() {return _nldJ2Hyperipv;};
  virtual const IPVariable* getInternalData()  const {return _nldJ2Hyperipv;};

 /* specific function */
  IPLocalDamageJ2Hyper* getIPLocalDamageJ2Hyper(){return _nldJ2Hyperipv;}
  const IPLocalDamageJ2Hyper* getIPLocalDamageJ2Hyper() const{return _nldJ2Hyperipv;}
  virtual ~localDamageJ2HyperDG3DIPVariable()
  {
    if (_nldJ2Hyperipv) delete _nldJ2Hyperipv;
    _nldJ2Hyperipv = NULL;
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_nldJ2Hyperipv)
      _nldJ2Hyperipv->setLocation(loc);
  };

  virtual bool dissipationIsActive() const {return _nldJ2Hyperipv->dissipationIsActive();};
  virtual void blockDissipation(const bool fl){   _nldJ2Hyperipv->blockDissipation(fl);}
  virtual bool dissipationIsBlocked() const { return _nldJ2Hyperipv->dissipationIsBlocked();}

  virtual void setBulkCriticalDamage(const double DT){_nldJ2Hyperipv->setCriticalDamage(DT);}; // set critical damage value
  virtual double getBulkCriticalDamage() const {return _nldJ2Hyperipv->getCriticalDamage();};

  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _nldJ2Hyperipv->damageEnergy();};

	// for path following based on  irreversibe energt
	virtual double irreversibleEnergy() const {return _nldJ2Hyperipv->irreversibleEnergy();};
	virtual double& getRefToIrreversibleEnergy() {return _nldJ2Hyperipv->getRefToIrreversibleEnergy();};

	virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _nldJ2Hyperipv->getConstRefToDIrreversibleEnergyDF();};
	virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _nldJ2Hyperipv->getRefToDIrreversibleEnergyDF();};

  virtual IPVariable* clone() const {return new localDamageJ2HyperDG3DIPVariable(*this);};
  virtual void restart();
};

class localDamageIsotropicElasticityDG3DIPVariable : public dG3DIPVariable
{

 protected:
  IPLocalDamageIsotropicElasticity *_localIPV;

 public:
	localDamageIsotropicElasticityDG3DIPVariable(const mlawLocalDamageHyperelastic & elasticLaw, const bool oninter=false);
  localDamageIsotropicElasticityDG3DIPVariable(const localDamageIsotropicElasticityDG3DIPVariable &source);
  virtual localDamageIsotropicElasticityDG3DIPVariable& operator=(const IPVariable &source);

  virtual IPVariable* getInternalData() {return _localIPV;};
  virtual const IPVariable* getInternalData()  const {return _localIPV;};

 /* specific function */
  IPLocalDamageIsotropicElasticity* getIPLocalDamageIsotropicElasticity(){return _localIPV;}
  const IPLocalDamageIsotropicElasticity* getIPLocalDamageIsotropicElasticity() const{return _localIPV;}
  virtual ~localDamageIsotropicElasticityDG3DIPVariable()
  {
    if (_localIPV){
     delete _localIPV;
     _localIPV = NULL;
    }
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_localIPV)
      _localIPV->setLocation(loc);
  };

  virtual bool dissipationIsActive() const {return _localIPV->dissipationIsActive();};

  virtual void blockDissipation(const bool fl){
    if (_localIPV) _localIPV->blockDissipation(fl);
  }
  virtual bool dissipationIsBlocked() const {
    if (_localIPV) return _localIPV->dissipationIsBlocked();
    else return false;
  }

  virtual void setBulkCriticalDamage(const double DT){_localIPV->setCriticalDamage(DT);}; // set critical damage value
  virtual double getBulkCriticalDamage() const {return _localIPV->getCriticalDamage();};

  virtual double get(const int i) const;
  virtual double defoEnergy() const {return _localIPV->defoEnergy();};
  virtual double plasticEnergy() const{return _localIPV->plasticEnergy();};
  virtual double damageEnergy() const {return _localIPV->damageEnergy();};

  // for path following based on irreversibe energt
  virtual double irreversibleEnergy() const {return _localIPV->irreversibleEnergy();};
  virtual double& getRefToIrreversibleEnergy() {return _localIPV->getRefToIrreversibleEnergy();};

  virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const
    {return _localIPV->getConstRefToDIrreversibleEnergyDF();};
  virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient()
    {return _localIPV->getRefToDIrreversibleEnergyDF();};

  virtual IPVariable* clone() const {return new localDamageIsotropicElasticityDG3DIPVariable(*this);};
  virtual void restart();
};



class ViscoelasticDG3DIPVariable : public dG3DIPVariable
{
 protected:
  IPViscoelastic *_Visipv;
 public:
  ViscoelasticDG3DIPVariable(const mlawViscoelastic &_Vislaw, const bool oninter=false);
  ViscoelasticDG3DIPVariable(const ViscoelasticDG3DIPVariable &source);
  virtual ViscoelasticDG3DIPVariable& operator=(const IPVariable &source);
  virtual ~ViscoelasticDG3DIPVariable(){if(_Visipv!=NULL) { delete _Visipv; _Visipv=NULL;}};

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_Visipv)
      _Visipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _Visipv;};
  virtual const IPVariable* getInternalData()  const {return _Visipv;};

 /* specific function */
  IPViscoelastic* getIPViscoelastic(){return _Visipv;} //output is an instance of IPViscoelastic
  const IPViscoelastic* getIPViscoelastic() const{return _Visipv;}
  virtual double get(const int i) const;
  virtual IPVariable* clone() const {return new ViscoelasticDG3DIPVariable(*this);};
  virtual void restart();
};

class EOSDG3DIPVariable : public dG3DIPVariable
{
 protected:
  IPEOS *_eosipv;
  STensor3 _PK1_vis; //viscous PK1 for AV case

 public:

  EOSDG3DIPVariable(double elementSize, const bool oninter=false);

  EOSDG3DIPVariable(double elementSize,IPVariable *IPv, const bool oninter=false);

  EOSDG3DIPVariable(const EOSDG3DIPVariable &source);
  virtual EOSDG3DIPVariable& operator=(const IPVariable &source);
  virtual ~EOSDG3DIPVariable(){if(_eosipv!=NULL) {delete _eosipv; _eosipv=NULL;}};

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_eosipv)
      _eosipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _eosipv;};
  virtual const IPVariable* getInternalData()  const {return _eosipv;};

 /* specific function */
  IPEOS* getIPEOS(){return _eosipv;}
  const IPEOS* getIPEOS() const{return _eosipv;}
  virtual double get(const int i) const;
  virtual void getCauchyStress(STensor3 &cauchy) const;//new version of Cauchy stress which substracts the AV part
  virtual STensor3 &getFirstPiolaViscous() {return _PK1_vis;}
  virtual IPVariable* clone() const {return new EOSDG3DIPVariable(*this);};
  virtual void restart();
};


class VUMATinterfaceDG3DIPVariable : public dG3DIPVariable
{
 protected:
  IPVUMATinterface *_vumatipv;
 public:
  VUMATinterfaceDG3DIPVariable(int _nsdv, double _size, const bool oninter=false);
  VUMATinterfaceDG3DIPVariable(const VUMATinterfaceDG3DIPVariable &source);
  virtual VUMATinterfaceDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_vumatipv)
      _vumatipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _vumatipv;};
  virtual const IPVariable* getInternalData()  const {return _vumatipv;};

 /* specific function */
  IPVUMATinterface* getIPVUMATinterface(){return _vumatipv;}
  const IPVUMATinterface* getIPVUMATinterface() const{return _vumatipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _vumatipv->damageEnergy();}
  virtual ~VUMATinterfaceDG3DIPVariable()
  {
     delete _vumatipv;
     _vumatipv= NULL;
  }
  virtual IPVariable* clone() const {return new VUMATinterfaceDG3DIPVariable(*this);};
  virtual void restart();
};

class TransverseIsotropicDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPTransverseIsotropic _tiipv;
 public:
  TransverseIsotropicDG3DIPVariable(const bool oninter=false);
  TransverseIsotropicDG3DIPVariable(const TransverseIsotropicDG3DIPVariable &source);
  virtual TransverseIsotropicDG3DIPVariable& operator=(const IPVariable &source);
  virtual ~TransverseIsotropicDG3DIPVariable(){};
  virtual IPVariable* getInternalData() {return &_tiipv;};
  virtual const IPVariable* getInternalData()  const {return &_tiipv;};

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _tiipv.setLocation(loc);
  };

 /* specific function */
  IPTransverseIsotropic* getIPTransverseIsotropic(){return &_tiipv;}
  const IPTransverseIsotropic* getIPTransverseIsotropic() const{return &_tiipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _tiipv.damageEnergy();}
  virtual IPVariable* clone() const {return new TransverseIsotropicDG3DIPVariable(*this);};
  virtual void restart();
};


class TransverseIsoCurvatureDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPTransverseIsoCurvature _tiipv;
 public:
  TransverseIsoCurvatureDG3DIPVariable(const SVector3 &GaussP, const SVector3 &Centroid,
                                       const SVector3 &PointStrart1, const SVector3 &PointStrart2, const double init_Angle, const bool oninter=false);
  TransverseIsoCurvatureDG3DIPVariable(const TransverseIsoCurvatureDG3DIPVariable &source);
  virtual TransverseIsoCurvatureDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _tiipv.setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return &_tiipv;};
  virtual const IPVariable* getInternalData()  const {return &_tiipv;};

 /* specific function */
  IPTransverseIsoCurvature* getIPTransverseIsoCurvature(){return &_tiipv;}
  const IPTransverseIsoCurvature* getIPTransverseIsoCurvature() const{return &_tiipv;}
  virtual ~TransverseIsoCurvatureDG3DIPVariable(){}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _tiipv.damageEnergy();};
  virtual IPVariable* clone() const {return new TransverseIsoCurvatureDG3DIPVariable(*this);};
  virtual void restart();
};

class TransverseIsoYarnBDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPTransverseIsoYarnB _tiipv;
 public:

  TransverseIsoYarnBDG3DIPVariable(const SVector3 &GaussP, const Yarn YD, const bool oninter=false);
  TransverseIsoYarnBDG3DIPVariable(const SVector3 &GaussP, const bool oninter=false);
  TransverseIsoYarnBDG3DIPVariable(const SVector3 &GaussP, const vector<double> Dir, const bool oninter=false);
  TransverseIsoYarnBDG3DIPVariable(const TransverseIsoYarnBDG3DIPVariable &source);

  TransverseIsoYarnBDG3DIPVariable& operator=(const IPVariable &source);


  // specific function
  IPTransverseIsoYarnB* getIPTransverseIsoYarnB(){return &_tiipv;}

  const IPTransverseIsoYarnB* getIPTransverseIsoYarnB() const{return &_tiipv;}
  virtual ~TransverseIsoYarnBDG3DIPVariable(){}
  //virtual double get(const int i) const;
  //virtual double defoEnergy() const;
  //virtual double plasticEnergy() const;

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _tiipv.setLocation(loc);
  };
  virtual IPVariable* getInternalData() {return &_tiipv;};
  virtual const IPVariable* getInternalData()  const {return &_tiipv;};


  virtual IPVariable* clone() const {return new TransverseIsoYarnBDG3DIPVariable(*this);};

  virtual void restart();
};

class AnisotropicDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPAnisotropic _tiipv;
 public:
  AnisotropicDG3DIPVariable(const bool oninter=false);
  AnisotropicDG3DIPVariable(const AnisotropicDG3DIPVariable &source);
  virtual AnisotropicDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _tiipv.setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return &_tiipv;};
  virtual const IPVariable* getInternalData()  const {return &_tiipv;};

 /* specific function */
  IPAnisotropic* getIPAnisotropic(){return &_tiipv;}
  const IPAnisotropic* getIPAnisotropic() const{return &_tiipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _tiipv.damageEnergy();}
  virtual IPVariable* clone() const {return new AnisotropicDG3DIPVariable(*this);};
  virtual void restart();
};


class AnisotropicStochDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPAnisotropicStoch _tiipv;
 public:
  AnisotropicStochDG3DIPVariable(const SVector3 &GaussP, const fullMatrix<double> &ExMat, const fullMatrix<double> &EyMat,
                                 const fullMatrix<double> &EzMat, const fullMatrix<double> &VxyMat, const fullMatrix<double> &VxzMat,
                                 const fullMatrix<double> &VyzMat, const fullMatrix<double> &MUxyMat, const fullMatrix<double> &MUxzMat,
                                 const fullMatrix<double> &MUyzMat, const double dx, const double dy, const double OrigX,
                                 const double OrigY, const int intpl,
                                 const bool oninter=false);
  AnisotropicStochDG3DIPVariable(const AnisotropicStochDG3DIPVariable &source);
  virtual AnisotropicStochDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _tiipv.setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return &_tiipv;};
  virtual const IPVariable* getInternalData()  const {return &_tiipv;};

 /* specific function */
  IPAnisotropicStoch* getIPAnisotropicStoch(){return &_tiipv;}
  const IPAnisotropicStoch* getIPAnisotropicStoch() const{return &_tiipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _tiipv.damageEnergy();};
  virtual IPVariable* clone() const {return new AnisotropicStochDG3DIPVariable(*this);};
  virtual void restart();
};


class ClusterDG3DIPVariable : public dG3DIPVariable // or store data in a different way
{
 protected:
  IPCluster *_clusteripv;
 public:
  ClusterDG3DIPVariable(IPCluster* ipc, const bool oninter=false);
  ClusterDG3DIPVariable(const ClusterDG3DIPVariable &source);
  virtual ClusterDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _clusteripv->setLocation(loc);
  };
  virtual ~ClusterDG3DIPVariable()
  {
    if (_clusteripv) delete _clusteripv;
    _clusteripv = NULL;
  }
  virtual IPVariable* getInternalData() {return _clusteripv;};
  virtual const IPVariable* getInternalData()  const {return _clusteripv;};

 /* specific function */
  IPCluster* getIPCluster(){return _clusteripv;}
  const IPCluster* getIPCluster() const{return _clusteripv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _clusteripv->damageEnergy();};
  virtual IPVariable* clone() const {return new ClusterDG3DIPVariable(*this);};
  virtual void restart();
};
/*
class ExtraDofDiffusionDG3DIPVariableBase : public dG3DIPVariable
{

 protected:


  double                       field;
  SVector3                     gradField;
  SVector3                     flux;
  STensor3                     dPdField;
	STensor33										 dPdGradField;
  STensor3                     dFluxdGradField;
  SVector3                     dFluxdField;
  STensor33                    dFluxdF;
  const STensor3               *linearK;
  double                       fieldSource;  //related to cp*dT/dt
  double                       dFieldSourcedField;
	SVector3                     dFieldSourcedGradField;
  STensor3                     dFieldSourcedF;

	double                       mechanicalSource; // related to convert mechanical energy to heat
  double                       dMechanicalSourcedField;
  SVector3                     dMechanicalSourcedGradField;
  STensor3                     dMechanicalSourcedF;

	double												Cp; // field capacity

 // for dg: jump in non local epl
  double                       fieldJump;
  double                       oneOverFieldJump;
  double                       dOneOverFieldJumpdFieldm;
  double                       dOneOverFieldJumpdFieldp;
  SVector3                     interfaceFlux;
  const STensor3	      *linearSymmetrizationCoupling;


 public:
  ExtraDofDiffusionDG3DIPVariableBase(const bool oninter=false);
  ExtraDofDiffusionDG3DIPVariableBase(const  ExtraDofDiffusionDG3DIPVariableBase &source);
  ExtraDofDiffusionDG3DIPVariableBase& operator=(const IPVariable &source);

  virtual ~ExtraDofDiffusionDG3DIPVariableBase()
  {
  }

  virtual double defoEnergy() const=0;
  virtual double plasticEnergy() const=0;
  virtual double damageEnergy() const = 0;

  virtual double get(const int i) const=0;
  virtual double getInternalEnergyExtraDofDiffusion() const = 0;



  virtual IPVariable* clone() const =0;
  virtual void restart();
};*/


//new ThermoMec
class ThermoMechanicsDG3DIPVariableBase : public dG3DIPVariable, public extraDofIPVariableBase
{

 protected:


 public:
  ThermoMechanicsDG3DIPVariableBase(const bool oninter=false);
  ThermoMechanicsDG3DIPVariableBase(const ThermoMechanicsDG3DIPVariableBase &source);
  ThermoMechanicsDG3DIPVariableBase& operator=(const IPVariable &source);

  virtual ~ThermoMechanicsDG3DIPVariableBase()
  {
  }
  virtual double getInternalEnergyExtraDofDiffusion() const = 0;
  virtual void getConstitutiveExtraDofFlux(const int index, SVector3& q) const {
    if (index > 0) Msg::Error("constitutive field with index %d is not defined",index);
    q = getConstRefToFlux()[index];
  }
  // constitutive field internal energy
  virtual double getConstitutiveExtraDofInternalEnergy(const int index) const {
    if (index > 0) Msg::Error("constitutive field with index %d is not defined",index);
    return getInternalEnergyExtraDofDiffusion();
  }
  // constitutive field capacity per unit field
  virtual double getConstitutiveExtraDofFieldCapacityPerUnitField(const int index) const {
    if (index > 0) Msg::Error("constitutive field with index %d is not defined",index);
    return getConstRefToExtraDofFieldCapacityPerUnitField()(index);
  };
  // mechanical source converted to this constitutive field
  virtual double getConstitutiveExtraDofMechanicalSource(const int index) const {
    if (index > 0) Msg::Error("constitutive field with index %d is not defined",index);
    return getConstRefToMechanicalSource()(index);
  };
  // non-constitutive field value
  virtual void getNonConstitutiveExtraDofFlux(const int index,SVector3& flux) const{
    Msg::Error("ExtraDofDiffusionDG3DIPVariableBase::getNonConstitutiveExtraDofFlux is not defined");
  }

  // Archiving data to be changed
  virtual const double getConstRefToTemperature() const {return getConstRefToField(0);}
  virtual double &getRefToTemperature() {return getRefToField(0);}

  virtual const SVector3 &getConstRefToGradT() const {return getConstRefToGradField()[0];}
  virtual SVector3 &getRefToGradT() {return getRefToGradField()[0];}

  virtual const SVector3 &getConstRefToThermalFlux() const {return getConstRefToFlux()[0];}
  virtual SVector3 &getRefToThermalFlux() {return getRefToFlux()[0];}

  virtual  STensor3 &getRefTodPdT() {return getRefTodPdField()[0];}

  virtual const  STensor3 &getConstRefTodThermalFluxdGradT() const {return  getConstRefTodFluxdGradField()[0][0];}
  virtual  STensor3 &getRefTodThermalFluxdGradT() {return  getRefTodFluxdGradField()[0][0];}

  virtual const  SVector3 &getConstRefTodThermalFluxdT() const {return  getConstRefTodFluxdField()[0][0];}
  virtual  SVector3 &getRefTodThermalFluxdT() {return  getRefTodFluxdField()[0][0];}

  virtual const  STensor33 &getConstRefTodThermalFluxdF() const {return  getConstRefTodFluxdF()[0];}
  virtual  STensor33 &getRefTodThermalFluxdF() {return  getRefTodFluxdF()[0];}

  virtual const double getConstRefToTemperatureJump() const {return getConstRefToFieldJump()(0);}
  virtual double &getRefToTemperatureJump() {return  getRefToFieldJump()(0);}

  virtual const double getConstRefToOneOverTemperatureJump() const {return getConstRefToEnergyConjugatedFieldJump()(0);}
  virtual double &getRefToOneOverTemperatureJump() {return  getRefToEnergyConjugatedFieldJump()(0);}

  virtual const double getConstRefTodOneOverTemperatureJumpdFieldm() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldm()[0][0];}
  virtual double &getRefTodOneOverTemperatureJumpdFieldm() {return  getRefTodEnergyConjugatedFieldJumpdFieldm()[0][0];}

  virtual const double getConstRefTodOneOverTemperatureJumpdFieldp() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldp()[0][0];}
  virtual double &getRefTodOneOverTemperaturedJumpdFieldp() {return  getRefTodEnergyConjugatedFieldJumpdFieldp()[0][0];}

  virtual const double getConstRefToThermalSource() const {return getConstRefToFieldSource()(0);}
  virtual double &getRefToThermalSource() {return  getRefToFieldSource()(0);}

  virtual const double getConstRefTodThermalSourcedField() const {return getConstRefTodFieldSourcedField()(0,0);}
  virtual double &getRefTodThermalSourcedField() {return  getRefTodFieldSourcedField()(0,0);}

  virtual const  STensor3 &getConstRefTodThermalSourcedF() const {return getConstRefTodFieldSourcedF()[0];}
  virtual  STensor3 &getRefTodThermalSourcedF() {return getRefTodFieldSourcedF()[0];}

  virtual const STensor3 *getConstRefToStiff_alphaDilatation() const {return getConstRefToLinearSymmetrizationCoupling()[0];}
  virtual void setRefToStiff_alphaDilatation(const STensor3 &eT) {setConstRefToLinearSymmetrizationCoupling(eT,0);}

  virtual const STensor3 *getConstRefToLinearConductivity() const {return getConstRefToLinearK()[0][0];}
  virtual void setRefToLinearConductivity(const STensor3 &eT) {setConstRefToLinearK(eT,0,0);}


  virtual double get(const int i) const;
  virtual IPVariable* clone() const =0;
  virtual void restart();
};


class LinearThermoMechanicsDG3DIPVariable : public ThermoMechanicsDG3DIPVariableBase
{

 protected:
  IPLinearThermoMechanics _linearTMIP;
  LinearThermoMechanicsDG3DIPVariable(const bool oninter=false) {Msg::Error("LinearThermoMechanicsDG3DIPVariable: Cannot use the default constructor");}
 public:
  LinearThermoMechanicsDG3DIPVariable(double tinitial, const bool oninter=false);
  LinearThermoMechanicsDG3DIPVariable(const LinearThermoMechanicsDG3DIPVariable &source);
  LinearThermoMechanicsDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    _linearTMIP.setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return &_linearTMIP;};
  virtual const IPVariable* getInternalData()  const {return &_linearTMIP;};

 /* specific function */
  IPLinearThermoMechanics* getIPLinearThermoMechanics(){return &_linearTMIP;}
  const IPLinearThermoMechanics* getIPLinearThermoMechanics() const{return &_linearTMIP;}
  virtual ~LinearThermoMechanicsDG3DIPVariable()
  {
  }
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double getInternalEnergyExtraDofDiffusion() const {return _linearTMIP.getThermalEnergy();};
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _linearTMIP.damageEnergy();};
  virtual IPVariable* clone() const {return new LinearThermoMechanicsDG3DIPVariable(*this);};
  virtual void restart();
  virtual int fractureEnergy(double* arrayEnergy) const{arrayEnergy[0]=_linearTMIP.getConstRefToFractureEnergy(); return 1;} //need it to get H1 norm in the interface  e.g. LinearThermoMech
};

class J2ThermoMechanicsDG3DIPVariable : public ThermoMechanicsDG3DIPVariableBase // or store data in a different way
{
 protected:
  IPJ2ThermoMechanics *_j2ipv;
  J2ThermoMechanicsDG3DIPVariable(const mlawJ2ThermoMechanics &_j2law, const bool oninter=false){Msg::Error("J2ThermoMechanicsDG3DIPVariable: Cannot use the default constructor");}
 public:
  J2ThermoMechanicsDG3DIPVariable(double tinitial,const mlawJ2ThermoMechanics &_j2law, const bool oninter=false);
  J2ThermoMechanicsDG3DIPVariable(const J2ThermoMechanicsDG3DIPVariable &source);
  J2ThermoMechanicsDG3DIPVariable& operator=(const IPVariable &source);
  virtual ~J2ThermoMechanicsDG3DIPVariable(){if(_j2ipv!=NULL) {delete _j2ipv; _j2ipv=NULL;}};

  virtual void setLocation(const IPVariable::LOCATION loc) {
    ThermoMechanicsDG3DIPVariableBase::setLocation(loc);
    if (_j2ipv)
      _j2ipv->setLocation(loc);
  };


  virtual IPVariable* getInternalData() {return _j2ipv;};
  virtual const IPVariable* getInternalData()  const {return _j2ipv;};

 /* specific function */
  IPJ2ThermoMechanics* getIPJ2ThermoMechanics(){return _j2ipv;}
  const IPJ2ThermoMechanics* getIPJ2ThermoMechanics() const{return _j2ipv;}
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _j2ipv->damageEnergy();}
  virtual double getInternalEnergyExtraDofDiffusion() const {return _j2ipv->getThermalEnergy();};
  virtual IPVariable* clone() const {return new J2ThermoMechanicsDG3DIPVariable(*this);};
  virtual void restart();
};

class J2FullThermoMechanicsDG3DIPVariable : public ThermoMechanicsDG3DIPVariableBase{
	protected:
		IPJ2ThermoMechanics _j2Thermo;

	public:
		J2FullThermoMechanicsDG3DIPVariable(const J2IsotropicHardening* j2harden, const bool oninter);
		J2FullThermoMechanicsDG3DIPVariable(const J2FullThermoMechanicsDG3DIPVariable& src);
		virtual J2FullThermoMechanicsDG3DIPVariable& operator =(const IPVariable& src);
		virtual ~J2FullThermoMechanicsDG3DIPVariable(){};

		virtual void setLocation(const IPVariable::LOCATION loc) {
			ThermoMechanicsDG3DIPVariableBase::setLocation(loc);
			_j2Thermo.setLocation(loc);
		};

    virtual IPVariable* getInternalData() {return &_j2Thermo;};
    virtual const IPVariable* getInternalData()  const {return &_j2Thermo;};

		IPJ2ThermoMechanics* getIPJ2ThermoMechanics(){return &_j2Thermo;}
		const IPJ2ThermoMechanics* getIPJ2ThermoMechanics() const{return &_j2Thermo;}
		virtual double get(const int i) const;
		virtual double defoEnergy() const {return _j2Thermo.defoEnergy();};
		virtual double plasticEnergy() const {return _j2Thermo.plasticEnergy();};
    virtual double damageEnergy() const {return _j2Thermo.damageEnergy();};
		virtual double getInternalEnergyExtraDofDiffusion() const {return _j2Thermo.getThermalEnergy();};
		virtual IPVariable* clone() const {return new J2FullThermoMechanicsDG3DIPVariable(*this);};
		virtual void restart();
};


class AnIsotropicTherMechDG3DIPVariable : public ThermoMechanicsDG3DIPVariableBase
{

 protected:
  IPAnIsotropicTherMech _AnIsotropicTMIP;

public:
  AnIsotropicTherMechDG3DIPVariable(const bool oninter=false) {Msg::Error("AnIsotropicTherMechDG3DIPVariable: Cannot use the default constructor");}
	AnIsotropicTherMechDG3DIPVariable(double tinitial, const bool oninter=false);
  AnIsotropicTherMechDG3DIPVariable(const AnIsotropicTherMechDG3DIPVariable &source);
  AnIsotropicTherMechDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    ThermoMechanicsDG3DIPVariableBase::setLocation(loc);
    _AnIsotropicTMIP.setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return &_AnIsotropicTMIP;};
  virtual const IPVariable* getInternalData()  const {return &_AnIsotropicTMIP;};

 /* specific function */
  IPAnIsotropicTherMech* getIPAnIsotropicTherMech(){return &_AnIsotropicTMIP;}
  const IPAnIsotropicTherMech* getIPAnIsotropicTherMech() const{return &_AnIsotropicTMIP;}
  virtual ~AnIsotropicTherMechDG3DIPVariable()
  {
  }
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double getInternalEnergyExtraDofDiffusion() const {return _AnIsotropicTMIP.getThermalEnergy();};
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _AnIsotropicTMIP.damageEnergy();};
  virtual IPVariable* clone() const {return new AnIsotropicTherMechDG3DIPVariable(*this);};
  virtual void restart();

};

class SMPDG3DIPVariable : public ThermoMechanicsDG3DIPVariableBase
{
  protected:
  IPSMP *_TMIPSMP;

	public:
	 SMPDG3DIPVariable(const mlawSMP &_lawSMP, const bool oninter=false){Msg::Error("MPDG3DIPVariable: Cannot use the default constructor");}
	 SMPDG3DIPVariable(double tinitial,const mlawSMP &_lawSMP, const bool oninter=false);
	 SMPDG3DIPVariable(const SMPDG3DIPVariable &source);
	 SMPDG3DIPVariable& operator=(const IPVariable &source);


  virtual void setLocation(const IPVariable::LOCATION loc) {
    ThermoMechanicsDG3DIPVariableBase::setLocation(loc);
    if (_TMIPSMP){
      _TMIPSMP->setLocation(loc);
    }
  };

  virtual IPVariable* getInternalData() {return _TMIPSMP;};
  virtual const IPVariable* getInternalData()  const {return _TMIPSMP;};

 /* specific function */
  IPSMP* getIPSMP(){return _TMIPSMP;}
  const IPSMP* getIPSMP() const{return _TMIPSMP;}
  virtual ~SMPDG3DIPVariable()
  {
    if(_TMIPSMP != NULL)
    { delete _TMIPSMP;
      _TMIPSMP = NULL;
    }
  }
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _TMIPSMP->damageEnergy();};
  virtual double getInternalEnergyExtraDofDiffusion() const {return _TMIPSMP->getThermalEnergy();};
  virtual double getRefToEquivalentPlasticDefo1() const {return _TMIPSMP->getConstRefToEquivalentPlasticDefo1();};
  virtual IPVariable* clone() const {return new SMPDG3DIPVariable(*this);};
  virtual void restart();
};





class PhenomenologicalSMPDG3DIPVariable : public ThermoMechanicsDG3DIPVariableBase
{
protected:
    IPPhenomenologicalSMP *_TMIPSMP;

public:
    PhenomenologicalSMPDG3DIPVariable(const mlawPhenomenologicalSMP &_lawSMP, const bool oninter=false)
                                        { Msg::Error("MPDG3DIPVariable: Cannot use the default constructor");};
    PhenomenologicalSMPDG3DIPVariable(double tinitial, const mlawPhenomenologicalSMP &_lawSMP, const bool oninter=false);
    PhenomenologicalSMPDG3DIPVariable(const PhenomenologicalSMPDG3DIPVariable &source);
    PhenomenologicalSMPDG3DIPVariable& operator=(const IPVariable &source);

    virtual void setLocation(const IPVariable::LOCATION loc)
    {
        ThermoMechanicsDG3DIPVariableBase::setLocation(loc);
        if (_TMIPSMP)
        {
            _TMIPSMP->setLocation(loc);
        }
    };

    virtual IPVariable* getInternalData() {return _TMIPSMP;};

    virtual const IPVariable* getInternalData()  const {return _TMIPSMP;};

    /* specific function */
    IPPhenomenologicalSMP* getIPPhenomenologicalSMP()
    {
        return _TMIPSMP;
    };

    const IPPhenomenologicalSMP* getIPPhenomenologicalSMP() const {return _TMIPSMP;};
    virtual ~PhenomenologicalSMPDG3DIPVariable()
    {
        if(_TMIPSMP != NULL)
        {
            delete _TMIPSMP;
            _TMIPSMP = NULL;
        }
    };

    virtual double get(const int i) const;
    virtual double defoEnergy() const;
    virtual double plasticEnergy() const;
    virtual double damageEnergy() const {return _TMIPSMP->damageEnergy();};
    virtual double getInternalEnergyExtraDofDiffusion() const {return _TMIPSMP->getThermalEnergy();};
    virtual IPVariable* clone() const {return new PhenomenologicalSMPDG3DIPVariable(*this);};
    virtual void restart();
};


class crystalPlasticityDG3DIPVariable : public dG3DIPVariable
{

 protected:
  IPCrystalPlasticity *_ipv;

 public:
  crystalPlasticityDG3DIPVariable(int _nsdv, double size, const bool oninter=false);
  crystalPlasticityDG3DIPVariable(const crystalPlasticityDG3DIPVariable &source);
  virtual crystalPlasticityDG3DIPVariable& operator=(const IPVariable &source);

 /* specific function */
  IPCrystalPlasticity* getIPCrystalPlasticity(){return _ipv;}
  const IPCrystalPlasticity* getIPCrystalPlasticity() const{return _ipv;}
  virtual ~crystalPlasticityDG3DIPVariable()
  {
    if (_ipv) delete _ipv;
    _ipv = NULL;
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_ipv)
      _ipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _ipv;};
  virtual const IPVariable* getInternalData()  const {return _ipv;};

  virtual bool dissipationIsActive() const {return _ipv->dissipationIsActive();};

  virtual void blockDissipation(const bool fl){if (_ipv) _ipv->blockDissipation(fl);};
  virtual bool dissipationIsBlocked() const {
    if (_ipv) return _ipv->dissipationIsBlocked();
    else return false;
  }

  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;

  virtual IPVariable* clone() const {return new crystalPlasticityDG3DIPVariable(*this);};
  virtual void restart();
};


class ElecTherMechDG3DIPVariableBase : public dG3DIPVariable
{

 protected:

 // double                       Temperature;
 //SVector3                     gradT;
 //SVector3                     fluxT;
 //STensor3                     dPdT;
 //STensor3                     dFluxTdGradT;
 //SVector3                     dFluxTdT;
 //STensor3                     dFluxTdGradV;
 //SVector3                     dFluxTdV;
 //STensor33                    dFluxTdF;
 // const STensor3               *lineark10;
 // const STensor3               *lineark20;
 // STensor3                     k10;
 //STensor3                     k20;
  //STensor3                     dk10dT;
  //STensor3                     dk20dT;
  //STensor3                     dk10dv;

  //STensor43                    dk10dF;
  //STensor43                    dk20dF;
 // for dg: jump in non local epl
 // double                       TemperatureJump;
  //SVector3                     interfaceFluxT; //should be interfaceFluxJy


  //double                       Voltage;
  //SVector3                     gradV;
  //SVector3                     fluxV;
  //STensor3                     dPdV;
  //STensor3                     dFluxVdGradV;
  //SVector3                     dFluxVdV;
  //STensor3                     dFluxVdGradT;
  //SVector3                     dFluxVdT;
  //STensor33                    dFluxVdF;
  //const STensor3               *linearl10;
  //const STensor3               *linearl20;
  //STensor3                     l10;
  //STensor3                     l20;
  //STensor3                     dl10dT;
  //STensor3                     dl20dT;
  //STensor3                     dl20dv;

  //STensor43                    dl10dF;
  //STensor43                    dl20dF;

  //double                       thermalSource;
  //double                       dThermalSourcedT;
  //STensor3                     dThermalSourcedF;
 // for dg: jump in non local epl
  //double                       VoltageJump;
  //SVector3                     interfaceFluxV;

  //double                       fvJump;
  //double                       fTJump;
  //double                       dfvjumpdvm;
  //double                       dfvjumpdvp;
  //double                       dfvjumpdTm;
  //double                       dfvjumpdTp;
 //double                       dfTjumpdTm;
 //double                       dfTjumpdTp;

  //STensor3 			dgradVdgradfV;
  //STensor3 			dgradVdgradfT;
  //STensor3 			dgradTdgradfV;
  //STensor3 			dgradTdgradfT;
  //double 			dVdfV;
  //double 			dVdfT;
  //double 			dTdfV;
  //double 			dTdfT;
  //SVector3 			dgradVdfV;
  //SVector3  			dgradVdfT;
  //SVector3  			dgradTdfT;
  //SVector3  			dgradTdfV;

  //double                       fT;
  //double                       fV;
  //SVector3                      gradfT;
  //SVector3                      gradfV;

  //SVector3 			fluxjy;
  //SVector3 			djydV;
  //STensor3 			djydgradV;
  //SVector3 			djydT;
  //STensor3 			djydgradT;
  //STensor33 			djydF;
  //STensor3 			jy1;
  //STensor3 			djy1dT;
  //STensor3 			djy1dV;
  //STensor43			djy1dF;
 // const STensor3		*Stiff_alphaDilatation;



 public:
  ElecTherMechDG3DIPVariableBase(const bool oninter=false);
  ElecTherMechDG3DIPVariableBase(const  ElecTherMechDG3DIPVariableBase &source);
  ElecTherMechDG3DIPVariableBase& operator=(const IPVariable &source);

  virtual ~ElecTherMechDG3DIPVariableBase()
  {

  }

  virtual double defoEnergy() const=0;
  virtual double plasticEnergy() const=0;
  virtual double damageEnergy() const = 0;
  virtual double get(const int i) const;

  // Archiving data to be changed

  virtual const double getConstRefToTemperature() const {return getConstRefToField(0);}
  virtual double &getRefToTemperature() {return getRefToField(0);}

  virtual const SVector3 &getConstRefToGradT() const {return getConstRefToGradField()[0];}
  virtual SVector3 &getRefToGradT() {return getRefToGradField()[0];}

 /* virtual const SVector3 &getConstRefToThermalFlux() const {return getConstRefToFlux()[0];}
  virtual SVector3 &getRefToThermalFlux() {return getRefToFlux()[0];}*/

  virtual  const STensor3 &getConstRefTodPdT() const {return getConstRefTodPdField()[0];}
  virtual  STensor3 &getRefTodPdT() {return getRefTodPdField()[0];}

 /* virtual const  STensor3 &getConstRefTodThermalFluxdGradT() const {return  getConstRefTodFluxdGradField()[0][0];}
  virtual  STensor3 &getRefTodThermalFluxdGradT() {return  getRefTodFluxdGradField()[0][0];}*/

  virtual const  SVector3 &getConstRefTodFluxTdV() const {return  getConstRefTodFluxdField()[0][1];}
  virtual  SVector3 &getRefTodFluxTdV() {return  getRefTodFluxdField()[0][1];}

  virtual const  STensor3 &getConstRefTodFluxTdGradV() const {return  getConstRefTodFluxdGradField()[0][1];}//dqdgradV,
  virtual  STensor3 &getRefTodFluxTdGradV() {return  getRefTodFluxdGradField()[0][1];}

/* virtual const  SVector3 &getConstRefTodFluxTdT() const {return  getConstRefTodFluxdField()[0][0];}
  virtual  SVector3 &getRefTodFluxTdT() {return  getRefTodFluxdField()[0][0];}*/

/*  virtual const  STensor33 &getConstRefTodThermalFluxdF() const {return  getConstRefTodFluxdF()[0];}
  virtual  STensor33 &getRefTodThermalFluxdF() {return  getRefTodFluxdF()[0];}*/

  /*virtual const STensor3 &getConstRefTolineark10() const {return *lineark10;}
  virtual void setRefTolineark10(const STensor3 &eT1) {lineark10=&eT1;}

  virtual const STensor3 &getConstRefTolineark20() const {return *lineark20;}
  virtual void setRefTolineark20(const STensor3 &eT2) {lineark20=&eT2;}*/

 /* virtual const  STensor3 &getConstRefToElecConductivity() const {return  l10;}
  virtual  STensor3 &getRefToElecConductivity() {return  l10;}

  virtual const  STensor3 &getConstRefToCrossElecConductivity() const {return  l20;}
  virtual  STensor3 &getRefToCrossElecConductivity() {return  l20;}*/

  virtual const  STensor3 *getConstRefToElecConductivity() const {return getConstRefToLinearK()[1][1];}
  virtual void setRefToElecConductivity(const STensor3 &eT) {return  setConstRefToLinearK(eT,1,1);}

  virtual const  STensor3 *getConstRefToCrossElecConductivity() const {return getConstRefToLinearK()[1][0];}
  virtual void setRefToCrossElecConductivity(const STensor3 &eT) {return  setConstRefToLinearK(eT,1,0);}

  virtual const  STensor3 *getConstRefToTherConductivity() const {return getConstRefToLinearK()[0][0];}
  virtual void setRefToTherConductivity(const STensor3 &eT) {return  setConstRefToLinearK(eT,0,0);}

  virtual const  STensor3 *getConstRefToCrossTherConductivity() const {return getConstRefToLinearK()[0][1];}
  virtual void setRefToCrossTherConductivity(const STensor3 &eT) {return  setConstRefToLinearK(eT,0,1);}

  virtual const  STensor3 &getConstRefTodElecConductivitydT() const {return  getConstRefTodLinearKdField()[1][1][0];}
  virtual  STensor3 &getRefTodElecConductivitydT() {return  getRefTodLinearKdField()[1][1][0];}

  virtual const  STensor3 &getConstRefTodCrossElecConductivitydT() const {return  getConstRefTodLinearKdField()[1][0][0];}
  virtual  STensor3 &getRefTodCrossElecConductivitydT() {return  getRefTodLinearKdField()[1][0][0];}

  virtual const  STensor3 &getConstRefTodTherConductivitydT() const {return  getConstRefTodLinearKdField()[0][0][0];}
  virtual  STensor3 &getRefTodTherConductivitydT() {return  getRefTodLinearKdField()[0][0][0];}

  virtual const  STensor3 &getConstRefTodCrossTherConductivitydT() const {return  getConstRefTodLinearKdField()[0][1][0];}
  virtual  STensor3 &getRefTodCrossTherConductivitydT() {return  getRefTodLinearKdField()[0][1][0];}

  virtual const  STensor3 &getConstRefTodTherConductivitydv() const {return  getConstRefTodLinearKdField()[0][0][1];}
  virtual  STensor3 &getRefTodTherConductivitydv() {return  getRefTodLinearKdField()[0][0][1];}

  virtual const  STensor3 &getConstRefTodCrossTherConductivitydv() const {return  getConstRefTodLinearKdField()[0][1][1];}
  virtual  STensor3 &getRefTodCrossTherConductivitydv() {return  getRefTodLinearKdField()[0][1][1];}

  virtual const  STensor3 &getConstRefTodElecConductivitydv() const {return  getConstRefTodLinearKdField()[1][1][1];}
  virtual  STensor3 &getRefTodElecConductivitydv() {return  getRefTodLinearKdField()[1][1][1];}

  virtual const  STensor3 &getConstRefTodCrossElecConductivitydv() const {return  getConstRefTodLinearKdField()[1][0][1];}
  virtual  STensor3 &getRefTodCrossElecConductivitydv() {return  getRefTodLinearKdField()[1][0][1];}

  virtual const  STensor43 &getConstRefTodElecConductivitydF() const {return  getConstRefTodLinearKdF()[1][1];}
  virtual  STensor43 &getRefTodElecConductivitydF() {return  getRefTodLinearKdF()[1][1];}

  virtual const  STensor43 &getConstRefTodCrossElecConductivitydF() const {return  getConstRefTodLinearKdF()[1][0];}
  virtual  STensor43 &getRefTodCrossElecConductivitydF() {return  getRefTodLinearKdF()[1][0];}

  virtual const  STensor43 &getConstRefTodTherConductivitydF() const {return  getConstRefTodLinearKdF()[0][0];}
  virtual  STensor43 &getRefTodTherConductivitydF() {return  getRefTodLinearKdF()[0][0];}

  virtual const  STensor43 &getConstRefTodCrossTherConductivitydF() const {return  getConstRefTodLinearKdF()[0][1];}
  virtual  STensor43 &getRefTodCrossTherConductivitydF() {return  getRefTodLinearKdF()[0][1];}

  virtual const double getConstRefTofvJump() const {return getConstRefToEnergyConjugatedFieldJump()(1);}
  virtual double &getRefTofvJump() {return  getRefToEnergyConjugatedFieldJump()(1);}

  virtual const double getConstRefTofTJump() const {return getConstRefToEnergyConjugatedFieldJump()(0);}
  virtual double &getRefTofTJump() {return  getRefToEnergyConjugatedFieldJump()(0);}

  virtual const double getConstRefToOneOverTemperatureJump() const {return getConstRefToEnergyConjugatedFieldJump()(0);}
  virtual double &getRefToOneOverTemperatureJump() {return  getRefToEnergyConjugatedFieldJump()(0);}

  virtual const double getConstRefTodOneOverTemperatureJumpdTm() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldm()[0][0];}
  virtual double &getRefTodOneOverTemperatureJumpdTm() {return  getRefTodEnergyConjugatedFieldJumpdFieldm()[0][0];}

  virtual const double getConstRefTodOneOverTemperatureJumpdTp() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldp()[0][0];}
  virtual double &getRefTodOneOverTemperatureJumpdTp() {return  getRefTodEnergyConjugatedFieldJumpdFieldp()[0][0];}

  virtual const double &getConstRefTodfvjumpdvm() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldm()[1][1];}
  virtual double &getRefTodfvjumpdvm() {return  getRefTodEnergyConjugatedFieldJumpdFieldm()[1][1];}

  virtual const double &getConstRefTodfvjumpdvp() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldp()[1][1];}
  virtual double &getRefTodfvjumpdvp() {return  getRefTodEnergyConjugatedFieldJumpdFieldp()[1][1];}

  virtual const double &getConstRefTodfvjumpdTm() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldm()[1][0];}
  virtual double &getRefTodfvjumpdTm() {return  getRefTodEnergyConjugatedFieldJumpdFieldm()[1][0];}

  virtual const double &getConstRefTodfvjumpdTp() const {return getConstRefTodEnergyConjugatedFieldJumpdFieldp()[1][0];}
  virtual double &getRefTodfvjumpdTp() {return  getRefTodEnergyConjugatedFieldJumpdFieldp()[1][0];}

  virtual const double getConstRefToTemperatureJump() const {return getConstRefToFieldJump()(0);}
  virtual double &getRefToTemperatureJump() {return  getRefToFieldJump()(0);}

  virtual const SVector3 &getConstRefTointerfaceFluxT() const {return getConstRefToInterfaceFlux()[0];}   // i didn't use it
  virtual SVector3 &getRefTointerfaceFluxT() {return getRefToInterfaceFlux()[0];}

  // for V
  virtual const double getConstRefToVoltage() const {return  getConstRefToField(1);}
  virtual double &getRefToVoltage() {return  getRefToField(1);}

  virtual const SVector3 &getConstRefToGradV() const {return getConstRefToGradField()[1];}
  virtual SVector3 &getRefToGradV() {return getRefToGradField()[1];}

  virtual const SVector3 &getConstRefToFluxV() const {return getConstRefToFlux()[1];}
  virtual SVector3 &getRefToFluxV() {return getRefToFlux()[1];}

  virtual const  STensor3 &getConstRefTodPdV() const {return getConstRefTodPdField()[1];}
  virtual  STensor3 &getRefTodPdV() {return getRefTodPdField()[1];}

  virtual const  STensor3 &getConstRefTodFluxVdGradV() const {return  getConstRefTodFluxdGradField()[1][1];}//dedgradV
  virtual  STensor3 &getRefTodFluxVdGradV() {return  getRefTodFluxdGradField()[1][1];}

  virtual const  STensor3 &getConstRefTodFluxVdGradT() const {return  getConstRefTodFluxdGradField()[1][0];}//dedgradT
  virtual  STensor3 &getRefTodFluxVdGradT() {return  getRefTodFluxdGradField()[1][0];}

  virtual const  SVector3 &getConstRefTodFluxVdV() const {return  getConstRefTodFluxdField()[1][1];}//dedv
  virtual  SVector3 &getRefTodFluxVdV() {return  getRefTodFluxdField()[1][1];}

  virtual const  SVector3 &getConstRefTodFluxVdT() const {return  getConstRefTodFluxdField()[1][0];}//dedv
  virtual  SVector3 &getRefTodFluxVdT() {return  getRefTodFluxdField()[1][0];}

  virtual const  STensor33 &getConstRefTodFluxVdF() const {return  getConstRefTodFluxdF()[1];}//dedf
  virtual  STensor33 &getRefTodFluxVdF() {return  getRefTodFluxdF()[1];}

 /* virtual const STensor3 &getConstRefTolinearl10() const {return *linearl10;}
  virtual void setRefTolinearl10(const STensor3 &eV1) {linearl10=&eV1;}

  virtual const STensor3 &getConstRefTolinearl20() const {return *linearl20;}
  virtual void setRefTolinearl20(const STensor3 &eV2) {linearl20=&eV2;}*/

  virtual const double getConstRefToVoltageJump() const {return getConstRefToFieldJump()(1);}
  virtual double &getRefToVoltageJump() {return  getRefToFieldJump()(1);}

  virtual const SVector3 &getConstRefTointerfaceFluxV() const {return getConstRefToInterfaceFlux()[1];}  // i didn't use it
  virtual SVector3 &getRefTointerfaceFluxV() {return getRefToInterfaceFlux()[1];}

  virtual const double getConstRefToThermalSource() const {return getConstRefToFieldSource()(0);}
  virtual double &getRefToThermalSource() {return  getRefToFieldSource()(0);}

  virtual const double getConstRefTodThermalSourcedT() const {return getConstRefTodFieldSourcedField()(0,0);}
  virtual double &getRefTodThermalSourcedT() {return  getRefTodFieldSourcedField()(0,0);}

  virtual const  STensor3 &getConstRefTodThermalSourcedF() const {return getConstRefTodFieldSourcedF()[0];}
  virtual  STensor3 &getRefTodThermalSourcedF() {return getRefTodFieldSourcedF()[0];}

 /* virtual const  SVector3 &getConstRefTodThermalSourcedGradV() const {return  dThermalSourcedGradV;}//dwdgradV
  virtual  SVector3 &getRefTodThermalSourcedGradV() {return  dThermalSourcedGradV;}

  virtual const  SVector3 &getConstRefTodThermalSourcedGradT() const {return  dThermalSourcedGradT;}//dwdgradT
  virtual  SVector3 &getRefTodThermalSourcedGradT() {return  dThermalSourcedGradT;}*/

  virtual const double getConstRefTofT() const {return  getConstRefToEnergyConjugatedField(0);}
  virtual double &getRefTofT() {return getRefToEnergyConjugatedField(0);}

  virtual const SVector3 &getConstRefToGradfT() const {return getConstRefToGradEnergyConjugatedField()[0];}
  virtual SVector3 &getRefToGradfT() {return getRefToGradEnergyConjugatedField()[0];}

  virtual const double getConstRefTofV() const {return  getConstRefToEnergyConjugatedField(1);}
  virtual double &getRefTofV() {return  getRefToEnergyConjugatedField(1);}

  virtual const SVector3 &getConstRefToGradfV() const {return getConstRefToGradEnergyConjugatedField()[1];}
  virtual SVector3 &getRefToGradfV() {return getRefToGradEnergyConjugatedField()[1];}

  virtual const STensor3 &getConstRefTodgradVdgradfV() const {return getConstRefTodGradFielddGradEnergyConjugatedField()[1][1];}
  virtual STensor3 &getRefTodgradVdgradfV() {return  getRefTodGradFielddGradEnergyConjugatedField()[1][1];}

  virtual const STensor3 &getConstRefTodgradVdgradfT() const {return getConstRefTodGradFielddGradEnergyConjugatedField()[1][0];}
  virtual STensor3 &getRefTodgradVdgradfT() {return  getRefTodGradFielddGradEnergyConjugatedField()[1][0];}

  virtual const STensor3 &getConstRefTodgradTdgradfT() const {return getConstRefTodGradFielddGradEnergyConjugatedField()[0][0];}
  virtual STensor3 &getRefTodgradTdgradfT() {return  getRefTodGradFielddGradEnergyConjugatedField()[0][0];}

  virtual const STensor3 &getConstRefTodgradTdgradfV() const {return getConstRefTodGradFielddGradEnergyConjugatedField()[0][1];}
  virtual STensor3 &getRefTodgradTdgradfV() {return  getRefTodGradFielddGradEnergyConjugatedField()[0][1];}

  virtual const double &getConstRefTodVdfV() const {return getConstRefTodFielddEnergyConjugatedField()[1][1];}
  virtual double &getRefTodVdfV() {return  getRefTodFielddEnergyConjugatedField()[1][1];}

  virtual const double &getConstRefTodVdfT() const {return getConstRefTodFielddEnergyConjugatedField()[1][0];}
  virtual double &getRefTodVdfT() {return  getRefTodFielddEnergyConjugatedField()[1][0];}

   virtual const double &getConstRefTodTdfT() const {return getConstRefTodFielddEnergyConjugatedField()[0][0];}
  virtual double &getRefTodTdfT() {return  getRefTodFielddEnergyConjugatedField()[0][0];}

  virtual const double &getConstRefTodTdfV() const {return getConstRefTodFielddEnergyConjugatedField()[0][1];}
  virtual double &getRefTodTdfV() {return  getRefTodFielddEnergyConjugatedField()[0][1];}

    virtual const SVector3  &getConstRefTodgradVdfV() const {return getConstRefTodGradFielddEnergyConjugatedField()[1][1];}
  virtual SVector3  &getRefTodgradVdfV() {return  getRefTodGradFielddEnergyConjugatedField()[1][1];}

  virtual const SVector3  &getConstRefTodgradVdfT() const {return getConstRefTodGradFielddEnergyConjugatedField()[1][0];}
  virtual SVector3  &getRefTodgradVdfT() {return  getRefTodGradFielddEnergyConjugatedField()[1][0];}

   virtual const SVector3  &getConstRefTodgradTdfT() const {return getConstRefTodGradFielddEnergyConjugatedField()[0][0];}
  virtual SVector3  &getRefTodgradTdfT() {return  getRefTodGradFielddEnergyConjugatedField()[0][0];}

  virtual const SVector3  &getConstRefTodgradTdfV() const {return getConstRefTodGradFielddEnergyConjugatedField()[0][1];}
  virtual SVector3  &getRefTodgradTdfV() {return  getRefTodGradFielddEnergyConjugatedField()[0][1];}

   virtual const SVector3  &getConstRefToFluxEnergy() const {return getConstRefToFlux()[0];}
   virtual SVector3 &getRefToFluxEnergy(){return  getRefToFlux()[0];}

   virtual const SVector3  &getConstRefTodFluxEnergydV() const {return getConstRefTodFluxdField()[0][1];}
   virtual SVector3 &getRefTodFluxEnergydV(){return  getRefTodFluxdField()[0][1];}

   virtual const STensor3 &getConstRefTodFluxEnergydGradV() const {return getConstRefTodFluxdGradField()[0][1];}
   virtual STensor3 &getRefTodFluxEnergydGradV(){return  getRefTodFluxdGradField()[0][1];}

   virtual const SVector3  &getConstRefTodFluxEnergydT() const {return getConstRefTodFluxdField()[0][0];}
   virtual SVector3 &getRefTodFluxEnergydT(){return  getRefTodFluxdField()[0][0];}

   virtual const STensor3 &getConstRefTodFluxEnergydGradT() const {return getConstRefTodFluxdGradField()[0][0];}
   virtual STensor3 &getRefTodFluxEnergydGradT(){return  getRefTodFluxdGradField()[0][0];}

   virtual const STensor33 &getConstRefTodFluxEnergydF() const {return getConstRefTodFluxdF()[0];}
   virtual STensor33 &getRefTodFluxEnergydF(){return  getRefTodFluxdF()[0];}

 /*  virtual const STensor3 *getConstRefToEnergyConductivity() const {return getConstRefToEnergyK()[0][0];}
   virtual void setRefToEnergyConductivity(const STensor3 &eT){return  setConstRefToEnergyK(eT,0,0);}

   virtual const STensor3 &getConstRefTodEnergyConductivitydT() const {return getConstRefTodEnergyKdField()[0][0][0];}
   virtual STensor3 &getRefTodEnergyConductivitydT(){return  getRefTodEnergyKdField()[0][0][0];}

   virtual const STensor3 &getConstRefTodEnergyConductivitydV() const {return getConstRefTodEnergyKdField()[0][0][1];}
   virtual STensor3 &getRefTodEnergyConductivitydV(){return getRefTodEnergyKdField()[0][0][1];}

   virtual const STensor43 &getConstRefTodEnergyConductivitydF() const {return getConstRefTodEnergyKdF()[0][0];}
   virtual STensor43 &getRefTodEnergyConductivitydF(){return getRefTodEnergyKdF()[0][0];}*/

   virtual const STensor3 &getConstRefTodFluxEnergydGradVdT() const {return getConstRefTodFluxdGradFielddField()[0][1][0];}
   virtual STensor3 &getRefTodFluxEnergydGradVdT(){return  getRefTodFluxdGradFielddField()[0][1][0];}

   virtual const STensor3 &getConstRefTodFluxEnergydGradVdV() const {return getConstRefTodFluxdGradFielddField()[0][1][1];}
   virtual STensor3 &getRefTodFluxEnergydGradVdV(){return getRefTodFluxdGradFielddField()[0][1][1];}

   virtual const STensor3 &getConstRefTodFluxEnergydGradTdT() const {return getConstRefTodFluxdGradFielddField()[0][0][0];}
   virtual STensor3 &getRefTodFluxEnergydGradTdT(){return  getRefTodFluxdGradFielddField()[0][0][0];}

   virtual const STensor3 &getConstRefTodFluxEnergydGradTdV() const {return getConstRefTodFluxdGradFielddField()[0][0][1];}
   virtual STensor3 &getRefTodFluxEnergydGradTdV(){return getRefTodFluxdGradFielddField()[0][0][1];}

   virtual const STensor43 &getConstRefTodFluxEnergydGradTdF() const {return getConstRefTodFluxdGradFielddF()[0][0];}
   virtual STensor43 &getRefTodFluxEnergydGradTdF(){return getRefTodFluxdGradFielddF()[0][0];}

   virtual const STensor43 &getConstRefTodFluxEnergydGradVdF() const {return getConstRefTodFluxdGradFielddF()[0][1];}
   virtual STensor43 &getRefTodFluxEnergydGradVdF(){return getRefTodFluxdGradFielddF()[0][1];}

   virtual const STensor3 *getConstRefToStiff_alphaDilatation() const {return getConstRefToLinearSymmetrizationCoupling()[0];}
   virtual void setRefToStiff_alphaDilatation(const STensor3 &eT) {setConstRefToLinearSymmetrizationCoupling(eT,0);}

   virtual IPVariable* clone() const =0;
   virtual void restart();
};


class LinearElecTherMechDG3DIPVariable : public ElecTherMechDG3DIPVariableBase
{

 protected:
  IPLinearElecTherMech _linearETMIP;


public:
  LinearElecTherMechDG3DIPVariable( const bool oninter=false){Msg::Error("LinearElecTherMechDG3DIPVariable: Cannot use the default constructor");}
  LinearElecTherMechDG3DIPVariable(double tinitial,double vinitial, const bool oninter=false);
  LinearElecTherMechDG3DIPVariable(const LinearElecTherMechDG3DIPVariable &source);
  LinearElecTherMechDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    ElecTherMechDG3DIPVariableBase::setLocation(loc);
    _linearETMIP.setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return &_linearETMIP;};
  virtual const IPVariable* getInternalData()  const {return &_linearETMIP;};


  IPLinearElecTherMech* getIPLinearElecTherMech(){return &_linearETMIP;}
  const IPLinearElecTherMech* getIPLinearElecTherMech() const{return &_linearETMIP;}
  virtual ~LinearElecTherMechDG3DIPVariable()
  {
  }
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _linearETMIP.damageEnergy();}
  virtual IPVariable* clone() const {return new LinearElecTherMechDG3DIPVariable(*this);};
  virtual void restart();
};


class AnIsotropicElecTherMechDG3DIPVariable : public ElecTherMechDG3DIPVariableBase
{

 protected:
  IPAnIsotropicElecTherMech *_ETMIP;


public:
 AnIsotropicElecTherMechDG3DIPVariable(const mlawAnIsotropicElecTherMech &_lawETM, const bool oninter=false){Msg::Error("LinearElecTherMechDG3DIPVariable: Cannot use the default constructor");}
 AnIsotropicElecTherMechDG3DIPVariable(double tinitial,double Vinitial,const mlawAnIsotropicElecTherMech &_lawETM, const bool oninter=false);
 AnIsotropicElecTherMechDG3DIPVariable(const AnIsotropicElecTherMechDG3DIPVariable &source);
 AnIsotropicElecTherMechDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    ElecTherMechDG3DIPVariableBase::setLocation(loc);
    if (_ETMIP)
      _ETMIP->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _ETMIP;};
  virtual const IPVariable* getInternalData()  const {return _ETMIP;};

  IPAnIsotropicElecTherMech* getIPAnIsotropicElecTherMech(){return _ETMIP;}
  const IPAnIsotropicElecTherMech* getIPAnIsotropicElecTherMech() const{return _ETMIP;}
  virtual ~AnIsotropicElecTherMechDG3DIPVariable()
  {
    if(_ETMIP != NULL)
    {
       delete _ETMIP;
       _ETMIP=NULL;
    }
  }
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _ETMIP->damageEnergy();};
  virtual IPVariable* clone() const {return new AnIsotropicElecTherMechDG3DIPVariable(*this);};
  virtual void restart();
};

class ElecSMPDG3DIPVariable : public ElecTherMechDG3DIPVariableBase
{

 protected:
  IPElecSMP *_ESMPIP;

public:
 ElecSMPDG3DIPVariable(const mlawElecSMP &_lawESMP, const bool oninter=false){Msg::Error("ElecSMPDG3DIPVariable: Cannot use the default constructor");}
 ElecSMPDG3DIPVariable(double tinitial,double vinitial,const mlawElecSMP &_lawESMP, const bool oninter=false);
 ElecSMPDG3DIPVariable(const ElecSMPDG3DIPVariable &source);
 ElecSMPDG3DIPVariable& operator=(const IPVariable &source);

  virtual void setLocation(const IPVariable::LOCATION loc) {
    ElecTherMechDG3DIPVariableBase::setLocation(loc);
    if (_ESMPIP)
      _ESMPIP->setLocation(loc);
  };


  virtual IPVariable* getInternalData() {return _ESMPIP;};
  virtual const IPVariable* getInternalData()  const {return _ESMPIP;};

  IPElecSMP* getIPElecSMP(){return _ESMPIP;}
  const IPElecSMP* getIPElecSMP() const{return _ESMPIP;}
  virtual ~ElecSMPDG3DIPVariable()
  {
    if(_ESMPIP != NULL)
    {
       delete _ESMPIP;
       _ESMPIP = NULL;
    }
  }
  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double getRefToEquivalentPlasticDefo1() const {return _ESMPIP->getConstRefToEquivalentPlasticDefo1();};
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _ESMPIP->damageEnergy();};
  virtual IPVariable* clone() const {return new ElecSMPDG3DIPVariable(*this);};
  virtual void restart();
};

#endif //DG3DIPVARIABLE
