//
//
// Description: Class of materials for non linear dg
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _DG3DMATERIALLAW_H_
#define _DG3DMATERIALLAW_H_
#ifndef SWIG
#include "dG3DIPVariable.h"
#include "mlawJ2linear.h"
#include "mlawVUMATinterface.h"
#include "mlawTransverseIsotropic.h"
#include "mlawTransverseIsoCurvature.h"
#include "mlawTransverseIsoYarnB.h"
#include "mlawAnisotropic.h"
#include "mlawAnisotropicStoch.h"
#include "j2IsotropicHardening.h"
#include "mlawLinearThermoMechanics.h"
#include "mlawJ2VMSmallStrain.h"
#include "mlawSMP.h"
#include "mlawPhenomenologicalSMP.h"
#include "mlawEOS.h"
#include "mlawCrystalPlasticity.h"
#include "mlawViscoelastic.h"
#include "mlawLinearElecTherMech.h"
#include "mlawAnIsotropicElecTherMech.h"
#include "mlawElecSMP.h"
#include "mlawAnIsotropicTherMech.h"
#include "mlawCohesive.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include "mlawJ2FullyCoupledThermoMechanics.h"
#include "mlawHyperelasticDamage.h"
#include "mlawClusterMaterialLaws.h"
#endif

class dG3DMaterialLaw : public materialLaw{
 public:

  STensor43 elasticStiffness;
  STensor43 TotalelasticStiffness;

protected:
  double _rho;
  bool _useBarF;

public:

#ifndef SWIG
  dG3DMaterialLaw(const int num, const double rho,
                   const bool init=true) : materialLaw(num,init), _rho(rho), _useBarF(false)
  {

  }
  dG3DMaterialLaw(const dG3DMaterialLaw &source) : materialLaw(source),
                                                      _rho(source._rho), elasticStiffness(source.elasticStiffness),TotalelasticStiffness(source.TotalelasticStiffness),
         _useBarF(source._useBarF)

  {

  }
  virtual ~dG3DMaterialLaw()
  {
  }
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const=0;
  // To allow initialization of bulk ip in case of fracture
  virtual void createIPVariable(IPVariable* &ipv, const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const=0;
  virtual void calledBy2lawFracture(){}
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw)=0;
  virtual double density() const{return _rho;}
  // As to be defined for explicit scheme. If law for implicit only
  // please define this function with return 0
  virtual double soundSpeed() const=0;
  // If law enable fracture the bool checkfrac allow to detect fracture. This bool is unused otherwise
  // The bool is necessery for computation of matrix by perturbation in this case the check of fracture
  // has not to be performed.
  virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true)=0;

  // 3D to 1D
  virtual void stress3DTo1D(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);

  // 3D to 1D with constant T
  virtual void stress3DTo1D_constantTriaxiality(double T, IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);

  // 3D to 2D
  virtual void stress3DTo2D(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);

  virtual void fillElasticStiffness(double E, double nu, STensor43 &tangent) const;
  virtual void setElasticStiffness(IPVariable* ipv) const {Msg::Error("define setElasticStiffness if using multiscale simulation");};
  virtual void initialIPVariable(IPVariable* ipv, bool stiff);
//  virtual const double bulkModulus() const {}
//  virtual void setBulkModulus(const double K){}
  // function to define when one IP is broken
  virtual materialLaw* clone() const = 0;
virtual void constitutive(const STensor3& F0,         // initial deformation gradient (input @ time n)
                            const STensor3& Fn,         // updated deformation gradient (input @ time n+1)
                            STensor3 &P,                // updated 1st Piola-Kirchhoff stress tensor (output)
                                                        // contains the initial values on input
                            const IPVariable *q0,       // array of initial internal variable
                            IPVariable *q1,              // updated array of internal variable (in ipvcur on output),
			    STensor43 &Tangent,         // constitutive tangents (output)
			    const bool stiff,
                            STensor43* elasticTangent = NULL
			    ) const
      {
        Msg::Error("dG3DMaterialLaw constitutive not defined");
      }
#endif // SWIG
  virtual bool getUseBarF() const { return _useBarF;}
  virtual void setUseBarF(bool useBarF) { _useBarF=useBarF;}
  virtual const materialLaw *getConstNonLinearSolverMaterialLaw() const      {
        Msg::Error("dG3DMaterialLaw getConstNonLinearSolverMaterialLaw() not defined");
      } // =0;
  virtual materialLaw *getNonLinearSolverMaterialLaw() {
        Msg::Error("dG3DMaterialLaw getNonLinearSolverMaterialLaw() not defined");
      }; //=0;
};

class dG3DLinearElasticMaterialLaw : public dG3DMaterialLaw{
	protected:
	#ifndef SWIG
    elasticPotential* _elasticLaw;
	#endif

	public:
		dG3DLinearElasticMaterialLaw(const int num, const double rho);
		dG3DLinearElasticMaterialLaw(const int num, const double rho, const double E, const double nu);
		void setElasticPotential(const elasticPotential* potential);
		#ifndef SWIG
		dG3DLinearElasticMaterialLaw(const dG3DLinearElasticMaterialLaw& src);
		virtual ~dG3DLinearElasticMaterialLaw(){
			if (_elasticLaw != NULL){
				delete _elasticLaw; _elasticLaw = NULL;
			}
		};
		virtual materialLaw::matname getType() const {return materialLaw::linearElastic;}
		virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  // To allow initialization of bulk ip in case of fracture
		virtual void createIPVariable(IPVariable* &ipv, const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
		virtual void initLaws(const std::map<int,materialLaw*> &maplaw);
		virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);
		virtual double scaleFactor() const;
		virtual double soundSpeed() const;
		virtual materialLaw* clone() const {return new dG3DLinearElasticMaterialLaw(*this);};
		virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const{}; // do nothing
    virtual bool withEnergyDissipation() const {return false;};
		

#endif //SWIG
};

class J2SmallStrainDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  #ifndef SWIG
   mlawJ2VMSmallStrain _j2law;
  #endif //SWIG

 public:
  J2SmallStrainDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const double sy0,const double h,const double tol=1.e-6,
                   const bool tangentByPart= false, const double pert=1e-8);
  J2SmallStrainDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,const double tol=1.e-6,
                   const bool tangentByPart= false, const double pert=1e-8);
 #ifndef SWIG
  J2SmallStrainDG3DMaterialLaw(const J2SmallStrainDG3DMaterialLaw &source);
  virtual ~J2SmallStrainDG3DMaterialLaw()
  {

  }
  // set the time of _j2law
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _j2law.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _j2law.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _j2law.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual const double bulkModulus() const {return _j2law.bulkModulus();}
	virtual double scaleFactor() const{return _j2law.scaleFactor();};
	virtual materialLaw* clone() const{return new J2SmallStrainDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _j2law.withEnergyDissipation();};

	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_j2law.setMacroSolver(sv);
	};
 #endif // SWIG
};

class J2LinearDG3DMaterialLaw : public dG3DMaterialLaw
{
    friend class EOSDG3DMaterialLaw;
 protected:
  mlawJ2linear _j2law;
  //double _E,_nu;
 private: // cache data
  //mutable STensor43 _tangent3D;
  //const STensor3 _ISTensor3;
  //mutable STensor3 tau,PKxyz,PKsh,invF,tauXYZ,Fp1,Fp0;
 public:
  J2LinearDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const double sy0,const double h,const double tol=1.e-6,
                   const bool matrixBypert=false, const double tolPert=1e-8);
  J2LinearDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,const double tol=1.e-6,
                   const bool matrixBypert=false, const double tolPert=1e-8);
  void setStrainOrder(const int order);
  void setViscosityEffect(const viscosityLaw& v, const double p);
  virtual const materialLaw *getConstNonLinearSolverMaterialLaw() const      {
      return &_j2law;
      } // =0;
  virtual materialLaw *getNonLinearSolverMaterialLaw() {
      return &_j2law;
      }; //=0;

 #ifndef SWIG
  J2LinearDG3DMaterialLaw(const J2LinearDG3DMaterialLaw &source);
  virtual ~J2LinearDG3DMaterialLaw()
  {

  }
  // set the time of _j2law
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _j2law.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _j2law.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _j2law.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual const double bulkModulus() const {return _j2law.bulkModulus();}
	virtual double scaleFactor() const{return _j2law.scaleFactor();}
	virtual materialLaw* clone() const{return new J2LinearDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _j2law.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_j2law.setMacroSolver(sv);
	};
 
 #endif // SWIG
};

class HyperViscoElasticDG3DMaterialLaw : public dG3DMaterialLaw{
  #ifndef SWIG
  protected:
    mlawHyperViscoElastic _viscoLaw;

  #endif //SWIG
  public:
    HyperViscoElasticDG3DMaterialLaw(const int num, const double rho, const double E,const double nu,
                        const bool matrixbyPerturbation = false, const double pert = 1e-8);
    void setStrainOrder(const int order);
    void setViscoelasticMethod(const int method);
    void setViscoElasticNumberOfElement(const int N);
    void setViscoElasticData(const int i, const double Ei, const double taui);
    void setViscoElasticData_Bulk(const int i, const double Ki, const double ki);
    void setViscoElasticData_Shear(const int i, const double Gi, const double gi);
    void setViscoElasticData(const std::string filename);

    #ifndef SWIG
     HyperViscoElasticDG3DMaterialLaw(const HyperViscoElasticDG3DMaterialLaw &source);
    virtual ~HyperViscoElasticDG3DMaterialLaw(){}
    // set the time of _j2law
    virtual void setTime(const double t,const double dtime){
      dG3DMaterialLaw::setTime(t,dtime);
      _viscoLaw.setTime(t,dtime);
    }
    virtual materialLaw::matname getType() const {return _viscoLaw.getType();}
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const{return _viscoLaw.soundSpeed();} // or change value ??
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual const double bulkModulus() const {return _viscoLaw.bulkModulus();}
    virtual double scaleFactor() const{return _viscoLaw.scaleFactor();}
    virtual materialLaw* clone() const{return new HyperViscoElasticDG3DMaterialLaw(*this);};
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
    virtual bool withEnergyDissipation() const {return _viscoLaw.withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      _viscoLaw.setMacroSolver(sv);
    };
    #endif //SWIG
};

class HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw  : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    mlawPowerYieldHyper _viscoLaw;
    #endif //SWIG
  public:
    HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw(const int num, const double rho, const double E,const double nu, const double tol =1e-6,
                        const bool matrixbyPerturbation = false, const double pert = 1e-8);
    void setStrainOrder(const int order);
    void setViscoelasticMethod(const int method);
    void setViscoElasticNumberOfElement(const int N);
    void setViscoElasticData(const int i, const double Ei, const double taui);
    void setViscoElasticData_Bulk(const int i, const double Ki, const double ki);
    void setViscoElasticData_Shear(const int i, const double Gi, const double gi);
    void setViscoElasticData(const std::string filename);

    void setCompressionHardening(const J2IsotropicHardening& comp);
    void setTractionHardening(const J2IsotropicHardening& trac);
    void setKinematicHardening(const kinematicHardening& kin);
    void setViscosityEffect(const viscosityLaw& v, const double p);

    void setYieldPowerFactor(const double n);
    void nonAssociatedFlowRuleFactor(const double b);
    void setPlasticPoissonRatio(const double nup);
    void setNonAssociatedFlow(const bool flag);

    #ifndef SWIG
    HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw(const HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw& src);
    virtual ~HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw(){};
    virtual void setTime(const double t,const double dtime){
      dG3DMaterialLaw::setTime(t,dtime);
      _viscoLaw.setTime(t,dtime);
    }
    virtual materialLaw::matname getType() const {return _viscoLaw.getType();}
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const{return _viscoLaw.soundSpeed();} // or change value ??
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual const double bulkModulus() const {return _viscoLaw.bulkModulus();}
    virtual double scaleFactor() const{return _viscoLaw.scaleFactor();}
    virtual materialLaw* clone() const{return new HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw(*this);};
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
    virtual bool withEnergyDissipation() const {return _viscoLaw.withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      _viscoLaw.setMacroSolver(sv);
    };
    #endif //SWIG
};

//======================================================ViscoelasticDG3DMaterialLaw===============================================================//
class ViscoelasticDG3DMaterialLaw : public dG3DMaterialLaw
{
    friend class EOSDG3DMaterialLaw;
 protected:
  mlawViscoelastic _Vislaw;
 private: // cache data

 public:

  ViscoelasticDG3DMaterialLaw(const int num,const double rho,
                              const double K, const std::string &array_eta, int array_size_eta, const std::string &array_mu,
                              const double tol=1.e-6, const bool pert= false, const double eps=1.e-8);


 #ifndef SWIG
  ViscoelasticDG3DMaterialLaw(const ViscoelasticDG3DMaterialLaw &source);
  virtual ~ViscoelasticDG3DMaterialLaw()
  {

  }
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _Vislaw.setTime(t,dtime);

  }
  virtual materialLaw::matname getType() const {return _Vislaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _Vislaw.soundSpeed();}
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual const double bulkModulus() const {return _Vislaw.bulkModulus();}
  virtual std::vector<double> read_file(const char* file_name, int file_size); //read from files
	virtual materialLaw* clone() const{return new ViscoelasticDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _Vislaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_Vislaw.setMacroSolver(sv);
	};
 #endif // SWIG
};


//--------------------------------------------EOSDG3DMaterialLaw (compatible with the other material laws)-------------------------//
class EOSDG3DMaterialLaw : public dG3DMaterialLaw
{
    friend class J2LinearDG3DMaterialLaw;
    friend class ViscoelasticDG3DMaterialLaw;
  protected:
  std::map<int,materialLaw*> maplaw;   // chosen deviatoric material law
  mlawEOS _EOSlaw;

  public:

//  static void cout2DMat(const STensor3 &a, const STensor3 &b, STensor3 &c);

  EOSDG3DMaterialLaw(const int num,
               const double rho,const double K, materialLaw *_mlaw, const double Pressure0 = 0.);
  // Accoustic EOS for non-artificial viscosity case
  EOSDG3DMaterialLaw(int flag, const int num,
          const double rho,const double P1, double P2, double P3, materialLaw *_mlaw, const double Pressure0 = 0.);
  // P1 = s, P2 = C0, P3 = Gamma: Hugoniot EOS for non-artificial viscosity case
  // P1 = K, P2 = c1, P3 = cl: Accoustic EOS for artificial viscosity case
  EOSDG3DMaterialLaw(int flag, const int num,
          const double rho,const double s, const double C0,double c1, double cl, const double Gamma, materialLaw *_mlaw);
  // Hugoniot EOS for artificial viscosity case
  EOSDG3DMaterialLaw(int flag, int flagC, const int num,
          const double rho,const double K, double c1, double cl, double ESqrd, double ESlin, materialLaw *_mlaw, const double Pressure0 = 0.);
 #ifndef SWIG
  EOSDG3DMaterialLaw(const EOSDG3DMaterialLaw &source);

  virtual ~EOSDG3DMaterialLaw()
  {

  }
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    maplaw.begin()->second->setTime(t,dtime);
    _EOSlaw.setTime(t,dtime);
  }

  virtual materialLaw::matname getType() const {return materialLaw::EOS;}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const {return _EOSlaw.soundSpeed();}
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual materialLaw* clone() const{return new EOSDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _EOSlaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_EOSlaw.setMacroSolver(sv);
		for (std::map<int,materialLaw*>::iterator it = maplaw.begin(); it != maplaw.end(); it++){
			it->second->setMacroSolver(sv);
		}
	};
 #endif // SWIG
};


class VUMATinterfaceDG3DMaterialLaw : public dG3DMaterialLaw
{

 protected:
  mlawVUMATinterface _vumatlaw;

 public:
  VUMATinterfaceDG3DMaterialLaw(const int num, const double rho,
                                const char *propName);
  virtual ~VUMATinterfaceDG3DMaterialLaw() {};

#ifndef SWIG
  VUMATinterfaceDG3DMaterialLaw(const VUMATinterfaceDG3DMaterialLaw &source);
  // set the time of _vumatlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _vumatlaw.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _vumatlaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _vumatlaw.soundSpeed();}
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual materialLaw* clone() const{return new VUMATinterfaceDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _vumatlaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_vumatlaw.setMacroSolver(sv);
	};
#endif
};

class TransverseIsotropicDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  mlawTransverseIsotropic _tilaw;
  double _E,_nu, _EA, _GA, _nu_minor;

 public:
  TransverseIsotropicDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu, const double EA,
		   const double GA, const double nu_minor,
                   const double Ax, const double Ay, const double Az);
  virtual const materialLaw *getConstNonLinearSolverMaterialLaw() const      {
      return &_tilaw;
      } // =0;
  virtual materialLaw *getNonLinearSolverMaterialLaw() {
      return &_tilaw;
      }; //=0;
 #ifndef SWIG
  TransverseIsotropicDG3DMaterialLaw(const TransverseIsotropicDG3DMaterialLaw &source);
  virtual ~TransverseIsotropicDG3DMaterialLaw(){}
  // set the time of _tilaw
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _tilaw.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _tilaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _tilaw.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual double scaleFactor() const{return _tilaw.scaleFactor();};
	virtual materialLaw* clone() const{return new TransverseIsotropicDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _tilaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_tilaw.setMacroSolver(sv);
	};

 
 #endif // SWIG
};




class TransverseIsoCurvatureDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  mlawTransverseIsoCurvature _tilaw;
  double _E,_nu, _EA, _GA, _nu_minor;

 public:
  TransverseIsoCurvatureDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu, const double EA,
		   const double GA, const double nu_minor,
                   const double Centroid_x, const double Centroid_y, const double Centroid_z,
                   const double PointStart1_x, const double PointStart1_y, const double PointStart1_z,
                   const double PointStart2_x,  const double PointStart2_y, const double PointStart2_z, const double init_Angle);
 #ifndef SWIG
  TransverseIsoCurvatureDG3DMaterialLaw(const TransverseIsoCurvatureDG3DMaterialLaw &source);
  virtual ~TransverseIsoCurvatureDG3DMaterialLaw(){}
  // set the time of _tilaw
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _tilaw.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _tilaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _tilaw.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual double scaleFactor() const{return _tilaw.scaleFactor();};
	virtual materialLaw* clone() const{return new TransverseIsoCurvatureDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _tilaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_tilaw.setMacroSolver(sv);
	};
 #endif // SWIG
};

class TransverseIsoYarnBDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  mlawTransverseIsoYarnB _tilaw;

 public:
  //Remarks: Two constructors are defined, the first one is for the yarn and the second one is for the matrix.
  //Constructor for yarn
  TransverseIsoYarnBDG3DMaterialLaw(int lawnum,//tag of the material law
                                    double rho, //material density
                                    int physicnum,//physic number of
                                    char* meshfile, //vector container of YarnDirections
                                    int isunidir,   // 0:Woven yarn. 1:Straight yarn. 2:Homogenuous material
                                    double rDSvDT,  //To initialize '_CriticalrDSvDT'
                                    double MaxStrainRate, //The maximal value of the strain rate
			            double Cxx11,
		                    double Cxx21,
			            double Cxx22,
			            double Cxx31,
			            double Cxx32,
			            double Cyy11,
			            double Cyy21,
			            double Cyy22,
			            double Cyy31,
			            double Cyy32,
			            double Cxy11,
			            double Cxy21,
			            double Cxy22,
			            double Cxy31,
			            double Cxy32,
			            double Cyz11,
			            double Cyz21,
			            double Cyz22,
			            double Cyz31,
			            double Cyz32,
                                    double nuxy,
                                    double nuxz,
                                    double nuyz);

  //Constructor for matrix
  TransverseIsoYarnBDG3DMaterialLaw(int lawnum,//tag of the material law
                                    double rho, //material density
                                    int physicnum,//physic number of
                                    double rDSvDT,  //To initialize '_CriticalrDSvDT'
                                    double MaxStrainRate, //The maximal value of the strain rate
			            double Cxx11,
		                    double Cxx21,
			            double Cxx22,
			            double Cxx31,
			            double Cxx32,
			            double Cyy11,
			            double Cyy21,
			            double Cyy22,
			            double Cyy31,
			            double Cyy32,
			            double Cxy11,
			            double Cxy21,
			            double Cxy22,
			            double Cxy31,
			            double Cxy32,
			            double Cyz11,
			            double Cyz21,
			            double Cyz22,
			            double Cyz31,
			            double Cyz32,
                                    double nuxy,
                                    double nuxz,
                                    double nuyz);

 #ifndef SWIG

  virtual ~TransverseIsoYarnBDG3DMaterialLaw(){}
  // set the time of _tilaw

   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _tilaw.setTime(t,dtime);
  }


  virtual materialLaw::matname getType() const{return _tilaw.getType();}

  virtual void createIPState(IPStateBase* &ips,
                             const bool* state_=NULL,
                             const MElement *ele=NULL,
                             const int nbFF_=0,
                             const IntPt *GP=NULL,
                             const int gpt = 0) const;

  virtual void createIPVariable(IPVariable* &ipv,
                                const MElement *ele,
                                const int nbFF,
                                const IntPt *GP,
                                const int gpt) const;

  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}

  virtual double soundSpeed() const{return _tilaw.soundSpeed();}
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual materialLaw* clone() const {return new TransverseIsoYarnBDG3DMaterialLaw(*this);};
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _tilaw.withEnergyDissipation();};
  TransverseIsoYarnBDG3DMaterialLaw(const TransverseIsoYarnBDG3DMaterialLaw &source);
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_tilaw.setMacroSolver(sv);
	};
 #endif // SWIG
};

class AnisotropicDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  mlawAnisotropic _tilaw;
 private: // cache data
  mutable STensor43 _tangent3D;
  const STensor3 _ISTensor3;
  mutable STensor3 tau,PKxyz,PKsh,invF,tauXYZ,Fp1,Fp0;
 public:
  AnisotropicDG3DMaterialLaw(const int num,const double rho,
				const double Ex,const double Ey,const double Ez,
				const double Vxy,const double Vxz,const double Vyz,
				const double MUxy,const double MUxz,const double MUyz,
                   		const double alpha, const double beta, const double gamma);
  AnisotropicDG3DMaterialLaw(const int num,const double rho,
				const double Ex,const double Ey,const double Ez,
				const double Vxy,const double Vxz,const double Vyz,
				const double MUxy,const double MUxz,const double MUyz,
                   		const double euler[3]);
 #ifndef SWIG
  AnisotropicDG3DMaterialLaw(const AnisotropicDG3DMaterialLaw &source);
  virtual ~AnisotropicDG3DMaterialLaw(){}
  // set the time of _tilaw
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _tilaw.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _tilaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _tilaw.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual double scaleFactor() const{return _tilaw.scaleFactor();}
	virtual materialLaw* clone() const{return new AnisotropicDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _tilaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_tilaw.setMacroSolver(sv);
	};
 #endif // SWIG
};


class AnisotropicStochDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  mlawAnisotropicStoch _tilaw;

 public:
  AnisotropicStochDG3DMaterialLaw(const int num, const double rho, const double alpha,
                                  const double beta, const double gamma, const double OrigX, const double OrigY, const char *propName, const int intpl);

 #ifndef SWIG
  AnisotropicStochDG3DMaterialLaw(const AnisotropicStochDG3DMaterialLaw &source);
  virtual ~AnisotropicStochDG3DMaterialLaw(){}
  // set the time of _tilaw
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _tilaw.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _tilaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _tilaw.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual double scaleFactor() const{return _tilaw.scaleFactor();}
	virtual materialLaw* clone() const{return new AnisotropicStochDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _tilaw.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_tilaw.setMacroSolver(sv);
	};
 #endif // SWIG
};


class ClusterDG3DMaterialLaw : public dG3DMaterialLaw
{
 protected:
  mlawClusterMaterialLaws _clusterlaw;

 public:
  ClusterDG3DMaterialLaw(const int num, const char *clusterName);
  void addClusterMaterialLaw(int clusterNb, int materialLawNb)
  {
    _clusterlaw.addClusterMaterialLaw(clusterNb, materialLawNb);
  }
  void addClusterEigenStrain(int clusterNb, double F11,double F12,double F13,double F21,double F22,double F23,double F31,double F32,double F33);
 #ifndef SWIG
  ClusterDG3DMaterialLaw(const ClusterDG3DMaterialLaw &source);
  virtual ~ClusterDG3DMaterialLaw(){}
  // set the time of _tilaw
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _clusterlaw.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _clusterlaw.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw)
  {
    _clusterlaw.initLaws(maplaw);
  }
  virtual double soundSpeed() const{return _clusterlaw.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual double scaleFactor() const{return _clusterlaw.scaleFactor();}
  virtual materialLaw* clone() const{return new ClusterDG3DMaterialLaw(*this);};
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _clusterlaw.withEnergyDissipation();};
  virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_clusterlaw.setMacroSolver(sv);
  };
 #endif // SWIG
};

class LocalDamageHyperelasticDG3DMaterialLaw: public dG3DMaterialLaw{
protected:
	#ifndef SWIG
		mlawLocalDamageHyperelastic* _nlLaw;
	#endif
	public:
		LocalDamageHyperelasticDG3DMaterialLaw(const int num, const double rho, const elasticPotential& elP, const DamageLaw &damLaw);

		#ifndef SWIG
		LocalDamageHyperelasticDG3DMaterialLaw(const LocalDamageHyperelasticDG3DMaterialLaw& src);
		virtual ~LocalDamageHyperelasticDG3DMaterialLaw(){
      if (_nlLaw) delete _nlLaw;
    };
		virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const {return _nlLaw->getType();};

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw) {}
    virtual double soundSpeed() const {return _nlLaw->soundSpeed();};
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
		virtual double scaleFactor() const {return _nlLaw->scaleFactor();}
		virtual materialLaw* clone() const{return new LocalDamageHyperelasticDG3DMaterialLaw(*this);};
		virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
    virtual bool withEnergyDissipation() const {return _nlLaw->withEnergyDissipation();};
		virtual void setMacroSolver(const nonLinearMechSolver* sv){
			dG3DMaterialLaw::setMacroSolver(sv);
			_nlLaw->setMacroSolver(sv);
		};
		#endif
};

class LocalAnisotropicDamageHyperelasticDG3DMaterialLaw: public LocalDamageHyperelasticDG3DMaterialLaw{

  public:
		LocalAnisotropicDamageHyperelasticDG3DMaterialLaw(const int num, const double rho, const elasticPotential& elP, const DamageLaw &damLaw);

		#ifndef SWIG
		LocalAnisotropicDamageHyperelasticDG3DMaterialLaw(const LocalAnisotropicDamageHyperelasticDG3DMaterialLaw& src): LocalDamageHyperelasticDG3DMaterialLaw(src){};
		virtual ~LocalAnisotropicDamageHyperelasticDG3DMaterialLaw(){
    };
    #endif //SWIG
};

class LocalDamageJ2HyperDG3DMaterialLaw : public dG3DMaterialLaw
{

 protected:
  mlawLocalDamageJ2Hyper *_nldJ2Hyperlaw; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

 public:
  LocalDamageJ2HyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,
                   const DamageLaw &_damLaw,const double tol=1.e-6);
  virtual ~LocalDamageJ2HyperDG3DMaterialLaw();

#ifndef SWIG
  LocalDamageJ2HyperDG3DMaterialLaw(const LocalDamageJ2HyperDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime);
  virtual materialLaw::matname getType() const;

  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual double scaleFactor() const{return _nldJ2Hyperlaw->scaleFactor();}
	virtual materialLaw* clone() const{ return new LocalDamageJ2HyperDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _nldJ2Hyperlaw->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_nldJ2Hyperlaw != NULL){
			_nldJ2Hyperlaw->setMacroSolver(sv);
		}
	}
#endif
};

class LinearThermoMechanicsDG3DMaterialLaw : public dG3DMaterialLaw // public materialLaw
{

 protected:
  mlawLinearThermoMechanics *_lawLinearTM; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

  STensor3                  linearK;
  STensor3		    Stiff_alphaDilatation;
 public:
  LinearThermoMechanicsDG3DMaterialLaw(const int num,const double rho, const double Ex, const double Ey, const double Ez, const double Vxy, const double Vxz, const double Vyz,
				       const double MUxy, const double MUxz, const double MUyz, const double alpha, const double beta, const double gamma,const double t0,
				       const double Kx,const double Ky,const double Kz,const double alphax,const double alphay,const double alphaz,const double cp);
  virtual ~LinearThermoMechanicsDG3DMaterialLaw() {
		if (_lawLinearTM != NULL){
			delete _lawLinearTM;
			_lawLinearTM = NULL;
		}
	};

#ifndef SWIG
 LinearThermoMechanicsDG3DMaterialLaw(const LinearThermoMechanicsDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawLinearTM->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawLinearTM->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawLinearTM->soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
	virtual double scaleFactor() const {return _lawLinearTM->shearModulus();};
  virtual int getNumberOfExtraDofsDiffusion() const {return 1;}
  virtual double getExtraDofStoredEnergyPerUnitField(double T) const;
	virtual double getInitialExtraDofStoredEnergyPerUnitField() const;
  virtual double defoEnergy(const STensor3& Fn,const SVector3& gradT) const;
  virtual double defoEnergy( const SVector3&ujump,const double &Tjump, const SVector3& N) const;
	virtual materialLaw* clone() const{ return new LinearThermoMechanicsDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _lawLinearTM->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_lawLinearTM!=NULL){
			_lawLinearTM->setMacroSolver(sv);
		}
	}
#endif
};

class J2ThermoMechanicsDG3DMaterialLaw : public dG3DMaterialLaw
{
    friend class EOSDG3DMaterialLaw;
 protected:
  mlawJ2ThermoMechanics _j2law;
  STensor3            linearK;
  STensor3	      Stiff_alphaDilatation;
 public:
  J2ThermoMechanicsDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,
                   const double t0,const double Kther, const double alphather, const double cp, const double tol=1.e-6);
 #ifndef SWIG
  J2ThermoMechanicsDG3DMaterialLaw(const J2ThermoMechanicsDG3DMaterialLaw &source);
  virtual ~J2ThermoMechanicsDG3DMaterialLaw()
  {

  }
  // set the time of _j2law
   virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _j2law.setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _j2law.getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _j2law.soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual const double bulkModulus() const {return _j2law.bulkModulus();}
	virtual double scaleFactor() const{return _j2law.shearModulus();};
  virtual int getNumberOfExtraDofsDiffusion() const {return 1;}
  virtual double getExtraDofStoredEnergyPerUnitField(double T) const;
	virtual double getInitialExtraDofStoredEnergyPerUnitField() const;
	virtual materialLaw* clone() const{ return new J2ThermoMechanicsDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _j2law.withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		_j2law.setMacroSolver(sv);
	}
 #endif // SWIG
};

class FullJ2ThermoMechanicsDG3DMaterialLaw : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    mlawJ2FullyCoupledThermoMechanics _j2FullThermo;
    #endif // SWIG

  public:
    FullJ2ThermoMechanicsDG3DMaterialLaw(const int num, const double E, const double nu,const double rho,
                               const double sy0,const double h,const double tol=1.e-6,
                               const bool matrixbyPerturbation = false, const double pert = 1e-8);
		void setStrainOrder(const int i);
    void setReferenceTemperature(const double Tr);
    void setIsotropicHardeningLaw(const J2IsotropicHardening& isoHard);

    void setReferenceThermalExpansionCoefficient(const double al);
    void setTemperatureFunction_ThermalExpansionCoefficient(const scalarFunction& Tfunc);

    void setReferenceThermalConductivity(const double KK);
    void setTemperatureFunction_ThermalConductivity(const scalarFunction& Tfunc);

    void setReferenceCp(const double Cp);
    void setTemperatureFunction_Cp(const scalarFunction& Tfunc);

    void setTemperatureFunction_Hardening(const scalarFunction& Tfunc);
    void setTemperatureFunction_InitialYieldStress(const scalarFunction& Tfunc);
    void setTemperatureFunction_BulkModulus(const scalarFunction& Tfunc);
    void setTemperatureFunction_ShearModulus(const scalarFunction& Tfunc);
    void setTaylorQuineyFactor(const double f);
    void setThermalEstimationPreviousConfig(const bool flag);
    #ifndef SWIG
    FullJ2ThermoMechanicsDG3DMaterialLaw(const FullJ2ThermoMechanicsDG3DMaterialLaw &source);
    virtual ~FullJ2ThermoMechanicsDG3DMaterialLaw(){}
    // set the time of _nldlaw
    virtual materialLaw::matname getType() const {return _j2FullThermo.getType();}
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const{return _j2FullThermo.soundSpeed();}
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true,const bool checkfrac=true);
    virtual double scaleFactor() const;
    virtual int getNumberOfExtraDofsDiffusion() const {return 1;}
    virtual double getInitialExtraDofStoredEnergyPerUnitField() const;
    virtual double getExtraDofStoredEnergyPerUnitField(double T) const;
    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime){
      dG3DMaterialLaw::setTime(t,dtime);
      _j2FullThermo.setTime(t,dtime);
    }
		virtual materialLaw* clone() const{ return new FullJ2ThermoMechanicsDG3DMaterialLaw(*this);};
		virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
    virtual bool withEnergyDissipation() const {return _j2FullThermo.withEnergyDissipation();};
		virtual void setMacroSolver(const nonLinearMechSolver* sv){
			dG3DMaterialLaw::setMacroSolver(sv);
			_j2FullThermo.setMacroSolver(sv);
		}
    #endif // SWIG
};



class mlawAnIsotropicTherMechDG3DMaterialLaw :public dG3DMaterialLaw
{

 protected:
   mlawAnIsotropicTherMech  *_lawAnTM;
   STensor3 	 Stiff_alphaDilatation;
   STensor3                  linearK;

 public:


  mlawAnIsotropicTherMechDG3DMaterialLaw(const int num,const double E, const double nu,const double rho,const double EA, const double GA,const double nu_minor,const double Ax,
					     const double Ay, const double Az, const double alpha, const double beta, const double gamma,const double t0,
					    const double kx,const double ky, const double kz,const double cp,const double alphath);

#ifndef SWIG
	virtual ~mlawAnIsotropicTherMechDG3DMaterialLaw() {
		if (_lawAnTM != NULL){
			delete _lawAnTM;
			_lawAnTM = NULL;
		};
	}
  mlawAnIsotropicTherMechDG3DMaterialLaw(const mlawAnIsotropicTherMechDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawAnTM->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawAnTM->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawAnTM->soundSpeed();} // or change value ??
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual int getNumberOfExtraDofsDiffusion() const {return 2;}
  double getExtraDofStoredEnergyPerUnitField(double T) const
  {
   Msg::Error("getExtraDofStoredEnergyPerUnitField not defined");
  }
	virtual materialLaw* clone() const{ return new mlawAnIsotropicTherMechDG3DMaterialLaw(*this);};
	virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual bool withEnergyDissipation() const {return _lawAnTM->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_lawAnTM!=NULL)
			_lawAnTM->setMacroSolver(sv);
	}
#endif
};
//class LinearSMPDG3DMaterialLaw : public LinearThermoMechanicsDG3DMaterialLaw  // public materialLaw
class SMPDG3DMaterialLaw :public dG3DMaterialLaw // ,public mlawSMP
{

 protected:
  mlawSMP *_lawTMSMP; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

  STensor3            linearK;
  STensor3 	      Stiff_alphaDilatation;
 public:

  SMPDG3DMaterialLaw(const int num,const double rho,const double alpha, const double beta, const double gamma ,const double t0, const double Kx,const double Ky, const double Kz,
       const double mu_groundState3,const double Im3, const double pi,const double Tr, const double Nmu_groundState2 ,const double mu_groundState2 ,
       const double Im2,const double Sgl2,const double Sr2,const double Delta,const double m2, const double epsilonr ,const double n,const double epsilonp02,
       const double alphar1,const double alphagl1,const double Ggl1,const double Gr1 ,const double Mgl1,const double Mr1,const double Mugl1,
       const double Mur1,const double epsilon01,const double Qgl1,const double Qr1,const double epsygl1 ,const double d1,
      const double m1,const double V1,const double alphap, const double  Sa0,const double ha1,const double b1,const double g1,
       const double phia01,const double Z1, const double r1,const double s1,const double Sb01,const double Hgl1,const double Lgl1,const double Hr1,
       const double Lr1,const double l1,const double Kb,const double be1,const double c0,const double wp,const double c1);

  void setStrainOrder(const int order);
  virtual void setMechanism2(bool mechanism2);

#ifndef SWIG
	virtual ~SMPDG3DMaterialLaw() {
		if (_lawTMSMP !=NULL) {delete _lawTMSMP; _lawTMSMP = NULL;};
	};
 SMPDG3DMaterialLaw(const SMPDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawTMSMP->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawTMSMP->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawTMSMP->soundSpeed();} // or change value ??
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual int getNumberOfExtraDofsDiffusion() const {return 1;}
  double getExtraDofStoredEnergyPerUnitField(double T) const;
	virtual materialLaw* clone() const{ return new SMPDG3DMaterialLaw(*this);};
  virtual bool withEnergyDissipation() const {return _lawTMSMP->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_lawTMSMP!=NULL)
			_lawTMSMP->setMacroSolver(sv);
	}
#endif
};


class PhenomenologicalSMPDG3DMaterialLaw :public dG3DMaterialLaw // ,public mlawSMP
{

 protected:
  mlawPhenomenologicalSMP *_lawTMSMP; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)

  STensor3            linearK;
  STensor3 	      Stiff_alphaDilatation;
 public:

  PhenomenologicalSMPDG3DMaterialLaw(const int num, const double rho, const double alpha, const double beta, const double gamma ,const double t0, const double Kxg,const double Kyg, const double Kzg, const double Kxr, const double Kyr, const double Kzr,const double TaylorQuiney, const double c, const double cp, const double cg, const double cr,const double Tg,const double Tc,const double xi,const double w1,const double w2, const double Eg, const double Er, const double nug, const double nur,const double alphag, const double alphar,const double alpha_trans, const double tau_y0);
  PhenomenologicalSMPDG3DMaterialLaw(const int num,const double rho,const J2IsotropicHardening &j2IH,const double alpha, const double beta, const double gamma,const double t0,const double Kxg,const double Kyg, const double Kzg, const double Kxr, const double Kyr, const double Kzr, const double TaylorQuiney, const double c, const double cp, const double cg, const double cr, const double Tg,const double Tc,const double xi, const double w1,const double w2,const double Eg, const double Er, const double nug, const double nur, const double alphag, const double alphar,const double alpha_trans,const double tau_y0);

  void setStrainOrder(const int order);
  virtual void setIsotropicHardeningLaw(const J2IsotropicHardening& isoHard);
  virtual void setTemperatureFunction_ThermalExpansionCoefficient_glassy(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_ThermalExpansionCoefficient_rubbery(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_ThermalConductivity_glassy(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_ThermalConductivity_rubbery(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_Cp_glassy(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_Cp_rubbery(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_Hardening(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_InitialYieldStress(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_BulkModulus_glassy(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_BulkModulus_rubbery(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_ShearModulus_glassy(const scalarFunction& Tfunc);
  virtual void setTemperatureFunction_ShearModulus_rubbery(const scalarFunction& Tfunc);

#ifndef SWIG

  virtual ~PhenomenologicalSMPDG3DMaterialLaw() {
    if (_lawTMSMP !=NULL) {delete _lawTMSMP; _lawTMSMP = NULL;};
  };
  PhenomenologicalSMPDG3DMaterialLaw(const PhenomenologicalSMPDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawTMSMP->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawTMSMP->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawTMSMP->soundSpeed();} // or change value ??
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual int getNumberOfExtraDofsDiffusion() const {return 1;}
  double getExtraDofStoredEnergyPerUnitField(double T) const;
  virtual materialLaw* clone() const{ return new PhenomenologicalSMPDG3DMaterialLaw(*this);};
  virtual bool withEnergyDissipation() const {return _lawTMSMP->withEnergyDissipation();};
  virtual void setMacroSolver(const nonLinearMechSolver* sv){
    dG3DMaterialLaw::setMacroSolver(sv);
    if (_lawTMSMP!=NULL)
      _lawTMSMP->setMacroSolver(sv);
  }
#endif
};

class crystalPlasticityDG3DMaterialLaw : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    mlawCrystalPlasticity *_cplaw; 
    #endif //SWIG

  public:
    crystalPlasticityDG3DMaterialLaw(const int num, const double rho, const double temperature,
                                const char *propName);
    #ifndef SWIG
    crystalPlasticityDG3DMaterialLaw(const crystalPlasticityDG3DMaterialLaw &source);
    virtual ~crystalPlasticityDG3DMaterialLaw();

    // set the time of _cplaw
    virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const;
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const;// or change value ??
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual double scaleFactor() const;
    virtual materialLaw* clone() const{return new crystalPlasticityDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const;
    virtual void setMacroSolver(const nonLinearMechSolver* sv);
    #endif
};


class LinearElecTherMechDG3DMaterialLaw : public dG3DMaterialLaw // public materialLaw
{

 protected:
  mlawLinearElecTherMech *_lawLinearETM; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)
  STensor3 	         Stiff_alphaDilatation;
  STensor3                lineark10, lineark20, linearl10, linearl20, jy1;
 public:
  LinearElecTherMechDG3DMaterialLaw(const int num,const double rho,const double Ex, const double Ey, const double Ez, const double Vxy, const double Vxz, const double Vyz,
		const double MUxy, const double MUxz, const double MUyz, const double alpha, const double beta, const double gamma,const double t0,const double Kx,
		const double Ky,const double Kz,const double alphax,const double alphay, const double alphaz,const  double lx,const double ly,const double lz,const double seebeck,
		const double c,const double v0);




#ifndef SWIG
	virtual ~LinearElecTherMechDG3DMaterialLaw() {
		if (_lawLinearETM !=NULL){ delete _lawLinearETM; _lawLinearETM = NULL;};
	}
 LinearElecTherMechDG3DMaterialLaw(const LinearElecTherMechDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawLinearETM->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawLinearETM->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawLinearETM->soundSpeed();} // or change value ??
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual int getNumberOfExtraDofsDiffusion() const {return 2;}
  double getExtraDofStoredEnergyPerUnitField(double T) const
  {
   Msg::Error("getExtraDofStoredEnergyPerUnitField not defined");
  }
	virtual materialLaw* clone() const{ return new LinearElecTherMechDG3DMaterialLaw(*this);};
  virtual bool withEnergyDissipation() const {return _lawLinearETM->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_lawLinearETM!=NULL)
			_lawLinearETM->setMacroSolver(sv);
	}
#endif
};




class mlawAnIsotropicElecTherMechDG3DMaterialLaw :public dG3DMaterialLaw
{

 protected:
   mlawAnIsotropicElecTherMech  *_lawETM;
   STensor3 	 Stiff_alphaDilatation;

 // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)
 //modified
  STensor3                Kref10, Kref20, Lref10, Lref20, jy1;
 public:


  mlawAnIsotropicElecTherMechDG3DMaterialLaw(const int num,const double E, const double nu,const double rho,const double EA, const double GA,const double nu_minor,const double Ax,
					     const double Ay, const double Az, const double alpha, const double beta, const double gamma,const double t0,const  double lx,
					     const  double ly,const  double lz,const double seebeck,const double kx,const double ky, const double kz,const double cp,const double alphath,const double v0);


#ifndef SWIG
 mlawAnIsotropicElecTherMechDG3DMaterialLaw(const mlawAnIsotropicElecTherMechDG3DMaterialLaw &source);
 virtual ~mlawAnIsotropicElecTherMechDG3DMaterialLaw() {
	 if (_lawETM != NULL){
		delete _lawETM;
		_lawETM = NULL;
	 }
 }
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawETM->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawETM->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawETM->soundSpeed();} // or change value ??
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual int getNumberOfExtraDofsDiffusion() const {return 2;}
  double getExtraDofStoredEnergyPerUnitField(double T) const
  {
   Msg::Error("getExtraDofStoredEnergyPerUnitField not defined");
  }
	virtual materialLaw* clone() const{ return new mlawAnIsotropicElecTherMechDG3DMaterialLaw(*this);};
  virtual bool withEnergyDissipation() const {return  _lawETM->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_lawETM!=NULL)
			_lawETM->setMacroSolver(sv);
	}
#endif
};



class mlawElecSMPDG3DMaterialLaw :public dG3DMaterialLaw
{

 protected:
   mlawElecSMP  *_lawETMSMP;
   STensor3	 Stiff_alphaDilatation;
 // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)
//modified
 STensor3                K10, K20, L10, L20, jy1;
 public:


  mlawElecSMPDG3DMaterialLaw(const int num,const double rho, const double alpha, const double beta, const double gamma,const double t0,const double Kx,const double Ky,
			 const double Kz, const double mu_groundState3,const double Im3, const double pi  ,const double Tr,
		          const double Nmu_groundState2,const double mu_groundState2, const double Im2,const double Sgl2,const double Sr2,const double Delta,const double m2, const double epsilonr,
			  const double n, const double epsilonp02,const double alphar1,const double alphagl1,const double Ggl1,const double Gr1,const double Mgl1,const double Mr1,const double Mugl1
			  ,const double Mur1, const double epsilon01,const double QglOnKb1,const double QrOnKb1,const double epsygl1 ,const double d1,const double m1,const double VOnKb1,
			  const double alphap,const double  Sa0,const double ha1,const double b1,const double g1,const double phia01,const double Z1,const double r1,const double s1,const double Sb01
			  ,const double Hgl1,const double Lgl1,const double Hr1,const double Lr1,const double l1,const double Kb,const double be1,const double c0,const double wp,const double c1
			  ,const double lx,const double ly,const double lz,const double seebeck,const double v0);

  void setStrainOrder(const int order);
  virtual void setMechanism2(bool mechanism2);

#ifndef SWIG
	virtual ~mlawElecSMPDG3DMaterialLaw() {
		if (_lawETMSMP != NULL){
			delete _lawETMSMP;
			_lawETMSMP = NULL;
		}
	};
 mlawElecSMPDG3DMaterialLaw(const mlawElecSMPDG3DMaterialLaw &source);
  // set the time of _nldlaw
  virtual void setTime(const double t,const double dtime){
    dG3DMaterialLaw::setTime(t,dtime);
    _lawETMSMP->setTime(t,dtime);
  }
  virtual materialLaw::matname getType() const {return _lawETMSMP->getType();}
  virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
  virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
  virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
  virtual double soundSpeed() const{return _lawETMSMP->soundSpeed();} // or change value ??
  virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
  virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
  virtual int getNumberOfExtraDofsDiffusion() const {return 2;}
  double getExtraDofStoredEnergyPerUnitField(double T) const
  {
   Msg::Error("getExtraDofStoredEnergyPerUnitField not defined");
  }
	virtual materialLaw* clone() const{ return new mlawElecSMPDG3DMaterialLaw(*this);};
  virtual bool withEnergyDissipation() const {return _lawETMSMP->withEnergyDissipation();};
	virtual void setMacroSolver(const nonLinearMechSolver* sv){
		dG3DMaterialLaw::setMacroSolver(sv);
		if (_lawETMSMP!=NULL)
			_lawETMSMP->setMacroSolver(sv);
	}
#endif
};

#endif
