//
// Author: Van Dung NGUYEN, (C) 2016
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef DG3DMULTISCALEMATERIALLAW_H_
#define DG3DMULTISCALEMATERIALLAW_H_

#include "dG3DMultiscaleIPVariable.h"
#include "dG3DMaterialLaw.h"
#include "numericalMaterial.h"
#include "elasticPotential.h"
#include "ipField.h"

class dG3DMultiscaleMaterialLawBase : public dG3DMaterialLaw, public numericalMaterial{
	#ifndef SWIG
	protected:
		virtual void fillElasticStiffness(double & rho, STensor43& L);

  protected:
    std::map<TwoNum, std::set<int> > _interfaceElementsToView;
    bool _allInterfacesAreViewed;
    std::set<int> _interfaceGPsToAllView;


	public:
		dG3DMultiscaleMaterialLawBase(const int lnum, const int tag);
		dG3DMultiscaleMaterialLawBase(const dG3DMultiscaleMaterialLawBase& src)
                            : dG3DMaterialLaw(src),numericalMaterial(src),
                            _blockDamageAfterFailureOnset(src._blockDamageAfterFailureOnset),
                            _lostSolutionUniquenssTolerance(src._lostSolutionUniquenssTolerance),
                            _failureCrFollowingInterfaceNormal(src._failureCrFollowingInterfaceNormal),
                            _allInterfacesAreViewed(src._allInterfacesAreViewed),
                           _interfaceElementsToView(src._interfaceElementsToView),
                           _interfaceGPsToAllView(src._interfaceGPsToAllView) {};

    virtual bool withEnergyDissipation() const  {
      for (int i=0; i< _allMaterialLaw.size(); i++){
        if (_allMaterialLaw[i]->withEnergyDissipation()){
          return true;
        }
      }
      return false;
    }

    virtual ~dG3DMultiscaleMaterialLawBase(){};

    virtual void createIPState(const bool isSolve, IPStateBase* &ips,const bool* state =NULL,
                                 const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt=0) const=0;

    const std::map<TwoNum, std::set<int> >& getInterfaceElementsToView() const {return _interfaceElementsToView;};
    const std::set<int> getInterfaceGPsToAllView() const {return _interfaceGPsToAllView;};
    bool allInterfacesAreViewed() const {return _allInterfacesAreViewed;}
	#endif //SWIG
    protected:
      bool _blockDamageAfterFailureOnset; // true if failure is blocked after failure onset
      double _lostSolutionUniquenssTolerance;
      bool _failureCrFollowingInterfaceNormal; // true if failure is checked based on interface normal

    public:
      void setBlockDamageAfterFailureOnset(const bool flg){ _blockDamageAfterFailureOnset = flg;};
      void setBlockDamageAfterFailureOnsetTolerance(const double tol){ _lostSolutionUniquenssTolerance = tol;};
      void setFailureCheckFollowingInterfaceNormal(const bool fl){_failureCrFollowingInterfaceNormal = fl;};
      bool getBlockDamageAfterFailureOnset() const {return _blockDamageAfterFailureOnset;};
      double getBlockDamageAfterFailureOnsetTolerance() const{return _lostSolutionUniquenssTolerance;}
      bool getFailureCheckFollowingInterfaceNormal() const{return _blockDamageAfterFailureOnset;};
      void setViewInterfaceElement(const int em, const int ep, const int gp);
      void setViewAllInterfaceElements(const bool fl, const int gp=0);
};

class dG3DElasticMultiscaleMaterialLaw : public dG3DMultiscaleMaterialLawBase{
  protected:
    elasticPotential* _elasticPotential; //

  public:
		dG3DElasticMultiscaleMaterialLaw(const int lnum, const int tag);
		#ifndef SWIG
		dG3DElasticMultiscaleMaterialLaw(const dG3DElasticMultiscaleMaterialLaw& src)
                            : dG3DMultiscaleMaterialLawBase(src){
      if (src._elasticPotential) _elasticPotential = src._elasticPotential->clone();
    };
    virtual ~dG3DElasticMultiscaleMaterialLaw(){
      if (_elasticPotential) delete _elasticPotential;
    };

		virtual materialLaw::matname getType() const {return materialLaw::linearElastic;}
		virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
		virtual void createIPState(const bool isSolve, IPStateBase* &ips,const bool* state =NULL,
                                 const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt=0) const
		{
			Msg::Error("numericalMaterial::createIPState is not used in this material law");
		};
	// To allow initialization of bulk ip in case of fracture
		virtual void createIPVariable(IPVariable* &ipv, const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
		virtual void initLaws(const std::map<int,materialLaw*> &maplaw);
		virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const{};
		virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);
		virtual double scaleFactor() const;
		virtual double soundSpeed() const;
		virtual bool isNumeric() const{return false;} // not true numeric material
		virtual materialLaw* clone() const {return new dG3DElasticMultiscaleMaterialLaw(*this);};
		#endif //SWIG
};


class dG3DMultiscaleMaterialLaw : public dG3DMultiscaleMaterialLawBase{
  public:
    dG3DMultiscaleMaterialLaw(const int lnum, const int tag);
    #ifndef SWIG
    dG3DMultiscaleMaterialLaw(const dG3DMultiscaleMaterialLaw& src)
                            : dG3DMultiscaleMaterialLawBase(src){};
    virtual ~dG3DMultiscaleMaterialLaw(){};
    virtual matname getType() const{return materialLaw::numeric;};

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;

    virtual void createIPState(const bool isSolve, IPStateBase* &ips,const bool* state =NULL,
                                 const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt=0) const;
    // To allow initialization of bulk ip in case of fracture
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void calledBy2lawFracture(){}
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw);
    virtual double density() const {return _rho;};
    // As to be defined for explicit scheme. If law for implicit only
    // please define this function with return 0
    virtual double soundSpeed() const {return 0;};
    // If law enable fracture the bool checkfrac allow to detect fracture. This bool is unused otherwise
    // The bool is necessery for computation of matrix by perturbation in this case the check of fracture
    // has not to be performed.
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
    virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual void setElasticStiffness(IPVariable* ipv) const;
    virtual bool isNumeric() const{return true;}
		virtual void initialIPVariable(IPVariable* ipv, bool stiff);

		virtual bool brokenCheck(const IPVariable* ipv) const;

		virtual materialLaw* clone() const {return new dG3DMultiscaleMaterialLaw(*this);};
    #endif // SWIG
};

class MultiscaleThermoMechanicsDG3DMaterialLaw : public dG3DMaterialLaw, public numericalMaterial{
	#ifndef SWIG
	protected:
		STensor3            linearK;
		STensor3	      Stiff_alphaDilatation;
		virtual void fillElasticStiffness(double & rho, STensor43& L){
			Msg::Error("MultiscaleThermoMechanicsDG3DMaterialLaw::fillElasticStiffness should not be used");
		};
		virtual void fillElasticStiffness(double& rho, STensor43& L, STensor3& Klinear, STensor3& alphaDilatation);

	#endif //SWIG


	public:
		MultiscaleThermoMechanicsDG3DMaterialLaw(const int num, const int tag);

		#ifndef SWIG
		MultiscaleThermoMechanicsDG3DMaterialLaw(const MultiscaleThermoMechanicsDG3DMaterialLaw&  src);
		virtual ~MultiscaleThermoMechanicsDG3DMaterialLaw(){}
		virtual matname getType() const{return materialLaw::numeric;};

    virtual bool withEnergyDissipation() const  {
      for (int i=0; i< _allMaterialLaw.size(); i++){
        if (_allMaterialLaw[i]->withEnergyDissipation()){
          return true;
        }
      }
      return false;
    }
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;

    virtual void createIPState(const bool isSolve, IPStateBase* &ips,const bool* state =NULL,
                                 const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt=0) const;
    // To allow initialization of bulk ip in case of fracture
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void calledBy2lawFracture(){}
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw);
    virtual double density() const {return _rho;};
    // As to be defined for explicit scheme. If law for implicit only
    // please define this function with return 0
    virtual double soundSpeed() const {return 0;};
    // If law enable fracture the bool checkfrac allow to detect fracture. This bool is unused otherwise
    // The bool is necessery for computation of matrix by perturbation in this case the check of fracture
    // has not to be performed.
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvprev) const;
    virtual void stress(IPVariable* ipv, const IPVariable* ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual void setElasticStiffness(IPVariable* ipv) const;
    virtual bool isNumeric() const{return true;}
		virtual void initialIPVariable(IPVariable* ipv, bool stiff);
		virtual materialLaw* clone() const {return new MultiscaleThermoMechanicsDG3DMaterialLaw(*this);};
		#endif //SWIG
};

#endif // DG3DMULTISCALEMATERIALLAW_H_
