//
// C++ Interface: dG3D domain with enhanced strain
//
// Description: Interface class for dg 3D
//
// Author:  <V.-D. NGUYEN>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "embeddedLocalizationBand.h"
#include "GaussIntegration.h"

void print(SPoint3 A, std::string str){
	printf("SPoint3 %s  [%e %e %e]\n",str.c_str(), A[0],A[1],A[2]);
};

double embeddedFiniteBand(MElement* ele, IntPt& GP, const SVector3& normal, const double bandwidth){
	// compute percentage of damage volume over whole volume of element ele
	
	if (normal.norm() < 1e-8) {
		Msg::Warning("zero normal vector has been set");
	return 0.;
	}
	
	double u = GP.pt[0]; double v = GP.pt[1]; double w = GP.pt[2];
	SPoint3 pt;
	ele->pnt(u,v,w,pt);
	
	double beta = 0;
	if (ele->getType() == TYPE_TRI){
		std::vector<MVertex*> vv;
		ele->getVertices(vv);
		
		MVertex* A(NULL), *B(NULL), *C(NULL);
		
		
		double v0 = normal[0]*(vv[0]->x() - pt.x()) + normal[1]*(vv[0]->y() - pt.y())+ normal[2]*(vv[0]->z() - pt.z());
		double v1 = normal[0]*(vv[1]->x() - pt.x()) + normal[1]*(vv[1]->y() - pt.y())+ normal[2]*(vv[1]->z() - pt.z());
		double v2 = normal[0]*(vv[2]->x() - pt.x()) + normal[1]*(vv[2]->y() - pt.y())+ normal[2]*(vv[2]->z() - pt.z());
		
		
		if (v0*v1 >= 0.){
			A  = vv[2]; B = vv[0]; C = vv[1];
		}
		else if (v0*v2 >= 0.){
			A  = vv[1]; B = vv[0]; C = vv[2];
		}
		else if (v1*v2 >= 0){
			A  = vv[0]; B = vv[1]; C = vv[2];
		}
		else{
			Msg::Error("wrong define %e %e %e",v0,v1,v2);
		}
		
		SVector3 AB(A->point(),B->point());
		SVector3 AC(A->point(),C->point());
		
		SVector3 faceNormal = crossprod(AB,AC);
		faceNormal.normalize();
		
		SVector3 dir = crossprod(normal,faceNormal);
		dir.normalize();
		
		SVector3 nAB = crossprod(faceNormal,AB);
		nAB.normalize();
		SVector3 nAC = crossprod(faceNormal,AC);
		nAC.normalize();
		
		SPoint3 Q(pt), R(pt);
		SVector3 PA(pt,A->point());
		double t = dot(nAB,PA)/dot(nAB,dir);
		for (int i=0; i<3; i++){
			Q[i] += dir[i]*t;
		}
		
		t = dot(nAC,PA)/dot(nAC,dir);
		for (int i=0; i<3; i++){
			R[i] += dir[i]*t;
		}
		beta = GP.weight*Q.distance(R)*bandwidth/ele->getVolume();
		
		/*
		printf("rank %d element = %d GP.weight %e bandwidth %e beta = %e volume %e \n",Msg::GetCommRank(),ele->getNum(),GP.weight,bandwidth,beta,ele->getVolume());
		nAC.print("nAC");
		dir.print("dir");
		normal.print("normal");
		print(A->point(),"A");
		print(B->point(),"B");
		print(C->point(),"C");
		print(pt,"P");
		print(Q,"Q");
		print(R,"R");
		 */ 
	}
	else{
		Msg::Error("element type %d has not been implemented");
	}
	
	return beta;
};

