//
// C++ Interface: multiple fields space to generat extra key at interface
////
//
// Author: Van Dung NGUYEN, (C) 2016
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef EXTRAFIELDFUNCTIONSPACE_H_
#define EXTRAFIELDFUNCTIONSPACE_H_

#include "MInterfaceElement.h"
#include "ThreeDLagrangeFunctionSpace.h"

class extraFieldFunctionSpace : public ThreeDLagrangeFunctionSpace{
  protected:
    virtual void getKeysOnElement(MElement *ele, std::vector<Dof> &keys) const{
      Msg::Error("This function is not implemented");
    }
	
	protected:
		const partDomain* _dom;

  public:
    extraFieldFunctionSpace(const partDomain* dom, int id, int ncomp) : ThreeDLagrangeFunctionSpace(id,ncomp,true,true),_dom(dom){};
    extraFieldFunctionSpace(const partDomain* dom, int id, int ncomp, int comp1) : ThreeDLagrangeFunctionSpace(id,ncomp,comp1,true,true),_dom(dom){};
    extraFieldFunctionSpace(const partDomain* dom, int id, int ncomp, int comp1, int comp2) : ThreeDLagrangeFunctionSpace(id,ncomp,comp1,comp2,true,true),_dom(dom){};
    extraFieldFunctionSpace(const partDomain* dom, int id, int ncomp, int comp1, int comp2, int comp3) : ThreeDLagrangeFunctionSpace(id,ncomp,comp1,comp2,comp3,true,true),_dom(dom){};
	
    virtual ~extraFieldFunctionSpace(){};

		virtual FunctionSpaceBase* clone(const int id) const{
			if (_ncomp == 1) return new extraFieldFunctionSpace(_dom,id,1,comp[0]);
			else if (_ncomp == 2) return new extraFieldFunctionSpace(_dom,id,2,comp[0],comp[1]);
			else if (_ncomp == 3) return new extraFieldFunctionSpace(_dom,id,3,comp[0],comp[1],comp[2]);
			else return new extraFieldFunctionSpace(_dom,id,_ncomp);
		}
    virtual int getNumKeys(MElement *ele) const {
      if (_dom->IsMultipleFieldFormulation()){
				if (dynamic_cast<MInterfaceElement*>(ele) == NULL) {
					Msg::Error("Multiple fields are now considered for interface element only");
				}
				
				if (_dom->getEnrichedUnknownLocation() == partDomain::NODE){
					return (_ncomp*ele->getNumVertices());
				}
				else if (_dom->getEnrichedUnknownLocation() == partDomain::GP){
					MInterfaceElement* iele = dynamic_cast<MInterfaceElement*>(ele);
					
					if (iele != NULL){
						// at interface element
						const dgPartDomain* dgDom = dynamic_cast<const dgPartDomain*>(_dom);
						if (dgDom){
							IntPt* GP = NULL;
							int npts = dgDom->getInterfaceGaussIntegrationRule()->getIntPoints(ele,&GP);
							return (npts*_ncomp);
						}
						else{
							Msg::Error("interface elements must be used with dgDom extraFieldFunctionSpace::getNumKeys");
						}
					}
					else {
						// at bulk element
						IntPt* GP = NULL;
						int npts = _dom->getBulkGaussIntegrationRule()->getIntPoints(ele,&GP);
						return (npts*_ncomp);
					}
					
				}
				else{
					Msg::Error("encriched location %d is not considered",_dom->getEnrichedUnknownLocation());
				}
				
			}
			else{
				return 0;
			}
    };

    virtual void getKeys(MElement *ele, std::vector<Dof> &keys) const{
			if (_dom->IsMultipleFieldFormulation()){
				if (_dom->getEnrichedUnknownLocation() == partDomain::NODE){
					// generate Dof at nodes
					for (int j=0;j<_ncomp;++j){
						for (int i=0;i<ele->getNumVertices();i++){
							keys.push_back(Dof(ele->getNum(),dG3DDof3IntType::createTypeWithThreeInts(comp[j],ifield[j],i)));
						}
					}
				}
				else if (_dom->getEnrichedUnknownLocation() == partDomain::GP){
					MInterfaceElement* iele = dynamic_cast<MInterfaceElement*>(ele);
					int npts = 0;
					IntPt* GP;
					if (iele != NULL){
						const dgPartDomain* dgDom = static_cast<const dgPartDomain*>(_dom);
						npts = dgDom->getInterfaceGaussIntegrationRule()->getIntPoints(ele,&GP);
					}
					else{
						npts = _dom->getBulkGaussIntegrationRule()->getIntPoints(ele,&GP);
					}
					// generate dof at GP
					for (int j=0;j<_ncomp;++j){
						for (int i=0;i<npts;i++){
							keys.push_back(Dof(ele->getNum(),dG3DDof3IntType::createTypeWithThreeInts(comp[j],ifield[j],i)));
						}
					}
				}
				
			}
    };
    
    virtual int getNumExtraKeysPerVertex() const {return _ncomp;};

};

#endif // EXTRAFIELDFUNCTIONSPACE_H_
