//
// C++ Interface: terms
////
//
// Author: Van Dung NGUYEN, (C) 2016
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "multipleFieldDG3DTerms.h"
#include "FractureCohesiveDG3DIPVariable.h"
#include "dG3DEnhancedStrainIPVariable.h"
#include "highOrderTensor.h"

void twoFieldDG3DForceBulk::get(MElement *ele,int npts,IntPt *GP,fullVector<double> &vFor) const
{
  const FunctionSpaceBase* space  = _dom->getFunctionSpace();
  int nbdof = space->getNumKeys(ele);
  int nbFF = ele->getNumShapeFunctions();
	
  vFor.resize(nbdof);
  vFor.setAll(0.);

  const AllIPState::ipstateElementContainer *vips = _ipf->getAips()->getIPstate(ele->getNum());

  for (int i = 0; i < npts; i++){
    const dG3DEnhancedStrainIPVariable *ipv     = static_cast<const dG3DEnhancedStrainIPVariable*>((*vips)[i]->getState(IPStateBase::current));
		
    double weight = GP[i].weight;
    double & detJ = ipv->getJacobianDeterminant(ele,GP[i]);
    double ratio = detJ*weight;
		
    const std::vector<TensorialTraits<double>::GradType> &Grads = ipv->gradf(space,ele,GP[i]);
    const std::vector<TensorialTraits<double>::ValType> &Vals = ipv->f(space,ele,GP[i]);
		
		const STensor3& P = ipv->getConstRefToFirstPiolaKirchhoffStress();
		for (int j=0; j<nbFF; j++){
			for (int k=0; k<3; k++){
				for (int l=0; l<3; l++){
					vFor(j+k*nbFF) += P(k,l)*Grads[j+0*nbFF](l)*ratio;
				}
			}
		}
    
		const STensor3& eF = ipv->getConstRefToEnhancedDeformationGradient();
		const STensor3& eFLocal = ipv->getConstRefToLocalEnhancedDeformationGradient();
		
		for (int j=0; j< nbFF; j++){
			for (int k=0; k<3; k++){
				for (int l=0; l<3; l++){
					int row = Tensor23::getIndex(k,l);
					vFor(j+row*nbFF + 3*nbFF) += _eqFactor*(eF(k,l) - eFLocal(k,l))*Vals[j+0*nbFF]*ratio;
				}
			}
		}
  }
}

void twoFieldDG3DStiffnessBulk::get(MElement *ele,int npts,IntPt *GP,fullMatrix<double> &mStiff) const
{
  const FunctionSpaceBase* space  = _dom->getFunctionSpace();
  int nbdof = space->getNumKeys(ele);
  int nbFF = ele->getNumShapeFunctions();
	
  mStiff.resize(nbdof,nbdof);
  mStiff.setAll(0.);
	
  const AllIPState::ipstateElementContainer *vips = _ipf->getAips()->getIPstate(ele->getNum());

  for (int i = 0; i < npts; i++){
    const dG3DEnhancedStrainIPVariable *ipv     = static_cast<const dG3DEnhancedStrainIPVariable*>((*vips)[i]->getState(IPStateBase::current));
    double weight = GP[i].weight;
    double & detJ = ipv->getJacobianDeterminant(ele,GP[i]);
    double ratio = detJ*weight;
		
		const std::vector<TensorialTraits<double>::GradType> &Grads = ipv->gradf(space,ele,GP[i]);
		const std::vector<TensorialTraits<double>::ValType> &Vals = ipv->f(space,ele,GP[i]);
		
		const STensor43& H = ipv->getConstRefToTangentModuli();
		const STensor43& enH = ipv->getConstRefToEnhancedTangentModuli();
		
		for (int j=0; j<nbFF; j++){
			for (int k=0; k<3; k++){
				for (int l=0; l<3; l++){
					// defo
					for (int p=0; p< nbFF; p++){
						for (int q=0; q<3; q++){
							for (int r=0; r<3; r++){
								mStiff(j+k*nbFF,p+q*nbFF) += H(k,l,q,r)*Grads[j+0*nbFF](l)*Grads[p+0*nbFF](r)*ratio;
							}
						}
					}
					// extra dofs
					for (int p=0; p< nbFF; p++){
						for (int q=0; q<3; q++){
							for (int r=0; r<3; r++){
								int col = Tensor23::getIndex(q,r);
								mStiff(j+k*nbFF,p+col*nbFF+3*nbFF) += enH(k,l,q,r)*Grads[j+0*nbFF](l)*Vals[p+0*nbFF]*ratio;
							}
						}
					}						
				}
			}
		}
		
		const STensor43& DeFLocalDdefo = ipv->getConstRefToDLocalEnhancedDeformationGradientDDeformationGradient();
		const STensor43& DeFLocalDeF = ipv->getConstRefToDLocalEnhancedDeformationGradientDEnhancedDeformationGradient();
		
		static STensor3 I3(1.);
		
		for (int j=0; j< nbFF; j++){
			for (int k=0; k<3; k++){
				for (int l=0; l<3; l++){
					int row = Tensor23::getIndex(k,l);
					// defo
					for (int p=0; p< nbFF; p++){
						for (int q=0; q<3; q++){
							for (int r=0; r<3; r++){
								mStiff(j+row*nbFF + 3*nbFF,p+q*nbFF) -= _eqFactor*DeFLocalDdefo(k,l,q,r)*Vals[j+0*nbFF]*Grads[p+0*nbFF](r)*ratio;
							}
						}
					}
					// extra field
					for (int p=0; p< nbFF; p++){
						for (int q=0; q<3; q++){
							for (int r=0; r<3; r++){
								int col = Tensor23::getIndex(q,r);
								mStiff(j+row*nbFF + 3*nbFF,p+col*nbFF+3*nbFF)  +=  _eqFactor*(I3(k,q)*I3(l,r) - DeFLocalDeF(k,l,q,r))*Vals[j+0*nbFF]*Vals[p+0*nbFF]*ratio;
							}
						}
					}
				}
			}
		}
	}
}


void twoFieldDG3DForceInter::get(MElement *ele, int npts, IntPt *GP, fullVector<double> &m)const{
  MInterfaceElement *ie = dynamic_cast<MInterfaceElement*>(ele);
  const int nbFFm = ie->getElem(0)->getNumVertices();
  const int nbFFp = ie->getElem(1)->getNumVertices();
  const int nbFF = ele->getNumVertices();
  
  const FunctionSpaceBase* minusSpace  = _dom->getFunctionSpaceMinus();
  const FunctionSpaceBase* plusSpace = _dom->getFunctionSpacePlus();
  const FunctionSpaceBase* spaceInter = _dom->getInterfaceFunctionSpace(); 

  const int nbdofm = minusSpace->getNumKeys(ie->getElem(0));
  const int nbdofp = plusSpace->getNumKeys(ie->getElem(1));
  const int nextra = spaceInter->getNumKeys(ele);

  m.resize(nbdofm+nbdofp+nextra,true);

  if(_dom->getFormulation()){
    // get value at gauss's point
    const AllIPState::ipstateElementContainer *vips = _ipf->getAips()->getIPstate(ele->getNum());
    // Values on minus and plus elements
    IntPt *GPm; IntPt *GPp;
    _dom->getInterfaceQuadrature()->getIntPoints(ie,GP,&GPm,&GPp);
    // loop on Gauss Point
    for(int i=0;i<npts; i++)
    {
      // IP
      const IPStateBase *ipsm        = (*vips)[i];
      const dG3DIPVariableBase *ipvm     = static_cast<const dG3DIPVariableBase*>(ipsm->getState(IPStateBase::current));

      const IPStateBase *ipsp        = (*vips)[i+npts];
      const dG3DIPVariableBase *ipvp     = static_cast<const dG3DIPVariableBase*>(ipsp->getState(IPStateBase::current));

      const std::vector<TensorialTraits<double>::ValType> &Valsm = ipvm->f(minusSpace,ie->getElem(0),GPm[i]);
      const std::vector<TensorialTraits<double>::ValType> &Valsp = ipvp->f(plusSpace,ie->getElem(1),GPp[i]);

      const std::vector<TensorialTraits<double>::GradType> &Gradsm = ipvm->gradf(minusSpace,ie->getElem(0),GPm[i]);
      const std::vector<TensorialTraits<double>::GradType> &Gradsp = ipvp->gradf(plusSpace,ie->getElem(1),GPp[i]);

      const std::vector<TensorialTraits<double>::ValType>& Vals = ipvm->f(spaceInter,ie,GP[i]);
      
      const double weight = GP[i].weight;
      const double& detJ = ipvm->getJacobianDeterminant(ie,GP[i]);
      double wJ = detJ*weight;

      static SVector3 normal;
      normal = ipvm->getConstRefToReferenceOutwardNormal();
      normal.normalize();

      
			bool broken = false;			
			const FractureCohesive3DIPVariable *ipvmf = dynamic_cast<const FractureCohesive3DIPVariable*>(ipvm);
      const FractureCohesive3DIPVariable *ipvpf = NULL; 
			if(ipvmf != NULL){
				broken = ipvmf->isbroken();
        ipvpf = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
			}
      
      // cohesive traction
      static SVector3 T;
      if (broken){
        T = ipvmf->getConstRefToInterfaceForce();
        T += ipvpf->getConstRefToInterfaceForce();
        T *= 0.5;
      }
      else{
        // compute from stress tensor
        const STensor3& Pm = ipvm->getConstRefToFirstPiolaKirchhoffStress();
        const STensor3& Pp = ipvp->getConstRefToFirstPiolaKirchhoffStress();
          
        for (int j=0; j<3; j++){
          T(j) = 0.;
          for (int k=0; k<3; k++){
            T(j) += 0.5*(Pm(j,k)+Pp(j,k))*normal(k);
          }
        }
      }
      
      const SVector3& ujump = ipvm->getConstRefToJump();      
      static SVector3 diffJump;
      diffJump = ujump;
      // if fracture
      if (broken){
        static SVector3 cjump;
        cjump = ipvmf->getConstRefToCohesiveJump();
        cjump += ipvpf->getConstRefToCohesiveJump();
        cjump *= (0.5);
        diffJump -= cjump;
      };
      
      for(int j=0;j<nbFFm;j++){
        for(int k=0;k<3;k++){
          m(j+k*nbFFm) -= T(k)*(Valsm[j+k*nbFFm]*wJ);
        }
      }
      for(int j=0;j<nbFFp;j++){
        for(int k=0;k<3;k++){
          m(j+k*nbFFp+nbdofm) += T(k)*(Valsp[j+k*nbFFp]*wJ);
        }
      }

    // constraint part 
			if (_dom->getEnrichedUnknownLocation() == partDomain::NODE){
				for (int j=0; j< nbFF; j++){
					for (int k=0; k<3; k++){
						m(j+k*nbFF+nbdofm+nbdofp) += _eqFactor*(diffJump(k)*Vals[j+k*nbFF]*wJ);
					}
				}
				
			}
			else if (_dom->getEnrichedUnknownLocation() == partDomain::GP){
				for (int k=0; k<3; k++){
					m(i+k*npts+nbdofm+nbdofp) += _eqFactor*diffJump(k)*wJ;
				}
			}

    }

  }
};


void twoFieldDG3DStiffnessInter::get(MElement *ele,int npts,IntPt *GP,fullMatrix<double> &m) const
{
  MInterfaceElement *ie = dynamic_cast<MInterfaceElement*>(ele);
  const int nbFFm = ie->getElem(0)->getNumVertices();
  const int nbFFp = ie->getElem(1)->getNumVertices();
  const int nbFF = ele->getNumVertices();

  const FunctionSpaceBase* minusSpace  = _dom->getFunctionSpaceMinus();
  const FunctionSpaceBase* plusSpace = _dom->getFunctionSpacePlus();
  const FunctionSpaceBase* spaceInter = _dom->getInterfaceFunctionSpace(); 

  const int nbdofm = minusSpace->getNumKeys(ie->getElem(0));
  const int nbdofp = plusSpace->getNumKeys(ie->getElem(1));
  const int nextra = spaceInter->getNumKeys(ele);

  m.resize(nbdofm+nbdofp+nextra,nbdofm+nbdofp+nextra,true);

  if(_dom->getFormulation())
  {
    // get value at gauss's point
    const AllIPState::ipstateElementContainer *vips = _ipf->getAips()->getIPstate(ele->getNum());
    // Values on minus and plus elements
    IntPt *GPm; IntPt *GPp;
    _dom->getInterfaceQuadrature()->getIntPoints(ie,GP,&GPm,&GPp);
    
    double hs = ie->getCharacteristicSize();

    // loop on Gauss Point
    for(int i=0;i<npts; i++)
    {
      // Coordonate of Gauss' point i

      // IP
      const IPStateBase *ipsm        = (*vips)[i];
      const dG3DIPVariableBase *ipvm     = static_cast<const dG3DIPVariableBase*>(ipsm->getState(IPStateBase::current));

      const IPStateBase *ipsp        = (*vips)[i+npts];
      const dG3DIPVariableBase *ipvp     = static_cast<const dG3DIPVariableBase*>(ipsp->getState(IPStateBase::current));


      const std::vector<TensorialTraits<double>::GradType> &Gradsm = ipvm->gradf(minusSpace,ie->getElem(0),GPm[i]);
      const std::vector<TensorialTraits<double>::GradType> &Gradsp = ipvp->gradf(plusSpace,ie->getElem(1),GPp[i]);
      
      const std::vector<TensorialTraits<double>::ValType> &Valsm = ipvm->f(minusSpace,ie->getElem(0),GPm[i]);
      const std::vector<TensorialTraits<double>::ValType> &Valsp = ipvp->f(plusSpace,ie->getElem(1),GPp[i]);
      
      const std::vector<TensorialTraits<double>::ValType>& Vals = ipvm->f(spaceInter,ie,GP[i]);
      
			const double weight = GP[i].weight;
      const double& detJ = ipvm->getJacobianDeterminant(ie,GP[i]);
      double wJ = detJ*weight;

      static SVector3 normal;
      normal =  ipvm->getConstRefToReferenceOutwardNormal();
      normal.normalize();

      bool  broken = false;			
      const FractureCohesive3DIPVariable *ipvmf = dynamic_cast<const FractureCohesive3DIPVariable*>(ipvm);
      const FractureCohesive3DIPVariable *ipvpf = NULL;
      if(ipvmf != NULL){
        broken = ipvmf->isbroken();
        ipvpf =  static_cast<const FractureCohesive3DIPVariable*>(ipvp); 
      }
			
      // cohesive jump      
      static STensor3 DdiffJumpDjump;
      static STensor3 DdiffJumpDinJump;
      STensorOperation::unity(DdiffJumpDjump);
      STensorOperation::zero(DdiffJumpDinJump);
      if (broken){
        DdiffJumpDinJump = ipvmf->getConstRefToDCohesiveJumpDIncompatibleJump();;
        DdiffJumpDinJump += ipvpf->getConstRefToDCohesiveJumpDIncompatibleJump();;
        DdiffJumpDinJump *= (-0.5/(_beta*hs));
      }
      // cohesive traction
      static STensor33 DTDFm;
      static STensor33 DTDFp;
      static STensor3 DTDinJump;

      if (broken){
        DTDFm = ipvmf->getConstRefToDInterfaceForceDDeformationGradient();
        DTDFp = ipvpf->getConstRefToDInterfaceForceDDeformationGradient();
        DTDinJump = ipvmf->getConstRefToDInterfaceForceDIncompatibleJump();
        DTDinJump += ipvpf->getConstRefToDInterfaceForceDIncompatibleJump();
        DTDinJump *= (0.5/(_beta*hs));
      }
      else{
        // computed from bulk law only$
        const STensor43& Hm = ipvm->getConstRefToTangentModuli();
        const STensor43& Hp = ipvp->getConstRefToTangentModuli();
        for (int j=0; j<3; j++){
          for (int p=0; p<3; p++){
            DTDinJump(j,p) = 0.;
            for (int q=0; q<3; q++){
              DTDFm(j,p,q) = 0.;
              DTDFp(j,p,q) = 0.;
              for (int k=0; k<3; k++){
                DTDFm(j,p,q) += Hm(j,k,p,q)*normal(k);
                DTDFp(j,p,q) += Hp(j,k,p,q)*normal(k);
                DTDinJump(j,p) += 0.5*(Hm(j,k,p,q)+Hp(j,k,p,q))*normal(k)*normal(q)/(_beta*hs);
              }
            }
          }
        }
      }
    
    // interface force
    // on F
      for(int j=0;j<nbFFm;j++){
        for(int k=0;k<3;k++){
          for (int p=0; p<nbFFm; p++){
            for (int q=0; q<3; q++){
              for (int r=0; r<3; r++){
                m(j+k*nbFFm,p+q*nbFFm) -= 0.5*DTDFm(k,q,r)*Gradsm[p+q*nbFFm](r)*(Valsm[j+k*nbFFm]*wJ);
              }
            }
          }

          for (int p=0; p< nbFFp; p++){
            for (int q=0; q<3; q++){
              for (int r=0; r<3; r++){
                m(j+k*nbFFm,p+q*nbFFp+nbdofm) -=0.5*DTDFp(k,q,r)*Gradsp[p+q*nbFFp](r)*(Valsm[j+k*nbFFm]*wJ);
              }
            }
          }
          
          // extra field
          if (_dom->getEnrichedUnknownLocation() == partDomain::NODE){
            for (int p=0; p< nbFF; p++){
							for (int q=0; q<3; q++){
								m(j+k*nbFFm, p+q*nbFF+nbdofm+nbdofp) -= DTDinJump(k,q)*Vals[p+q*nbFF]*(Valsm[j+k*nbFFm]*wJ);
							}
						}
          }
          else if ( _dom->getEnrichedUnknownLocation() == partDomain::GP){
            for (int q=0; q<3; q++){
							m(j+k*nbFFm, i+q*npts+nbdofm+nbdofp) -= DTDinJump(k,q)*(Valsm[j+k*nbFFm]*wJ);
						}
          };
        }
      }
      for(int j=0;j<nbFFp;j++){
        for(int k=0;k<3;k++){
          for (int p=0; p<nbFFm; p++){
            for (int q=0; q<3; q++){
              for (int r=0; r<3; r++){
                m(j+k*nbFFp+nbdofm,p+q*nbFFm) += 0.5*DTDFm(k,q,r)*Gradsm[p+q*nbFFm](r)*(Valsp[j+k*nbFFp]*wJ);
              }
            }
          }

          for (int p=0; p<nbFFp; p++){
            for (int q=0; q<3; q++){
              for (int r=0; r<3; r++){
                m(j+k*nbFFp+nbdofm,p+q*nbFFp+nbdofm) += 0.5*DTDFp(k,q,r)*Gradsp[p+q*nbFFp](r)*(Valsp[j+k*nbFFp]*wJ);
              }
            }
          }
          
          if (_dom->getEnrichedUnknownLocation() == partDomain::NODE){
            for (int p=0; p< nbFF; p++){
              for (int q=0; q<3; q++){
                m(j+k*nbFFp+nbdofm, p+q*nbFF+nbdofm+nbdofp) += DTDinJump(k,q)*Vals[p+q*nbFF]*(Valsp[j+k*nbFFp]*wJ);
              }
            }
          }
          else if ( _dom->getEnrichedUnknownLocation() == partDomain::GP){
            for (int q=0; q<3; q++){
              m(j+k*nbFFp+nbdofm, i+q*npts+nbdofm+nbdofp) += DTDinJump(k,q)*(Valsp[j+k*nbFFp]*wJ);
            }
          }
          
        }
      }
      
			

      if (_dom->getEnrichedUnknownLocation() == partDomain::NODE){
				// constraint part
				for (int j=0; j< nbFF; j++){
					for (int k=0; k<3; k++){
						for (int p=0; p<nbFFm; p++){
							for (int q=0; q<3; q++){
								m(j+k*nbFF+nbdofm+nbdofp,p+q*nbFFm) -= _eqFactor*DdiffJumpDjump(k,q)*Valsm[p+q*nbFFm]*Vals[j+k*nbFF]*wJ;
							}
						}

						for (int p=0; p<nbFFp; p++){
							for (int q=0; q<3; q++){
								m(j+k*nbFF+nbdofm+nbdofp,p+q*nbFFp+nbdofm) += _eqFactor*DdiffJumpDjump(k,q)*Valsp[p+q*nbFFp]*Vals[j+k*nbFF]*wJ;
							}
						}

						for (int p=0; p<nbFF; p++){
							for (int q=0; q<3; q++){
								m(j+k*nbFF+nbdofm+nbdofp,p+q*nbFF+nbdofm+nbdofp) += _eqFactor*DdiffJumpDinJump(k,q)*Vals[p+q*nbFF]*Vals[j+k*nbFF]*wJ;
							}
						}
					}
				}
			}
			else if (_dom->getEnrichedUnknownLocation() == partDomain::GP){
				for (int k=0; k<3; k++){
					for (int p=0; p<nbFFm; p++){
						for (int q=0; q<3; q++){
							m(i+k*npts+nbdofm+nbdofp,p+q*nbFFm) -= _eqFactor*DdiffJumpDjump(k,q)*Valsm[p+q*nbFFm]*wJ;
						}
					}

					for (int p=0; p<nbFFp; p++){
						for (int q=0; q<3; q++){
							m(i+k*npts+nbdofm+nbdofp,p+q*nbFFp+nbdofm) += _eqFactor*DdiffJumpDjump(k,q)*Valsp[p+q*nbFFp]*wJ;
						}
					}

					for (int q=0; q<3; q++){
						m(i+k*npts+nbdofm+nbdofp,i+q*npts+nbdofm+nbdofp) += _eqFactor*DdiffJumpDinJump(k,q)*wJ;
					}

				}
			}
    }
  }
  //m.print("mf");
};