//
// C++ Interface: terms
////
//
// Author: Van Dung NGUYEN, (C) 2016
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef MULTIPLEFIELDDG3DTERMS_H_
#define MULTIPLEFIELDDG3DTERMS_H_

#include "nlTerms.h"
#include "ipField.h"
#include "partDomain.h"
#include "dG3DTerms.h"

class twoFieldDG3DForceBulk : public nonLinearTermBase<double>{
	protected:
		const partDomain* _dom;
    const IPField* _ipf;
		double _eqFactor;
		
	public:
		twoFieldDG3DForceBulk(const partDomain* d, const IPField *ip, const double fact) : _dom(d),_ipf(ip),_eqFactor(fact){}
		virtual ~twoFieldDG3DForceBulk(){}
		virtual void get(MElement *ele,int npts,IntPt *GP,fullVector<double> &m) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullVector<double> > &mv) const
		{
			Msg::Error("Define me get by integration point twoFieldDG3DForceBulk");
		}
		virtual void set(const fullVector<double> *datafield) {};
    virtual const bool isData() const {return false;};
		
		virtual LinearTermBase<double>* clone () const
		{
			return new twoFieldDG3DForceBulk(_dom,_ipf,_eqFactor);
		}
};

class twoFieldDG3DStiffnessBulk : public BiNonLinearTermBase {
	protected:
		const partDomain* _dom;
    const IPField* _ipf;
		double _eqFactor;
		
	public:
		twoFieldDG3DStiffnessBulk(const partDomain* d, const IPField *ip, const double fact) : _dom(d),_ipf(ip),_eqFactor(fact){}
																	
		virtual void get(MElement *ele,int npts,IntPt *GP,fullMatrix<double> &m) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullMatrix<double> > &mv) const
		{
			Msg::Error("Define me get by gauss point twoFieldDG3DStiffnessBulk");
		}
		virtual void set(const fullVector<double> *datafield) {};
    virtual const bool isData() const {return false;}; 
		
		virtual BilinearTermBase* clone () const
		{
			return new twoFieldDG3DStiffnessBulk(_dom,_ipf,_eqFactor);
		}
		
};

class twoFieldDG3DForceInter : public nonLinearTermBase<double>{
  protected:
    const dgPartDomain* _dom;
    const IPField* _ipf;
    double _eqFactor;
    double _beta; // stability parameter

  public:
    twoFieldDG3DForceInter(const dgPartDomain* dom, const IPField* ip, const double eq, const double b): _dom(dom),_ipf(ip),_eqFactor(eq),_beta(b){}

    virtual ~twoFieldDG3DForceInter() {}

    virtual void get(MElement *ele, int npts, IntPt *GP, fullVector<double> &v)const;
    virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullVector<double> > &mv) const
    {
      Msg::Error("Define me get by integration point twoFieldDG3DForceInter");
    }

    virtual void set(const fullVector<double> *datafield) {};
    virtual const bool isData() const {return false;};
 
    virtual LinearTermBase<double>* clone () const
    {
      return new twoFieldDG3DForceInter(_dom,_ipf,_eqFactor,_beta);
    }
};

class  twoFieldDG3DStiffnessInter : public BiNonLinearTermBase{
  protected:
    const dgPartDomain* _dom;
    const IPField* _ipf;
    double _eqFactor;
    double _beta;

  public:
    twoFieldDG3DStiffnessInter(const dgPartDomain* dom, const IPField* ip, const double fact, const double b): _dom(dom),_ipf(ip),_eqFactor(fact),_beta(b){}
    virtual ~twoFieldDG3DStiffnessInter(){}
    virtual void get(MElement *ele,int npts,IntPt *GP, fullMatrix<double> &m)const;
    virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullMatrix<double> > &mv) const
    {
      Msg::Error("Define me ?? get by integration point twoFieldDG3DForceInter");
    }

    virtual void set(const fullVector<double> *datafield) {};
    virtual const bool isData() const {return false;}; 
    virtual BilinearTermBase* clone () const
    {
      return new twoFieldDG3DStiffnessInter(_dom,_ipf,_eqFactor,_beta);
    }

};

#endif // MULTIPLEFIELDDG3DTERMS_H_
