//
// C++ Interface: ipvariable
//
// Description: Class with definition of ipvarible for dG3D
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef NONLOCALDAMAGEDG3DIPVARIABLE_H_
#define NONLOCALDAMAGEDG3DIPVARIABLE_H_
#include "dG3DIPVariable.h"
#include "ipNonLocalDamageJ2Hyper.h"
#include "ipNonLocalDamageIsotropicElasticity.h"
#include "ipNonLocalPorosity.h"
#include "mlawNonLocalDamageJ2Hyper.h"
#include "mlawNonLocalDamageJ2SmallStrain.h"
#include "mlawNonLocalDamageIsotropicElasticity.h"
#include "mlawNonLocalPorous.h"
#include "mlawHyperelasticDamage.h"
#include "ipNonLocalDamage.h"
#include "mlawNonLocalPorousWithFailure.h"


class virtualNonLocalDamageDG3DIPVariable : public dG3DIPVariableBase{
  protected:
    nonlocalData _nonlocalData; // al zero
    dG3DIPVariableBase* _ipvBase;
    std::vector<double> localVar;
    std::vector<double> nonLocalVar;
    const std::vector<STensor3>& characteristicLength;
    std::vector<double> DirrEnergDNonlocalVar;
    constitutiveExtraDofDiffusionData *_constitutiveExtraDofDiffusionData;
    std::vector<double> constitutiveExtraDofVar;
    std::vector<double> DirrEnergDConstitutiveExtraDofDiffusion;

  public:
    virtualNonLocalDamageDG3DIPVariable(const int num, const dG3DIPVariableBase* base, const std::vector<STensor3>& length, int numExtra=0);
    virtualNonLocalDamageDG3DIPVariable(const virtualNonLocalDamageDG3DIPVariable& src);
    virtual virtualNonLocalDamageDG3DIPVariable& operator =(const IPVariable& src);
    virtual ~virtualNonLocalDamageDG3DIPVariable();

    dG3DIPVariableBase* getIPVBase() {return _ipvBase;}
    const dG3DIPVariableBase* getIPVBase() const {return _ipvBase;}

    virtual IPVariable* getInternalData() {return _ipvBase->getInternalData();};
    virtual const IPVariable* getInternalData()  const {return _ipvBase->getInternalData();};

    #if defined(HAVE_MPI)
    // by default in MPI there is no value that are communicate between rank.
    // but you can communicate values by filling these functions for your IPVariable
    virtual int numberValuesToCommunicateMPI()const{return _ipvBase->numberValuesToCommunicateMPI();}
    virtual void fillValuesToCommunicateMPI(double *arrayMPI)const{_ipvBase->fillValuesToCommunicateMPI(arrayMPI);};
    virtual void getValuesFromMPI(const double *arrayMPI){_ipvBase->getValuesFromMPI(arrayMPI);};
    #endif // HAVE_MPI

    virtual void Delete(const bool fl){
      dG3DIPVariableBase::Delete(fl);
      _ipvBase->Delete(fl);
    }
    virtual void setLocation(const IPVariable::LOCATION loc){
      dG3DIPVariableBase::setLocation(loc);
      _ipvBase->setLocation(loc);
    };
    virtual void setSolver(nonLinearMechSolver* s) {
      dG3DIPVariableBase::setSolver(s);
      _ipvBase->setSolver(s);
    };

    virtual bool dissipationIsActive() const {return _ipvBase->dissipationIsActive();};

    virtual void blockDissipation(const bool bl){
      dG3DIPVariableBase::blockDissipation(bl);
      _ipvBase->blockDissipation(bl);
    }
    virtual bool dissipationIsBlocked() const {return _ipvBase->dissipationIsBlocked();}

    virtual int fractureEnergy(double* arrayEnergy) const{return _ipvBase->fractureEnergy(arrayEnergy);}

    virtual const bool isInterface() const {return _ipvBase->isInterface();};

    // Archiving data
    virtual double get(const int i) const;
    virtual double defoEnergy() const {return _ipvBase->defoEnergy();};
    virtual double plasticEnergy() const {return _ipvBase->plasticEnergy();};
    virtual double damageEnergy() const {return _ipvBase->damageEnergy();};
    virtual double vonMises() const {return _ipvBase->vonMises();};
    virtual void getCauchyStress(STensor3 &cauchy) const {_ipvBase->getCauchyStress(cauchy);};
    virtual double getJ() const {return _ipvBase->getJ();};

    virtual double getBarJ() const {return _ipvBase->getBarJ();};
    virtual double &getRefToBarJ() {return _ipvBase->getRefToBarJ();};
    virtual double getLocalJ() const {return _ipvBase->getLocalJ();};
    virtual double &getRefToLocalJ() {return _ipvBase->getRefToLocalJ();};
    virtual void getBackLocalDeformationGradient(STensor3& localF) const 
                   {_ipvBase->getBackLocalDeformationGradient(localF);};

    virtual const STensor3 &getConstRefToDeformationGradient() const {return _ipvBase->getConstRefToDeformationGradient();};
    virtual STensor3 &getRefToDeformationGradient() {return _ipvBase->getRefToDeformationGradient();};

    virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStress() const{return _ipvBase->getConstRefToFirstPiolaKirchhoffStress();};
    virtual STensor3 &getRefToFirstPiolaKirchhoffStress() {return _ipvBase->getRefToFirstPiolaKirchhoffStress();};

    virtual const STensor43 &getConstRefToTangentModuli() const {return _ipvBase->getConstRefToTangentModuli();};
    virtual STensor43 &getRefToTangentModuli() {return _ipvBase->getRefToTangentModuli();};

     virtual const STensor43 &getConstRefToElasticTangentModuli() const {return _ipvBase->getConstRefToElasticTangentModuli();};
    virtual STensor43 &getRefToElasticTangentModuli() {return _ipvBase->getRefToElasticTangentModuli();};

    virtual const STensor43 &getConstRefToDGElasticTangentModuli() const {return _ipvBase->getConstRefToDGElasticTangentModuli();};
    virtual void setRefToDGElasticTangentModuli(const STensor43 &eT) {_ipvBase->setRefToDGElasticTangentModuli(eT);};

    virtual const SVector3 &getConstRefToReferenceOutwardNormal() const {return _ipvBase->getConstRefToReferenceOutwardNormal();};
    virtual SVector3 &getRefToReferenceOutwardNormal() {return _ipvBase->getRefToReferenceOutwardNormal();};

    virtual const SVector3 &getConstRefToCurrentOutwardNormal() const {return _ipvBase->getConstRefToCurrentOutwardNormal();};
    virtual SVector3 &getRefToCurrentOutwardNormal() {return _ipvBase->getRefToCurrentOutwardNormal();};

    virtual const SVector3 &getConstRefToJump() const {return _ipvBase->getConstRefToJump();};
    virtual SVector3 &getRefToJump() {return _ipvBase->getRefToJump();};

    // non local
    virtual int getNumberNonLocalVariable() const {return _nonlocalData.numNonLocalVariable;};
    //
    virtual double getConstRefToLocalVariable(const int idex) const {return localVar[idex];};
    virtual double& getRefToLocalVariable(const int idex) {return localVar[idex];};;
    //
    virtual double getConstRefToNonLocalVariable(const int idex) const {return nonLocalVar[idex];};
    virtual double& getRefToNonLocalVariable(const int idex) {return nonLocalVar[idex];};

    virtual const std::vector<STensor3> &getConstRefToDLocalVariableDStrain() const {return _nonlocalData.dLocalVariableDStrain;}
    virtual std::vector<STensor3> &getRefToDLocalVariableDStrain() {return _nonlocalData.dLocalVariableDStrain;}

    virtual const std::vector<STensor3> &getConstRefToDStressDNonLocalVariable() const {return _nonlocalData.dStressDNonLocalVariable;}
    virtual std::vector<STensor3> &getRefToDStressDNonLocalVariable() {return _nonlocalData.dStressDNonLocalVariable;}

    virtual const fullMatrix<double> &getConstRefToDLocalVariableDNonLocalVariable() const {return _nonlocalData.dLocalVariableDNonLocalVariable;}
    virtual fullMatrix<double> &getRefToDLocalVariableDNonLocalVariable() {return _nonlocalData.dLocalVariableDNonLocalVariable;}

    virtual const fullVector<double> &getConstRefToNonLocalJump() const {return _nonlocalData.nonLocalJump;}
    virtual fullVector<double> &getRefToNonLocalJump() {return _nonlocalData.nonLocalJump;}

    virtual const std::vector<SVector3> &getConstRefToGradNonLocalVariable() const {return _nonlocalData.gradNonLocalVariable;}
    virtual std::vector<SVector3> &getRefToGradNonLocalVariable() {return _nonlocalData.gradNonLocalVariable;}
    //coupling non local with constitutiveExtraDof
    virtual const fullMatrix<double> & getConstRefTodLocalVariableDExtraDofDiffusionField() const {return _nonlocalData.dLocalVariableDExtraDofDiffusionField;}
    virtual fullMatrix<double> & getRefTodLocalVariableDExtraDofDiffusionField(){return _nonlocalData.dLocalVariableDExtraDofDiffusionField;}


    virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idx) const {return characteristicLength[idx];};

    //-----------------get 1st piola before AV ------------//

    virtual const STensor3 &getConstRefToFirstPiolaKirchhoffStressb4AV() const {return _ipvBase->getConstRefToFirstPiolaKirchhoffStressb4AV();};
    virtual STensor3 &getRefToFirstPiolaKirchhoffStressb4AV() {return _ipvBase->getRefToFirstPiolaKirchhoffStressb4AV();};

    virtual void setBulkCriticalDamage(const double DT){_ipvBase->setBulkCriticalDamage(DT);}; // set critical damage value
    virtual double getBulkCriticalDamage() const {return _ipvBase->getBulkCriticalDamage();};

    virtual void setNonLocalToLocal(const bool fl){_ipvBase->setNonLocalToLocal(fl);};
    virtual bool getNonLocalToLocal() const {return _ipvBase->getNonLocalToLocal();};

    // for multiple field formulation
    // incompatible jump
    virtual const SVector3& getConstRefToIncompatibleJump() const {return _ipvBase->getConstRefToIncompatibleJump();};
    virtual SVector3& getRefToIncompatibleJump() {return _ipvBase->getRefToIncompatibleJump();};

    // for path following based on  irreversibe energt
    virtual double irreversibleEnergy() const {return _ipvBase->irreversibleEnergy();};
    virtual double& getRefToIrreversibleEnergy() {return _ipvBase->getRefToIrreversibleEnergy();};

    virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _ipvBase->getConstRefToDIrreversibleEnergyDDeformationGradient();};
    virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _ipvBase->getRefToDIrreversibleEnergyDDeformationGradient();};

    // irreversible ennergy depend on nonlocal variale
    virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const {return DirrEnergDNonlocalVar[index];};
    virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) {return DirrEnergDNonlocalVar[index];};

    // irreversible ennergy depend on extra dof
    virtual const double& getConstRefToDIrreversibleEnergyDConstitutiveExtraDofDiffusion(const int index) const {return DirrEnergDConstitutiveExtraDofDiffusion[index];};
    virtual double& getRefToDIrreversibleEnergyDConstitutiveExtraDofDiffusion(const int index) {return DirrEnergDConstitutiveExtraDofDiffusion[index];};

    virtual IPVariable* clone() const {return new virtualNonLocalDamageDG3DIPVariable(*this);};

    virtual void restart();

  // extradof
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodLinearKdField() const {return _ipvBase->getConstRefTodLinearKdField();}; 
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodLinearKdField() {return _ipvBase->getRefTodLinearKdField();}; 
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodLinearKdF() const {return _ipvBase->getConstRefTodLinearKdF();}; 
  virtual std::vector<std::vector<STensor43>>  &getRefTodLinearKdF() {return _ipvBase->getRefTodLinearKdF();}; 
/*  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodEnergyKdField() const {return _ipvBase->getConstRefTodEnergyKdField();}; 
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodEnergyKdField() {return _ipvBase->getRefTodEnergyKdField();}; 
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodEnergyKdF() const {return _ipvBase->getConstRefTodEnergyKdF();}; 
  virtual std::vector<std::vector<STensor43>>  &getRefTodEnergyKdF() {return _ipvBase->getRefTodEnergyKdF();}; */
  virtual const std::vector<std::vector<std::vector<STensor3> > >  &getConstRefTodFluxdGradFielddField() const {return _ipvBase->getConstRefTodFluxdGradFielddField();}; 
  virtual std::vector<std::vector<std::vector<STensor3> > >  &getRefTodFluxdGradFielddField() {return _ipvBase->getRefTodFluxdGradFielddField();}; 
  virtual const std::vector<std::vector<STensor43>>  &getConstRefTodFluxdGradFielddF() const {return _ipvBase->getConstRefTodFluxdGradFielddF();}; 
  virtual std::vector<std::vector<STensor43>>  &getRefTodFluxdGradFielddF() {return _ipvBase->getRefTodFluxdGradFielddF();};
  virtual int getNumConstitutiveExtraDofDiffusionVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      return 0;
    else
      return _constitutiveExtraDofDiffusionData->getConstitutiveExtraDofDiffusionVariable();
  }
  virtual double getConstRefToField(const int idex) const {return constitutiveExtraDofVar[idex];};
  virtual double &getRefToField(const int idex) {return constitutiveExtraDofVar[idex];};
  virtual const std::vector<SVector3>  &getConstRefToGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradField;
  }
  virtual std::vector<SVector3>  &getRefToGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradField;
  }
  virtual const std::vector<SVector3>  &getConstRefToFlux() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToFlux() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->flux;
  }
  virtual std::vector<SVector3>  &getRefToFlux()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToFlux() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->flux;
  }
  virtual const std::vector<STensor3>  &getConstRefTodPdField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodPdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdField;
  }
  virtual std::vector<STensor3>  &getRefTodPdField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodPdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdField;
  }
  virtual const std::vector<STensor33>  &getConstRefTodPdGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodPdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdGradField;
  }
  virtual std::vector<STensor33>  &getRefTodPdGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodPdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dPdGradField;
  }
  virtual const std::vector< std::vector<STensor3> >  &getConstRefTodFluxdGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradField;
  }
  virtual std::vector<std::vector<STensor3> >  &getRefTodFluxdGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdGradField;
  }
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFluxdField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdField;
  }
  virtual std::vector<std::vector<SVector3> >  &getRefTodFluxdField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdField;
  }
  virtual const std::vector<STensor33>   &getConstRefTodFluxdF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdF;
  }
  virtual std::vector<STensor33>   &getRefTodFluxdF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdF;
  }
  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToLinearK() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTolinearK() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->linearK;
  }
  virtual void setConstRefToLinearK(const STensor3 &eT, int i, int j)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTolinearK() : _constitutiveExtraDofDiffusionData not created");
    else
    {
      if(i < getNumConstitutiveExtraDofDiffusionVariable() && j< getNumConstitutiveExtraDofDiffusionVariable())
         _constitutiveExtraDofDiffusionData->linearK[i][j] = &eT;
      else
         Msg::Error("getConstRefTolinearK() : index out of range");
    }
  }
/*  virtual const std::vector<std::vector<const STensor3*> > &getConstRefToEnergyK() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyK() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->EnergyK;
  }
  virtual void setConstRefToEnergyK(const STensor3 &eT, int i, int j)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyK() : _constitutiveExtraDofDiffusionData not created");
    else
    {
      if(i < getNumConstitutiveExtraDofDiffusionVariable() && j< getNumConstitutiveExtraDofDiffusionVariable())
         _constitutiveExtraDofDiffusionData->linearK[i][j] = &eT;
      else
         Msg::Error("getConstRefToEnergyK() : index out of range");
    }
  }*/
  virtual const fullVector<double>  &getConstRefToFieldSource() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToFieldSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldSource;
  }
  virtual fullVector<double>  &getRefToFieldSource()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToFieldSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldSource;
  }
  virtual const fullMatrix<double>  &getConstRefTodFieldSourcedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFieldSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedField;
  }
  virtual fullMatrix<double>  &getRefTodFieldSourcedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedField;
  }
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodFieldSourcedGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedGradField;
  }
  virtual std::vector<std::vector<SVector3> >  &getRefTodFieldSourcedGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedGradField;
  }
  virtual const std::vector<STensor3>  &getConstRefTodFieldSourcedF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFieldSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedF;
  }
  virtual std::vector<STensor3>  &getRefTodFieldSourcedF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedF;
  }
  virtual const fullVector<double>  &getConstRefToMechanicalSource() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToMechanicalSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->mechanicalSource;
  }
  virtual fullVector<double>  &getRefToMechanicalSource()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToMechanicalSource() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->mechanicalSource;
  }
  virtual const fullMatrix<double>  &getConstRefTodMechanicalSourcedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedField;
  }
  virtual fullMatrix<double>  &getRefTodMechanicalSourcedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedField;
  }
  virtual const std::vector<std::vector<SVector3> >  &getConstRefTodMechanicalSourcedGradField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedGradField;
  }
  virtual std::vector<std::vector<SVector3> >  &getRefTodMechanicalSourcedGradField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedGradField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedGradField;
  }
  virtual const std::vector<STensor3>  &getConstRefTodMechanicalSourcedF() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedF;
  }
  virtual std::vector<STensor3>  &getRefTodMechanicalSourcedF()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedF() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedF;
  }
  virtual const fullVector<double>  &getConstRefToExtraDofFieldCapacityPerUnitField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToExtraDofFieldCapacityPerUnitField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldCapacityPerUnitField;
  }
  virtual fullVector<double>  &getRefToExtraDofFieldCapacityPerUnitField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToExtraDofFieldCapacityPerUnitField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldCapacityPerUnitField;
  }
  virtual const fullVector<double>  &getConstRefToFieldJump() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldJump;
  }
  virtual fullVector<double>  &getRefToFieldJump()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->fieldJump;
  }
  virtual const fullVector<double>  &getConstRefToOneOverFieldJump() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToOneOverFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->oneOverFieldJump;
  }
  virtual fullVector<double>  &getRefToOneOverFieldJump()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToOneOverFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->oneOverFieldJump;
  }
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldm() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodOneOverFieldJumpdFieldm : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldm;
  }
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldm()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodOneOverFieldJumpdFieldm() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldm;
  }
  virtual const fullMatrix<double>  &getConstRefTodOneOverFieldJumpdFieldp() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodOneOverFieldJumpdFieldp : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldp;
  }
  virtual fullMatrix<double>  &getRefTodOneOverFieldJumpdFieldp()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodOneOverFieldJumpdFieldp() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dOneOverFieldJumpdFieldp;
  }
  virtual const std::vector<SVector3>  &getConstRefToInterfaceFlux() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToInterfaceFlux : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->interfaceFlux;
  }
  virtual std::vector<SVector3>  &getRefToInterfaceFlux()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToInterfaceFlux() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->interfaceFlux;
  }
  virtual const std::vector< const STensor3*> &getConstRefToLinearSymmetrizationCoupling() const 
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToLinearSymmetrizationCoupling() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->linearSymmetrizationCoupling;
  }
  virtual void setConstRefToLinearSymmetrizationCoupling(const STensor3 &eT, int i) 
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("setConstRefToLinearSymmetrizationCoupling() : _constitutiveExtraDofDiffusionData not created");
    else
    { 
      if(i < getNumConstitutiveExtraDofDiffusionVariable())
         _constitutiveExtraDofDiffusionData->linearSymmetrizationCoupling[i] = &eT;
      else
         Msg::Error("setConstRefToLinearSymmetrizationCoupling() : index out of range");
    }
  }
  //coupling constitutive extra dof with nonLocal
  virtual const std::vector<std::vector<SVector3> > & getConstRefTodFluxdNonLocalVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFluxdNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdNonLocalVariable;
  }
  virtual std::vector<std::vector<SVector3> > & getRefTodFluxdNonLocalVariable()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFluxdNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFluxdNonLocalVariable;
  }
  virtual const fullMatrix<double> &getConstRefTodFieldSourcedNonLocalVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFieldSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedNonLocalVariable;
  }
  virtual const fullMatrix<double> &getRefTodFieldSourcedNonLocalVariable()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFieldSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFieldSourcedNonLocalVariable;
  }
  virtual const fullMatrix<double> &getConstRefTodMechanicalSourcedNonLocalVariable() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodMechanicalSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedNonLocalVariable;
  }
  virtual fullMatrix<double> &getRefTodMechanicalSourcedNonLocalVariable()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodMechanicalSourcedNonLocalVariable : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dMechanicalSourcedNonLocalVariable;
  }

  //Energy Conjugated Field (fT & fv)

  virtual double getConstRefToEnergyConjugatedField(const int idex) const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedField(idex);
  }
  virtual double  &getRefToEnergyConjugatedField(const int idex)
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedField(idex);
  }
  virtual const std::vector<SVector3>  &getConstRefToGradEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradEnergyConjugatedField;
  }
  virtual std::vector<SVector3>  &getRefToGradEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->gradEnergyConjugatedField;
  }
  virtual const  std::vector<std::vector<double>>  &getConstRefTodFielddEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFielddEnergyConjugatedField;
  }
  virtual std::vector<std::vector<double>>  &getRefTodFielddEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dFielddEnergyConjugatedField;
  }
  virtual const std::vector<std::vector<SVector3>>  &getConstRefTodGradFielddEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodGradFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddEnergyConjugatedField;
  }
  virtual std::vector<std::vector<SVector3>>  &getRefTodGradFielddEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodGradFielddEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddEnergyConjugatedField;
  }
  virtual const std::vector<std::vector<STensor3>>  &getConstRefTodGradFielddGradEnergyConjugatedField() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodGradFielddGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddGradEnergyConjugatedField;
  }
  virtual std::vector<std::vector<STensor3>>  &getRefTodGradFielddGradEnergyConjugatedField()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodGradFielddGradEnergyConjugatedField() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dGradFielddGradEnergyConjugatedField;
  }
  virtual const fullVector<double>  &getConstRefToEnergyConjugatedFieldJump() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefToEnergyConjugatedFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedFieldJump;
  } 
  virtual fullVector<double>  &getRefToEnergyConjugatedFieldJump()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefToEnergyConjugatedFieldJump() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->energyConjugatedFieldJump;
  } 
  virtual const  std::vector<std::vector<double>>  &getConstRefTodEnergyConjugatedFieldJumpdFieldm() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodEnergyConjugatedFieldJumpdFieldm() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldm;
  } 
  virtual std::vector<std::vector<double>>  &getRefTodEnergyConjugatedFieldJumpdFieldm()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodEnergyConjugatedFieldJumpdFieldm() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldm;
  } 
  virtual const  std::vector<std::vector<double>>  &getConstRefTodEnergyConjugatedFieldJumpdFieldp() const
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getConstRefTodEnergyConjugatedFieldJumpdFieldp() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldp;
  }
  virtual std::vector<std::vector<double>>  &getRefTodEnergyConjugatedFieldJumpdFieldp()
  {
    if(_constitutiveExtraDofDiffusionData==NULL)
      Msg::Error("getRefTodEnergyConjugatedFieldJumpdFieldp() : _constitutiveExtraDofDiffusionData not created");
    else
      return _constitutiveExtraDofDiffusionData->dEnergyConjugatedFieldJumpdFieldp;
  }

};

class nonLocalDamageDG3DIPVariable : public dG3DIPVariable
{

 protected:
  IPNonLocalDamage *_nldipv;

 public:
  nonLocalDamageDG3DIPVariable(int _nsdv, const bool oninter=false);
  nonLocalDamageDG3DIPVariable(const nonLocalDamageDG3DIPVariable &source);
  virtual nonLocalDamageDG3DIPVariable& operator=(const IPVariable &source);

 /* specific function */
  IPNonLocalDamage* getIPNonLocalDamage(){return _nldipv;}
  const IPNonLocalDamage* getIPNonLocalDamage() const{return _nldipv;}
  virtual ~nonLocalDamageDG3DIPVariable()
  {
    if (_nldipv) delete _nldipv;
    _nldipv = NULL;
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_nldipv)
      _nldipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _nldipv;};
  virtual const IPVariable* getInternalData()  const {return _nldipv;};

  virtual bool dissipationIsActive() const {return _nldipv->dissipationIsActive();};

  virtual void blockDissipation(const bool fl){if (_nldipv) _nldipv->blockDissipation(fl);};
  virtual bool dissipationIsBlocked() const {
    if (_nldipv) return _nldipv->dissipationIsBlocked();
    else return false;
  }

  virtual double getConstRefToLocalVariable(const int idex) const ;
  virtual double& getRefToLocalVariable(const int idex);

  virtual double getConstRefToNonLocalVariable(const int idex) const;
  virtual double& getRefToNonLocalVariable(const int idex);

  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return getIPNonLocalDamage()->damageEnergy();};
  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const
  {
    if (idex == 0)
      return getIPNonLocalDamage()->_nldCharacteristicLengthMatrix;
    else
      Msg::Error("the non-local variable %d is not defined",idex);
  }

  virtual IPVariable* clone() const {return new nonLocalDamageDG3DIPVariable(*this);};
  virtual void restart();
};

class nonLocalDamageIsotropicElasticityDG3DIPVariable : public dG3DIPVariable
{

 protected:
  IPNonLocalDamageIsotropicElasticity *_nldIsotropicElasticityipv;

 public:
  nonLocalDamageIsotropicElasticityDG3DIPVariable(const mlawNonlocalDamageHyperelastic & elasticLaw, const bool oninter=false);
  nonLocalDamageIsotropicElasticityDG3DIPVariable(const mlawNonLocalDamageIsotropicElasticity & isoElastLaw, const bool oninter=false);
  nonLocalDamageIsotropicElasticityDG3DIPVariable(const nonLocalDamageIsotropicElasticityDG3DIPVariable &source);
  virtual nonLocalDamageIsotropicElasticityDG3DIPVariable& operator=(const IPVariable &source);

 /* specific function */
  IPNonLocalDamageIsotropicElasticity* getIPNonLocalDamageIsotropicElasticity(){return _nldIsotropicElasticityipv;}
  const IPNonLocalDamageIsotropicElasticity* getIPNonLocalDamageIsotropicElasticity() const{return _nldIsotropicElasticityipv;}
  virtual ~nonLocalDamageIsotropicElasticityDG3DIPVariable()
  {
    if (_nldIsotropicElasticityipv){
     delete _nldIsotropicElasticityipv;
     _nldIsotropicElasticityipv = NULL;
    }
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_nldIsotropicElasticityipv)
      _nldIsotropicElasticityipv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _nldIsotropicElasticityipv;};
  virtual const IPVariable* getInternalData()  const {return _nldIsotropicElasticityipv;};

  virtual bool dissipationIsActive() const {return _nldIsotropicElasticityipv->dissipationIsActive();};

  virtual void blockDissipation(const bool fl){
    if (_nldIsotropicElasticityipv) _nldIsotropicElasticityipv->blockDissipation(fl);
  }
  virtual bool dissipationIsBlocked() const {
    if (_nldIsotropicElasticityipv) return _nldIsotropicElasticityipv->dissipationIsBlocked();
    else return false;
  }

  virtual void setBulkCriticalDamage(const double DT){_nldIsotropicElasticityipv->setCriticalDamage(DT);}; // set critical damage value
  virtual double getBulkCriticalDamage() const {return _nldIsotropicElasticityipv->getCriticalDamage();};

  virtual void setNonLocalToLocal(const bool fl){_nldIsotropicElasticityipv->setNonLocalToLocal(fl);};
  virtual bool getNonLocalToLocal() const {return _nldIsotropicElasticityipv->getNonLocalToLocal();};

  virtual double getConstRefToLocalVariable(const int idex) const ;
  virtual double& getRefToLocalVariable(const int idex);

  virtual double getConstRefToNonLocalVariable(const int idex) const;
  virtual double& getRefToNonLocalVariable(const int idex);

  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _nldIsotropicElasticityipv->damageEnergy();};
  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const
  {
    if (idex == 0)
      return _nldIsotropicElasticityipv->getConstRefToCharacteristicLength();
    else
      Msg::Error("the non-local variable %d is not defined",idex);
  }

  // for path following based on irreversibe energt
  virtual double irreversibleEnergy() const {return _nldIsotropicElasticityipv->irreversibleEnergy();};
  virtual double& getRefToIrreversibleEnergy() {return _nldIsotropicElasticityipv->getRefToIrreversibleEnergy();};

  virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const
    {return _nldIsotropicElasticityipv->getConstRefToDIrreversibleEnergyDF();};
  virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient()
    {return _nldIsotropicElasticityipv->getRefToDIrreversibleEnergyDF();};

  virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const
  {
    if (index == 0)
      {
      return _nldIsotropicElasticityipv->getDIrreversibleEnergyDNonLocalVariable();
      }
    else{Msg::Error("the non-local variable %d is not defined",index);}
  };
  virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index)
  {
    if (index == 0)
    {
      return _nldIsotropicElasticityipv->getRefToDIrreversibleEnergyDNonLocalVariable();
    }
    else{Msg::Error("the non-local variable %d is not defined",index);}
  };

  virtual IPVariable* clone() const {return new nonLocalDamageIsotropicElasticityDG3DIPVariable(*this);};
  virtual void restart();
};

class nonLocalDamageJ2HyperDG3DIPVariable : public dG3DIPVariable
{

 protected:
  IPNonLocalDamageJ2Hyper *_nldJ2Hyperipv;

 public:
  nonLocalDamageJ2HyperDG3DIPVariable(const mlawNonLocalDamageJ2Hyper &_j2law, const bool oninter=false);
  nonLocalDamageJ2HyperDG3DIPVariable(const mlawNonLocalDamageJ2SmallStrain &_j2law, const bool oninter=false);
  nonLocalDamageJ2HyperDG3DIPVariable(const nonLocalDamageJ2HyperDG3DIPVariable &source);
  virtual nonLocalDamageJ2HyperDG3DIPVariable& operator=(const IPVariable &source);

 /* specific function */
  IPNonLocalDamageJ2Hyper* getIPNonLocalDamageJ2Hyper(){return _nldJ2Hyperipv;}
  const IPNonLocalDamageJ2Hyper* getIPNonLocalDamageJ2Hyper() const{return _nldJ2Hyperipv;}
  virtual ~nonLocalDamageJ2HyperDG3DIPVariable()
  {
    if (_nldJ2Hyperipv) delete _nldJ2Hyperipv;
    _nldJ2Hyperipv = NULL;
  }

  virtual void blockDissipation(const bool fl){
    if (_nldJ2Hyperipv) _nldJ2Hyperipv->blockDissipation(fl);
  }
  virtual bool dissipationIsBlocked() const {
    if (_nldJ2Hyperipv) return _nldJ2Hyperipv->dissipationIsBlocked();
    else return false;
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_nldJ2Hyperipv)
      _nldJ2Hyperipv->setLocation(loc);
  };
  virtual IPVariable* getInternalData() {return _nldJ2Hyperipv;};
  virtual const IPVariable* getInternalData()  const {return _nldJ2Hyperipv;};

  virtual bool dissipationIsActive() const {return _nldJ2Hyperipv->dissipationIsActive();};

  virtual void setBulkCriticalDamage(const double DT){_nldJ2Hyperipv->setCriticalDamage(DT);}; // set critical damage value
  virtual double getBulkCriticalDamage() const {return _nldJ2Hyperipv->getCriticalDamage();};

  virtual void setNonLocalToLocal(const bool fl){_nldJ2Hyperipv->setNonLocalToLocal(fl);};
  virtual bool getNonLocalToLocal() const {return _nldJ2Hyperipv->getNonLocalToLocal();};

  virtual double getConstRefToLocalVariable(const int idex) const ;
  virtual double& getRefToLocalVariable(const int idex);

  virtual double getConstRefToNonLocalVariable(const int idex) const;
  virtual double& getRefToNonLocalVariable(const int idex);

  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _nldJ2Hyperipv->damageEnergy();}
  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const
  {
    if (idex == 0)
      return _nldJ2Hyperipv->getConstRefToCharacteristicLength();
    else
      Msg::Error("the non-local variable %d is not defined",idex);
  }

	// for path following based on  irreversibe energt
	virtual double irreversibleEnergy() const {return _nldJ2Hyperipv->irreversibleEnergy();};
	virtual double& getRefToIrreversibleEnergy() {return _nldJ2Hyperipv->getRefToIrreversibleEnergy();};

	virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _nldJ2Hyperipv->getConstRefToDIrreversibleEnergyDF();};
	virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _nldJ2Hyperipv->getRefToDIrreversibleEnergyDF();};

	virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const {
		if (index == 0){
			return _nldJ2Hyperipv->getDIrreversibleEnergyDNonLocalVariable();
		}
		else{
			Msg::Error("the non-local variable %d is not defined",index);
		}
	};
	virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) {
		if (index == 0){
			return _nldJ2Hyperipv->getRefToDIrreversibleEnergyDNonLocalVariable();
		}
		else{
			Msg::Error("the non-local variable %d is not defined",index);
		}
	};

  virtual IPVariable* clone() const {return new nonLocalDamageJ2HyperDG3DIPVariable(*this);};
  virtual void restart();
};

class nonLocalPorosityDG3DIPVariable : public dG3DIPVariable
{
 protected:
  IPNonLocalPorosity *_nldPorousipv;
  mlawNonLocalPorosity::nonLocalMethod _nonlocalMethod;

 public:
  nonLocalPorosityDG3DIPVariable(const bool oninter, const int numLocalVar, const int numExtraDofDiffusion);
  nonLocalPorosityDG3DIPVariable(const mlawNonLocalPorosity &porousLaw,const double fvInitial, const bool oninter=false, const int numNonLocalVar = 1, const int numExtraDofDiffusion=0);
  nonLocalPorosityDG3DIPVariable(const nonLocalPorosityDG3DIPVariable &source);
  virtual nonLocalPorosityDG3DIPVariable& operator=(const IPVariable &source);

  virtual IPVariable* getInternalData()  {return _nldPorousipv;};
	virtual const IPVariable* getInternalData()  const {return _nldPorousipv;};

 /* specific function */
  IPNonLocalPorosity* getIPNonLocalPorosity(){return _nldPorousipv;}
  const IPNonLocalPorosity* getIPNonLocalPorosity() const {return _nldPorousipv;}
  virtual ~nonLocalPorosityDG3DIPVariable()
  {
    if (_nldPorousipv != NULL){
      delete _nldPorousipv; _nldPorousipv=NULL;
    }
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_nldPorousipv)
      _nldPorousipv->setLocation(loc);
  };

  virtual bool dissipationIsActive() const {return _nldPorousipv->dissipationIsActive();};

  virtual void blockDissipation(const bool fl){
    if (_nldPorousipv) _nldPorousipv->blockDissipation(fl);
  }
  virtual bool dissipationIsBlocked() const{
    if (_nldPorousipv) return _nldPorousipv->dissipationIsBlocked();
    else return false;
  }

  virtual void setNonLocalToLocal(const bool fl){ if (_nldPorousipv) _nldPorousipv->setNonLocalToLocal(fl);};
  virtual bool getNonLocalToLocal() const {
    if (_nldPorousipv) return _nldPorousipv->getNonLocalToLocal();
    else return false;
  };

  virtual double getConstRefToLocalVariable(const int idex) const ;
  virtual double& getRefToLocalVariable(const int idex);

  virtual double getConstRefToNonLocalVariable(const int idex) const;
  virtual double& getRefToNonLocalVariable(const int idex);


  virtual double get(const int i) const;
  virtual double defoEnergy() const;
  virtual double plasticEnergy() const;
  virtual double damageEnergy() const {return _nldPorousipv->damageEnergy();};
  virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const
  {
    return _nldPorousipv->getConstRefToCharacteristicLength(idex);
  }
  virtual IPVariable* clone() const {return new nonLocalPorosityDG3DIPVariable(*this);};

  // for path following based on  irreversibe energt
	virtual double irreversibleEnergy() const {return _nldPorousipv->irreversibleEnergy();};
	virtual double& getRefToIrreversibleEnergy() {return _nldPorousipv->getRefToIrreversibleEnergy();};

	virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _nldPorousipv->getConstRefToDIrreversibleEnergyDF();};
	virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _nldPorousipv->getRefToDIrreversibleEnergyDF();};
  virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const {
    return _nldPorousipv->getConstRefToDIrreversibleEnergyDNonLocalVariable(index);
	};
	virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index) {
    return _nldPorousipv->getRefToDIrreversibleEnergyDNonLocalVariable(index);
	};


  virtual void restart();
  virtual void setRefToLinearConductivity(const STensor3 &eT) {setConstRefToLinearK(eT,0,0);}
  virtual void setRefToStiff_alphaDilatation(const STensor3 &eT) {setConstRefToLinearSymmetrizationCoupling(eT,0);}
protected:
  nonLocalPorosityDG3DIPVariable(const mlawNonLocalPorosity &pLaw, const bool oninter=false, const int numLocalVar = 1, const int numExtraDofDiffusion=0);

};

class nonLocalPorosityWithCleavageDG3DIPVariable : public nonLocalPorosityDG3DIPVariable{
  protected:
    IPNonLocalPorosityWithCleavage* _cleavageNonLocalIPv;
  public:
    nonLocalPorosityWithCleavageDG3DIPVariable(const mlawNonLocalPorousWithCleavageFailure &porousLaw,const double fvInitial, const bool oninter=false, const int numNonLocalVar = 1, const int numExtraDofDiffusion=0);
    nonLocalPorosityWithCleavageDG3DIPVariable(const nonLocalPorosityWithCleavageDG3DIPVariable &source);
    virtual nonLocalPorosityWithCleavageDG3DIPVariable& operator=(const IPVariable &source);
    virtual ~nonLocalPorosityWithCleavageDG3DIPVariable();

    virtual double get(const int i) const;

    IPNonLocalPorosityWithCleavage* getIPNonLocalPorosityWithCleavage(){return dynamic_cast<IPNonLocalPorosityWithCleavage*>(_nldPorousipv);}
    const IPNonLocalPorosityWithCleavage* getIPNonLocalPorosityWithCleavage() const {return dynamic_cast<const IPNonLocalPorosityWithCleavage*>(_nldPorousipv);}

    virtual double getConstRefToLocalVariable(const int idex) const ;
    virtual double& getRefToLocalVariable(const int idex);

    virtual double getConstRefToNonLocalVariable(const int idex) const;
    virtual double& getRefToNonLocalVariable(const int idex);

    virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const;

    virtual IPVariable* clone() const {return new nonLocalPorosityWithCleavageDG3DIPVariable(*this);};
    virtual void restart();
};
#endif //NONLOCALDAMAGEDG3DIPVARIABLE_H_
