//
// C++ Interface: materialLaw
//
// Description: Class with definition of materialLaw for shell
//
//
// Author:  <Gauthier BECKER>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "nonLocalDamageDG3DMaterialLaw.h"
#include "ipstate.h"
#include "MInterfaceElement.h"
#include "ipField.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <string>
#include "nonLocalDamageDG3DIPVariable.h"
#include "dG3DCohesiveIPVariable.h"
#include "FractureCohesiveDG3DIPVariable.h"


virtualNonLocalDamageDG3DMaterialLaw::virtualNonLocalDamageDG3DMaterialLaw(const int num, const dG3DMaterialLaw& base, const int numNLVariable):
  dG3DMaterialLaw(num,base.density(),true), numNonlocalVar(numNLVariable)
{
  elasticStiffness  = (base.elasticStiffness);
  TotalelasticStiffness = (base.TotalelasticStiffness);
  _lawBase = dynamic_cast<dG3DMaterialLaw*>(base.clone());
  characteristicLength.resize(numNLVariable,STensor3(0.));
};

virtualNonLocalDamageDG3DMaterialLaw::virtualNonLocalDamageDG3DMaterialLaw(const virtualNonLocalDamageDG3DMaterialLaw &source):
  dG3DMaterialLaw(source), numNonlocalVar(source.numNonlocalVar),characteristicLength(source.characteristicLength){
  _lawBase = NULL;
  if (source._lawBase){
    _lawBase = dynamic_cast<dG3DMaterialLaw*>(source._lawBase->clone());
  }
};

virtualNonLocalDamageDG3DMaterialLaw::~virtualNonLocalDamageDG3DMaterialLaw(){
  if (_lawBase != NULL) delete _lawBase; _lawBase = NULL;
};

void virtualNonLocalDamageDG3DMaterialLaw::setNonLocalLengthScale(const int index, const CLengthLaw &cLLaw){
  IPCLength* ipCL = NULL;
  cLLaw.createIPVariable(ipCL);
  cLLaw.computeCL(0.,*ipCL);
  characteristicLength[index] = ipCL->getConstRefToCL();
  characteristicLength[index].print("characteristicLength");
  if (ipCL!= NULL) delete ipCL;
};

void virtualNonLocalDamageDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  IPStateBase* ipsBase= NULL;
  _lawBase->createIPState(ipsBase,state_,ele,nbFF_,GP,gpt);

  dG3DIPVariableBase* dgi = static_cast<dG3DIPVariableBase*>(ipsBase->getState(IPStateBase::initial));
  dG3DIPVariableBase* dg1 = static_cast<dG3DIPVariableBase*>(ipsBase->getState(IPStateBase::previous));
  dG3DIPVariableBase* dg2 = static_cast<dG3DIPVariableBase*>(ipsBase->getState(IPStateBase::current));


  IPVariable* ipvi = new  virtualNonLocalDamageDG3DIPVariable(numNonlocalVar,dgi,characteristicLength);
  IPVariable* ipv1 = new  virtualNonLocalDamageDG3DIPVariable(numNonlocalVar,dg1,characteristicLength);
  IPVariable* ipv2 = new  virtualNonLocalDamageDG3DIPVariable(numNonlocalVar,dg2,characteristicLength);

  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
  delete ipsBase;
}

void virtualNonLocalDamageDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  IPVariable* ipvBase = NULL;
  _lawBase->createIPVariable(ipvBase,ele,nbFF_,GP,gpt);
  dG3DIPVariableBase* dgipv = static_cast<dG3DIPVariableBase*>(ipvBase);
  if(ipv !=NULL) delete ipv;
  ipv = new  virtualNonLocalDamageDG3DIPVariable(numNonlocalVar,dgipv,characteristicLength);
  delete ipvBase;
}

void virtualNonLocalDamageDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  virtualNonLocalDamageDG3DIPVariable* ipvcur;
  const virtualNonLocalDamageDG3DIPVariable* ipvprev;

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<virtualNonLocalDamageDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const virtualNonLocalDamageDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<virtualNonLocalDamageDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const virtualNonLocalDamageDG3DIPVariable*>(ipvp);
  }
  _lawBase->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void virtualNonLocalDamageDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  virtualNonLocalDamageDG3DIPVariable* ipvcur;
  const virtualNonLocalDamageDG3DIPVariable* ipvprev;

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<virtualNonLocalDamageDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const virtualNonLocalDamageDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<virtualNonLocalDamageDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const virtualNonLocalDamageDG3DIPVariable*>(ipvp);
  }

  IPVariable* dgIPvCur = static_cast<IPVariable*>(ipvcur->getIPVBase());
  const IPVariable* dgIPvPrev = static_cast<const IPVariable*>(ipvprev->getIPVBase());

  _lawBase->stress(dgIPvCur,dgIPvPrev,stiff,checkfrac);
};



NonLocalDamageDG3DMaterialLaw::NonLocalDamageDG3DMaterialLaw(const int num, const double rho, const char *propName) :
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldlaw = new mlawNonLocalDamage(num,rho,propName);
  double nu = _nldlaw->poissonRatio();
  double mu = _nldlaw->shearModulus();
  double E = 2.*mu*(1.+nu);
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalDamageDG3DMaterialLaw::~NonLocalDamageDG3DMaterialLaw()
{
  if (_nldlaw !=NULL) delete _nldlaw;
  _nldlaw = NULL;
};


NonLocalDamageDG3DMaterialLaw::NonLocalDamageDG3DMaterialLaw(const NonLocalDamageDG3DMaterialLaw &source) :
                                                             dG3DMaterialLaw(source)
{
	_nldlaw = NULL;
	if (source._nldlaw != NULL){
		_nldlaw = dynamic_cast<materialLawWithDamage*>(source._nldlaw->clone());
	}

}

void NonLocalDamageDG3DMaterialLaw::setTime(const double t,const double dtime){
  dG3DMaterialLaw::setTime(t,dtime);
  _nldlaw->setTime(t,dtime);
  return;
}

materialLaw::matname NonLocalDamageDG3DMaterialLaw::getType() const {return _nldlaw->getType();}

bool NonLocalDamageDG3DMaterialLaw::withEnergyDissipation() const {return _nldlaw->withEnergyDissipation();};

void NonLocalDamageDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;

  int nsdv = _nldlaw->getNsdv();
  IPVariable* ipvi = new  nonLocalDamageDG3DIPVariable(nsdv,inter);
  IPVariable* ipv1 = new  nonLocalDamageDG3DIPVariable(nsdv,inter);
  IPVariable* ipv2 = new  nonLocalDamageDG3DIPVariable(nsdv,inter);

  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
  _nldlaw->createIPState((static_cast <nonLocalDamageDG3DIPVariable*> (ipvi))->getIPNonLocalDamage(),
                         (static_cast <nonLocalDamageDG3DIPVariable*> (ipv1))->getIPNonLocalDamage(),
             			 (static_cast <nonLocalDamageDG3DIPVariable*> (ipv2))->getIPNonLocalDamage());

}

void NonLocalDamageDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL) inter=false;

  ipv = new  nonLocalDamageDG3DIPVariable(_nldlaw->getNsdv(),inter);

  IPNonLocalDamage * ipvnl = static_cast <nonLocalDamageDG3DIPVariable*>(ipv)->getIPNonLocalDamage();
  _nldlaw->createIPVariable(ipvnl,ele,nbFF_,GP,gpt);
}


double NonLocalDamageDG3DMaterialLaw::soundSpeed() const{return _nldlaw->soundSpeed();}

void NonLocalDamageDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  nonLocalDamageDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageDG3DIPVariable*>(ipvp);
  }
  _nldlaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());

};

void NonLocalDamageDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  nonLocalDamageDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageDG3DIPVariable*>(ipvp);
  }

  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPNonLocalDamage* q1 = ipvcur->getIPNonLocalDamage();
  const IPNonLocalDamage* q0 = ipvprev->getIPNonLocalDamage();

  /* compute stress */
  _nldlaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                        ipvcur->getRefToDLocalVariableDStrain()[0],ipvcur->getRefToDStressDNonLocalVariable()[0],
                        ipvcur->getRefToDLocalVariableDNonLocalVariable()(0,0),stiff);

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);

}

double NonLocalDamageDG3DMaterialLaw::scaleFactor() const{return _nldlaw->scaleFactor();};

void NonLocalDamageDG3DMaterialLaw::setMacroSolver(const nonLinearMechSolver* sv){
	dG3DMaterialLaw::setMacroSolver(sv);
	if (_nldlaw != NULL){
		_nldlaw->setMacroSolver(sv);
	}
};

//**** NonLocalDamageDG3DMaterialLaw_Stochastic
NonLocalDamageDG3DMaterialLaw_Stoch::NonLocalDamageDG3DMaterialLaw_Stoch(const int num, const double rho, const char *propName,
                     const double Ori_x, const double Ori_y, const double Ori_z,const double Lx,const double Ly,const double Lz,
                     const double dx, const double dy, const double dz,  const double euler0,const double euler1,
                     const double euler2,const char *Geo, const char *RandProp) : NonLocalDamageDG3DMaterialLaw(num,rho,propName){
  if(_nldlaw != NULL) delete _nldlaw;
  _nldlaw = new mlawNonLocalDamage_Stoch(num,rho,propName,Ori_x, Ori_y, Ori_z, Lx, Ly, Lz, dx, dy, dz,
                                        euler0,euler1,euler2, Geo, RandProp);

  double nu = _nldlaw->poissonRatio();
  double mu = _nldlaw->shearModulus();
  double E = 2.*mu*(1.+nu);
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalDamageDG3DMaterialLaw_Stoch::NonLocalDamageDG3DMaterialLaw_Stoch(const int num, const double rho, const char *propName, const double Ori_x, const double Ori_y, const double Ori_z,const double Lx,const double Ly,const double Lz, const char *RandProp, const int intpl) : NonLocalDamageDG3DMaterialLaw(num,rho,propName){
  if(_nldlaw != NULL) delete _nldlaw;
  _nldlaw = new mlawNonLocalDamage_Stoch(num,rho,propName,Ori_x, Ori_y, Ori_z, Lx, Ly, Lz, RandProp,intpl);
  double nu = _nldlaw->poissonRatio();
  double mu = _nldlaw->shearModulus();
  double E = 2.*mu*(1.+nu);
  fillElasticStiffness(E, nu, elasticStiffness);
}



NonLocalDamageDG3DMaterialLaw_Stoch::~NonLocalDamageDG3DMaterialLaw_Stoch()
{

 if (_nldlaw !=NULL) delete _nldlaw;
  _nldlaw = NULL;

};


NonLocalDamageDG3DMaterialLaw_Stoch::NonLocalDamageDG3DMaterialLaw_Stoch(const NonLocalDamageDG3DMaterialLaw_Stoch &source) :
                                                             NonLocalDamageDG3DMaterialLaw(source) {}

void NonLocalDamageDG3DMaterialLaw_Stoch::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;
  SPoint3 pt_Global;
  ele->pnt(GP->pt[0],GP->pt[1],GP->pt[2],pt_Global);
  SVector3 GaussP(pt_Global);

  int nsdv = _nldlaw->getNsdv();
  IPVariable* ipvi = new  nonLocalDamageDG3DIPVariable(nsdv,inter);
  IPVariable* ipv1 = new  nonLocalDamageDG3DIPVariable(nsdv,inter);
  IPVariable* ipv2 = new  nonLocalDamageDG3DIPVariable(nsdv,inter);

  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
 static_cast<mlawNonLocalDamage_Stoch*>(_nldlaw)->createIPState(GaussP, (static_cast <nonLocalDamageDG3DIPVariable*> (ipvi))->getIPNonLocalDamage(),
                         (static_cast <nonLocalDamageDG3DIPVariable*> (ipv1))->getIPNonLocalDamage(),
             			 (static_cast <nonLocalDamageDG3DIPVariable*> (ipv2))->getIPNonLocalDamage());

}

void NonLocalDamageDG3DMaterialLaw_Stoch::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL) inter=false;
  SPoint3 pt_Global;
  ele->pnt(GP->pt[0],GP->pt[1],GP->pt[2],pt_Global);
  SVector3 GaussP(pt_Global);

  ipv = new  nonLocalDamageDG3DIPVariable(_nldlaw->getNsdv(),inter);

  IPNonLocalDamage * ipvnl = static_cast <nonLocalDamageDG3DIPVariable*>(ipv)->getIPNonLocalDamage();
  static_cast<mlawNonLocalDamage_Stoch*>(_nldlaw)->createIPVariable(GaussP, ipvnl,ele,nbFF_,GP,gpt);
}
// *********

NonLocalDamageHyperelasticDG3DMaterialLaw::NonLocalDamageHyperelasticDG3DMaterialLaw(const int num, const double rho, const elasticPotential& elP, const CLengthLaw &cLLaw,
                                         const DamageLaw &damLaw): dG3DMaterialLaw(num,rho,true){
  _nlLaw = new mlawNonlocalDamageHyperelastic(num,rho,elP,cLLaw,damLaw);

  double E = elP.getYoungModulus();
	double nu = elP.getPoissonRatio();
	fillElasticStiffness(E, nu, elasticStiffness);
};
NonLocalDamageHyperelasticDG3DMaterialLaw::NonLocalDamageHyperelasticDG3DMaterialLaw(const NonLocalDamageHyperelasticDG3DMaterialLaw& src):
dG3DMaterialLaw(src)
{
  _nlLaw = NULL;
  if (src._nlLaw != NULL){
    _nlLaw = dynamic_cast<mlawNonlocalDamageHyperelastic*>(src._nlLaw->clone());
  }
};
void NonLocalDamageHyperelasticDG3DMaterialLaw::setTime(const double t,const double dtime)
{
	dG3DMaterialLaw::setTime(t,dtime);
	_nlLaw->setTime(t,dtime);
}

void NonLocalDamageHyperelasticDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
	bool inter=true;
	const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
	if(iele==NULL) inter=false;

	IPVariable* ipvi = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nlLaw,inter);
	IPVariable* ipv1 = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nlLaw,inter);
	IPVariable* ipv2 = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nlLaw,inter);
	if(ips != NULL) delete ips;
	ips = new IP3State(state_,ipvi,ipv1,ipv2);
}

void NonLocalDamageHyperelasticDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
	bool inter=true;
	const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
	if(iele == NULL) inter=false;

	if(ipv !=NULL) delete ipv;
	ipv = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nlLaw,inter);
}

void NonLocalDamageHyperelasticDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {
      ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp->getIPvBulk());
      const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
      ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
  }
  else
  {
      ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipv);
      ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvp);
  }

  _nlLaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());

};

void NonLocalDamageHyperelasticDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
    /* get ipvariable */
    nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
    const nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

    FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
    if(ipvtmp !=NULL)
		{
				ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp->getIPvBulk());
				const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
				ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
		}
    else
		{
				ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipv);
				ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvp);
		}

    /* Deformation gradient F */
    const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
    const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

    /* data for non local isotropic elastic law */
    IPNonLocalDamageIsotropicElasticity* q1 = ipvcur->getIPNonLocalDamageIsotropicElasticity();
    const IPNonLocalDamageIsotropicElasticity* q0 = ipvprev->getIPNonLocalDamageIsotropicElasticity();

    /* compute stress */
    _nlLaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                                 ipvcur->getRefToDLocalVariableDStrain()[0],ipvcur->getRefToDStressDNonLocalVariable()[0],
                                 ipvcur->getRefToDLocalVariableDNonLocalVariable()(0,0),stiff);

    ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);   // Is it correct ????

}

NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw::NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw(const int num, const double rho,
                                        const elasticPotential& elP, const CLengthLaw &cLLaw,
                                         const DamageLaw &damLaw): NonLocalDamageHyperelasticDG3DMaterialLaw(num,rho,elP,cLLaw,damLaw){
  if (_nlLaw != NULL) delete _nlLaw;
  _nlLaw = new mlawNonlocalAnisotropicDamageHyperelastic(num,rho,elP,cLLaw,damLaw);
};


NonLocalDamageIsotropicElasticityDG3DMaterialLaw::NonLocalDamageIsotropicElasticityDG3DMaterialLaw(
    const int num,const double rho, double E,const double nu, const CLengthLaw &cLLaw,const DamageLaw &damLaw) :
    dG3DMaterialLaw(num,rho,true)
{
    _nldIsotropicElasticitylaw = new mlawNonLocalDamageIsotropicElasticity(num,rho,E,nu,cLLaw,damLaw);
    fillElasticStiffness(E, nu, elasticStiffness); // Is it usefull ?
}

NonLocalDamageIsotropicElasticityDG3DMaterialLaw::~NonLocalDamageIsotropicElasticityDG3DMaterialLaw()
{
		if (_nldIsotropicElasticitylaw!=NULL){
			delete _nldIsotropicElasticitylaw;
			_nldIsotropicElasticitylaw = NULL;
		}
};

NonLocalDamageIsotropicElasticityDG3DMaterialLaw::NonLocalDamageIsotropicElasticityDG3DMaterialLaw(const NonLocalDamageIsotropicElasticityDG3DMaterialLaw &source) :
    dG3DMaterialLaw(source)
{
	_nldIsotropicElasticitylaw = NULL;
	if (source._nldIsotropicElasticitylaw != NULL){
    _nldIsotropicElasticitylaw = new mlawNonLocalDamageIsotropicElasticity(*(source._nldIsotropicElasticitylaw));
	}
}

void NonLocalDamageIsotropicElasticityDG3DMaterialLaw::setNonLocalLimitingMethod(const int num){_nldIsotropicElasticitylaw->setNonLocalLimitingMethod(num);};

void NonLocalDamageIsotropicElasticityDG3DMaterialLaw::setTime(const double t,const double dtime)
{
    dG3DMaterialLaw::setTime(t,dtime);
    _nldIsotropicElasticitylaw->setTime(t,dtime);
}

materialLaw::matname NonLocalDamageIsotropicElasticityDG3DMaterialLaw::getType() const
{
    return _nldIsotropicElasticitylaw->getType();
}

void NonLocalDamageIsotropicElasticityDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
    // check if interface element or not
    bool inter=true;
    const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
    if(iele==NULL) inter=false;

    IPVariable* ipvi = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nldIsotropicElasticitylaw,inter);
    IPVariable* ipv1 = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nldIsotropicElasticitylaw,inter);
    IPVariable* ipv2 = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nldIsotropicElasticitylaw,inter);
    if(ips != NULL) delete ips;
    ips = new IP3State(state_,ipvi,ipv1,ipv2);

    _nldIsotropicElasticitylaw->createIPState((static_cast <nonLocalDamageIsotropicElasticityDG3DIPVariable*> (ipvi))->getIPNonLocalDamageIsotropicElasticity(),
                                  (static_cast <nonLocalDamageIsotropicElasticityDG3DIPVariable*> (ipv1))->getIPNonLocalDamageIsotropicElasticity(),
                                  (static_cast <nonLocalDamageIsotropicElasticityDG3DIPVariable*> (ipv2))->getIPNonLocalDamageIsotropicElasticity());

}

void NonLocalDamageIsotropicElasticityDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
    // check if interface element or not
    bool inter=true;
    const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
    if(iele == NULL) inter=false;

    if(ipv !=NULL) delete ipv;
    ipv = new  nonLocalDamageIsotropicElasticityDG3DIPVariable(*_nldIsotropicElasticitylaw,inter);
    IPNonLocalDamageIsotropicElasticity * ipvnl = static_cast <nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipv)->getIPNonLocalDamageIsotropicElasticity();
    _nldIsotropicElasticitylaw->createIPVariable(ipvnl);
}

double NonLocalDamageIsotropicElasticityDG3DMaterialLaw::soundSpeed() const
{
    return _nldIsotropicElasticitylaw->soundSpeed();
}

void NonLocalDamageIsotropicElasticityDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
    nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
    const nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

    FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
    if(ipvtmp !=NULL)
        {

            ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp->getIPvBulk());
            const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
            ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
        }
    else
        {
            ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipv);
            ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvp);
        }

    _nldIsotropicElasticitylaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void NonLocalDamageIsotropicElasticityDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
    /* get ipvariable */
    nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
    const nonLocalDamageIsotropicElasticityDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

    FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
    if(ipvtmp !=NULL)
        {

            ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp->getIPvBulk());
            const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
            ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
        }
    else
        {
            ipvcur = static_cast<nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipv);
            ipvprev = static_cast<const nonLocalDamageIsotropicElasticityDG3DIPVariable*>(ipvp);
        }

    /* Deformation gradient F */
    const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
    const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

    /* data for non local isotropic elastic law */
    IPNonLocalDamageIsotropicElasticity* q1 = ipvcur->getIPNonLocalDamageIsotropicElasticity();
    const IPNonLocalDamageIsotropicElasticity* q0 = ipvprev->getIPNonLocalDamageIsotropicElasticity();

    /* compute stress */
    _nldIsotropicElasticitylaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                                 ipvcur->getRefToDLocalVariableDStrain()[0],ipvcur->getRefToDStressDNonLocalVariable()[0],
                                 ipvcur->getRefToDLocalVariableDNonLocalVariable()(0,0),stiff);

    ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);   // Is it correct ????

}

//
//
NonLocalDamageJ2SmallStrainDG3DMaterialLaw::NonLocalDamageJ2SmallStrainDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,
                    const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol,
                    const bool matrixBypert, const double tolPert) :
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldJ2smallStrain = new mlawNonLocalDamageJ2SmallStrain(num,E,nu,rho,_j2IH,_cLLaw,_damLaw,tol,matrixBypert,tolPert);
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalDamageJ2SmallStrainDG3DMaterialLaw::~NonLocalDamageJ2SmallStrainDG3DMaterialLaw() {
	if (_nldJ2smallStrain!=NULL){
		delete _nldJ2smallStrain;
		_nldJ2smallStrain = NULL;
	}
};


NonLocalDamageJ2SmallStrainDG3DMaterialLaw::NonLocalDamageJ2SmallStrainDG3DMaterialLaw(const NonLocalDamageJ2SmallStrainDG3DMaterialLaw &source) :
                                                             dG3DMaterialLaw(source)
{
	_nldJ2smallStrain = NULL;
	if (source._nldJ2smallStrain != NULL){
		_nldJ2smallStrain = new mlawNonLocalDamageJ2SmallStrain(*(source._nldJ2smallStrain));
	}
}

void NonLocalDamageJ2SmallStrainDG3DMaterialLaw::setTime(const double t,const double dtime){
  dG3DMaterialLaw::setTime(t,dtime);
  _nldJ2smallStrain->setTime(t,dtime);
}

materialLaw::matname
NonLocalDamageJ2SmallStrainDG3DMaterialLaw::getType() const {return _nldJ2smallStrain->getType();}

void NonLocalDamageJ2SmallStrainDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;
  IPVariable* ipvi = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2smallStrain,inter);
  IPVariable* ipv1 = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2smallStrain,inter);
  IPVariable* ipv2 = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2smallStrain,inter);
  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
  _nldJ2smallStrain->createIPState((static_cast <nonLocalDamageJ2HyperDG3DIPVariable*> (ipvi))->getIPNonLocalDamageJ2Hyper(),
                         (static_cast <nonLocalDamageJ2HyperDG3DIPVariable*> (ipv1))->getIPNonLocalDamageJ2Hyper(),
             			 (static_cast <nonLocalDamageJ2HyperDG3DIPVariable*> (ipv2))->getIPNonLocalDamageJ2Hyper());

}

void NonLocalDamageJ2SmallStrainDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }
  ipv = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2smallStrain,inter);
  IPNonLocalDamageJ2Hyper * ipvnl = static_cast <nonLocalDamageJ2HyperDG3DIPVariable*>(ipv)->getIPNonLocalDamageJ2Hyper();
  _nldJ2smallStrain->createIPVariable(ipvnl);
}

double
NonLocalDamageJ2SmallStrainDG3DMaterialLaw::soundSpeed() const{return _nldJ2smallStrain->soundSpeed();} // or change value ??


void NonLocalDamageJ2SmallStrainDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  nonLocalDamageJ2HyperDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageJ2HyperDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvp);
  }

  _nldJ2smallStrain->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void NonLocalDamageJ2SmallStrainDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  nonLocalDamageJ2HyperDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageJ2HyperDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvp);
  }

  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPNonLocalDamageJ2Hyper* q1 = ipvcur->getIPNonLocalDamageJ2Hyper();
  const IPNonLocalDamageJ2Hyper* q0 = ipvprev->getIPNonLocalDamageJ2Hyper();

  STensor43& elasticL = ipvcur->getRefToElasticTangentModuli();

  /* compute stress */
  _nldJ2smallStrain->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                        ipvcur->getRefToDLocalVariableDStrain()[0],ipvcur->getRefToDStressDNonLocalVariable()[0],
                        ipvcur->getRefToDLocalVariableDNonLocalVariable()(0,0),stiff,&elasticL);

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);

}

//
NonLocalDamageJ2HyperDG3DMaterialLaw::NonLocalDamageJ2HyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,
                    const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol,
                    const bool matrixBypert, const double tolPert) :
                                                              dG3DMaterialLaw(num,rho,true)
{
  _nldJ2Hyperlaw = new mlawNonLocalDamageJ2Hyper(num,E,nu,rho,_j2IH,_cLLaw,_damLaw,tol,matrixBypert,tolPert);
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalDamageJ2HyperDG3DMaterialLaw::~NonLocalDamageJ2HyperDG3DMaterialLaw() {
	if (_nldJ2Hyperlaw!=NULL){
		delete _nldJ2Hyperlaw;
		_nldJ2Hyperlaw = NULL;
	}
};


NonLocalDamageJ2HyperDG3DMaterialLaw::NonLocalDamageJ2HyperDG3DMaterialLaw(const NonLocalDamageJ2HyperDG3DMaterialLaw &source) :
                                                             dG3DMaterialLaw(source)
{
	_nldJ2Hyperlaw = NULL;
	if (source._nldJ2Hyperlaw != NULL){
		_nldJ2Hyperlaw = new mlawNonLocalDamageJ2Hyper(*(source._nldJ2Hyperlaw));
	}
}

void NonLocalDamageJ2HyperDG3DMaterialLaw::setStrainOrder(const int order){
  _nldJ2Hyperlaw->setStrainOrder(order);
};

void
NonLocalDamageJ2HyperDG3DMaterialLaw::setTime(const double t,const double dtime){
  dG3DMaterialLaw::setTime(t,dtime);
  _nldJ2Hyperlaw->setTime(t,dtime);
}

materialLaw::matname
NonLocalDamageJ2HyperDG3DMaterialLaw::getType() const {return _nldJ2Hyperlaw->getType();}

void NonLocalDamageJ2HyperDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;
  IPVariable* ipvi = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2Hyperlaw,inter);
  IPVariable* ipv1 = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2Hyperlaw,inter);
  IPVariable* ipv2 = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2Hyperlaw,inter);
  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
  _nldJ2Hyperlaw->createIPState((static_cast <nonLocalDamageJ2HyperDG3DIPVariable*> (ipvi))->getIPNonLocalDamageJ2Hyper(),
                         (static_cast <nonLocalDamageJ2HyperDG3DIPVariable*> (ipv1))->getIPNonLocalDamageJ2Hyper(),
             			 (static_cast <nonLocalDamageJ2HyperDG3DIPVariable*> (ipv2))->getIPNonLocalDamageJ2Hyper());

}

void NonLocalDamageJ2HyperDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }
  ipv = new  nonLocalDamageJ2HyperDG3DIPVariable(*_nldJ2Hyperlaw,inter);
  IPNonLocalDamageJ2Hyper * ipvnl = static_cast <nonLocalDamageJ2HyperDG3DIPVariable*>(ipv)->getIPNonLocalDamageJ2Hyper();
  _nldJ2Hyperlaw->createIPVariable(ipvnl);
}

double
NonLocalDamageJ2HyperDG3DMaterialLaw::soundSpeed() const{return _nldJ2Hyperlaw->soundSpeed();} // or change value ??

void NonLocalDamageJ2HyperDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */
  nonLocalDamageJ2HyperDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageJ2HyperDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvp);
  }
  _nldJ2Hyperlaw->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};
void NonLocalDamageJ2HyperDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */
  nonLocalDamageJ2HyperDG3DIPVariable* ipvcur; //= static_cast < nonLocalDamageDG3DIPVariable *> (ipv);
  const nonLocalDamageJ2HyperDG3DIPVariable* ipvprev; //= static_cast <const  nonLocalDamageDG3DIPVariable *> (ipvp);

  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);
  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalDamageJ2HyperDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalDamageJ2HyperDG3DIPVariable*>(ipvp);
  }

  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPNonLocalDamageJ2Hyper* q1 = ipvcur->getIPNonLocalDamageJ2Hyper();
  const IPNonLocalDamageJ2Hyper* q0 = ipvprev->getIPNonLocalDamageJ2Hyper();

  STensor43& elasticL = ipvcur->getRefToElasticTangentModuli();

  /* compute stress */
  _nldJ2Hyperlaw->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                        ipvcur->getRefToDLocalVariableDStrain()[0],ipvcur->getRefToDStressDNonLocalVariable()[0],
                        ipvcur->getRefToDLocalVariableDNonLocalVariable()(0,0),stiff,&elasticL);

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);

}

//

NonLocalPorousDuctileDamageDG3DMaterialLaw::NonLocalPorousDuctileDamageDG3DMaterialLaw(const int num, const double rho, const bool init):
  dG3DMaterialLaw(num,rho,init), _nldPorous(NULL),_useI1J2J3Implementation(false){};
NonLocalPorousDuctileDamageDG3DMaterialLaw::NonLocalPorousDuctileDamageDG3DMaterialLaw(const NonLocalPorousDuctileDamageDG3DMaterialLaw &source):
  dG3DMaterialLaw(source),_imposedInitialPorosity(source._imposedInitialPorosity),linearK(source.linearK),Stiff_alphaDilatation(source.Stiff_alphaDilatation),
  _useI1J2J3Implementation(source._useI1J2J3Implementation)
{
  if (source._nldPorous != NULL){
    _nldPorous =dynamic_cast<mlawNonLocalPorosity*>(source._nldPorous->clone());
  }
  else{
    _nldPorous = NULL;
  }
};
NonLocalPorousDuctileDamageDG3DMaterialLaw::~NonLocalPorousDuctileDamageDG3DMaterialLaw(){
  if (_nldPorous!=NULL) {delete _nldPorous; _nldPorous = NULL;};
};


void NonLocalPorousDuctileDamageDG3DMaterialLaw::useI1J2J3Implementation(const bool fl){
  _useI1J2J3Implementation = fl;
  if (_useI1J2J3Implementation){
    Msg::Info("I1J2J3 implementation is used !!!!!!!!!!");
  }
}

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setStressFormulation(const int fm){
  _nldPorous->setStressFormulation(fm);
}

void NonLocalPorousDuctileDamageDG3DMaterialLaw::activateVoidNucleationDuringCoalescence(const bool fl){
  _nldPorous->activateVoidNucleationDuringCoalescence(fl);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setOrderForLogExp(const int newOrder){
  _nldPorous->setOrderForLogExp(newOrder);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setViscosityLaw(const viscosityLaw& visco){
  _nldPorous->setViscosityLaw(visco);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setNucleationLaw(const NucleationLaw& added_GdnLaw){
  _nldPorous->setNucleationLaw(added_GdnLaw);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setNucleationCriterion(const NucleationCriterionLaw& nuclCriterion){
  _nldPorous->setNucleationCriterion(nuclCriterion);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setScatterredInitialPorosity(const double fmin, const double fmax){
    _nldPorous->setScatterredInitialPorosity(fmin,fmax);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setCoalescenceLaw(const CoalescenceLaw& added_coalesLaw){
  _nldPorous->setCoalescenceLaw(added_coalesLaw);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setVoidEvolutionLaw(const voidStateEvolutionLaw& added_evolutionLaw){
  _nldPorous->setVoidEvolutionLaw(added_evolutionLaw);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setSubStepping(const bool fl, const int maxNumStep){
  _nldPorous->setSubStepping(fl,maxNumStep);
}

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setPostBlockedDissipationBehavior(const int method){
  _nldPorous->setPostBlockedDissipationBehavior(method);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setFailureTolerance(const double NewFailureTol){
  _nldPorous->setFailureTolerance(NewFailureTol);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setLocalFailurePorosity(const double newFvMax){
  _nldPorous->setLocalFailurePorosity(newFvMax);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setCorrectedRegularizedFunction(const scalarFunction& fct){
  _nldPorous->setCorrectedRegularizedFunction(fct);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setNonLocalMethod(const int i) {
  _nldPorous->setNonLocalMethod(i);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setShearPorosityGrowthFactor(const double k){
  Msg::Info("set shear contribution in porosity growth kw = %e",k);
  _nldPorous->setShearPorosityGrowthFactor(k);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setStressTriaxialityFunction_kw(const scalarFunction& fct){
  _nldPorous->setStressTriaxialityFunction_kw(fct);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setLodeFunction(const scalarFunction& fct){
  _nldPorous->setLodeFunction(fct);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::clearCLengthLaw(){
  _nldPorous->clearCLengthLaw();
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setCLengthLaw(const CLengthLaw& clength){
  _nldPorous->setCLengthLaw(clength);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setReferenceT(const double T){
  _nldPorous->setReferenceT(T);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setInitialPorosityOnElement(const int ele, const double fInit){
  Msg::Info("set initial porosity: ele = %d f0 = %e",ele,fInit);
  _imposedInitialPorosity[ele] = fInit;
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTemperatureFunction_rho(const scalarFunction& Tfunc)
{
  _nldPorous->setTemperatureFunction_rho(Tfunc);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTemperatureFunction_E(const scalarFunction& Tfunc)
{
  _nldPorous->setTemperatureFunction_E(Tfunc);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTemperatureFunction_nu(const scalarFunction& Tfunc)
{
  _nldPorous->setTemperatureFunction_nu(Tfunc);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTemperatureFunction_alp(const scalarFunction& Tfunc)
{
  _nldPorous->setTemperatureFunction_alp(Tfunc);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::setThermomechanicsCoupling(const bool fl){
  _nldPorous->setThermomechanicsCoupling(fl);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setThermalExpansionCoefficient(const double alp){
  _nldPorous->setThermalExpansionCoefficient(alp);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setReferenceThermalConductivity(const double KK){
  _nldPorous->setReferenceThermalConductivity(KK);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTemperatureFunction_ThermalConductivity(const scalarFunction& Tfunc){
  _nldPorous->setTemperatureFunction_ThermalConductivity(Tfunc);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setReferenceCp(const double Cp){
  _nldPorous->setReferenceCp(Cp);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTemperatureFunction_Cp(const scalarFunction& Tfunc){
  _nldPorous->setTemperatureFunction_Cp(Tfunc);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setThermalEstimationPreviousConfig(const bool flag){
  _nldPorous->setThermalEstimationPreviousConfig(flag);
};
void NonLocalPorousDuctileDamageDG3DMaterialLaw::setTaylorQuineyFactor(const double f){
  _nldPorous->setTaylorQuineyFactor(f);
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;
  double fvInit = _nldPorous->getInitialPorosity();
  std::map<int,double>::const_iterator it = _imposedInitialPorosity.find(ele->getNum());
  if (it != _imposedInitialPorosity.end()){
    fvInit = it->second;
  }

  int numNL=_nldPorous->getNumNonLocalVariables();
  int numExtraDof=0;
  if(_nldPorous->isThermomechanicallyCoupled())
    numExtraDof=1;

  IPVariable* ipvi = new  nonLocalPorosityDG3DIPVariable(*_nldPorous,fvInit,inter,numNL,numExtraDof);
  IPVariable* ipv1 = new  nonLocalPorosityDG3DIPVariable(*_nldPorous,fvInit,inter,numNL,numExtraDof);
  IPVariable* ipv2 = new  nonLocalPorosityDG3DIPVariable(*_nldPorous,fvInit,inter,numNL,numExtraDof);

  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
}

void NonLocalPorousDuctileDamageDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }
  double fvInit = _nldPorous->getInitialPorosity();
  std::map<int,double>::const_iterator it = _imposedInitialPorosity.find(ele->getNum());
  if (it != _imposedInitialPorosity.end()){
    fvInit = it->second;
  }
  int numNL=_nldPorous->getNumNonLocalVariables();
  int numExtraDof=0;
  if(_nldPorous->isThermomechanicallyCoupled())
    numExtraDof=1;
  
  if(ipv !=NULL) delete ipv;
  ipv = new  nonLocalPorosityDG3DIPVariable(*_nldPorous,fvInit,inter,numNL,numExtraDof);
}

void NonLocalPorousDuctileDamageDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */

  nonLocalPorosityDG3DIPVariable* ipvcur;
  const nonLocalPorosityDG3DIPVariable* ipvprev;
  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);

  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalPorosityDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalPorosityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalPorosityDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalPorosityDG3DIPVariable*>(ipvp);
  }
  _nldPorous->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::forceCoalescence(IPVariable*ipv, const IPVariable* ipvp) const
{
  nonLocalPorosityDG3DIPVariable* ipvcur;
  const nonLocalPorosityDG3DIPVariable* ipvprev;
  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);

  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalPorosityDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalPorosityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalPorosityDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalPorosityDG3DIPVariable*>(ipvp);
  }
  
   /* data for J2 law */
  IPNonLocalPorosity* q1 = ipvcur->getIPNonLocalPorosity();
  const IPNonLocalPorosity* q0 = ipvprev->getIPNonLocalPorosity();
  if(!_nldPorous->isThermomechanicallyCoupled())
  {
    _nldPorous->forceCoalescence(q1,q0,NULL);
  }
  else
  {
    double T = ipvcur->getConstRefToField(0);
    _nldPorous->forceCoalescence(q1,q0,&T);
  }
  
};

void NonLocalPorousDuctileDamageDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */

  nonLocalPorosityDG3DIPVariable* ipvcur;
  const nonLocalPorosityDG3DIPVariable* ipvprev;
  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);

  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalPorosityDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalPorosityDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalPorosityDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalPorosityDG3DIPVariable*>(ipvp);
  }


  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPNonLocalPorosity* q1 = ipvcur->getIPNonLocalPorosity();
  const IPNonLocalPorosity* q0 = ipvprev->getIPNonLocalPorosity();

  // set additional kinematic variables
	q1->setRefToCurrentOutwardNormal(ipvcur->getConstRefToCurrentOutwardNormal());
	q1->setRefToReferenceOutwardNormal(ipvcur->getConstRefToReferenceOutwardNormal());
  q1->setRefToDeformationGradient(ipvcur->getConstRefToDeformationGradient());


  /* compute stress */
  if(!_nldPorous->isThermomechanicallyCoupled())
  {
    if (_useI1J2J3Implementation){
      _nldPorous->I1J2J3_constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                          ipvcur->getRefToDLocalVariableDStrain(),ipvcur->getRefToDStressDNonLocalVariable(),
                          ipvcur->getRefToDLocalVariableDNonLocalVariable(),stiff);
    }
    else{
      if (_nldPorous->getNonLocalMethod() == mlawNonLocalPorosity::LOCAL){
        _nldPorous->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),stiff);
      }
      else {
        _nldPorous->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                          ipvcur->getRefToDLocalVariableDStrain(),ipvcur->getRefToDStressDNonLocalVariable(),
                          ipvcur->getRefToDLocalVariableDNonLocalVariable(),stiff);
      }
    }

  }
  else
  {
    // set temperature
    q1->setTemperature(ipvcur->getConstRefToField(0));
    if (_nldPorous->getNonLocalMethod() == mlawNonLocalPorosity::LOCAL){
      Msg::Error("local model for thermomechanics has not been implemented!!!");
    }
    else{
      _nldPorous->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,
                             ipvcur->getRefToTangentModuli(), ipvcur->getRefToDLocalVariableDStrain(),
                             ipvcur->getRefToDStressDNonLocalVariable(), ipvcur->getRefToDLocalVariableDNonLocalVariable(),
                             stiff,ipvcur->getRefTodPdField()[0],ipvcur->getRefTodPdGradField()[0],
                             ipvcur->getRefTodLocalVariableDExtraDofDiffusionField(),
                             ipvprev->getConstRefToField(0),ipvcur->getConstRefToField(0),// T0 and T
                             ipvprev->getConstRefToGradField()[0],ipvcur->getConstRefToGradField()[0],// gradT0 and gradT
                             ipvcur->getRefToFlux()[0],ipvcur->getRefTodFluxdF()[0],
                             ipvcur->getRefTodFluxdNonLocalVariable()[0], ipvcur->getRefTodFluxdField()[0][0],
                             ipvcur->getRefTodFluxdGradField()[0][0],// flux and derivatives
                             ipvcur->getRefToFieldSource()(0),
                             ipvcur->getRefTodFieldSourcedF()[0],ipvcur->getRefTodFieldSourcedNonLocalVariable(),
                             ipvcur->getRefTodFieldSourcedField()(0,0),ipvcur->getRefTodFieldSourcedGradField()[0][0],//thermal source
                             ipvcur->getRefToMechanicalSource()(0), // mechanical source
                             ipvcur->getRefTodMechanicalSourcedF()[0],
                             ipvcur->getRefTodMechanicalSourcedNonLocalVariable(),
                              ipvcur->getRefTodMechanicalSourcedField()(0,0),ipvcur->getRefTodMechanicalSourcedGradField()[0][0]
                             );
    }
    ipvcur->setRefToLinearConductivity(this->linearK);
    ipvcur->setRefToStiff_alphaDilatation(this->Stiff_alphaDilatation);
  };

  ipvcur->setRefToDGElasticTangentModuli(this->elasticStiffness);
};

//
NonLocalDamageGursonDG3DMaterialLaw::NonLocalDamageGursonDG3DMaterialLaw(const int num,const double rho,
                    const double E,const double nu,
                    const double q1, const double q2, const double q3, const double fVinitial,
                    const J2IsotropicHardening &_j2IH,
                    const double tol,
                    const bool matrixByPert, const double pert) :
      NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalDamageGurson(num,E,nu,rho,q1, q2, q3,
                                              fVinitial,_j2IH,zeroL,tol,matrixByPert,pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
}



NonLocalDamageGursonDG3DMaterialLaw::NonLocalDamageGursonDG3DMaterialLaw(const int num,const double rho,
                    const double E,const double nu,
                    const double q1, const double q2, const double q3, const double fVinitial,
                    const J2IsotropicHardening &_j2IH,
                    const CLengthLaw &_cLLaw, const double tol,
                    const bool matrixByPert, const double pert) :
   NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalDamageGurson(num,E,nu,rho,q1, q2, q3,
                                              fVinitial,_j2IH,_cLLaw,tol,matrixByPert,pert);
  fillElasticStiffness(E, nu, elasticStiffness);
}



NonLocalDamageGursonDG3DMaterialLaw::NonLocalDamageGursonDG3DMaterialLaw(const int num,const double rho,
                   const double E,const double nu,
                   const double q1, const double q2, const double q3, const double fVinitial,
                   const J2IsotropicHardening &_j2IH,
                   const CLengthLaw &_cLLaw, double TaylorQuineyFactor,
                   const double KK,const double cp,const double alp,
                   const double tol, const bool matrixByPert,const double pert) :
   NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{

  _nldPorous = new mlawNonLocalDamageGurson(num,E,nu,rho,q1, q2, q3,
                                              fVinitial,_j2IH,_cLLaw,tol,matrixByPert,pert);
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
}


NonLocalDamageGursonDG3DMaterialLaw::NonLocalDamageGursonDG3DMaterialLaw(const NonLocalDamageGursonDG3DMaterialLaw &source) :
  NonLocalPorousDuctileDamageDG3DMaterialLaw(source)
{

}

NonLocalDamageGursonDG3DMaterialLaw::~NonLocalDamageGursonDG3DMaterialLaw(){}


void NonLocalDamageGursonDG3DMaterialLaw::setTemperatureFunction_q1(const scalarFunction& Tfunc)
{
  static_cast<mlawNonLocalDamageGurson*>(_nldPorous)->setTemperatureFunction_q1(Tfunc);
};

void NonLocalDamageGursonDG3DMaterialLaw::setTemperatureFunction_q2(const scalarFunction& Tfunc)
{
  static_cast<mlawNonLocalDamageGurson*>(_nldPorous)->setTemperatureFunction_q2(Tfunc);
};

void NonLocalDamageGursonDG3DMaterialLaw::setTemperatureFunction_q3(const scalarFunction& Tfunc)
{
  static_cast<mlawNonLocalDamageGurson*>(_nldPorous)->setTemperatureFunction_q3(Tfunc);
};

NonLocalPorousThomasonDG3DMaterialLaw::NonLocalPorousThomasonDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalPorousThomasonLaw(num,E,nu,rho,fVinitial,lambda0,j2IH,zeroL,tol,matrixByPert,pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
}


NonLocalPorousThomasonDG3DMaterialLaw::NonLocalPorousThomasonDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousThomasonLaw(num,E,nu,rho,fVinitial,lambda0,j2IH,cLLaw,tol,matrixByPert,pert);
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousThomasonDG3DMaterialLaw::NonLocalPorousThomasonDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousThomasonLaw(num,E,nu,rho,fVinitial,lambda0,j2IH,cLLaw,tol,matrixByPert,pert);
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
}



NonLocalPorousThomasonDG3DMaterialLaw::NonLocalPorousThomasonDG3DMaterialLaw(const NonLocalPorousThomasonDG3DMaterialLaw &source) :
                                                             NonLocalPorousDuctileDamageDG3DMaterialLaw(source)
{

}

NonLocalPorousThomasonDG3DMaterialLaw::~NonLocalPorousThomasonDG3DMaterialLaw(){

}
void NonLocalPorousThomasonDG3DMaterialLaw::setYieldSurfaceExponent(const double newN)
{
  static_cast<mlawNonLocalPorousThomasonLaw*>(_nldPorous)->setYieldSurfaceExponent(newN);
};

NonLocalPorousThomasonWithMPSDG3DMaterialLaw::NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalPorousThomasonLawWithMPS(num,E,nu,rho,fVinitial,lambda0,j2IH,zeroL,tol,matrixByPert,pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}


NonLocalPorousThomasonWithMPSDG3DMaterialLaw::NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousThomasonLawWithMPS(num,E,nu,rho,fVinitial,lambda0,j2IH,cLLaw,tol,matrixByPert,pert);
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}

NonLocalPorousThomasonWithMPSDG3DMaterialLaw::NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousThomasonLawWithMPS(num,E,nu,rho,fVinitial,lambda0,j2IH,cLLaw,tol,matrixByPert,pert);
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}



NonLocalPorousThomasonWithMPSDG3DMaterialLaw::NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const NonLocalPorousThomasonWithMPSDG3DMaterialLaw &source) :
                                                             NonLocalPorousDuctileDamageDG3DMaterialLaw(source)
{

}

NonLocalPorousThomasonWithMPSDG3DMaterialLaw::~NonLocalPorousThomasonWithMPSDG3DMaterialLaw(){

}
void NonLocalPorousThomasonWithMPSDG3DMaterialLaw::setYieldSurfaceExponent(const double newN)
{
  static_cast<mlawNonLocalPorousThomasonLaw*>(_nldPorous)->setYieldSurfaceExponent(newN);
};

//
NonLocalPorousThomasonWithMSSDG3DMaterialLaw::NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalPorousThomasonLawWithMSS(num,E,nu,rho,fVinitial,lambda0,j2IH,zeroL,tol,matrixByPert,pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}


NonLocalPorousThomasonWithMSSDG3DMaterialLaw::NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousThomasonLawWithMSS(num,E,nu,rho,fVinitial,lambda0,j2IH,cLLaw,tol,matrixByPert,pert);
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}

NonLocalPorousThomasonWithMSSDG3DMaterialLaw::NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol, const bool matrixByPert, const double pert):
                                                              NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousThomasonLawWithMSS(num,E,nu,rho,fVinitial,lambda0,j2IH,cLLaw,tol,matrixByPert,pert);
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}



NonLocalPorousThomasonWithMSSDG3DMaterialLaw::NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const NonLocalPorousThomasonWithMSSDG3DMaterialLaw &source) :
                                                             NonLocalPorousDuctileDamageDG3DMaterialLaw(source)
{

}

NonLocalPorousThomasonWithMSSDG3DMaterialLaw::~NonLocalPorousThomasonWithMSSDG3DMaterialLaw(){

}

//

NonLocalPorousCoupledDG3DMaterialLaw::NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalPorousCoupledLaw(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0,
                                                    j2IH, zeroL, tol, matrixbyPerturbation, pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
}

//
NonLocalPorousCoupledDG3DMaterialLaw::NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousCoupledLaw(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  fillElasticStiffness(E, nu, elasticStiffness);
}

//
NonLocalPorousCoupledDG3DMaterialLaw::NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousCoupledLaw(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0, kappa,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousCoupledDG3DMaterialLaw::NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousDuctileDamageDG3DMaterialLaw(num,rho,true)
{
  _nldPorous = new mlawNonLocalPorousCoupledLaw(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0, kappa,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousCoupledDG3DMaterialLaw::NonLocalPorousCoupledDG3DMaterialLaw(const NonLocalPorousCoupledDG3DMaterialLaw &source) :
                                                             NonLocalPorousDuctileDamageDG3DMaterialLaw(source)
{

}

NonLocalPorousCoupledDG3DMaterialLaw::~NonLocalPorousCoupledDG3DMaterialLaw(){

}

void NonLocalPorousCoupledDG3DMaterialLaw::setYieldSurfaceExponent(const double newN){
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setYieldSurfaceExponent(newN);
};

void NonLocalPorousCoupledDG3DMaterialLaw::setYieldOffset(const bool fl){
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setYieldOffset(fl);
};

void NonLocalPorousCoupledDG3DMaterialLaw::setBulkCouplingBehaviour(const bool fl)
{
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setBulkCouplingBehaviour(fl);
}

void NonLocalPorousCoupledDG3DMaterialLaw::setCftOffset(const bool fl){
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setCftOffset(fl);
};

void NonLocalPorousCoupledDG3DMaterialLaw::setLodeOffset(const bool fl){
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setLodeOffset(fl);
}
void NonLocalPorousCoupledDG3DMaterialLaw::useTwoYieldRegularization(const bool fl, const double n)
{
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->useTwoYieldRegularization(fl,n);
}

void NonLocalPorousCoupledDG3DMaterialLaw::setCrackTransition(const bool fl, const double beta){
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setCrackTransition(fl,beta);
}

void NonLocalPorousCoupledDG3DMaterialLaw::setCoalescenceTolerance(const double t){
	static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setCoalescenceTolerance(t);
};

void NonLocalPorousCoupledDG3DMaterialLaw::setGrowthVoidEvolutionLaw(const voidStateEvolutionLaw& voidtateLaw)
{
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setGrowthVoidEvolutionLaw(voidtateLaw);
};
void NonLocalPorousCoupledDG3DMaterialLaw::setCoalescenceVoidEvolutionLaw(const voidStateEvolutionLaw& voidtateLaw)
{
  static_cast<mlawNonLocalPorousCoupledLaw*>(_nldPorous)->setCoalescenceVoidEvolutionLaw(voidtateLaw);
};


NonLocalPorousCoupledWithMPSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,j2IH,tol,matrixbyPerturbation,pert)
{
  
  if (_nldPorous != NULL) delete _nldPorous;
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0,
                                                    j2IH, zeroL, tol, matrixbyPerturbation, pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}

//
NonLocalPorousCoupledWithMPSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,j2IH,cLLaw,tol,matrixbyPerturbation,pert)
{
  if (_nldPorous != NULL) delete _nldPorous;
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}

//
NonLocalPorousCoupledWithMPSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,kappa,j2IH,cLLaw,tol,matrixbyPerturbation,pert)
{
  if (_nldPorous != NULL) delete _nldPorous;
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0, kappa,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  _useI1J2J3Implementation = true;
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousCoupledWithMPSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,kappa,j2IH,cLLaw,TaylorQuineyFactor,KK,cp,alp,tol,matrixbyPerturbation,pert)
{
  if (_nldPorous != NULL) delete _nldPorous;
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0, kappa,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  _useI1J2J3Implementation = true;
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousCoupledWithMPSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const NonLocalPorousCoupledWithMPSDG3DMaterialLaw &source) :
                                                             NonLocalPorousCoupledDG3DMaterialLaw(source)
{

}

NonLocalPorousCoupledWithMPSDG3DMaterialLaw::~NonLocalPorousCoupledWithMPSDG3DMaterialLaw(){

};

//
NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,j2IH,tol,matrixbyPerturbation,pert)
{
  
  if (_nldPorous != NULL) delete _nldPorous;
  ZeroCLengthLaw zeroL(num);
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPSAndMSS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0,
                                                    j2IH, zeroL, tol, matrixbyPerturbation, pert);
  _nldPorous->setNonLocalMethod(0); // local model
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}

//
NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,j2IH,cLLaw,tol,matrixbyPerturbation,pert)
{
  if (_nldPorous != NULL) delete _nldPorous;
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPSAndMSS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  fillElasticStiffness(E, nu, elasticStiffness);
  _useI1J2J3Implementation = true;
}

//
NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,kappa,j2IH,cLLaw,tol,matrixbyPerturbation,pert)
{
  if (_nldPorous != NULL) delete _nldPorous;
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPSAndMSS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0, kappa,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  _useI1J2J3Implementation = true;
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol, const bool matrixbyPerturbation, const double pert):
                                NonLocalPorousCoupledDG3DMaterialLaw(num,E,nu,rho,q1,q2,q3,fVinitial,lambda0,kappa,j2IH,cLLaw,TaylorQuineyFactor,KK,cp,alp,tol,matrixbyPerturbation,pert)
{
  if (_nldPorous != NULL) delete _nldPorous;
  _nldPorous = new mlawNonLocalPorousCoupledLawWithMPSAndMSS(num, E, nu, rho, q1, q2, q3, fVinitial, lambda0, kappa,
                                                    j2IH, cLLaw, tol, matrixbyPerturbation, pert);
  _useI1J2J3Implementation = true;
  _nldPorous->setThermomechanicsCoupling(true);
  _nldPorous->setTaylorQuineyFactor(TaylorQuineyFactor);
  _nldPorous->setReferenceThermalConductivity(KK);
  _nldPorous->setReferenceCp(cp);
  _nldPorous->setThermalExpansionCoefficient(alp);
  //
  Stiff_alphaDilatation=_nldPorous->getStiff_alphaDilatation();
  linearK  = _nldPorous->getInitialConductivityTensor();
  fillElasticStiffness(E, nu, elasticStiffness);
}

NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw &source) :
                                                             NonLocalPorousCoupledDG3DMaterialLaw(source)
{

}

NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::~NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(){

}

void NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::setShearFactor(double g)
{
  static_cast<mlawNonLocalPorousCoupledLawWithMPSAndMSS*>(_nldPorous)->setShearFactor(g);
}

void NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::setVoidLigamentExponent(double e)
{
  static_cast<mlawNonLocalPorousCoupledLawWithMPSAndMSS*>(_nldPorous)->setVoidLigamentExponent(e);
}

void NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::freezeCoalescenceState(const bool fl)
{
  static_cast<mlawNonLocalPorousCoupledLawWithMPSAndMSS*>(_nldPorous)->freezeCoalescenceState(fl);
}

void NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::setNeckingCoalescenceLaw(const CoalescenceLaw& added_coalsLaw)
{
  static_cast<mlawNonLocalPorousCoupledLawWithMPSAndMSS*>(_nldPorous)->setNeckingCoalescenceLaw(added_coalsLaw);
};
void NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::setShearCoalescenceLaw(const CoalescenceLaw& added_coalsLaw)
{
  static_cast<mlawNonLocalPorousCoupledLawWithMPSAndMSS*>(_nldPorous)->setShearCoalescenceLaw(added_coalsLaw);
};
void NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw::setShearVoidEvolutionLaw(const voidStateEvolutionLaw& voidtateLaw)
{
  static_cast<mlawNonLocalPorousCoupledLawWithMPSAndMSS*>(_nldPorous)->setShearVoidEvolutionLaw(voidtateLaw);
};


NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(const int num, NonLocalPorousDuctileDamageDG3DMaterialLaw& law,
  const CLengthLaw &cLLaw,const DamageLaw &damLaw,double sigmac):
  dG3DMaterialLaw(num,law.density(),true){
  _nlPorousWithCleavage = new mlawNonLocalPorousWithCleavageFailure(*law.getNonLocalPorosityLaw(),cLLaw,damLaw,sigmac);
};

NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(const int num, NonLocalPorousDuctileDamageDG3DMaterialLaw& law,
  const CLengthLaw &cLLaw,const DamageLaw &damLaw, const FailureCriterionBase& failureLaw):
  dG3DMaterialLaw(num,law.density(),true){
  _nlPorousWithCleavage = new mlawNonLocalPorousWithCleavageFailure(*law.getNonLocalPorosityLaw(),cLLaw,damLaw,failureLaw);
};

NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(const NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw& src):
dG3DMaterialLaw(src){
  if (src._nlPorousWithCleavage != NULL){
    _nlPorousWithCleavage = dynamic_cast<mlawNonLocalPorousWithCleavageFailure*>(src._nlPorousWithCleavage->clone());
  }
  else{
    _nlPorousWithCleavage = NULL;
  }
};

NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::~NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(){
  if (_nlPorousWithCleavage !=NULL){delete _nlPorousWithCleavage; _nlPorousWithCleavage = NULL;};
};

void NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::createIPState(IPStateBase* &ips,const bool* state_,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  // check interface element
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele==NULL) inter=false;
  double fvInit = _nlPorousWithCleavage->getInitialPorosity();

  int numNL=_nlPorousWithCleavage->getNumNonLocalVariables();
  int numExtraDof=0;

  IPVariable* ipvi = new  nonLocalPorosityWithCleavageDG3DIPVariable(*_nlPorousWithCleavage,fvInit,inter,numNL,numExtraDof);
  IPVariable* ipv1 = new  nonLocalPorosityWithCleavageDG3DIPVariable(*_nlPorousWithCleavage,fvInit,inter,numNL,numExtraDof);
  IPVariable* ipv2 = new  nonLocalPorosityWithCleavageDG3DIPVariable(*_nlPorousWithCleavage,fvInit,inter,numNL,numExtraDof);

  if(ips != NULL) delete ips;
  ips = new IP3State(state_,ipvi,ipv1,ipv2);
}

void NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const
{
  if(ipv !=NULL) delete ipv;
  bool inter=true;
  const MInterfaceElement *iele = dynamic_cast<const MInterfaceElement*>(ele);
  if(iele == NULL){
    inter=false;
  }
  double fvInit = _nlPorousWithCleavage->getInitialPorosity();
  int numNL=_nlPorousWithCleavage->getNumNonLocalVariables();
  int numExtraDof=0;

  ipv = new  nonLocalPorosityWithCleavageDG3DIPVariable(*_nlPorousWithCleavage,fvInit,inter,numNL,numExtraDof);

}

void NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const{
  /* get ipvariable */

  nonLocalPorosityWithCleavageDG3DIPVariable* ipvcur;
  const nonLocalPorosityWithCleavageDG3DIPVariable* ipvprev;
  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);

  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalPorosityWithCleavageDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalPorosityWithCleavageDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalPorosityWithCleavageDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalPorosityWithCleavageDG3DIPVariable*>(ipvp);
  }
  _nlPorousWithCleavage->checkInternalState(ipvcur->getInternalData(),ipvprev->getInternalData());
};

void NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw::stress(IPVariable* ipv, const IPVariable* ipvp, const bool stiff, const bool checkfrac)
{
  /* get ipvariable */

  nonLocalPorosityWithCleavageDG3DIPVariable* ipvcur;
  const nonLocalPorosityWithCleavageDG3DIPVariable* ipvprev;
  FractureCohesive3DIPVariable* ipvtmp = dynamic_cast<FractureCohesive3DIPVariable*>(ipv);

  if(ipvtmp !=NULL)
  {

    ipvcur = static_cast<nonLocalPorosityWithCleavageDG3DIPVariable*>(ipvtmp->getIPvBulk());
    const FractureCohesive3DIPVariable *ipvtmp2 = static_cast<const FractureCohesive3DIPVariable*>(ipvp);
    ipvprev = static_cast<const nonLocalPorosityWithCleavageDG3DIPVariable*>(ipvtmp2->getIPvBulk());
   }
  else
  {
    ipvcur = static_cast<nonLocalPorosityWithCleavageDG3DIPVariable*>(ipv);
    ipvprev = static_cast<const nonLocalPorosityWithCleavageDG3DIPVariable*>(ipvp);
  }


  /* strain xyz */
  const STensor3& F1 = ipvcur->getConstRefToDeformationGradient();
  const STensor3& F0 = ipvprev->getConstRefToDeformationGradient();

  /* data for J2 law */
  IPNonLocalPorosityWithCleavage* q1 = ipvcur->getIPNonLocalPorosityWithCleavage();
  const IPNonLocalPorosityWithCleavage* q0 = ipvprev->getIPNonLocalPorosityWithCleavage();

  // set additional kinematic variables
	q1->setRefToCurrentOutwardNormal(ipvcur->getConstRefToCurrentOutwardNormal());
	q1->setRefToReferenceOutwardNormal(ipvcur->getConstRefToReferenceOutwardNormal());
  q1->setRefToDeformationGradient(ipvcur->getConstRefToDeformationGradient());


  /* compute stress */

  if (_nlPorousWithCleavage->getNonLocalMethod() == mlawNonLocalPorosity::LOCAL){
    _nlPorousWithCleavage->getNonLocalPorosityLaw()->constitutive(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),stiff);
  }
  else {
    _nlPorousWithCleavage->predictorCorrector(F0,F1,ipvcur->getRefToFirstPiolaKirchhoffStress(),q0,q1,ipvcur->getRefToTangentModuli(),
                      ipvcur->getRefToDLocalVariableDStrain(),ipvcur->getRefToDStressDNonLocalVariable(),
                      ipvcur->getRefToDLocalVariableDNonLocalVariable(),stiff);
  }


};
