//
//
// Description: Class of materials for non linear dg
//
//
// Author:  <L. Noels>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _NONLOCALDAMAGEDG3DMATERIALLAW_H_
#define _NONLOCALDAMAGEDG3DMATERIALLAW_H_

#include "dG3DMaterialLaw.h"
#include "scalarFunction.h"
#ifndef SWIG
#include "NucleationLaw.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include "mlawHyperelasticDamage.h"
#include "mlawNonLocalDamage.h"
#include "mlawNonLocalDamage_Stoch.h"
#include "nonLocalDamageLaw.h"
#include "mlawNonLocalDamageJ2Hyper.h"
#include "mlawNonLocalDamageJ2SmallStrain.h"
#include "mlawNonLocalDamageIsotropicElasticity.h"
#include "mlawNonLocalDamageGurson.h"
#include "mlawNonLocalPorousCoalescence.h"
#include "mlawNonLocalPorousCoupled.h"
#include "mlawNonLocalPorousWithFailure.h"
#endif

class virtualNonLocalDamageDG3DMaterialLaw : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    dG3DMaterialLaw* _lawBase;
    int numNonlocalVar;
    std::vector<STensor3> characteristicLength;
    #endif //SWIG

  public:
    virtualNonLocalDamageDG3DMaterialLaw(const int num, const dG3DMaterialLaw& base, const int numNLVariable);
    void setNonLocalLengthScale(const int index, const CLengthLaw &cLLaw);
    #ifndef SWIG
    virtualNonLocalDamageDG3DMaterialLaw(const virtualNonLocalDamageDG3DMaterialLaw &source);
    virtual ~virtualNonLocalDamageDG3DMaterialLaw();
    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime){
      dG3DMaterialLaw::setTime(t,dtime);
      if (_lawBase){
        _lawBase->setTime(t,dtime);
      }
    };
    virtual materialLaw::matname getType() const {return _lawBase->getType();};

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){ _lawBase->initLaws(maplaw);};
    virtual double soundSpeed() const{return _lawBase->soundSpeed();};
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual double scaleFactor() const{return _lawBase->scaleFactor();};
    virtual bool isHighOrder() const { return _lawBase->isHighOrder();};
    virtual bool isNumeric() const {return _lawBase->isNumeric();};
    virtual int getNumberOfExtraDofsDiffusion() const {return _lawBase->getNumberOfExtraDofsDiffusion();}
    virtual double getInitialExtraDofStoredEnergyPerUnitField() const {return _lawBase->getInitialExtraDofStoredEnergyPerUnitField();}
    virtual double getExtraDofStoredEnergyPerUnitField(double T) const {return _lawBase->getExtraDofStoredEnergyPerUnitField(T);}
    virtual double getCharacteristicLength() const {return _lawBase->getCharacteristicLength();};
    virtual materialLaw* clone() const{ return new virtualNonLocalDamageDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const {return _lawBase->withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      if (_lawBase != NULL){
        _lawBase->setMacroSolver(sv);
      }
    }
    virtual bool brokenCheck(const IPVariable* ipv) const{return _lawBase->brokenCheck(ipv);};
    #endif
};

class NonLocalDamageDG3DMaterialLaw : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    materialLawWithDamage *_nldlaw; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)
    #endif //SWIG

  public:
    NonLocalDamageDG3DMaterialLaw(const int num, const double rho,
                                const char *propName);
    #ifndef SWIG
    NonLocalDamageDG3DMaterialLaw(const NonLocalDamageDG3DMaterialLaw &source);
    virtual ~NonLocalDamageDG3DMaterialLaw();

    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const;
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const;// or change value ??
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual double scaleFactor() const;
    virtual materialLaw* clone() const{return new NonLocalDamageDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const;
    virtual void setMacroSolver(const nonLinearMechSolver* sv);
    #endif
};


class NonLocalDamageDG3DMaterialLaw_Stoch : public NonLocalDamageDG3DMaterialLaw{
  public:
    NonLocalDamageDG3DMaterialLaw_Stoch(const int num, const double rho, const char *propName,
                        const double Ori_x, const double Ori_y, const double Ori_z,
                        const double Lx,const double Ly,const double Lz, 
                        const double dx,const double dy, const double dz, const double euler0= 0.0, const double euler1= 0.0, 
                        const double euler2= 0.0, const char *Geo=NULL, const char *RandProp=NULL);

    NonLocalDamageDG3DMaterialLaw_Stoch(const int num, const double rho, const char *propName,
                        const double Ori_x, const double Ori_y, const double Ori_z,
                        const double Lx,const double Ly,const double Lz,
                        const char *RandProp, const int intpl);


    #ifndef SWIG
    NonLocalDamageDG3DMaterialLaw_Stoch(const NonLocalDamageDG3DMaterialLaw_Stoch &source);
    virtual ~NonLocalDamageDG3DMaterialLaw_Stoch();
    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual materialLaw* clone() const{return new NonLocalDamageDG3DMaterialLaw_Stoch(*this);};
    #endif
};


class NonLocalDamageHyperelasticDG3DMaterialLaw: public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
		mlawNonlocalDamageHyperelastic* _nlLaw;
    #endif

  public:
		NonLocalDamageHyperelasticDG3DMaterialLaw(const int num, const double rho, const elasticPotential& elP, const CLengthLaw &cLLaw,
                                         const DamageLaw &damLaw);
		#ifndef SWIG
		NonLocalDamageHyperelasticDG3DMaterialLaw(const NonLocalDamageHyperelasticDG3DMaterialLaw& src);
		virtual ~NonLocalDamageHyperelasticDG3DMaterialLaw(){
      if (_nlLaw) delete _nlLaw;
    };
		virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const {return _nlLaw->getType();};

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw) {}
    virtual double soundSpeed() const {return _nlLaw->soundSpeed();};
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
		virtual double scaleFactor() const {return _nlLaw->scaleFactor();}
		virtual materialLaw* clone() const{return new NonLocalDamageHyperelasticDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const {return _nlLaw->withEnergyDissipation();};
		virtual void setMacroSolver(const nonLinearMechSolver* sv){
			dG3DMaterialLaw::setMacroSolver(sv);
			_nlLaw->setMacroSolver(sv);
		};
		#endif
};

class NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw: public NonLocalDamageHyperelasticDG3DMaterialLaw{
  public:
		NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw(const int num, const double rho,
                                        const elasticPotential& elP, const CLengthLaw &cLLaw,
                                         const DamageLaw &damLaw);
		#ifndef SWIG
		NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw(const NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw& src): NonLocalDamageHyperelasticDG3DMaterialLaw(src){};
		virtual ~NonLocalAnisotropicDamageHyperelasticDG3DMaterialLaw(){
    };
    #endif //SWIG
};

class NonLocalDamageIsotropicElasticityDG3DMaterialLaw : public dG3DMaterialLaw
{
  protected:
    #ifndef SWIG
    mlawNonLocalDamageIsotropicElasticity * _nldIsotropicElasticitylaw;
    #endif //SWIG

  public:
    NonLocalDamageIsotropicElasticityDG3DMaterialLaw(const int num,const double rho,
                                         double E,const double nu, const CLengthLaw &cLLaw,
                                         const DamageLaw &damLaw);
    //  specific functions
    virtual void setNonLocalLimitingMethod(const int num);
    #ifndef SWIG
    NonLocalDamageIsotropicElasticityDG3DMaterialLaw(const NonLocalDamageIsotropicElasticityDG3DMaterialLaw &source);
    virtual ~NonLocalDamageIsotropicElasticityDG3DMaterialLaw();
    virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const;

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw) {}
    virtual double soundSpeed() const;
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
		virtual double scaleFactor() const {return _nldIsotropicElasticitylaw->scaleFactor();}
		virtual materialLaw* clone() const{return new NonLocalDamageIsotropicElasticityDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const {return _nldIsotropicElasticitylaw->withEnergyDissipation();};
		virtual void setMacroSolver(const nonLinearMechSolver* sv){
			dG3DMaterialLaw::setMacroSolver(sv);
			if (_nldIsotropicElasticitylaw != NULL){
				_nldIsotropicElasticitylaw->setMacroSolver(sv);
			}
		};
    #endif // SWIG
};

class NonLocalDamageJ2SmallStrainDG3DMaterialLaw : public dG3DMaterialLaw
{
  protected:
    #ifndef SWIG
    mlawNonLocalDamageJ2SmallStrain *_nldJ2smallStrain;
    #endif //SWIG
  public:
    NonLocalDamageJ2SmallStrainDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,
                   const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol=1.e-6,
                   const bool matrixBypert = false, const double tolpert=1e-8);
    #ifndef SWIG
    NonLocalDamageJ2SmallStrainDG3DMaterialLaw(const NonLocalDamageJ2SmallStrainDG3DMaterialLaw &source);
    virtual ~NonLocalDamageJ2SmallStrainDG3DMaterialLaw();
    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const;

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const;
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual double scaleFactor() const{return _nldJ2smallStrain->scaleFactor();};
    virtual materialLaw* clone() const{ return new NonLocalDamageJ2SmallStrainDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const {return _nldJ2smallStrain->withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      if (_nldJ2smallStrain != NULL){
        _nldJ2smallStrain->setMacroSolver(sv);
      }
    }
    #endif
};


class NonLocalDamageJ2HyperDG3DMaterialLaw : public dG3DMaterialLaw
{
  protected:
    #ifndef SWIG
    mlawNonLocalDamageJ2Hyper *_nldJ2Hyperlaw;
    #endif //SWIG
  public:
    NonLocalDamageJ2HyperDG3DMaterialLaw(const int num,const double rho,
                   double E,const double nu,const J2IsotropicHardening &_j2IH,
                   const CLengthLaw &_cLLaw,const DamageLaw &_damLaw,const double tol=1.e-6,
                   const bool matrixBypert = false, const double tolpert=1e-8);
    void setStrainOrder(const int order);
    #ifndef SWIG
    NonLocalDamageJ2HyperDG3DMaterialLaw(const NonLocalDamageJ2HyperDG3DMaterialLaw &source);
    virtual ~NonLocalDamageJ2HyperDG3DMaterialLaw();
    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime);
    virtual materialLaw::matname getType() const;

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const;
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual double scaleFactor() const{return _nldJ2Hyperlaw->scaleFactor();};
    virtual materialLaw* clone() const{ return new NonLocalDamageJ2HyperDG3DMaterialLaw(*this);};
    virtual bool withEnergyDissipation() const {return _nldJ2Hyperlaw->withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      if (_nldJ2Hyperlaw != NULL){
        _nldJ2Hyperlaw->setMacroSolver(sv);
      }
    }
    #endif
};

class NonLocalPorousDuctileDamageDG3DMaterialLaw : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    mlawNonLocalPorosity *_nldPorous; // pointer to allow to choose between LLN style or Gmsh Style. The choice is performed by the constructor (2 constructors with != arguments)
    std::map<int, double> _imposedInitialPorosity;
    STensor3              linearK;
    STensor3		  Stiff_alphaDilatation;
    bool _useI1J2J3Implementation;
    #endif //SWIG

  public:
    #ifndef SWIG
    NonLocalPorousDuctileDamageDG3DMaterialLaw(const int num, const double rho, const bool init = true);
    NonLocalPorousDuctileDamageDG3DMaterialLaw(const NonLocalPorousDuctileDamageDG3DMaterialLaw &source);
    virtual ~NonLocalPorousDuctileDamageDG3DMaterialLaw();

    mlawNonLocalPorosity* getNonLocalPorosityLaw() {return _nldPorous;};
    const mlawNonLocalPorosity* getNonLocalPorosityLaw () const {return _nldPorous;};
    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime){
      dG3DMaterialLaw::setTime(t,dtime);
      _nldPorous->setTime(t,dtime);
    }
    virtual materialLaw::matname getType() const {return _nldPorous->getType();}

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const{return _nldPorous->soundSpeed();} // or change value ??
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;

    virtual double scaleFactor() const {return _nldPorous->scaleFactor();}

    virtual bool withEnergyDissipation() const {return _nldPorous->withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      if (_nldPorous != NULL){
        _nldPorous->setMacroSolver(sv);
      }
    }
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual void forceCoalescence(IPVariable*ipv, const IPVariable*ipvprev) const;
    virtual materialLaw* clone() const  = 0;
    #endif // SWIG
    void setStressFormulation(const int fm);
    void activateVoidNucleationDuringCoalescence(const bool fl);
    void useI1J2J3Implementation(const bool fl);
    void setOrderForLogExp(const int newOrder);
    void setViscosityLaw(const viscosityLaw& vico);
    void setNucleationLaw(const NucleationLaw& added_GdnLaw);
    void setNucleationCriterion(const NucleationCriterionLaw& nuclCriterion);
    void setScatterredInitialPorosity(const double fmin, const double fmax);
    void setCoalescenceLaw(const CoalescenceLaw& added_coalesLaw);
    void setVoidEvolutionLaw(const voidStateEvolutionLaw& added_evolutionLaw);
    void setSubStepping(const bool fl, const int maxNumStep);
    void setPostBlockedDissipationBehavior(const int method);
    void setFailureTolerance(const double NewFailureTol);
    void setLocalFailurePorosity(const double newFvMax);
    void setCorrectedRegularizedFunction(const scalarFunction& fct);
    void setNonLocalMethod(const int i);
    void setShearPorosityGrowthFactor(const double k);
    void setStressTriaxialityFunction_kw(const scalarFunction& fct);
    void setLodeFunction(const scalarFunction& fct);
    void clearCLengthLaw();
    void setCLengthLaw(const CLengthLaw& clength);
    void setInitialPorosityOnElement(const int ele, const double fInit);

    // temperature dependence
    void setReferenceT(const double T);
    void setTemperatureFunction_rho(const scalarFunction& Tfunc);
    void setTemperatureFunction_E(const scalarFunction& Tfunc);
    void setTemperatureFunction_nu(const scalarFunction& Tfunc);
    void setTemperatureFunction_alp(const scalarFunction& Tfunc);
    void setThermomechanicsCoupling(const bool fl);
    void setThermalExpansionCoefficient(const double alp);
    void setReferenceThermalConductivity(const double KK);
    void setTemperatureFunction_ThermalConductivity(const scalarFunction& Tfunc);
    void setReferenceCp(const double Cp);
    void setTemperatureFunction_Cp(const scalarFunction& Tfunc);
    void setThermalEstimationPreviousConfig(const bool flag);
    void setTaylorQuineyFactor(const double f);
};


class NonLocalDamageGursonDG3DMaterialLaw : public NonLocalPorousDuctileDamageDG3DMaterialLaw
{


public:
    NonLocalDamageGursonDG3DMaterialLaw(const int num,const double rho,
                   const double E,const double nu,
                   const double q1, const double q2, const double q3, const double fVinitial,
                   const J2IsotropicHardening &_j2IH,
                   const double tol=1.e-6, const bool matrixByPert=false, const double pert = 1e-8);

    NonLocalDamageGursonDG3DMaterialLaw(const int num,const double rho,
                   const double E,const double nu,
                   const double q1, const double q2, const double q3, const double fVinitial,
                   const J2IsotropicHardening &_j2IH,
                   const CLengthLaw &_cLLaw, const double tol=1.e-6, const bool matrixByPert=false, const double pert = 1e-8);

    NonLocalDamageGursonDG3DMaterialLaw(const int num,const double rho,
                   const double E,const double nu,
                   const double q1, const double q2, const double q3, const double fVinitial,
                   const J2IsotropicHardening &_j2IH,
                   const CLengthLaw &_cLLaw, double TaylorQuineyFactor,
                   const double KK,const double cp,const double alp,
                   const double tol=1.e-6, const bool matrixByPert=false, const double pert = 1e-8);


    void setTemperatureFunction_q1(const scalarFunction& Tfunc);
    void setTemperatureFunction_q2(const scalarFunction& Tfunc);
    void setTemperatureFunction_q3(const scalarFunction& Tfunc);

    #ifndef SWIG
    NonLocalDamageGursonDG3DMaterialLaw(const NonLocalDamageGursonDG3DMaterialLaw &source);
    virtual ~NonLocalDamageGursonDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalDamageGursonDG3DMaterialLaw(*this);};
    #endif
};



class NonLocalPorousThomasonDG3DMaterialLaw : public NonLocalPorousDuctileDamageDG3DMaterialLaw
{
  public:
    NonLocalPorousThomasonDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);

    NonLocalPorousThomasonDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);
    
    NonLocalPorousThomasonDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);

    void setYieldSurfaceExponent(const double newN);
    #ifndef SWIG
    NonLocalPorousThomasonDG3DMaterialLaw(const NonLocalPorousThomasonDG3DMaterialLaw &source);
    virtual ~NonLocalPorousThomasonDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalPorousThomasonDG3DMaterialLaw(*this);};
    #endif
};

class NonLocalPorousThomasonWithMPSDG3DMaterialLaw : public NonLocalPorousDuctileDamageDG3DMaterialLaw
{
  public:
    NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);

    NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);
    
    NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);

    void setYieldSurfaceExponent(const double newN);
    #ifndef SWIG
    NonLocalPorousThomasonWithMPSDG3DMaterialLaw(const NonLocalPorousThomasonWithMPSDG3DMaterialLaw &source);
    virtual ~NonLocalPorousThomasonWithMPSDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalPorousThomasonWithMPSDG3DMaterialLaw(*this);};
    #endif
};

class NonLocalPorousThomasonWithMSSDG3DMaterialLaw : public NonLocalPorousDuctileDamageDG3DMaterialLaw
{
  public:
    NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);

    NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);
    
    NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol=1.e-8, const bool matrixByPert = false, const double pert = 1e-8);

    #ifndef SWIG
    NonLocalPorousThomasonWithMSSDG3DMaterialLaw(const NonLocalPorousThomasonWithMSSDG3DMaterialLaw &source);
    virtual ~NonLocalPorousThomasonWithMSSDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalPorousThomasonWithMSSDG3DMaterialLaw(*this);};
    #endif
};

class NonLocalPorousCoupledDG3DMaterialLaw : public NonLocalPorousDuctileDamageDG3DMaterialLaw
{
  public:
    NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);
    
    NonLocalPorousCoupledDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    void setYieldSurfaceExponent(const double newN);
    void setCrackTransition(const bool fl, const double beta=0.);
    void setLodeOffset(const bool fl);
    void setCftOffset(const bool fl);
    void setYieldOffset(const bool fl);
    void setBulkCouplingBehaviour(const bool fl);
    void setCoalescenceTolerance(const double t);
    void useTwoYieldRegularization(const bool fl, const double n);
    
    void setGrowthVoidEvolutionLaw(const voidStateEvolutionLaw& voidtateLaw);
    void setCoalescenceVoidEvolutionLaw(const voidStateEvolutionLaw& voidtateLaw);

    #ifndef SWIG
    NonLocalPorousCoupledDG3DMaterialLaw(const NonLocalPorousCoupledDG3DMaterialLaw &source);
    virtual ~NonLocalPorousCoupledDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalPorousCoupledDG3DMaterialLaw(*this);};
    #endif
};

class NonLocalPorousCoupledWithMPSDG3DMaterialLaw : public NonLocalPorousCoupledDG3DMaterialLaw
{
  public:
    NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);
    
    NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);
    
    #ifndef SWIG
    NonLocalPorousCoupledWithMPSDG3DMaterialLaw(const NonLocalPorousCoupledWithMPSDG3DMaterialLaw &source);
    virtual ~NonLocalPorousCoupledWithMPSDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalPorousCoupledWithMPSDG3DMaterialLaw(*this);};
    #endif
};


class NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw : public NonLocalPorousCoupledDG3DMaterialLaw
{
  public:
    NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);

    NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);
    
    NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const int num,const double E,const double nu, const double rho,
                      const double q1, const double q2, const double q3,
                      const double fVinitial, const double lambda0, const double kappa, const J2IsotropicHardening &j2IH, const CLengthLaw &cLLaw,
                      double TaylorQuineyFactor, const double KK,const double cp,const double alp,
                      const double tol=1.e-8, const bool matrixbyPerturbation = false, const double pert = 1e-8);
    
    void setShearFactor(double g);
    void setVoidLigamentExponent(double e);
    void freezeCoalescenceState(const bool fl);
    void setNeckingCoalescenceLaw(const CoalescenceLaw& added_coalsLaw); 
    void setShearCoalescenceLaw(const CoalescenceLaw& added_coalsLaw); 
    void setShearVoidEvolutionLaw(const voidStateEvolutionLaw& voidtateLaw);
    #ifndef SWIG
    NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(const NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw &source);
    virtual ~NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw();
    virtual materialLaw* clone() const{ return new NonLocalPorousCoupledWithMPSAndMSSDG3DMaterialLaw(*this);};
    #endif
};


class NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw : public dG3DMaterialLaw{
  protected:
    #ifndef SWIG
    mlawNonLocalPorousWithCleavageFailure* _nlPorousWithCleavage;
    #endif // SWIG

  public:
    NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(const int num, NonLocalPorousDuctileDamageDG3DMaterialLaw& law,
                            const CLengthLaw &cLLaw,const DamageLaw &damLaw,double sigmac);
    NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(const int num, NonLocalPorousDuctileDamageDG3DMaterialLaw& law,
                            const CLengthLaw &cLLaw,const DamageLaw &damLaw, const FailureCriterionBase& failureLaw);
    #ifndef SWIG
    NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(const NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw& src);
    virtual ~NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw();

    // set the time of _nldlaw
    virtual void setTime(const double t,const double dtime){
      dG3DMaterialLaw::setTime(t,dtime);
      _nlPorousWithCleavage->setTime(t,dtime);
    }
    virtual materialLaw::matname getType() const {return _nlPorousWithCleavage->getType();}

    virtual void createIPState(IPStateBase* &ips,const bool* state_=NULL,const MElement *ele=NULL, const int nbFF_=0, const IntPt *GP=NULL, const int gpt = 0) const;
    virtual void createIPVariable(IPVariable* &ipv,const MElement *ele, const int nbFF_, const IntPt *GP, const int gpt) const;
    virtual void initLaws(const std::map<int,materialLaw*> &maplaw){}
    virtual double soundSpeed() const{return _nlPorousWithCleavage->soundSpeed();} // or change value ??
    virtual void checkInternalState(IPVariable* ipv, const IPVariable* ipvp) const;

    virtual double scaleFactor() const {return _nlPorousWithCleavage->scaleFactor();}

    virtual bool withEnergyDissipation() const {return _nlPorousWithCleavage->withEnergyDissipation();};
    virtual void setMacroSolver(const nonLinearMechSolver* sv){
      dG3DMaterialLaw::setMacroSolver(sv);
      if (_nlPorousWithCleavage != NULL){
        _nlPorousWithCleavage->setMacroSolver(sv);
      }
    }
    virtual void stress(IPVariable*ipv, const IPVariable*ipvprev, const bool stiff=true, const bool checkfrac=true);
    virtual materialLaw* clone() const  {return new NonLocalPorousDuctileDamageWithCleavageDG3DMaterialLaw(*this);};
    #endif // SWIG
};
#endif //_NONLOCALDAMAGEDG3DMATERIALLAW_H_
