//
// C++ Interface: nonLocalDamageQuadYieldHyperDG3DIPVariable
//
// Description:
//
//
// Author: V.D. NGUYEN 2014
//
// Copyright: See COPYING file that comes with this distribution
//
#include "nonLocalDamageHyperelasticDG3DIPVariable.h"
#include "ipField.h"
#include "restartManager.h"
nonLocalDamageHyperelasticDG3DIPVariable::nonLocalDamageHyperelasticDG3DIPVariable(const mlawNonLocalDamagePowerYieldHyper &mlaw, const bool oninter)
          :dG3DIPVariable(oninter,1)
{
  _nldHyperipv = new IPHyperViscoElastoPlasticNonLocalDamage(mlaw.getConstRefToCompressionHardening(),
                                                            mlaw.getConstRefToTractionHardening(),
                                                            mlaw.getConstRefToShearHardening(),
                                                            mlaw.getConstRefToKinematicHardening(),
                                                            mlaw.getViscoElasticNumberOfElement(),
                                                            mlaw.getCLengthLaw(),mlaw.getDamageLaw());
}

  nonLocalDamageHyperelasticDG3DIPVariable::nonLocalDamageHyperelasticDG3DIPVariable(const nonLocalDamageHyperelasticDG3DIPVariable &source):
          dG3DIPVariable(source)
{
  _nldHyperipv = new IPHyperViscoElastoPlasticNonLocalDamage(*source._nldHyperipv);
}

nonLocalDamageHyperelasticDG3DIPVariable& nonLocalDamageHyperelasticDG3DIPVariable::operator=(const IPVariable &source){
  dG3DIPVariable::operator=(source);
  const nonLocalDamageHyperelasticDG3DIPVariable* src = dynamic_cast<const nonLocalDamageHyperelasticDG3DIPVariable*>(&source);
  if(src != NULL)
  {
    _nldHyperipv->operator= (*dynamic_cast<const IPVariable*>(src->_nldHyperipv));
  }
  return *this;
}

double nonLocalDamageHyperelasticDG3DIPVariable::get(const int i) const
{
  if(i == IPField::PLASTICSTRAIN)
  {
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getCurrentPlasticStrain();
  }
  else if (i == IPField::NONLOCAL_PLASTICSTRAIN){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getEffectivePlasticStrain();
  }
  else if(i == IPField::DAMAGE)
  {
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getDamage();
  }
  else if (i == IPField::DAMAGE_0){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getDamage();
  }
  else if (i == IPField::PLASTIC_POISSON_RATIO){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->_nup;
  }
  else if (i == IPField::PLASTICSTRAIN_COMPRESSION){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getConstRefToCompressionPlasticStrain();
  }
  else if (i == IPField::PLASTICSTRAIN_TRACTION){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getConstRefToTractionPlasticStrain();
  }
  else if (i == IPField::PLASTICSTRAIN_SHEAR){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getConstRefToShearPlasticStrain();
  }
  else if (i == IPField::BACKSTRESS){
   if (getIPHyperViscoElastoPlasticNonLocalDamage()->_ipKinematic!= NULL)
      return getIPHyperViscoElastoPlasticNonLocalDamage()->_ipKinematic->getR();
    else
      return 0.;
  }
  else if (i == IPField::PLASTIC_RATE){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getConstRefToPlasticDeformationRate();
  }
  else if (i == IPField::K_XX){
    const STensor3& K = getIPHyperViscoElastoPlasticNonLocalDamage()->_kirchhoff;
    return K(0,0);
  }
  else if (i == IPField::K_YY){
    const STensor3& K = getIPHyperViscoElastoPlasticNonLocalDamage()->_kirchhoff;
    return K(1,1);
  }
  else if (i == IPField::K_ZZ){
    const STensor3& K = getIPHyperViscoElastoPlasticNonLocalDamage()->_kirchhoff;
    return K(2,2);
  }
  else if (i == IPField::K_XY){
    const STensor3& K = getIPHyperViscoElastoPlasticNonLocalDamage()->_kirchhoff;
    return K(0,1);
  }
  else if (i == IPField::K_YZ){
    const STensor3& K = getIPHyperViscoElastoPlasticNonLocalDamage()->_kirchhoff;
    return K(1,2);
  }
  else if (i == IPField::K_XZ){
    const STensor3& K = getIPHyperViscoElastoPlasticNonLocalDamage()->_kirchhoff;
    return K(0,2);
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}
double nonLocalDamageHyperelasticDG3DIPVariable::defoEnergy() const
{
  return getIPHyperViscoElastoPlasticNonLocalDamage()->defoEnergy();
}
double nonLocalDamageHyperelasticDG3DIPVariable::plasticEnergy() const
{
  return getIPHyperViscoElastoPlasticNonLocalDamage()->plasticEnergy();
}

double nonLocalDamageHyperelasticDG3DIPVariable::getConstRefToLocalVariable(const int idex) const{
  if (idex == 0){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getCurrentPlasticStrain();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};
double& nonLocalDamageHyperelasticDG3DIPVariable::getRefToLocalVariable(const int idex){
  if (idex == 0){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getRefToCurrentPlasticStrain();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};

double nonLocalDamageHyperelasticDG3DIPVariable::getConstRefToNonLocalVariable(const int idex) const{
  if (idex == 0){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getEffectivePlasticStrain();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};
double& nonLocalDamageHyperelasticDG3DIPVariable::getRefToNonLocalVariable(const int idex){
  if (idex == 0){
    return getIPHyperViscoElastoPlasticNonLocalDamage()->getRefToEffectivePlasticStrain();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};

void nonLocalDamageHyperelasticDG3DIPVariable::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_nldHyperipv);
  return;
}


localDamageHyperelasticDG3DIPVariableWithFailure::localDamageHyperelasticDG3DIPVariableWithFailure(const mlawLocalDamagePowerYieldHyperWithFailure &mlaw, const bool oninter)
          :dG3DIPVariable(oninter)
{
  _localIPv = new IPHyperViscoElastoPlasticMultipleLocalDamage(mlaw.getConstRefToCompressionHardening(),
                                                            mlaw.getConstRefToTractionHardening(),
                                                            mlaw.getConstRefToShearHardening(),
                                                            mlaw.getConstRefToKinematicHardening(),
                                                            mlaw.getViscoElasticNumberOfElement(),
                                                            mlaw.getDamageLaw());
}

localDamageHyperelasticDG3DIPVariableWithFailure::localDamageHyperelasticDG3DIPVariableWithFailure(const localDamageHyperelasticDG3DIPVariableWithFailure &source):
          dG3DIPVariable(source)
{
  _localIPv = new IPHyperViscoElastoPlasticMultipleLocalDamage(*source._localIPv);
}

localDamageHyperelasticDG3DIPVariableWithFailure& localDamageHyperelasticDG3DIPVariableWithFailure::operator=(const IPVariable &source){
  dG3DIPVariable::operator=(source);
  const localDamageHyperelasticDG3DIPVariableWithFailure* src = dynamic_cast<const localDamageHyperelasticDG3DIPVariableWithFailure*>(&source);
  if(src != NULL)
  {
    _localIPv->operator= (*dynamic_cast<const IPVariable*>(src->_localIPv));
  }
  return *this;
}

double localDamageHyperelasticDG3DIPVariableWithFailure::get(const int i) const
{
  if(i == IPField::PLASTICSTRAIN)
  {
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getConstRefToEqPlasticStrain();
  }
  else if (i == IPField::FAILURE_ONSET){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getFailureOnset();
  }
  else if (i == IPField::FAILURE_PLASTICSTRAIN){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getFailurePlasticity();
  }
  else if(i == IPField::DAMAGE)
  {
    double OneminusD = 1;
    int n =  this->getIPHyperViscoElastoPlasticMultipleLocalDamage()->getNumNonLocalVariable();
    for (int iter =0; iter < n; iter ++){
      double Diter = this->getIPHyperViscoElastoPlasticMultipleLocalDamage()->getDamage(iter);
      OneminusD *= (1.- Diter);
    }
    return (1.- OneminusD);
  }
  else if (i == IPField::DAMAGE_0){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getDamage(0);
  }
  else if (i == IPField::DAMAGE_1){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getDamage(1);
  }
  else if (i == IPField::PLASTIC_POISSON_RATIO){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->_nup;
  }
  else if (i == IPField::PLASTICSTRAIN_COMPRESSION){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getConstRefToCompressionPlasticStrain();
  }
  else if (i == IPField::PLASTICSTRAIN_TRACTION){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getConstRefToTractionPlasticStrain();
  }
  else if (i == IPField::PLASTICSTRAIN_SHEAR){
    return getIPHyperViscoElastoPlasticMultipleLocalDamage()->getConstRefToShearPlasticStrain();
  }
  else if (i == IPField::BACKSTRESS){
   if (getIPHyperViscoElastoPlasticMultipleLocalDamage()->_ipKinematic!= NULL)
      return getIPHyperViscoElastoPlasticMultipleLocalDamage()->_ipKinematic->getR();
    else
      return 0.;
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}

void localDamageHyperelasticDG3DIPVariableWithFailure::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_localIPv);
  return;
}




nonLocalDamageHyperelasticDG3DIPVariableWithFailure::nonLocalDamageHyperelasticDG3DIPVariableWithFailure(const mlawNonLocalDamagePowerYieldHyperWithFailure &mlaw, const bool oninter)
          :dG3DIPVariable(oninter,2)
{
  _nldHyperipv = new IPHyperViscoElastoPlasticMultipleNonLocalDamage(mlaw.getConstRefToCompressionHardening(),
                                                            mlaw.getConstRefToTractionHardening(),
                                                            mlaw.getConstRefToShearHardening(),
                                                            mlaw.getConstRefToKinematicHardening(),
                                                            mlaw.getViscoElasticNumberOfElement(),
                                                            mlaw.getCLengthLaw(),mlaw.getDamageLaw());
}

  nonLocalDamageHyperelasticDG3DIPVariableWithFailure::nonLocalDamageHyperelasticDG3DIPVariableWithFailure(const nonLocalDamageHyperelasticDG3DIPVariableWithFailure &source):
          dG3DIPVariable(source)
{
  _nldHyperipv = new IPHyperViscoElastoPlasticMultipleNonLocalDamage(*source._nldHyperipv);
}

nonLocalDamageHyperelasticDG3DIPVariableWithFailure& nonLocalDamageHyperelasticDG3DIPVariableWithFailure::operator=(const IPVariable &source){
  dG3DIPVariable::operator=(source);
  const nonLocalDamageHyperelasticDG3DIPVariableWithFailure* src = dynamic_cast<const nonLocalDamageHyperelasticDG3DIPVariableWithFailure*>(&source);
  if(src != NULL)
  {
    _nldHyperipv->operator= (*dynamic_cast<const IPVariable*>(src->_nldHyperipv));
  }
  return *this;
}

double nonLocalDamageHyperelasticDG3DIPVariableWithFailure::get(const int i) const
{
  if(i == IPField::PLASTICSTRAIN)
  {
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getCurrentPlasticStrain();
  }
  else if (i == IPField::NONLOCAL_PLASTICSTRAIN){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getEffectivePlasticStrain();
  }
  else if (i == IPField::FAILURE_ONSET){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getFailureOnset();
  }
  else if (i == IPField::FAILURE_PLASTICSTRAIN){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getFailurePlasticity();
  }
  else if (i == IPField::NONLOCAL_FAILURE_PLASTICSTRAIN){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getNonLocalFailurePlasticity();
  }
  else if(i == IPField::DAMAGE)
  {
    double OneminusD = 1;
    int n =  this->getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getNumNonLocalVariable();
    for (int iter =0; iter < n; iter ++){
      double Diter = this->getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getDamage(iter);
      OneminusD *= (1.- Diter);
    }
    return (1.- OneminusD);
  }
  else if (i == IPField::DAMAGE_0){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getDamage(0);
  }
  else if (i == IPField::DAMAGE_1){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getDamage(1);
  }
  else if (i == IPField::PLASTIC_POISSON_RATIO){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_nup;
  }
  else if (i == IPField::PLASTICSTRAIN_COMPRESSION){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getConstRefToCompressionPlasticStrain();
  }
  else if (i == IPField::PLASTICSTRAIN_TRACTION){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getConstRefToTractionPlasticStrain();
  }
  else if (i == IPField::PLASTICSTRAIN_SHEAR){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getConstRefToShearPlasticStrain();
  }
  else if (i == IPField::BACKSTRESS){
   if (getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_ipKinematic!= NULL)
      return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_ipKinematic->getR();
    else
      return 0.;
  }
  else if (i == IPField::PLASTIC_RATE){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getConstRefToPlasticDeformationRate();
  }
  else if (i == IPField::K_XX){
    const STensor3& K = getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_kirchhoff;
    return K(0,0);
  }
  else if (i == IPField::K_YY){
    const STensor3& K = getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_kirchhoff;
    return K(1,1);
  }
  else if (i == IPField::K_ZZ){
    const STensor3& K = getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_kirchhoff;
    return K(2,2);
  }
  else if (i == IPField::K_XY){
    const STensor3& K = getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_kirchhoff;
    return K(0,1);
  }
  else if (i == IPField::K_YZ){
    const STensor3& K = getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_kirchhoff;
    return K(1,2);
  }
  else if (i == IPField::K_XZ){
    const STensor3& K = getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->_kirchhoff;
    return K(0,2);
  }
  else
  {
    return dG3DIPVariable::get(i);
  }
}
double nonLocalDamageHyperelasticDG3DIPVariableWithFailure::defoEnergy() const
{
  return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->defoEnergy();
}
double nonLocalDamageHyperelasticDG3DIPVariableWithFailure::plasticEnergy() const
{
  return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->plasticEnergy();
}

double nonLocalDamageHyperelasticDG3DIPVariableWithFailure::getConstRefToLocalVariable(const int idex) const{
  if (idex == 0){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getCurrentPlasticStrain();
  }
  else if (idex == 1){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getFailurePlasticity();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};
double& nonLocalDamageHyperelasticDG3DIPVariableWithFailure::getRefToLocalVariable(const int idex){
  if (idex == 0){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getRefToCurrentPlasticStrain();
  }
  else if (idex == 1){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getRefToFailurePlasticity();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};

double nonLocalDamageHyperelasticDG3DIPVariableWithFailure::getConstRefToNonLocalVariable(const int idex) const{
  if (idex == 0){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getEffectivePlasticStrain();
  }
  else if (idex == 1){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getNonLocalFailurePlasticity();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};
double& nonLocalDamageHyperelasticDG3DIPVariableWithFailure::getRefToNonLocalVariable(const int idex){
  if (idex == 0){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getRefToEffectivePlasticStrain();
  }
  else if (idex == 1){
    return getIPHyperViscoElastoPlasticMultipleNonLocalDamage()->getRefToNonLocalFailurePlasticity();
  }
  else{
    Msg::Error("the non-local variable %d is not defined",idex);
  }
};

void nonLocalDamageHyperelasticDG3DIPVariableWithFailure::restart()
{
  dG3DIPVariable::restart();
  restartManager::restart(_nldHyperipv);
  return;
}

