//
// C++ Interface: nonLocalDamageQuadYieldHyperDG3DIPVariable
//
// Description:
//
//
// Author: V.D. NGUYEN 2014
//
// Copyright: See COPYING file that comes with this distribution
//
#ifndef NONLOCALDAMAGEHYPERELASTICDG3DIPVARIABLE_H_
#define NONLOCALDAMAGEHYPERELASTICDG3DIPVARIABLE_H_

#include "nonLocalDamageDG3DIPVariable.h"
#include "mlawNonLocalDamageHyperelastic.h"

class nonLocalDamageHyperelasticDG3DIPVariable: public dG3DIPVariable{
  protected:
    IPHyperViscoElastoPlasticNonLocalDamage *_nldHyperipv;

  public:
    nonLocalDamageHyperelasticDG3DIPVariable(const mlawNonLocalDamagePowerYieldHyper &mlaw, const bool oninter=false);
    nonLocalDamageHyperelasticDG3DIPVariable(const nonLocalDamageHyperelasticDG3DIPVariable &source);
    virtual nonLocalDamageHyperelasticDG3DIPVariable& operator=(const IPVariable &source);

   /* specific function */
    IPHyperViscoElastoPlasticNonLocalDamage* getIPHyperViscoElastoPlasticNonLocalDamage(){return _nldHyperipv;}
    const IPHyperViscoElastoPlasticNonLocalDamage* getIPHyperViscoElastoPlasticNonLocalDamage() const{return _nldHyperipv;}
    virtual ~nonLocalDamageHyperelasticDG3DIPVariable()
    {
      if (_nldHyperipv)
      delete _nldHyperipv;
      _nldHyperipv = NULL;
    }

    virtual void setLocation(const IPVariable::LOCATION loc) {
      dG3DIPVariable::setLocation(loc);
      if (_nldHyperipv)
        _nldHyperipv->setLocation(loc);
    };

    virtual IPVariable* getInternalData() {return _nldHyperipv;};
    virtual const IPVariable* getInternalData()  const {return _nldHyperipv;};

    virtual bool dissipationIsActive() const {return _nldHyperipv->dissipationIsActive();};
    virtual void blockDissipation(const bool fl){   _nldHyperipv->blockDissipation(fl);}
    virtual bool dissipationIsBlocked() const { return _nldHyperipv->dissipationIsBlocked();}

     //---get effective data when using coupled damage model
    virtual void setBulkCriticalDamage(const double DT){_nldHyperipv->setCriticalDamage(DT);};
    virtual double getBulkCriticalDamage() const {return _nldHyperipv->getCriticalDamage();};

    virtual double getConstRefToLocalVariable(const int idex) const;
    virtual double& getRefToLocalVariable(const int idex);

    virtual double getConstRefToNonLocalVariable(const int idex) const;
    virtual double& getRefToNonLocalVariable(const int idex);


    virtual double get(const int i) const;
    virtual double defoEnergy() const;
    virtual double plasticEnergy() const;
    virtual double damageEnergy() const {return _nldHyperipv->damageEnergy();};
    virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const
    {
      if (idex == 0)
        return _nldHyperipv->getConstRefToCharacteristicLength();
      else
        Msg::Error("the non-local damge id = %d is not defined",idex);
    }

    // for path following based on irreversibe energt
    virtual double irreversibleEnergy() const {return _nldHyperipv->irreversibleEnergy();};
    virtual double& getRefToIrreversibleEnergy() {return _nldHyperipv->getRefToIrreversibleEnergy();};

    virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const
      {return _nldHyperipv->getConstRefToDIrreversibleEnergyDF();};
    virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient()
      {return _nldHyperipv->getRefToDIrreversibleEnergyDF();};

    virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const
    {
      if (index == 0)
        {
        return _nldHyperipv->getDIrreversibleEnergyDNonLocalVariable();
        }
      else{Msg::Error("the non-local variable %d is not defined",index);}
    };
    virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index)
    {
      if (index == 0)
      {
        return _nldHyperipv->getRefToDIrreversibleEnergyDNonLocalVariable();
      }
      else{Msg::Error("the non-local variable %d is not defined",index);}
    };

    virtual IPVariable* clone() const {return new nonLocalDamageHyperelasticDG3DIPVariable(*this);};
    virtual void restart();
};

class localDamageHyperelasticDG3DIPVariableWithFailure : public dG3DIPVariable{

 protected:
  IPHyperViscoElastoPlasticMultipleLocalDamage *_localIPv;

 public:
  localDamageHyperelasticDG3DIPVariableWithFailure(const mlawLocalDamagePowerYieldHyperWithFailure &mlaw, const bool oninter=false);
  localDamageHyperelasticDG3DIPVariableWithFailure(const localDamageHyperelasticDG3DIPVariableWithFailure &source);
  virtual localDamageHyperelasticDG3DIPVariableWithFailure& operator=(const IPVariable &source);

 /* specific function */
  IPHyperViscoElastoPlasticMultipleLocalDamage* getIPHyperViscoElastoPlasticMultipleLocalDamage(){return _localIPv;}
  const IPHyperViscoElastoPlasticMultipleLocalDamage* getIPHyperViscoElastoPlasticMultipleLocalDamage() const{return _localIPv;}
  virtual ~localDamageHyperelasticDG3DIPVariableWithFailure()
  {
    if (_localIPv) delete _localIPv;
    _localIPv = NULL;
  }

  virtual void setLocation(const IPVariable::LOCATION loc) {
    dG3DIPVariable::setLocation(loc);
    if (_localIPv)
      _localIPv->setLocation(loc);
  };

  virtual IPVariable* getInternalData() {return _localIPv;};
  virtual const IPVariable* getInternalData()  const {return _localIPv;};

  virtual bool dissipationIsActive() const {return _localIPv->dissipationIsActive();};
  virtual void blockDissipation(const bool fl){   _localIPv->blockDissipation(fl);}
  virtual bool dissipationIsBlocked() const { return _localIPv->dissipationIsBlocked();}

  virtual void setBulkCriticalDamage(const double DT){_localIPv->setCriticalDamage(DT);}; // set critical damage value
  virtual double getBulkCriticalDamage() const {return _localIPv->getCriticalDamage();};

  virtual double get(const int i) const;
  virtual double defoEnergy() const {return _localIPv->defoEnergy();}
  virtual double plasticEnergy() const {return _localIPv->plasticEnergy();};
  virtual double damageEnergy() const {return _localIPv->damageEnergy();};

	// for path following based on  irreversibe energt
	virtual double irreversibleEnergy() const {return _localIPv->irreversibleEnergy();};
	virtual double& getRefToIrreversibleEnergy() {return _localIPv->getRefToIrreversibleEnergy();};

	virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {return _localIPv->getConstRefToDIrreversibleEnergyDF();};
	virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient() {return _localIPv->getRefToDIrreversibleEnergyDF();};

  virtual IPVariable* clone() const {return new localDamageHyperelasticDG3DIPVariableWithFailure(*this);};
  virtual void restart();
};


class nonLocalDamageHyperelasticDG3DIPVariableWithFailure: public dG3DIPVariable{
  protected:
    IPHyperViscoElastoPlasticMultipleNonLocalDamage *_nldHyperipv;

  public:
    nonLocalDamageHyperelasticDG3DIPVariableWithFailure(const mlawNonLocalDamagePowerYieldHyperWithFailure &mlaw, const bool oninter=false);
    nonLocalDamageHyperelasticDG3DIPVariableWithFailure(const nonLocalDamageHyperelasticDG3DIPVariableWithFailure &source);
    virtual nonLocalDamageHyperelasticDG3DIPVariableWithFailure& operator=(const IPVariable &source);

   /* specific function */
    IPHyperViscoElastoPlasticMultipleNonLocalDamage* getIPHyperViscoElastoPlasticMultipleNonLocalDamage(){return _nldHyperipv;}
    const IPHyperViscoElastoPlasticMultipleNonLocalDamage* getIPHyperViscoElastoPlasticMultipleNonLocalDamage() const{return _nldHyperipv;}
    virtual ~nonLocalDamageHyperelasticDG3DIPVariableWithFailure()
    {
      if (_nldHyperipv)
      delete _nldHyperipv;
      _nldHyperipv = NULL;
    }

    virtual void setLocation(const IPVariable::LOCATION loc) {
      dG3DIPVariable::setLocation(loc);
      if (_nldHyperipv)
        _nldHyperipv->setLocation(loc);
    };

    virtual IPVariable* getInternalData() {return _nldHyperipv;};
    virtual const IPVariable* getInternalData()  const {return _nldHyperipv;};

    virtual bool dissipationIsActive() const {return _nldHyperipv->dissipationIsActive();};
    virtual void blockDissipation(const bool fl){   _nldHyperipv->blockDissipation(fl);}
    virtual bool dissipationIsBlocked() const { return _nldHyperipv->dissipationIsBlocked();}

     //---get effective data when using coupled damage model
    virtual void setBulkCriticalDamage(const double DT){_nldHyperipv->setCriticalDamage(DT);};
    virtual double getBulkCriticalDamage() const {return _nldHyperipv->getCriticalDamage();};

    virtual double getConstRefToLocalVariable(const int idex) const;
    virtual double& getRefToLocalVariable(const int idex);

    virtual double getConstRefToNonLocalVariable(const int idex) const;
    virtual double& getRefToNonLocalVariable(const int idex);


    virtual double get(const int i) const;
    virtual double defoEnergy() const;
    virtual double plasticEnergy() const;
    virtual double damageEnergy() const {return _nldHyperipv->damageEnergy();};

    virtual const STensor3 &getConstRefToCharacteristicLengthMatrix(const int idex) const
    {
      return _nldHyperipv->getConstRefToCharacteristicLength(idex);
    }

    // for path following based on irreversibe energt
    virtual double irreversibleEnergy() const {return _nldHyperipv->irreversibleEnergy();};
    virtual double& getRefToIrreversibleEnergy() {return _nldHyperipv->getRefToIrreversibleEnergy();};

    virtual const STensor3& getConstRefToDIrreversibleEnergyDDeformationGradient() const {
      return _nldHyperipv->getConstRefToDIrreversibleEnergyDF();
    };
    virtual STensor3& getRefToDIrreversibleEnergyDDeformationGradient(){
      return _nldHyperipv->getRefToDIrreversibleEnergyDF();
    };

    virtual const double& getConstRefToDIrreversibleEnergyDNonLocalVariable(const int index) const
    {
      return _nldHyperipv->getDIrreversibleEnergyDNonLocalVariable(index);
    };
    virtual double& getRefToDIrreversibleEnergyDDNonLocalVariable(const int index)
    {
      return _nldHyperipv->getRefToDIrreversibleEnergyDNonLocalVariable(index);
    };

    virtual IPVariable* clone() const {return new nonLocalDamageHyperelasticDG3DIPVariableWithFailure(*this);};
    virtual void restart();
};

#endif // NONLOCALDAMAGEHYPERELASTICDG3DIPVARIABLE_H_
