//
//
// Description: pathfollowing terms
//
// Author:  <Van Dung NGUYEN>, (C) 2016
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef PATHFOLLOWINGTERMS_H_
#define PATHFOLLOWINGTERMS_H_

#include "dG3DTerms.h"
#include "ipField.h"
#include "dG3DCohesiveBandIPVariable.h"

class dG3DDissipationPathFollowingBulkScalarTerm : public ScalarTermBase<double>{
	protected:
		const IPField* _ipf;
		const partDomain* _dom;
			
	public:
		dG3DDissipationPathFollowingBulkScalarTerm(const partDomain* dom, const IPField* ip):_dom(dom),_ipf(ip){}
		virtual ~dG3DDissipationPathFollowingBulkScalarTerm(){}
		virtual void get(MElement *ele, int npts, IntPt *GP, double &val) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<double> &vval) const {
			Msg::Error("dG3DDissipationPathFollowingBulkScalarTerm::get(MElement *ele, int npts, IntPt *GP, std::vector<T2> &vval) is not defined");
		};
		virtual ScalarTermBase<double>* clone () const {return new dG3DDissipationPathFollowingBulkScalarTerm(_dom,_ipf);};
};

class dG3DDissipationPathFollowingBulkLinearTerm : public  LinearTermBase<double>{
	protected:
		const IPField* _ipf;
		const partDomain* _dom;
		
	public:
		dG3DDissipationPathFollowingBulkLinearTerm(const partDomain* dom,const IPField* ip): LinearTermBase<double>(),_dom(dom),_ipf(ip){};
		virtual ~dG3DDissipationPathFollowingBulkLinearTerm(){}
		
		virtual void get(MElement *ele,int npts,IntPt *GP,fullVector<double> &m) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullVector<double> > &mv) const
		{
			Msg::Error("Define me get by integration point dG3DDissipationPathFollowingBulkLinearTerm");
		}
		
		virtual LinearTermBase<double>* clone () const
		{
			return new dG3DDissipationPathFollowingBulkLinearTerm(_dom,_ipf);
		}
};

class dG3DEnhancedStrainDissipationPathFollowingBulkScalarTerm : public dG3DDissipationPathFollowingBulkScalarTerm{
	protected:
		
			
	public:
		dG3DEnhancedStrainDissipationPathFollowingBulkScalarTerm(const partDomain* dom, const IPField* ip):dG3DDissipationPathFollowingBulkScalarTerm(dom,ip){}
		virtual ~dG3DEnhancedStrainDissipationPathFollowingBulkScalarTerm(){}
		virtual void get(MElement *ele, int npts, IntPt *GP, double &val) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<double> &vval) const {
			Msg::Error("dG3DEnhancedStrainDissipationPathFollowingBulkScalarTerm::get(MElement *ele, int npts, IntPt *GP, std::vector<T2> &vval) is not defined");
		};
		virtual ScalarTermBase<double>* clone () const {return new dG3DEnhancedStrainDissipationPathFollowingBulkScalarTerm(_dom,_ipf);};
};

class dG3DEnhancedStrainDissipationPathFollowingBulkLinearTerm : public  dG3DDissipationPathFollowingBulkLinearTerm{
	protected:
		
		
	public:
		dG3DEnhancedStrainDissipationPathFollowingBulkLinearTerm(const partDomain* dom,const IPField* ip): dG3DDissipationPathFollowingBulkLinearTerm(dom,ip){};
		virtual ~dG3DEnhancedStrainDissipationPathFollowingBulkLinearTerm(){}
		
		virtual void get(MElement *ele,int npts,IntPt *GP,fullVector<double> &m) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullVector<double> > &mv) const
		{
			Msg::Error("Define me get by integration point dG3DEnhancedStrainDissipationPathFollowingBulkLinearTerm");
		}
		
		virtual LinearTermBase<double>* clone () const
		{
			return new dG3DEnhancedStrainDissipationPathFollowingBulkLinearTerm(_dom,_ipf);
		}
};

class dG3DDissipationPathFollowingBoundScalarTerm : public ScalarTermBase<double>{
	protected:
		const IPField* _ipf;
		const dgPartDomain* _dom;
	
	public:
		dG3DDissipationPathFollowingBoundScalarTerm(const dgPartDomain* dom, const IPField* ip):_ipf(ip),_dom(dom){}
		virtual ~dG3DDissipationPathFollowingBoundScalarTerm(){}
		virtual void get(MElement *ele, int npts, IntPt *GP, double &val) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<double> &vval) const {
			Msg::Error("dG3DDissipationPathFollowingBoundScalarTerm::get(MElement *ele, int npts, IntPt *GP, std::vector<T2> &vval) is not defined");
		};
		virtual ScalarTermBase<double>* clone () const {return new dG3DDissipationPathFollowingBoundScalarTerm(_dom,_ipf);};
};

class dG3DDissipationPathFollowingBoundLinearTerm : public  LinearTermBase<double>{
	protected:
		const IPField* _ipf;
		const dgPartDomain* _dom;
		
	public:
		dG3DDissipationPathFollowingBoundLinearTerm(const dgPartDomain* dom,const IPField* ip):_ipf(ip),_dom(dom){};
		virtual ~dG3DDissipationPathFollowingBoundLinearTerm(){}
		
		virtual void get(MElement *ele,int npts,IntPt *GP,fullVector<double> &m) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullVector<double> > &mv) const
		{
			Msg::Error("Define me get by integration point dG3DDissipationPathFollowingBoundLinearTerm");
		}
		
		virtual LinearTermBase<double>* clone () const
		{
			return new dG3DDissipationPathFollowingBoundLinearTerm(_dom,_ipf);
		}
};

class twoFieldDG3DDissipationPathFollowingBoundScalarTerm : public dG3DDissipationPathFollowingBoundScalarTerm {
	
	public:
		twoFieldDG3DDissipationPathFollowingBoundScalarTerm(const dgPartDomain* dom, const IPField* ip): 
						dG3DDissipationPathFollowingBoundScalarTerm(dom,ip){}
		virtual ~twoFieldDG3DDissipationPathFollowingBoundScalarTerm(){}
		virtual void get(MElement *ele, int npts, IntPt *GP, double &val) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<double> &vval) const {
			Msg::Error("twoFieldDG3DDissipationPathFollowingBoundScalarTerm::get(MElement *ele, int npts, IntPt *GP, std::vector<T2> &vval) is not defined");
		};
		virtual ScalarTermBase<double>* clone () const {return new twoFieldDG3DDissipationPathFollowingBoundScalarTerm(_dom,_ipf);};
}; 

class twoFieldDG3DDissipationPathFollowingBoundLinearTerm : public  dG3DDissipationPathFollowingBoundLinearTerm{
	protected:
		
	public:
		twoFieldDG3DDissipationPathFollowingBoundLinearTerm(const dgPartDomain* dom,const IPField* ip): 
								dG3DDissipationPathFollowingBoundLinearTerm(dom,ip){};
		virtual ~twoFieldDG3DDissipationPathFollowingBoundLinearTerm(){}
		
		virtual void get(MElement *ele,int npts,IntPt *GP,fullVector<double> &m) const;
		virtual void get(MElement *ele, int npts, IntPt *GP, std::vector<fullVector<double> > &mv) const
		{
			Msg::Error("Define me get by integration point twoFieldDG3DDissipationPathFollowingBoundLinearTerm");
		}
				
		virtual LinearTermBase<double>* clone () const
		{
			return new twoFieldDG3DDissipationPathFollowingBoundLinearTerm(_dom,_ipf);
		}
};


#endif // PATHFOLLOWINGTERMS_H_
